Package: EcoEnsemble
Title: A General Framework for Combining Ecosystem Models
Version: 1.2.0
Authors@R: 
    c(person(given = "Michael A.",
                family = "Spence",
                role = c("aut", "cre"),
                email = "michael.spence@cefas.gov.uk",
                comment = c(ORCID = "0000-0002-3445-7979")),
            person(given = "James A.",
                family = "Martindale",
                role = c("aut"),
                comment = c(ORCID = "0000-0002-1913-5592")),
            person(given = "Michael J.",
                family = "Thomson",
                role = c("aut"),
                comment = c(ORCID = "0000-0003-0284-0129")),
            person(given = "Thomas I. J.",
                family = "Bartos",
                role = c("aut")),
            person(given = "Luke E. K.",
                family = "Broadbent",
                role = c("aut")))
Description: Fit and sample from the ensemble model described in Spence et al (2018): "A general framework for combining ecosystem models"<doi:10.1111/faf.12310>.
License: GPL (>= 3)
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Biarch: true
URL: https://github.com/CefasRepRes/EcoEnsemble
BugReports: https://github.com/CefasRepRes/EcoEnsemble/issues
Depends: R (>= 3.5.0)
Imports: methods, Rcpp, matrixcalc, RcppParallel (>= 5.0.1), rstan (>=
        2.26.0), rstantools (>= 2.1.1), dplyr, ggplot2, reshape2,
        tibble, cowplot, posterior
LinkingTo: BH (>= 1.66.0), Rcpp, RcppEigen (>= 0.3.3.3.0), RcppParallel
        (>= 5.0.1), rstan (>= 2.26.0), StanHeaders (>= 2.26.0)
SystemRequirements: GNU make
Suggests: rmarkdown, kableExtra, knitr, testthat (>= 3.0.0), mgcv
Config/testthat/edition: 3
VignetteBuilder: knitr
Collate: 'DiscrepancyPrior-IndLTPrior-class.R'
        'DiscrepancyPrior-IndSTPrior-class.R'
        'DiscrepancyPrior-ShaSTPrior-class.R'
        'DiscrepancyPrior-TruthPrior-class.R' 'EcoEnsemble-package.R'
        'EnsemblePrior-class.R' 'EnsembleData-class.R'
        'EnsembleFit-class.R' 'EnsembleSample-class.R'
        'Prior_sampling.R' 'RcppExports.R' 'data.R' 'fit_ensemble.R'
        'fit_ensemble_withdrivers.R' 'generate_simulator_stan_data.R'
        'generate_simulator_stan_data_withdrivers.R'
        'get_samples_array.R' 'get_stan_outputs.R'
        'get_stan_outputs_withdrivers.R' 'plot_functions.R'
        'plot_functions_withdrivers.R' 'prior_functions.R'
        'stan_sampling_utils.R' 'stanmodels.R' 'validation_functions.R'
        'validation_functions_withdrivers.R'
        'validation_prior_functions.R'
NeedsCompilation: yes
Packaged: 2026-04-27 15:18:32 UTC; LB19
Author: Michael A. Spence [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3445-7979>),
  James A. Martindale [aut] (ORCID:
    <https://orcid.org/0000-0002-1913-5592>),
  Michael J. Thomson [aut] (ORCID:
    <https://orcid.org/0000-0003-0284-0129>),
  Thomas I. J. Bartos [aut],
  Luke E. K. Broadbent [aut]
Maintainer: Michael A. Spence <michael.spence@cefas.gov.uk>
Repository: CRAN
Date/Publication: 2026-04-27 17:00:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-27 21:24:52 UTC; unix
Archs: EcoEnsemble.so.dSYM

Package: EcoEnsemble
Title: A General Framework for Combining Ecosystem Models
Version: 1.2.0
Authors@R: 
    c(person(given = "Michael A.",
                family = "Spence",
                role = c("aut", "cre"),
                email = "michael.spence@cefas.gov.uk",
                comment = c(ORCID = "0000-0002-3445-7979")),
            person(given = "James A.",
                family = "Martindale",
                role = c("aut"),
                comment = c(ORCID = "0000-0002-1913-5592")),
            person(given = "Michael J.",
                family = "Thomson",
                role = c("aut"),
                comment = c(ORCID = "0000-0003-0284-0129")),
            person(given = "Thomas I. J.",
                family = "Bartos",
                role = c("aut")),
            person(given = "Luke E. K.",
                family = "Broadbent",
                role = c("aut")))
Description: Fit and sample from the ensemble model described in Spence et al (2018): "A general framework for combining ecosystem models"<doi:10.1111/faf.12310>.
License: GPL (>= 3)
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Biarch: true
URL: https://github.com/CefasRepRes/EcoEnsemble
BugReports: https://github.com/CefasRepRes/EcoEnsemble/issues
Depends: R (>= 3.5.0)
Imports: methods, Rcpp, matrixcalc, RcppParallel (>= 5.0.1), rstan (>=
        2.26.0), rstantools (>= 2.1.1), dplyr, ggplot2, reshape2,
        tibble, cowplot, posterior
LinkingTo: BH (>= 1.66.0), Rcpp, RcppEigen (>= 0.3.3.3.0), RcppParallel
        (>= 5.0.1), rstan (>= 2.26.0), StanHeaders (>= 2.26.0)
SystemRequirements: GNU make
Suggests: rmarkdown, kableExtra, knitr, testthat (>= 3.0.0), mgcv
Config/testthat/edition: 3
VignetteBuilder: knitr
Collate: 'DiscrepancyPrior-IndLTPrior-class.R'
        'DiscrepancyPrior-IndSTPrior-class.R'
        'DiscrepancyPrior-ShaSTPrior-class.R'
        'DiscrepancyPrior-TruthPrior-class.R' 'EcoEnsemble-package.R'
        'EnsemblePrior-class.R' 'EnsembleData-class.R'
        'EnsembleFit-class.R' 'EnsembleSample-class.R'
        'Prior_sampling.R' 'RcppExports.R' 'data.R' 'fit_ensemble.R'
        'fit_ensemble_withdrivers.R' 'generate_simulator_stan_data.R'
        'generate_simulator_stan_data_withdrivers.R'
        'get_samples_array.R' 'get_stan_outputs.R'
        'get_stan_outputs_withdrivers.R' 'plot_functions.R'
        'plot_functions_withdrivers.R' 'prior_functions.R'
        'stan_sampling_utils.R' 'stanmodels.R' 'validation_functions.R'
        'validation_functions_withdrivers.R'
        'validation_prior_functions.R'
NeedsCompilation: yes
Packaged: 2026-04-27 15:18:32 UTC; LB19
Author: Michael A. Spence [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3445-7979>),
  James A. Martindale [aut] (ORCID:
    <https://orcid.org/0000-0002-1913-5592>),
  Michael J. Thomson [aut] (ORCID:
    <https://orcid.org/0000-0003-0284-0129>),
  Thomas I. J. Bartos [aut],
  Luke E. K. Broadbent [aut]
Maintainer: Michael A. Spence <michael.spence@cefas.gov.uk>
Repository: CRAN
Date/Publication: 2026-04-27 17:00:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-01 22:11:02 UTC; unix
Archs: EcoEnsemble.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(plot,EnsembleSample)
export(EnsembleData)
export(EnsembleFit)
export(EnsemblePrior)
export(EnsembleSample)
export(IndLTPrior)
export(IndSTPrior)
export(KalmanFilter_back)
export(ShaSTPrior)
export(TruthPrior)
export(fit_ensemble_model)
export(gen_sample)
export(generate_sample)
export(get_mcmc_ensemble_model)
export(get_mle)
export(get_parameters)
export(get_transformed_data)
export(get_transformed_data_dri)
export(prior_ensemble_model)
export(sample_prior)
exportClasses(EnsembleData)
exportClasses(EnsembleFit)
exportClasses(EnsemblePrior)
exportClasses(EnsembleSample)
exportClasses(IndLTPrior)
exportClasses(IndSTPrior)
exportClasses(ShaSTPrior)
exportClasses(TruthPrior)
import(Rcpp)
import(cowplot)
import(dplyr)
import(ggplot2)
import(matrixcalc)
import(methods)
import(reshape2)
import(tibble)
importClassesFrom(rstan,stanfit)
importFrom(cowplot,plot_grid)
importFrom(rstan,optimizing)
importFrom(rstan,sampling)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(stats,rnorm)
useDynLib(EcoEnsemble, .registration = TRUE)

# EcoEnsemble

EcoEnsemble is an R package to set up, fit and sample from the ensemble framework described in [Spence et al (2018)](https://onlinelibrary.wiley.com/doi/abs/10.1111/faf.12310) for time series outputs.

## Installation
You can install the development version of EcoEnsemble using the `devtools` package:
``` r
library(devtools)
install_github("CefasRepRes/EcoEnsemble")
```

## Example
Fitting an ensemble model in EcoEnsemble is done in three main steps:

1. Eliciting priors on discrepancy terms: This is done by using the `EnsemblePrior()` constructor.
2. Fitting the ensemble model: Using the `fit_ensemble_model()` function with simulator outputs, observations and prior information. The ensemble model can be fit, obtaining either the point estimate, which maximises the posterior density, or running Markov chain Monte Carlo to generate a sample from the posterior denisty of the ensemble model.
3. Sampling the latent variables from the fitted model: Using the `generate_sample()` function with the fitted ensemble object, the discrepancy terms and the ensemble's best guess of the truth can be generated. Similarly to `fit_ensemble_model()`, this can either be a point estimate or a full sample.


We illustrate this process with datasets included with the package. EcoEnsemble comes loaded with the predicted biomasses of 4 species from 4 different mechanistic models of fish populations in the North Sea. It also includes statistical estimates of the biomasses from single-species stock assessments, and covariances for the model outputs and assessments. The models are run for different time periods and different species.

```{r}
library(EcoEnsemble)
# Outputs from mizer. These are logs of the biomasses for each year in a simulation.
print(SSB_miz)
```


|   |N.pout | Herring   |Cod  |Sole|
|---|---    |---        | --- |--- | 
|1984| 10.31706 |13.33601 |10.80006 |10.98139|
|1985| 12.07673 |13.63592 |10.46646 |10.87285|
|...|...|...|...|...|
|2049| 12.46354 |14.02923 |12.27473 |10.80954|
|2050| 12.46509 |14.03027 |12.27422 |10.81003|


To encode prior beliefs about how model discrepancies are related to one another, use the `EnsemblePrior()` constructor. Default values are available.

```{r}
priors <- EnsemblePrior(4)
```

or custom priors can be specified.

``` {r}
#Endoding prior beliefs. Details of the meanings of these terms can be found in the vignette or the documentation
num_species <- 4
priors <- EnsemblePrior(
  d = num_species,
  ind_st_params = IndSTPrior("lkj",  list(3, 2), 3, AR_params = c(1,1)),
  ind_lt_params = IndLTPrior(
    "beta",
    list(c(10,4,8, 7),c(2,3,1, 4)),
    list(matrix(5, num_species, num_species),
         matrix(0.5, num_species, num_species))
  ),
  sha_st_params = ShaSTPrior("inv_wishart",list(2, 1/3),list(5, diag(num_species))),
  sha_lt_params = 5,
  truth_params = TruthPrior(num_species, 10, list(3, 3), list(10, diag(num_species)))
)
```

This creates an `EnsemblePrior` object, which we can use to fit the ensemble model using the `fit_ensemble_model()` function and the data loaded with the package. When running a full MCMC sampling of the posterior, this step may take some time. Samples can then be generated from the resulting object using the `generate_sample()` function.


```{r}
fit <- fit_ensemble_model(observations = list(SSB_obs, Sigma_obs),
                          simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
                                            list(SSB_lm,  Sigma_lm,  "LeMans"),
                                            list(SSB_miz, Sigma_miz, "mizer"),
                                            list(SSB_fs,  Sigma_fs,  "FishSUMS")),
                          priors = priors)
samples <- generate_sample(fit)
```
This produces an `EnsembleSample` object containing samples of the ensemble model predictions. These can be viewed by calling the `plot()` function on this object. For a full MCMC sample, this includes ribbons giving quantiles of the ensemble outputs. If only maximising the posterior density, then only the single ouput is plotted.
```{r}
plot(samples)
```

![](man/figures/fitted_model.bmp)

## References
Spence, M. A., J. L. Blanchard, A. G. Rossberg, M. R. Heath, J. J. Heymans, S. Mackinson, N. Serpetti, D. C. Speirs, R. B. Thorpe, and P. G. Blackwell. 2018. “A General Framework for Combining Ecosystem Models.” *Fish and Fisheries* 19: 1013–42.

## ---- echo=FALSE,warning=FALSE,error=FALSE,message=FALSE----------------------
library(rstan)
library(EcoEnsemble)

## ----eval=FALSE---------------------------------------------------------------
#  inv_wish_st_mod <- stan_model(model_code="data {
#    int<lower=0> N;
#    real nu;
#    matrix[N,N] mat_cov;
#  }
#  parameters {
#    cov_matrix[N] Sigma;
#  }
#  model {
#    Sigma ~ inv_wishart(nu, mat_cov);
#  }
#  ")

## ----eval=FALSE---------------------------------------------------------------
#  fit.iw <- sampling(inv_wish_st_mod,data=list(N=4,nu=2*4,mat_cov=diag(4)))

## ----echo=FALSE---------------------------------------------------------------
load("data/ExploringPriors_invWishartDensities.RData")

## ----eval = FALSE-------------------------------------------------------------
#  ex.fit.iw <- extract(fit.iw)
#  plot(density(ex.fit.iw$Sigma[,1,1]),main="",xlab=expression(lambda),xlim=c(0,2))

## ----plotDensity, echo = FALSE------------------------------------------------
plot(invWishartDensity,main="",xlab=expression(lambda),xlim=c(0,2))

## ----eval = FALSE-------------------------------------------------------------
#  sd.2050 <- sqrt(33*ex.fit.iw$Sigma[,1,1])
#  plot(density(sqrt(33*ex.fit.iw$Sigma[,1,1])),main="",
#       xlab="standard deviation in 2050",xlim=c(0,5))

## ----plotSDDensity, echo = FALSE----------------------------------------------
plot(density(sd.2050),main="",xlab="standard deviation in 2050",xlim=c(0,5))

## -----------------------------------------------------------------------------
x <- seq(5,20,length.out=1000)
plot(x,colMeans(sapply(x, function(xs){dnorm(xs,tail(SSB_obs$Cod,n=1),sd.2050)})),
     type="l",axes=F,ylab="density",xlab="Cod in 2050 (1000 tonnes)")
labsx <- log(signif(exp(seq(5,20,length.out=7)),1))
axis(1,at=labsx,labels=c(exp(labsx)/1000))
axis(2)
abline(v=tail(SSB_obs$Cod,n=1))
abline(v=range(SSB_obs$Cod),lty=2)
box()

## -----------------------------------------------------------------------------
xs <- seq(0,5,0.001)
plot(xs,dgamma(xs,1,1),xlab=expression(pi[gamma,i]^2),ylab="density",type="l")

## -----------------------------------------------------------------------------
pis <- rgamma(1e5,1,1)
plot(density(sqrt(2 * pis)),xlab="Standard deviation",ylab="Density",main="")

## -----------------------------------------------------------------------------
xs <- seq(0,5,length.out=1000)
plot(xs,2*colMeans(sapply(xs, function(x){dnorm(x,0,sqrt(2*pis))})),
     type="l",axes=T,ylab="density",xlab="Difference of two models")

## ----eval=FALSE---------------------------------------------------------------
#  cor_pri_st <- stan_model(model_code = " functions{
#    real priors_cor_hierarchical_beta(matrix ind_st_cor, int N, matrix M){
#      real log_prior = 0;
#      for (i in 1:(N-1)){
#        for (j in (i+1):N){
#          log_prior += beta_lpdf(0.5*(ind_st_cor[i, j] + 1)| M[i, j], M[j, i] );
#        }
#      }
#      return log_prior;
#    }
#  }
#  data {
#  
#    vector[4] cor_p;
#  }
#  
#  parameters {
#    matrix <lower=0>[5,5] beta_pars;
#    corr_matrix[5] rho[4];
#  }
#  model {
#  
#    for (i in 1:4){
#      for (j in (i+1):5){
#        beta_pars[i,j] ~ gamma(cor_p[1],cor_p[2]);
#        beta_pars[j,i] ~ gamma(cor_p[3],cor_p[4]);
#      }
#    }
#    for(i in 1:4){
#      target += priors_cor_hierarchical_beta(rho[i],5,beta_pars);
#      diagonal(beta_pars) ~ std_normal();
#    }
#  }
#  ")
#  fit_cor <- sampling(cor_pri_st,data=list(cor_p=0.1 * c(1,1,1,1)),chains=4)
#  ex.fit <- extract(fit_cor)
#  def.par <- par(no.readonly=TRUE) #Old pars to reset afterwards
#  par(mfrow=c(2,2))
#  plot(density(ex.fit$beta_pars[,1,2]),xlab=expression(alpha[rho]),main="")
#  plot(density(log(ex.fit$beta_pars[,1,2]/ex.fit$beta_pars[,2,1])),main="",
#       xlab="Expected log odds")
#  plot(density(ex.fit$rho[,1,1,2]),xlab=expression(rho),main="")
#  xs <- seq(-1,1,length.out=100)
#  lines(xs,dbeta((xs+1)/2,2,2)/2,col="red")
#  plot(density(apply(ex.fit$rho[,,1,2],1,var)),xlim=c(0,0.35),main="",
#       xlab=expression(var(rho)))
#  par(def.par)

## ----echo=FALSE, fig.dim = c(7,6)---------------------------------------------
def.par <- par(no.readonly=TRUE) #old pars
load("data/ExploringPriors_HierBetaDensities.RData")
par(mfrow=c(2,2))
plot(beta_marginal,xlab=expression(alpha[rho]),main="")
plot(expected_log_odds,main="",xlab="Expected log odds")
plot(rho_density,xlab=expression(rho),main="")
xs <- seq(-1,1,length.out=100)
lines(xs,dbeta((xs+1)/2,2,2)/2,col="red")
plot(rho_var_density,xlim=c(0,0.35),main="",xlab=expression(var(rho)))

## ----eval=FALSE---------------------------------------------------------------
#  st_mod1 <- stan_model(model_code = "data {
#    vector[4] gam_p;
#  }
#  parameters {
#    vector [5] log_sha_st_var[4];
#    vector[5] gamma_mean;
#    vector<lower=0> [5] gamma_var;
#  }
#  model {
#    gamma_mean ~ normal(gam_p[1],gam_p[2]);
#    gamma_var ~ inv_gamma(gam_p[3],gam_p[4]);
#    for (i in 1:4){
#      log_sha_st_var[i] ~ normal(gamma_mean,sqrt(gamma_var));
#  
#    }
#  }
#  generated quantities{
#    vector[5] sha_st_var[4];
#    for (i in 1:4){
#      sha_st_var[i] = exp(log_sha_st_var[i]);
#    }
#  }
#  ")
#  test.fit_norm <- sampling(st_mod1,data=list(gam_p=c(-3,1,8,4)),chains=4)
#  ex.fit <- extract(test.fit_norm)
#  def.par <- par(no.readonly=TRUE) #old pars
#  par(mfrow=c(2,2))
#  plot(density(ex.fit$gamma_mean[,1]),main="",xlab=expression(mu[pi]))
#  plot(density(ex.fit$gamma_var[,1]),xlim=c(0,1),main="",xlab=expression(sigma[pi]^2))
#  plot(density(ex.fit$sha_st_var[,1,1]),xlim=c(0,1),main="",xlab=expression(pi["k,i"]^2))
#  plot(density(apply(ex.fit$sha_st_var[,,1],1,var)),xlim=c(0,0.2),
#       xlab=expression(var(pi["k,i"]^2)) ,main="")
#  par(def.par)

## ----echo = FALSE,fig.dim=c(7, 6)---------------------------------------------
load("data/ExploringPriors_HierGammaDensities.RData")
par(mfrow=c(2,2))
plot(gamma_mean,main="",xlab=expression(mu[pi]))
plot(gamma_var,xlim=c(0,1),main="",xlab=expression(sigma[pi]^2))
plot(sha_st_var,xlim=c(0,1),main="",xlab=expression(pi["k,i"]^2))
plot(sha_st_var_var,xlim=c(0,0.2),xlab=expression(var(pi["k,i"]^2)) ,main="")

## -----------------------------------------------------------------------------
hist(rbeta(1e5,2,2)*2 - 1,probability = T,xlab=expression(r["k,i"]),main="")

## -----------------------------------------------------------------------------
plot(density(rgamma(1e5,1,10)),xlim=c(0,0.5), main="",xlab=expression(pi[eta]^2))

## -----------------------------------------------------------------------------
plot(density(sqrt(rgamma(1e5,1,10))),xlim=c(0,0.5), main="",xlab=expression(pi[eta]))

## -----------------------------------------------------------------------------
hist(rbeta(1e5,2,2)*2 - 1,probability = T,xlab=expression(r["k,i"]),main="")

## ----eval=FALSE,fig.dim=c(7,6)------------------------------------------------
#  priors <- EnsemblePrior(4,
#                          ind_st_params =IndSTPrior("hierarchical",list(-3,1,8,4),
#                                                    list(0.1,0.1,0.1,0.1),AR_params=c(2,2)),
#                          ind_lt_params = IndLTPrior("lkj",list(1,1),1),
#                          sha_st_params = ShaSTPrior("lkj",list(1,10),1,AR_params=c(2,2)),
#                          sha_lt_params = 5
#  )
#  prior_density <- prior_ensemble_model(priors, M = 4)
#  samples <- sample_prior(observations = list(SSB_obs, Sigma_obs),
#               simulators = list(list(SSB_ewe, Sigma_ewe,"EwE"),
#                     list(SSB_fs ,  Sigma_fs,"FishSums"),
#                     list(SSB_lm ,  Sigma_lm,"LeMans"),
#                     list(SSB_miz, Sigma_miz,"mizer")),
#               priors=priors,
#               sam_priors = prior_density)
#  plot(samples)

## ----echo =FALSE,fig.dim=c(7,6)-----------------------------------------------
knitr::include_graphics("data/p_priors.png")

## ----setup_lies,warning=FALSE-------------------------------------------------
library(EcoEnsemble)
set.seed(1234)

## ----eval = FALSE, message=FALSE, warning=FALSE, results='hide', silent=TRUE----
#  cor_pri_st <- rstan::stan_model(model_code = " functions{
#    real priors_cor_hierarchical_beta(matrix ind_st_cor, int N, matrix M){
#      real log_prior = 0;
#      for (i in 1:(N-1)){
#        for (j in (i+1):N){
#          log_prior += beta_lpdf(0.5*(ind_st_cor[i, j] + 1)| M[i, j], M[j, i] );
#        }
#      }
#      return log_prior;
#    }
#  
#    real beta_conj_prior(real alpha, real betas, real r, real s, real k){
#      real rl = 1/(1 + exp(-r));
#      real sl = 1/(1 + exp(-s));
#      real p = (sl * rl)^k;
#      real q = (sl * (1 - rl))^k;
#      real ret = alpha * log(p) + betas * log(q) - k * lbeta(alpha,betas);
#      return ret;
#    }
#  
#  }
#  
#  data {
#    vector[3] cor_p;
#  }
#  
#  parameters {
#    matrix <lower=0>[5,5] beta_pars;
#    corr_matrix[5] rho[4];
#  }
#  
#  model {
#    for (i in 1:4){
#      for (j in (i+1):5){
#        target += beta_conj_prior(beta_pars[i,j], beta_pars[j,i], cor_p[1], cor_p[2], cor_p[3]);
#      }
#    }
#  
#    for (i in 1:4){
#      target += priors_cor_hierarchical_beta(rho[i],5,beta_pars);
#      diagonal(beta_pars) ~ std_normal();
#    }
#  
#  }
#  
#  generated quantities {
#  
#    matrix [5,5] rhovars;
#    for (i in 1:4){
#      for (j in (i+1):5){
#        rhovars[i,j] = 4 * (beta_pars[i,j] * beta_pars[j,i])/(square(beta_pars[i,j] + beta_pars[j,i]) * (beta_pars[i,j] + beta_pars[j,i] + 1));
#        rhovars[j,i] = (2 * beta_pars[i,j]/(beta_pars[i,j] + beta_pars[j,i])) - 1;
#      }
#    }
#  
#    for (i in 1:5){
#      rhovars[i,i] = 4;
#    }
#  
#  }
#  ")
#  library(ggplot2)
#  rhoplots <- list()
#  kvals <- c(0.05, 5)
#  parvals <- do.call(expand.grid, c(rep(list(c(0.25, 3)), 2), list(kvals)))
#  #Sampling and gathering outputs for plotting
#  for (i in 1:nrow(parvals)){
#    fit_cor <- rstan::sampling(cor_pri_st, data = list(cor_p=as.numeric(parvals[i,])), iter = 2000, chains=4)
#    ex.fit <- rstan::extract(fit_cor)
#    rho_density <- density(ex.fit$rho[,1,1,2], from = -1, to = 1)
#    rhoplot_data <- data.frame(rho_density$x, rho_density$y)
#    names(rhoplot_data) <- c("rho", "Density")
#    rhoplot_data <- cbind(rhoplot_data, rep(parvals[i,1], nrow(rhoplot_data)), rep(parvals[i,2], nrow(rhoplot_data)), rep(parvals[i,3], nrow(rhoplot_data)))
#    names(rhoplot_data)[3:5] <- c("r", "s", "k")
#    rhoplots[[i]] <- rhoplot_data
#  }
#  #Construct plots
#  rhoplot_data <- do.call(rbind, rhoplots)
#  rhoplot_data <- cbind(rhoplot_data, rep("Beta conjugate prior", nrow(rhoplot_data)))
#  names(rhoplot_data)[6] <- "Prior"
#  unif_range <- seq(-1, 1, length.out = nrow(rhoplots[[1]]))
#  unif_data <- data.frame(cbind(rep(unif_range, nrow(parvals)), rep(dbeta((unif_range+1)/2,5/2,5/2)/2, nrow(parvals)), rhoplot_data[,3:5], rep("Uniform prior", nrow(rhoplot_data))))
#  names(unif_data) <- names(rhoplot_data)
#  rhoplot_data <- rbind(rhoplot_data, unif_data)
#  rhoplot1 <- ggplot(rhoplot_data[which(rhoplot_data$k == kvals[1]),]) + geom_area(aes(x = rho, y = Density, color = Prior, fill = Prior), alpha = 0.3, position = "identity") + scale_x_continuous(bquote(rho), sec.axis = sec_axis(~., name = "r", breaks = NULL, labels = NULL)) + scale_y_continuous(sec.axis = sec_axis(~., name = "s", breaks = NULL, labels = NULL)) + theme(aspect.ratio = 1) + facet_grid(rows = vars(r), cols = vars(s)) + scale_color_manual(values = c("blue", "green")) + scale_fill_manual(values = c("blue", "green"))
#  rhoplot2 <- ggplot(rhoplot_data[which(rhoplot_data$k == kvals[2]),], aes(x = rho, y = Density)) + geom_area(aes(color = Prior, fill = Prior), alpha = 0.3, position = "identity") + scale_x_continuous(bquote(rho), sec.axis = sec_axis(~., name = "r", breaks = NULL, labels = NULL)) + scale_y_continuous(sec.axis = sec_axis(~., name = "s", breaks = NULL, labels = NULL)) + theme(aspect.ratio = 1) + facet_grid(rows = vars(r), cols = vars(s)) + scale_color_manual(values = c("blue", "green")) + scale_fill_manual(values = c("blue", "green"))

## ---- echo = FALSE, out.width="700px", out.height="700px"---------------------
knitr::include_graphics("data/rhoplot1.png")

## ---- eval = FALSE, fig.dim = c(7,6), echo = FALSE----------------------------
#  rhoplot1

## ---- echo = FALSE, out.width="700px", out.height="700px"---------------------
knitr::include_graphics("data/rhoplot2.png")

## ---- eval = FALSE, fig.dim = c(7,6), echo = FALSE----------------------------
#  rhoplot2

## ---- message=FALSE, warning=FALSE,eval=FALSE, results='hide', silent=TRUE----
#  kvals <- c(0.1, 0.2, 0.4, 0.8, 1.6, 3.2)
#  #parvals <- cbind(rep(0.3, 6), rep(-1, 6), kvals)
#  parvals <- cbind(rep(-0.3, 6), rep(3, 6), kvals)
#  rhovarplots <- list()
#  rhomeanplots <- list()
#  for (i in 1:6){
#    fit_cor <- rstan::sampling(cor_pri_st, data = list(cor_p=parvals[i,]), iter = 2000, chains=4)
#    ex.fit <- rstan::extract(fit_cor)
#    rhovar_density <- density(ex.fit$rhovars[,1,2])
#    rhomean_density <- density(ex.fit$rhovars[,2,1])
#    rhovar_data <- data.frame(cbind(rhovar_density$x, rhovar_density$y, rep(kvals[i], length(rhovar_density$x))))
#    rhomean_data <- data.frame(cbind(rhomean_density$x, rhomean_density$y, rep(kvals[i], length(rhomean_density$x))))
#    names(rhovar_data) <- c("Variance", "Density", "k")
#    names(rhomean_data) <- c("Expectation", "Density", "k")
#    rhovarplots[[i]] <- rhovar_data
#    rhomeanplots[[i]] <- rhomean_data
#  }
#  rhovarplot_data <- do.call(rbind, rhovarplots)
#  rhomeanplot_data <- do.call(rbind, rhomeanplots)
#  rhovarplot <- ggplot(rhovarplot_data) + geom_area(aes(x = Variance, y = Density), color = "blue", fill = "blue", alpha = 0.3, position = "identity") + geom_vline(xintercept = 0.2, color = "green", linetype = "dashed", linewidth = 0.8) + facet_wrap(vars(k), nrow = 2, ncol = 3) + scale_x_continuous(sec.axis = sec_axis(~., name = "k", breaks = NULL, labels = NULL))
#  rhomeanplot <- ggplot(rhomeanplot_data) + geom_area(aes(x = Expectation, y = Density), color = "blue", fill = "blue", alpha = 0.3, position = "identity") + geom_vline(xintercept = 0, color = "green", linetype = "dashed", linewidth = 0.8) + facet_wrap(vars(k), nrow = 2, ncol = 3) + scale_x_continuous(sec.axis = sec_axis(~., name = "k", breaks = NULL, labels = NULL))

## ---- echo = FALSE, out.width="600px", out.height = "500px"-------------------
knitr::include_graphics("data/rhovarplot.png")

## ---- eval = FALSE, echo = FALSE----------------------------------------------
#  rhovarplot

## ---- echo = FALSE, out.width = "600px", out.height = "500px"-----------------
knitr::include_graphics("data/rhomeanplot.png")

## ---- eval=FALSE, fig.dim = c(8,5), echo = FALSE------------------------------
#  rhomeanplot

## ---- eval = FALSE, message = FALSE, warning = FALSE, silent = TRUE, results = 'hide'----
#  fit_cor <- rstan::sampling(cor_pri_st, data = list(cor_p=c(0.25, 3, 4), iter = 2000, chains=4))
#  ex.fit <- rstan::extract(fit_cor)
#  rhoplot_data  <- data.frame(ex.fit$rho[,1,1,2])
#  names(rhoplot_data) <- "rho"
#  rhoplot <- ggplot(rhoplot_data) + geom_histogram(aes(x = rho), color = "blue", fill = "blue", alpha = 0.3, binwidth = 0.05) + scale_x_continuous(bquote(rho))

## ---- echo = FALSE, out.width = "400px", out.height = "400px"-----------------
knitr::include_graphics("data/rhoplot.png")

## ---- eval = FALSE, echo = FALSE, warning = FALSE, fig.dim = c(4,3)-----------
#  rhoplot

## ----eval=FALSE,message=FALSE, warning = FALSE, results = 'hide',silent=TRUE----
#  priors <- EnsemblePrior(4,
#                          ind_st_params =IndSTPrior("hierarchical_beta_conjugate",list(-3,1,8,4),
#                                                    list(0.25,3,4),AR_params=c(2,2)),
#                          ind_lt_params = IndLTPrior("lkj",list(1,1),1),
#                          sha_st_params = ShaSTPrior("lkj",list(1,10),1,AR_params=c(2,2)),
#                          sha_lt_params = 5
#  )
#  prior_density <- prior_ensemble_model(priors, M = 4)
#  samples <- sample_prior(observations = list(SSB_obs, Sigma_obs),
#               simulators = list(list(SSB_ewe, Sigma_ewe,"EwE"),
#                     list(SSB_fs ,  Sigma_fs,"FishSums"),
#                     list(SSB_lm ,  Sigma_lm,"LeMans"),
#                     list(SSB_miz, Sigma_miz,"mizer")),
#               priors=priors,
#               sam_priors = prior_density)

## ---- eval = FALSE, fig.dim = c(7, 6)-----------------------------------------
#  plot(samples)

## ---- echo = FALSE, out.height="500px", out.width="800px"---------------------
knitr::include_graphics("data/p_priors_beta_conj.png")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup_lies, warning=FALSE------------------------------------------------
library(EcoEnsemble)
library(mgcv)
library(ggplot2)
library(cowplot)
set.seed(1234)

## ----configure_priors---------------------------------------------------------
d <- 3 # The number of variables.

priors <- EnsemblePrior(d,
                        ind_st_params = IndSTPrior(AR_params=c(10,2)), 
                        ind_lt_params = IndLTPrior("beta",
                                                   cor_params = list(matrix(5, d, d), 
                                                                     matrix(1, d, d))),
                        sha_st_params = ShaSTPrior(AR_params=c(10,2)),
)

## ----sampling priors, eval = FALSE--------------------------------------------
# M <- 4 # The number of models we are considering.
# prior_density <- prior_ensemble_model(priors, M = M)
# ex.fit <- rstan::extract(prior_density$samples) # Extracted samples

## ----truth_best_guesses, eval = FALSE-----------------------------------------
# Time <- 50
# 
# true_par <-which.max(ex.fit$lp__)
# 
# latent <- matrix(NA, Time, (M+2)*d)
# 
# #Priors on initial values
# latent[1, 1:d] <- rnorm(d, 0, 1)
# latent[1, -(1:d)] <- t(chol(ex.fit$SIGMA_init[true_par,-(1:d) ,-(1:d)]))
#                                           %*% rnorm((M+1)*d, 0, 1)
# 
# #Find all the latent values of the dynamic linear model
# SIGMA <- ex.fit$SIGMA[true_par,,]
# SIGMA_chol <- t(chol(SIGMA))
# for (t in 2:Time) {
#   latent[t,] <- ex.fit$AR_params[true_par,] * latent[t-1,] + SIGMA_chol
#                                           %*% rnorm((M+2)*d, 0, 1)
# }
# 
# #The truth is simply the first d values
# truth_latent <- latent[,1:d]
# 
# #The best guesses are sums of the truth and discrepancies
# simulator_best_guesses <- array(NA,dim=c(Time,d,M))
# for(i in 1:M){
#   simulator_best_guesses[,,i] <- t(
#     t(latent[,1:d] + latent[,(d+1):(2*d)] + latent[,(1+i) * d + (1:d)]) +
#       ex.fit$ind_lt[true_par,i,] + ex.fit$sha_lt[true_par,] )
# }
# 

## ----load_datasets, echo = FALSE----------------------------------------------
#Here I will load in the data
M <- 4
Time <- 50
load("data/SyntheticData_PriorSamples.RData")

## ----plot_data, fig.dim = c(7, 4)---------------------------------------------
plot(truth_latent[,1], ylim=c(-2.5, 3), pch = 19)
lines(simulator_best_guesses[,1,1], col = 2)
lines(simulator_best_guesses[,1,2], col = 3)
lines(simulator_best_guesses[,1,3], col = 4)
lines(simulator_best_guesses[,1,4], col = 5)


## ----add_noise----------------------------------------------------------------
Times_obs <- round(Time * 0.8)
obs <- matrix(NA,Times_obs,d)
for(i in 1:d){
  g1 <- gam(y~s(x,k = 15),data=data.frame(x=1:Times_obs,y = truth_latent[1:Times_obs,i]))
  obs[,i] <- predict(g1)
}

obs.cov <- cov(obs - truth_latent[1:Times_obs,])


model.cov <- array(0,dim=c(M,d,d))
models_output <- array(NA,dim=c(M,Time,d))
for (j in 1:M){
  for(i in 1:d){
    g1 <- gam(y~s(x, k = 15),data=data.frame(x=1:Time,y=simulator_best_guesses[,i,j]))
    models_output[j,,i] <- predict(g1)
  }
  model.cov[j,,] <- cov(models_output[j,,] - simulator_best_guesses[,,j])
}


## ----plot_obs, fig.dim = c(7, 6)----------------------------------------------
#Observations and model outputs
plot(obs[,1], ylim=c(-2.5, 3), xlim = c(1,Time), pch = 19)
lines(models_output[1,,1], col=2, lwd = 2)
lines(models_output[2,,1], col=3, lwd = 2)
lines(models_output[3,,1], col=4, lwd = 2)
lines(models_output[4,,1], col=5, lwd = 2)

## -----------------------------------------------------------------------------
#Create the data frames that we'll use for EcoEnsemble
val_obs <- data.frame(obs); cov_obs <- obs.cov
val_model_1 <- data.frame(models_output[1,,]); cov_model_1 <- model.cov[1,,]
val_model_2 <- data.frame(models_output[2,,]); cov_model_2 <- model.cov[2,,]
val_model_3 <- data.frame(models_output[3,,]); cov_model_3 <- model.cov[3,,]
val_model_4 <- data.frame(models_output[4,,]); cov_model_4 <- model.cov[4,,]

#Set the dimnames to ensure EcoEnsemble can identify the information.
SPECIES_NAMES <- c("Species 1", "Species 2", "Species 3")
dimnames(val_obs) <- list(paste(1:Times_obs), SPECIES_NAMES)
dimnames(val_model_1) <- list(paste(1:Time), SPECIES_NAMES)
dimnames(val_model_2) <- list(paste(1:Time), SPECIES_NAMES)
dimnames(val_model_3) <- list(paste(1:Time), SPECIES_NAMES)
dimnames(val_model_4) <- list(paste(1:Time), SPECIES_NAMES)

dimnames(cov_obs) <- list(SPECIES_NAMES, SPECIES_NAMES)
dimnames(cov_model_1) <- list(SPECIES_NAMES, SPECIES_NAMES)
dimnames(cov_model_2) <- list(SPECIES_NAMES, SPECIES_NAMES)
dimnames(cov_model_3) <- list(SPECIES_NAMES, SPECIES_NAMES)
dimnames(cov_model_4) <- list(SPECIES_NAMES, SPECIES_NAMES)


## ----fitting, eval = FALSE----------------------------------------------------
# fit <- fit_ensemble_model(observations = list(val_obs, cov_obs),
#                           simulators = list(list(val_model_1, cov_model_1, "Model 1"),
#                                             list(val_model_2, cov_model_2, "Model 2"),
#                                             list(val_model_3, cov_model_3, "Model 3"),
#                                             list(val_model_4, cov_model_4, "Model 4")),
#                           priors = EnsemblePrior(d),
#                           control = list(adapt_delta = 0.9))
# # Use `sampler = "kalman"` here to retain the Kalman filter implementation
# fit_kalman <- fit_ensemble_model(observations = list(val_obs, cov_obs),
#                                  simulators = list(list(val_model_1, cov_model_1, "Model 1"),
#                                                    list(val_model_2, cov_model_2, "Model 2"),
#                                                    list(val_model_3, cov_model_3, "Model 3"),
#                                                    list(val_model_4, cov_model_4, "Model 4")),
#                                  priors = EnsemblePrior(d),
#                                  control = list(adapt_delta = 0.9),
#                                  sampler = "kalman")
# samples <- generate_sample(fit)

## ----results, eval = FALSE----------------------------------------------------
# var_index <- 1
# 
# df <- data.frame("Year" = paste(1:Time),
#                  "Ensemble" = apply(samples@mle[, var_index, ], 1, median),
#                  "Lower"  = apply(samples@samples[, var_index, ], 1, quantile, 0.1),
#                  "Upper"  = apply(samples@samples[, var_index, ], 1, quantile, 0.9),
#                  "Actual" = truth_latent[,var_index])
# df$Year <- as.numeric(df$Year)
# df <-  reshape2::melt(df, id.vars=c("Year", "Lower", "Upper"), variable.name="Simulator")
# 
# # We only want uncertainty bands for the Ensemble values, else we set zero width bands
# df[df$Simulator == "Actual", c("Lower", "Upper")] <- df[df$Simulator != "Actual", "value"]
# 
# ggplot(df, aes(x=`Year`, y=`value`)) +
#   geom_line(aes(group=`Simulator`,colour=`Simulator`)) +
#   geom_ribbon(aes(ymin=`Lower`, ymax =`Upper`, fill = `Simulator`), alpha=0.2)
# 

## ----plot_truth, echo = FALSE, fig.dim = c(7, 4)------------------------------
knitr::include_graphics("data/p_truth.png")

## ----eval = FALSE-------------------------------------------------------------
# fit_M1 <- fit_ensemble_model(observations = list(val_obs, cov_obs),
#                              simulators = list(list(val_model_1, cov_model_1, "Model 1")),
#                              priors = EnsemblePrior(d),
#                              control = list(adapt_delta = 0.9))
# samples_M1 <- generate_sample(fit_M1)
# 
# fit_M2 <- fit_ensemble_model(observations = list(val_obs, cov_obs),
#                              simulators = list(list(val_model_1, cov_model_1, "Model 1"),
#                                             list(val_model_2, cov_model_2, "Model 2")),
#                           priors = EnsemblePrior(d),
#                           control = list(adapt_delta = 0.9))
# samples_M2 <- generate_sample(fit_M2)
# 
# fit_M3 <- fit_ensemble_model(observations = list(val_obs, cov_obs),
#                           simulators = list(list(val_model_1, cov_model_1, "Model 1"),
#                                             list(val_model_2, cov_model_2, "Model 2"),
#                                             list(val_model_3, cov_model_3, "Model 3")),
#                           priors = EnsemblePrior(d),
#                           control = list(adapt_delta = 0.9))
# samples_M3 <- generate_sample(fit_M3)
# # Each of the fits above accepts `sampler = "kalman"` when the Kalman filter
# # variant is preferred over explicit latent state sampling.
# 
# plot_grid(
#   plot(samples_M1, variable=1) + ggtitle("1 Model")  + theme(legend.position = "none"),
#   plot(samples_M2, variable=1) + ggtitle("2 Models") + theme(legend.position = "none"),
#   plot(samples_M3, variable=1) + ggtitle("3 Models") + theme(legend.position = "none"),
#   plot(samples   , variable=1) + ggtitle("4 Models") + theme(legend.position = "none"))

## ----model_comparison, echo = FALSE, fig.dim = c(7, 6)------------------------
knitr::include_graphics("data/p_NumberOfModels.png")

## ----quantify_variance, eval = FALSE------------------------------------------
# def.par <- par(no.readonly=TRUE) #old pars
# par(mfrow = c(d, 1))
# legend_ys <- c(0.4, 0.18, 0.12)
# for(i in 1:d){
#   plot.ts(apply(samples_M1@samples[40:50,i,],1, var),
#           main = paste0("Species ", i), ylab="Variance" )
#   lines(apply(samples_M2@samples[40:50,i,],1, var), col = 2)
#   lines(apply(samples_M3@samples[40:50,i,],1, var), col = 3)
#   lines(apply(samples@samples[40:50,i,],1, var), col = 4)
#   legend(1, legend_ys[i], legend = c("1 Model", "2 Models", "3 Models", "4 Models"),
#           col = 1:4, lty = 1)
# }
# par(def.par)

## ----plot_variances, echo=FALSE, fig.dim = c(7,6)-----------------------------
load("data/SyntheticData_AddingModelsData.RData")
def.par <- par(no.readonly=TRUE) #old pars
par(mfrow = c(d, 1), mar = c(4, 4, 1.4, 1.4))
legend_ys <- c(0.4, 0.18, 0.12)
for(i in 1:d){
  plot.ts(var_m1[, i], 
          main = paste0("Species ", i), ylab="Variance" )
  lines(var_m2[, i], col = 2)
  lines(var_m3[, i], col = 3)
  lines(var_m4[, i], col = 4)
  legend(1, legend_ys[i], legend = c("1 Model", "2 Models", "3 Models", "4 Models"),
          col = 1:4, lty = 1)
}
par(def.par)

## ---- include = FALSE---------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup_lies, warning=FALSE------------------------------------------------
library(EcoEnsemble)
library(mgcv)
library(ggplot2)
library(cowplot)
set.seed(1234)

## ----configure_priors---------------------------------------------------------
d <- 3 # The number of variables.

priors <- EnsemblePrior(d,
                        ind_st_params = IndSTPrior(AR_params=c(10,2)), 
                        ind_lt_params = IndLTPrior("beta",
                                                   cor_params = list(matrix(5, d, d), 
                                                                     matrix(1, d, d))),
                        sha_st_params = ShaSTPrior(AR_params=c(10,2)),
)

## ----sampling priors, eval = FALSE--------------------------------------------
#  M <- 4 # The number of models we are considering.
#  prior_density <- prior_ensemble_model(priors, M = M)
#  ex.fit <- rstan::extract(prior_density$samples) # Extracted samples

## ----truth_best_guesses, eval = FALSE-----------------------------------------
#  Time <- 50
#  
#  true_par <-which.max(ex.fit$lp__)
#  
#  latent <- matrix(NA, Time, (M+2)*d)
#  
#  #Priors on initial values
#  latent[1, 1:d] <- rnorm(d, 0, 1)
#  latent[1, -(1:d)] <- t(chol(ex.fit$SIGMA_init[true_par,-(1:d) ,-(1:d)]))
#                                            %*% rnorm((M+1)*d, 0, 1)
#  
#  #Find all the latent values of the dynamic linear model
#  SIGMA <- ex.fit$SIGMA[true_par,,]
#  SIGMA_chol <- t(chol(SIGMA))
#  for (t in 2:Time) {
#    latent[t,] <- ex.fit$AR_params[true_par,] * latent[t-1,] + SIGMA_chol
#                                            %*% rnorm((M+2)*d, 0, 1)
#  }
#  
#  #The truth is simply the first d values
#  truth_latent <- latent[,1:d]
#  
#  #The best guesses are sums of the truth and discrepancies
#  simulator_best_guesses <- array(NA,dim=c(Time,d,M))
#  for(i in 1:M){
#    simulator_best_guesses[,,i] <- t(
#      t(latent[,1:d] + latent[,(d+1):(2*d)] + latent[,(1+i) * d + (1:d)]) +
#        ex.fit$ind_lt[true_par,i,] + ex.fit$sha_lt[true_par,] )
#  }
#  

## ----load_datasets, echo = FALSE----------------------------------------------
#Here I will load in the data
M <- 4
Time <- 50
load("data/SyntheticData_PriorSamples.RData")

## ----plot_data, fig.dim = c(7, 4)---------------------------------------------
plot(truth_latent[,1], ylim=c(-2.5, 3), pch = 19)
lines(simulator_best_guesses[,1,1], col = 2)
lines(simulator_best_guesses[,1,2], col = 3)
lines(simulator_best_guesses[,1,3], col = 4)
lines(simulator_best_guesses[,1,4], col = 5)


## ----add_noise----------------------------------------------------------------
Times_obs <- round(Time * 0.8)
obs <- matrix(NA,Times_obs,d)
for(i in 1:d){
  g1 <- gam(y~s(x,k = 15),data=data.frame(x=1:Times_obs,y = truth_latent[1:Times_obs,i]))
  obs[,i] <- predict(g1)
}

obs.cov <- cov(obs - truth_latent[1:Times_obs,])


model.cov <- array(0,dim=c(M,d,d))
models_output <- array(NA,dim=c(M,Time,d))
for (j in 1:M){
  for(i in 1:d){
    g1 <- gam(y~s(x, k = 15),data=data.frame(x=1:Time,y=simulator_best_guesses[,i,j]))
    models_output[j,,i] <- predict(g1)
  }
  model.cov[j,,] <- cov(models_output[j,,] - simulator_best_guesses[,,j])
}


## ----plot_obs, fig.dim = c(7, 6)----------------------------------------------
#Observations and model outputs
plot(obs[,1], ylim=c(-2.5, 3), xlim = c(1,Time), pch = 19)
lines(models_output[1,,1], col=2, lwd = 2)
lines(models_output[2,,1], col=3, lwd = 2)
lines(models_output[3,,1], col=4, lwd = 2)
lines(models_output[4,,1], col=5, lwd = 2)

## -----------------------------------------------------------------------------
#Create the data frames that we'll use for EcoEnsemble
val_obs <- data.frame(obs); cov_obs <- obs.cov
val_model_1 <- data.frame(models_output[1,,]); cov_model_1 <- model.cov[1,,]
val_model_2 <- data.frame(models_output[2,,]); cov_model_2 <- model.cov[2,,]
val_model_3 <- data.frame(models_output[3,,]); cov_model_3 <- model.cov[3,,]
val_model_4 <- data.frame(models_output[4,,]); cov_model_4 <- model.cov[4,,]

#Set the dimnames to ensure EcoEnsemble can identify the information.
SPECIES_NAMES <- c("Species 1", "Species 2", "Species 3")
dimnames(val_obs) <- list(paste(1:Times_obs), SPECIES_NAMES)
dimnames(val_model_1) <- list(paste(1:Time), SPECIES_NAMES)
dimnames(val_model_2) <- list(paste(1:Time), SPECIES_NAMES)
dimnames(val_model_3) <- list(paste(1:Time), SPECIES_NAMES)
dimnames(val_model_4) <- list(paste(1:Time), SPECIES_NAMES)

dimnames(cov_obs) <- list(SPECIES_NAMES, SPECIES_NAMES)
dimnames(cov_model_1) <- list(SPECIES_NAMES, SPECIES_NAMES)
dimnames(cov_model_2) <- list(SPECIES_NAMES, SPECIES_NAMES)
dimnames(cov_model_3) <- list(SPECIES_NAMES, SPECIES_NAMES)
dimnames(cov_model_4) <- list(SPECIES_NAMES, SPECIES_NAMES)


## ----fitting, eval = FALSE----------------------------------------------------
#  fit <- fit_ensemble_model(observations = list(val_obs, cov_obs),
#                            simulators = list(list(val_model_1, cov_model_1, "Model 1"),
#                                              list(val_model_2, cov_model_2, "Model 2"),
#                                              list(val_model_3, cov_model_3, "Model 3"),
#                                              list(val_model_4, cov_model_4, "Model 4")),
#                            priors = EnsemblePrior(d),
#                            control = list(adapt_delta = 0.9))
#  samples <- generate_sample(fit)

## ----results, eval = FALSE----------------------------------------------------
#  var_index <- 1
#  
#  df <- data.frame("Year" = paste(1:Time),
#                   "Ensemble" = apply(samples@mle[, var_index, ], 1, median),
#                   "Lower"  = apply(samples@samples[, var_index, ], 1, quantile, 0.1),
#                   "Upper"  = apply(samples@samples[, var_index, ], 1, quantile, 0.9),
#                   "Actual" = truth_latent[,var_index])
#  df$Year <- as.numeric(df$Year)
#  df <-  reshape2::melt(df, id.vars=c("Year", "Lower", "Upper"), variable.name="Simulator")
#  
#  # We only want uncertainty bands for the Ensemble values, else we set zero width bands
#  df[df$Simulator == "Actual", c("Lower", "Upper")] <- df[df$Simulator != "Actual", "value"]
#  
#  ggplot(df, aes(x=`Year`, y=`value`)) +
#    geom_line(aes(group=`Simulator`,colour=`Simulator`)) +
#    geom_ribbon(aes(ymin=`Lower`, ymax =`Upper`, fill = `Simulator`), alpha=0.2)
#  

## ----plot_truth, echo = FALSE, fig.dim = c(7, 4)------------------------------
knitr::include_graphics("data/p_truth.png")

## ----eval = FALSE-------------------------------------------------------------
#  fit_M1 <- fit_ensemble_model(observations = list(val_obs, cov_obs),
#                               simulators = list(list(val_model_1, cov_model_1, "Model 1")),
#                               priors = EnsemblePrior(d),
#                               control = list(adapt_delta = 0.9))
#  samples_M1 <- generate_sample(fit_M1)
#  
#  fit_M2 <- fit_ensemble_model(observations = list(val_obs, cov_obs),
#                               simulators = list(list(val_model_1, cov_model_1, "Model 1"),
#                                              list(val_model_2, cov_model_2, "Model 2")),
#                            priors = EnsemblePrior(d),
#                            control = list(adapt_delta = 0.9))
#  samples_M2 <- generate_sample(fit_M2)
#  
#  fit_M3 <- fit_ensemble_model(observations = list(val_obs, cov_obs),
#                            simulators = list(list(val_model_1, cov_model_1, "Model 1"),
#                                              list(val_model_2, cov_model_2, "Model 2"),
#                                              list(val_model_3, cov_model_3, "Model 3")),
#                            priors = EnsemblePrior(d),
#                            control = list(adapt_delta = 0.9))
#  samples_M3 <- generate_sample(fit_M3)
#  
#  plot_grid(
#    plot(samples_M1, variable=1) + ggtitle("1 Model")  + theme(legend.position = "none"),
#    plot(samples_M2, variable=1) + ggtitle("2 Models") + theme(legend.position = "none"),
#    plot(samples_M3, variable=1) + ggtitle("3 Models") + theme(legend.position = "none"),
#    plot(samples   , variable=1) + ggtitle("4 Models") + theme(legend.position = "none"))

## ----model_comparison, echo = FALSE, fig.dim = c(7, 6)------------------------
knitr::include_graphics("data/p_NumberOfModels.png")

## ----quantify_variance, eval = FALSE------------------------------------------
#  def.par <- par(no.readonly=TRUE) #old pars
#  par(mfrow = c(d, 1))
#  legend_ys <- c(0.4, 0.18, 0.12)
#  for(i in 1:d){
#    plot.ts(apply(samples_M1@samples[40:50,i,],1, var),
#            main = paste0("Species ", i), ylab="Variance" )
#    lines(apply(samples_M2@samples[40:50,i,],1, var), col = 2)
#    lines(apply(samples_M3@samples[40:50,i,],1, var), col = 3)
#    lines(apply(samples@samples[40:50,i,],1, var), col = 4)
#    legend(1, legend_ys[i], legend = c("1 Model", "2 Models", "3 Models", "4 Models"),
#            col = 1:4, lty = 1)
#  }
#  par(def.par)

## ----plot_variances, echo=FALSE, fig.dim = c(7,6)-----------------------------
load("data/SyntheticData_AddingModelsData.RData")
def.par <- par(no.readonly=TRUE) #old pars
par(mfrow = c(d, 1), mar = c(4, 4, 1.4, 1.4))
legend_ys <- c(0.4, 0.18, 0.12)
for(i in 1:d){
  plot.ts(var_m1[, i], 
          main = paste0("Species ", i), ylab="Variance" )
  lines(var_m2[, i], col = 2)
  lines(var_m3[, i], col = 3)
  lines(var_m4[, i], col = 4)
  legend(1, legend_ys[i], legend = c("1 Model", "2 Models", "3 Models", "4 Models"),
          col = 1:4, lty = 1)
}
par(def.par)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message = FALSE,
  fig.width = 8,
  fig.height = 5
)

library(ggplot2)
library(dplyr)
library(knitr)
library(kableExtra)

pair_results  <- readRDS("data/pair_results.rds")
plot_df2       <- readRDS("data/plot_df2.rds")
table_diag    <- readRDS("data/table_diag.rds")
section_sizes <- readRDS("data/section_sizes.rds")
diag_table    <- readRDS("data/diag_table.rds")
plot_df_ess <- readRDS("data/plot_df_ess.rds")

## ----setup, include=FALSE-----------------------------------------------------
library(EcoEnsemble)

## ----eval=F-------------------------------------------------------------------
# #generate priors
# priors <- EnsemblePrior(4)
# 
# #run the model
# fit_sample <- fit_ensemble_model(observations = list(SSB_obs, Sigma_obs),
#                                  simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
#                                                    list(SSB_lm,  Sigma_lm,  "LeMans"),
#                                                    list(SSB_miz, Sigma_miz, "mizer"),
#                                                    list(SSB_fs,  Sigma_fs,  "FishSUMS")),
#                                  priors = priors,
#                                  sampler = "kalman")

## ----echo=F, out.width = "75%",fig.align ='center'----------------------------

# ggplot2::ggplot(plot_df_ess %>%
#     dplyr::filter(setup == "Model + hierarchical") %>%
#     dplyr::mutate(sampler = dplyr::if_else(sampler == "E", "Explicit", sampler))%>%mutate(sampler = dplyr::if_else(sampler == "K", "Kalman", sampler))
#                 , ggplot2::aes(x = setup, y = value, fill = sampler)) +
#   ggplot2::geom_boxplot(
#     position = ggplot2::position_dodge(width = 0.72),
#     width = 0.58,
#     outlier.shape = NA
#   ) +
#   ggplot2::facet_wrap(metric ~ ., scales = "free_y", nrow=1) +
#   ggplot2::labs(
#     x = NULL,
#     y = NULL,
#     fill = "Sampler"
#   ) +
#   ggplot2::theme_bw(base_size = 11) +
#   ggplot2::theme(
#     legend.position = "top",
#     strip.placement = "outside",
#     strip.background = ggplot2::element_blank(),
#     panel.spacing.y = grid::unit(0.15, "lines"),
#     plot.margin = grid::unit(c(0.3, 0.4, 0.2, 0.2), "cm")
#   )+ ggplot2::theme(
#     axis.title.x = ggplot2::element_blank(),
#     axis.text.x  = ggplot2::element_blank(),
#     axis.ticks.x = ggplot2::element_blank()
#   )

knitr::include_graphics("data/explicit_plot1.png")


## ----echo=F, out.width = "85%",fig.align ='center'----------------------------
knitr::include_graphics("data/explicit_plot4.png")

## ----echo=F-------------------------------------------------------------------
table_diag %>%
    select(-section) %>%
    mutate(across(where(is.numeric), ~ formatC(., format = "fg", digits = 3))) %>%
    kbl(
        booktabs = TRUE,
        align = c("l", rep("r", 8)),
        col.names = c("Diagnostic", "E", "K", "E", "K", "E", "K", "E", "K")
    )  %>%
    add_header_above(c(
        " " = 1,
        "Drivers" = 2,
        "Model" = 2,
        "Model + hierarchical" = 2,
        "Drivers + hierarchical" = 2
    )) %>%
    kable_styling(
        full_width = FALSE,
        font_size = 10,
        latex_options = "hold_position"
    ) %>%
    row_spec(0, bold = TRUE) %>%
    pack_rows("Run time", 1, section_sizes["Run time"]) %>%
    pack_rows(
        "Convergence",
        section_sizes["Run time"] + 1,
        section_sizes["Run time"] + section_sizes["Convergence"]
    ) %>%
    pack_rows(
        "Bulk diagnostics",
        section_sizes["Run time"] + section_sizes["Convergence"] + 1,
        section_sizes["Run time"] + section_sizes["Convergence"] + section_sizes["Bulk diagnostics"]
    ) %>%
    pack_rows(
        "Tail diagnostics",
        section_sizes["Run time"] + section_sizes["Convergence"] + section_sizes["Bulk diagnostics"] + 1,
        sum(section_sizes)
    ) %>%
    column_spec(1, width = "4.2cm") %>%
    column_spec(2:9, width = "1.5cm")


## ----echo=F-------------------------------------------------------------------
diag_table %>%
    mutate(across(where(is.numeric), ~ formatC(., format = "fg", digits = 3))) %>%
    knitr::kable(
        booktabs = TRUE,
        align = c("l", rep("r", 8)),
        col.names = c("Diagnostic", "E", "K", "E", "K", "E", "K", "E", "K")
    ) %>%
  kableExtra::add_header_above(c(
    " " = 1,
    "Drivers" = 2,
    "Model" = 2,
    "Model + hierarchical" = 2,
    "Drivers + hierarchical" = 2
  )) %>%
  kableExtra::kable_styling(
    full_width = FALSE,
    font_size = 10,
    latex_options = "hold_position"
  ) %>%
  kableExtra::column_spec(1, width = "4.2cm") %>%
  kableExtra::column_spec(2:9, width = "1.5cm")

## ----setup_lies, warning=FALSE, message=FALSE, silent = TRUE------------------
library(EcoEnsemble)
library(mgcv)
library(ggplot2)
library(cowplot)
set.seed(5678)

## ----configure_priors, eval = FALSE-------------------------------------------
#  d <- 3 # The number of variables.
#  
#  priors <- EnsemblePrior(d,
#                          ind_st_params = IndSTPrior(AR_params=c(10,2)),
#                          ind_lt_params = IndLTPrior("beta",
#                                                     cor_params = list(matrix(5, d, d),
#                                                                       matrix(1, d, d)),var_params = list(1,0.5)),
#                          sha_st_params = ShaSTPrior(AR_params=c(10,2)),
#  )

## ----sampling priors, eval = FALSE--------------------------------------------
#  M <- 3 # The number of simulators we are considering.
#  MM <- 2 # The number of drivers we are considering.
#  prior_density <- prior_ensemble_model(priors, M = M, MM = MM)
#  ex.fit <- rstan::extract(prior_density$samples) # Extracted samples

## ----truth_best_guesses, eval = FALSE-----------------------------------------
#  Time <- 50
#  
#  true_par <-which.max(ex.fit$lp__)
#  
#  latent <- matrix(NA, Time, (M+MM+2)*d)
#  
#  #Priors on initial values
#  latent[1, 1:d] <- rnorm(d, 0, 1)
#  latent[1, -(1:d)] <- t(chol(ex.fit$SIGMA_init[true_par,-(1:d) ,-(1:d)])) %*% rnorm((M+MM+1)*d, 0, 1)
#  
#  #Find all the latent values of the dynamic linear model
#  SIGMA <- ex.fit$SIGMA[true_par,,]
#  SIGMA_chol <- t(chol(SIGMA))
#  for (t in 2:Time) {
#    latent[t,] <- ex.fit$AR_params[true_par,] * latent[t-1,] + SIGMA_chol %*% rnorm((M+MM+2)*d, 0, 1)
#  }
#  
#  #The truth is simply the first d values
#  truth_latent <- latent[,1:d]
#  
#  #The best guesses are sums of the truth and discrepancies
#  simulator_best_guesses <- array(NA,dim=c(Time,d,M*MM))
#  for(i in 1:M){
#    for (j in 1:MM){
#      simulator_best_guesses[,,MM*(i-1)+j] <- t(
#        t(latent[,1:d] + latent[,(d+1):(2*d)] + latent[,(i + 1)*d + (1:d)] + latent[,(M + j + 1)*d + (1:d)]) +
#          ex.fit$ind_lt[true_par,i,] + ex.fit$ind_lt_dri[true_par,j,] + ex.fit$sha_lt[true_par,])
#    }
#  }

## ----plot_data, eval = FALSE--------------------------------------------------
#  plotlist <- list()
#  for (sim in 1:M){
#    for (dri in 1:MM){
#      plotlist[[MM*(sim-1)+dri]] <- cbind(c(1:Time),simulator_best_guesses[,1,MM*(sim-1)+dri],rep(sim,Time),rep(dri,Time),rep("sim",Time))
#    }
#  }
#  plot_df <- data.frame(rbind(do.call(rbind, plotlist), cbind(c(1:Time), truth_latent[,1], rep(0,Time), rep(0,Time), rep("truth",Time))))
#  names(plot_df) <- c("Year", "Value", "Simulator", "Driver", "Type")
#  plot_df$Year <- as.numeric(plot_df$Year)
#  plot_df$Value <- as.numeric(plot_df$Value)
#  bgplot <- ggplot2::ggplot(plot_df) + geom_line(data = plot_df[which(plot_df$Type == "sim"),], aes(x = Year, y = Value, color = Simulator, linetype = Driver), linewidth = 0.8) + geom_line(data = plot_df[which(plot_df$Type == "truth"),], aes(x = Year, y = Value), linewidth = 1, color = "purple")

## ---- echo = FALSE, out.width="500px", out.height="500px"---------------------
knitr::include_graphics("data/bgplot.png")

## ----add_noise, eval = FALSE--------------------------------------------------
#  Times_obs <- round(Time * 0.8)
#  obs <- matrix(NA,Times_obs,d)
#  for(i in 1:d){
#    g1 <- gam(y~s(x,k = 15),data=data.frame(x=1:Times_obs,y = truth_latent[1:Times_obs,i]))
#    obs[,i] <- predict(g1)
#  }
#  
#  obs.cov <- cov(obs - truth_latent[1:Times_obs,])
#  
#  
#  model.cov <- array(0,dim=c(M*MM,d,d))
#  models_output <- array(NA,dim=c(M*MM,Time,d))
#  for (j in 1:(M*MM)){
#    for(i in 1:d){
#      g1 <- gam(y~s(x, k = 15),data=data.frame(x=1:Time,y=simulator_best_guesses[,i,j]))
#      models_output[j,,i] <- predict(g1)
#    }
#    model.cov[j,,] <- cov(models_output[j,,] - simulator_best_guesses[,,j])
#  }

## ----plot_obs, eval = FALSE---------------------------------------------------
#  #Observations and model outputs
#  plotlist <- list()
#  for (sim in 1:M){
#    for (dri in 1:MM){
#      plotlist[[MM*(sim-1)+dri]] <- cbind(c(1:Time),models_output[MM*(sim-1)+dri,,1],rep(sim,Time),rep(dri,Time),rep("sim",Time),rep("observed", Time))
#    }
#  }
#  plot_df <- data.frame(rbind(do.call(rbind, plotlist), cbind(c(1:Time), c(obs[,1],rep(0, Time - max(Times_obs))), rep(0,Time), rep(0,Time), rep("truth",Time), c(rep("observed", max(Times_obs)),rep("unobserved", Time - max(Times_obs))))))
#  names(plot_df) <- c("Year", "Value", "Simulator", "Driver", "Type", "Obs_Status")
#  plot_df$Year <- as.numeric(plot_df$Year)
#  plot_df$Value <- as.numeric(plot_df$Value)
#  obsplot <- ggplot(plot_df) + geom_line(data=plot_df[which(plot_df$Type == "sim"),], aes(x = Year, y = Value, color = Simulator, linetype = Driver), linewidth = 0.8) + geom_point(data = plot_df[intersect(which(plot_df$Type == "truth"),which(plot_df$Obs_Status == "observed")),], aes(x = Year, y = Value), color = "purple")

## ---- echo = FALSE, out.width="500px", out.height="500px"---------------------
knitr::include_graphics("data/obsplot.png")

## ---- eval = FALSE------------------------------------------------------------
#  #Create the data frames that we'll use for EcoEnsemble
#  val_obs <- data.frame(obs); cov_obs <- obs.cov
#  val_model_11 <- data.frame(models_output[1,,]); cov_model_11 <- model.cov[1,,]
#  val_model_12 <- data.frame(models_output[2,,]); cov_model_12 <- model.cov[2,,]
#  val_model_21 <- data.frame(models_output[3,,]); cov_model_21 <- model.cov[3,,]
#  val_model_22 <- data.frame(models_output[4,,]); cov_model_22 <- model.cov[4,,]
#  val_model_31 <- data.frame(models_output[5,,]); cov_model_31 <- model.cov[5,,]
#  val_model_32 <- data.frame(models_output[6,,]); cov_model_32 <- model.cov[6,,]
#  
#  #Set the dimnames to ensure EcoEnsemble can identify the information.
#  SPECIES_NAMES <- c("Species 1", "Species 2", "Species 3")
#  dimnames(val_obs) <- list(paste(1:Times_obs), SPECIES_NAMES)
#  dimnames(val_model_11) <- list(paste(1:Time), SPECIES_NAMES)
#  dimnames(val_model_21) <- list(paste(1:Time), SPECIES_NAMES)
#  dimnames(val_model_12) <- list(paste(1:Time), SPECIES_NAMES)
#  dimnames(val_model_22) <- list(paste(1:Time), SPECIES_NAMES)
#  dimnames(val_model_31) <- list(paste(1:Time), SPECIES_NAMES)
#  dimnames(val_model_32) <- list(paste(1:Time), SPECIES_NAMES)
#  
#  dimnames(cov_obs) <- list(SPECIES_NAMES, SPECIES_NAMES)
#  dimnames(cov_model_11) <- list(SPECIES_NAMES, SPECIES_NAMES)
#  dimnames(cov_model_21) <- list(SPECIES_NAMES, SPECIES_NAMES)
#  dimnames(cov_model_12) <- list(SPECIES_NAMES, SPECIES_NAMES)
#  dimnames(cov_model_22) <- list(SPECIES_NAMES, SPECIES_NAMES)
#  dimnames(cov_model_31) <- list(SPECIES_NAMES, SPECIES_NAMES)
#  dimnames(cov_model_32) <- list(SPECIES_NAMES, SPECIES_NAMES)
#  
#  val_model_1 <- list(val_model_11, val_model_12)
#  val_model_2 <- list(val_model_21, val_model_22)
#  val_model_3 <- list(val_model_31, val_model_32)
#  cov_model_1 <- list(cov_model_11, cov_model_12)
#  cov_model_2 <- list(cov_model_21, cov_model_22)
#  cov_model_3 <- list(cov_model_31, cov_model_32)

## ----fitting, eval = FALSE----------------------------------------------------
#  fit <- fit_ensemble_model(observations = list(val_obs, cov_obs),
#                                               simulators = list(list(val_model_1, cov_model_1, "Simulator 1", c("Driver 1", "Driver 2")),
#                                                                 list(val_model_2, cov_model_2, "Simulator 2", c("Driver 1", "Driver 2")),
#                                                                 list(val_model_3, cov_model_3, "Simulator 3", c("Driver 1", "Driver 2"))
#                                               ),
#                                               priors = EnsemblePrior(d),
#                                               control = list(adapt_delta = 0.9),drivers=T)
#  samples <- generate_sample(fit)

## ----results, eval = FALSE----------------------------------------------------
#  df_list <- list()
#  for (var_index in 1:3){
#    df <- data.frame("Year" = paste(1:Time),
#                     "Ensemble" = apply(samples@mle[, var_index, ], 1, median),
#                     "Lower"  = apply(samples@samples[, var_index, ], 1, quantile, 0.1),
#                     "Upper"  = apply(samples@samples[, var_index, ], 1, quantile, 0.9),
#                     "Actual" = truth_latent[,var_index])
#    df$Year <- as.numeric(df$Year)
#    df <-  reshape2::melt(df, id.vars=c("Year", "Lower", "Upper"), variable.name="Simulator")
#    # We only want uncertainty bands for the Ensemble values, else we set zero width bands
#    df[df$Simulator == "Actual", c("Lower", "Upper")] <- df[df$Simulator != "Actual", "value"]
#    df$Species = rep(SPECIES_NAMES[var_index], Time)
#    df_list[[var_index]] <- ggplot(df, aes(x=`Year`, y=`value`)) +
#    geom_line(aes(group=`Simulator`,colour=`Simulator`)) +
#    geom_ribbon(aes(ymin=`Lower`, ymax =`Upper`, fill = `Simulator`), alpha=0.2) + ggplot2::ggtitle(SPECIES_NAMES[var_index]) + ggplot2::theme(legend.position = "right")
#  }
#  
#  legend <- cowplot::get_plot_component(df_list[[1]], "guide-box-right")
#  df_list <- lapply(df_list, function(x) {x + ggplot2::theme(legend.position = "none")})
#  output_plot_nolegend <- cowplot::plot_grid(plotlist=df_list, nrow = 3, ncol = 1)
#  output_plot <- cowplot::plot_grid(output_plot_nolegend, legend, rel_widths = c(4,1), nrow = 1, ncol = 2)

## ----plot_truth, echo = FALSE, out.width="600px", out.height="600px"----------
knitr::include_graphics("data/output_plot.png")

## ---- eval = FALSE------------------------------------------------------------
#  samples_array <- as.array(fit@samples)
#  #Drop last element of third dimension as this is "lp__"
#  max(apply(samples_array[,,-dim(samples_array)[3]], 3, rstan::Rhat), na.rm = TRUE)
#  min(apply(samples_array[,,-dim(samples_array)[3]], 3, rstan::ess_bulk), na.rm = TRUE)
#  min(apply(samples_array[,,-dim(samples_array)[3]], 3, rstan::ess_tail), na.rm = TRUE)

## ----echo=FALSE---------------------------------------------------------------
1.007311; 836.737; 559.6022

## ---- eval = FALSE------------------------------------------------------------
#  rstan::check_divergences(fit@samples)

## ---- echo = FALSE------------------------------------------------------------
message(sprintf("%s of %s iterations ended with a divergence (%s%%).", 
            5, 4000, 100 * 5/4000), "\nTry increasing 'adapt_delta' to remove the divergences.")

## ---- eval = FALSE, fig.dim = c(7,4)------------------------------------------
#  rstan::traceplot(fit@samples, pars = "ind_st_sd[3,2]")

## ---- echo = FALSE, out.width = "500px", out.height = "300px"-----------------
knitr::include_graphics("data/drivers_trace.png")

## ----include=FALSE------------------------------------------------------------
#Load in the data and load the library
library(EcoEnsemble)
#load("data/vignette_plots.Rdata")

## ----label="Observations"-----------------------------------------------------
SSB_obs
Sigma_obs

## ----label="EwESigma"---------------------------------------------------------
Sigma_ewe

## ----label="lmSigma"----------------------------------------------------------
Sigma_lm

## ----label="mizSigma"---------------------------------------------------------
Sigma_miz

## ----label="fsSigma"----------------------------------------------------------
Sigma_fs

## ----model_outputs, warning=FALSE, fig.dim = c(7, 3), message = FALSE, echo = FALSE----
library(tibble)
library(dplyr)
library(reshape2)
library(ggplot2)

#Inlcude years in the data frames
SSB_obs_tmp <- rownames_to_column(SSB_obs, var = "Year")
SSB_ewe_tmp <- rownames_to_column(SSB_ewe, var = "Year")
SSB_miz_tmp <- rownames_to_column(SSB_miz, var = "Year")
SSB_lm_tmp <- rownames_to_column(SSB_lm, var = "Year")
SSB_fs_tmp <- rownames_to_column(SSB_fs, var = "Year")

#Join dataframes together
df_all <- SSB_obs_tmp %>% 
    full_join(SSB_ewe_tmp, by = "Year", suffix = c("","_ewe")) %>%
    full_join(SSB_miz_tmp, by = "Year", suffix = c("","_miz")) %>%
    full_join(SSB_lm_tmp, by = "Year", suffix = c("","_lm")) %>%
    full_join(SSB_fs_tmp, by = "Year", suffix = c("","_fs"))

#Melt into long data format for ggplot
df_all <- melt(df_all, id.vars = "Year")
colnames(df_all) <- c("Year", "Simulator", "logSSB")
df_all$Year <- as.numeric(df_all$Year)

#Finally create the plots
df_n_pout <- df_all[grepl("N.pout", df_all$Simulator), ]
p1 <- ggplot(data=df_n_pout, aes(x=`Year`, y=`logSSB`, na.rm = TRUE)) + 
  geom_line(aes(group=`Simulator`,colour=`Simulator`)) +
  ggtitle("Norway pout")

df_herring <- df_all[grepl("Herring", df_all$Simulator), ]
p2 <- ggplot(data=df_herring, aes(x=`Year`, y=`logSSB`, na.rm = TRUE)) +    
  geom_line(aes(group=`Simulator`,colour=`Simulator`)) + 
  ggtitle("Herring")

df_cod <- df_all[grepl("Cod", df_all$Simulator), ]
p3 <- ggplot(data=df_cod, aes(x=`Year`, y=`logSSB`, na.rm = TRUE)) +    
  geom_line(aes(group=`Simulator`,colour=`Simulator`))+
  ggtitle("Cod")

df_sole <- df_all[grepl("Sole", df_all$Simulator), ]
p4 <- ggplot(data=df_sole, aes(x=`Year`, y=`logSSB`, na.rm = TRUE)) +    
  geom_line(aes(group=`Simulator`,colour=`Simulator`)) +
  ggtitle("Sole")

cowplot::plot_grid(p1, p2, p3, p4, ncol = 2, nrow = 2)


## -----------------------------------------------------------------------------
#Setting priors for an ensemble model with 4 variables of interest.
priors <- EnsemblePrior(4)

## -----------------------------------------------------------------------------
priors_custom <- EnsemblePrior(
                        d = 4,
                        ind_st_params = IndSTPrior("lkj", list(25, 0.25), 30),
                        ind_lt_params = IndLTPrior(
                          "beta",
                          list(c(25, 25, 25, 10),c(0.25, 0.25, 0.25, 0.1)),
                          list(matrix(40, 4, 4), matrix(40, 4, 4))
                          ),
                        sha_st_params = ShaSTPrior("lkj", list(25, 0.25), 30),
                        sha_lt_params = 3)

## ---- label = "fit", eval=FALSE-----------------------------------------------
#  fit_sample <- fit_ensemble_model(observations = list(SSB_obs, Sigma_obs),
#                                   simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
#                                                     list(SSB_lm,  Sigma_lm,  "LeMans"),
#                                                     list(SSB_miz, Sigma_miz, "mizer"),
#                                                     list(SSB_fs,  Sigma_fs,  "FishSUMS")),
#                                   priors = priors)
#  samples <- generate_sample(fit_sample)

## ---- label = "Initial_plots", eval=FALSE-------------------------------------
#  plot(samples)

## ----fig.dim = c(7, 4), echo=FALSE, warning = FALSE---------------------------
knitr::include_graphics("data/plot_initial_outputs.png")

## ---- label = "Initial_plots_changed", fig.dim = c(7, 4), eval=FALSE----------
#  plot(samples, variable = "Cod", quantiles = c(0.25, 0.75)) +
#      ggplot2::theme_classic() +
#      ggplot2::scale_color_brewer(palette="Set2")  +
#      ggplot2::ylab("SSB (log tonnes)")

## ----fig.dim = c(7, 4), echo=FALSE, warning = FALSE---------------------------
knitr::include_graphics("data/plot_initial_customised.png")

## ----visualise_priors, eval= FALSE--------------------------------------------
#  prior_model <- prior_ensemble_model(priors = priors, M = 4)
#  prior_samples <- sample_prior(observations = list(SSB_obs, Sigma_obs),
#                                   simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
#                                                     list(SSB_lm,  Sigma_lm,  "LeMans"),
#                                                     list(SSB_miz, Sigma_miz, "mizer"),
#                                                     list(SSB_fs,  Sigma_fs,  "FishSUMS")),
#                                   priors = priors, prior_model)
#  

## ----plot_priors, fig.dim = c(7, 4), eval = FALSE-----------------------------
#  plot(prior_samples)

## ----fig.dim = c(7, 6), echo = FALSE, warning = FALSE-------------------------
knitr::include_graphics("data/plot_priors_only.png")

## ---- label = "create_data_manual"--------------------------------------------
ens_data <- EnsembleData(observations = list(SSB_obs, Sigma_obs),
                          simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
                                            list(SSB_lm,  Sigma_lm,  "LeMans"),
                                            list(SSB_miz, Sigma_miz, "mizer"),
                                            list(SSB_fs,  Sigma_fs,  "FishSUMS")),
                          priors = priors)


## ---- label = "fitting_ensemble_DIY", eval=FALSE------------------------------
#  mod <- get_mcmc_ensemble_model(priors)
#  samples <- rstan::sampling(mod, data = ens_data@stan_input)
#  fit <- EnsembleFit(ens_data, samples = samples)

## ---- eval = FALSE------------------------------------------------------------
#  # Generating samples using DIY functions
#  transf_data <- get_transformed_data(fit)
#  mle_sample <- gen_sample(1, ex.fit = rstan::extract(samples), transformed_data = transf_data,
#                           time = ens_data@stan_input$time)

## ---- eval=FALSE--------------------------------------------------------------
#  fit <- fit_ensemble_model(observations = observations, simulators = simulators,
#                            priors = priors)
#  samples_tmp <- generate_sample(fit)
#  samples <- samples_tmp@samples

## ----echo=FALSE,warning=FALSE,error=FALSE,message=FALSE-----------------------
library(rstan)
library(EcoEnsemble)

## ----eval=FALSE---------------------------------------------------------------
# inv_wish_st_mod <- stan_model(model_code="data {
#   int<lower=0> N;
#   real nu;
#   matrix[N,N] mat_cov;
# }
# parameters {
#   cov_matrix[N] Sigma;
# }
# model {
#   Sigma ~ inv_wishart(nu, mat_cov);
# }
# ")

## ----eval=FALSE---------------------------------------------------------------
# fit.iw <- sampling(inv_wish_st_mod,data=list(N=4,nu=2*4,mat_cov=diag(4)))

## ----echo=FALSE---------------------------------------------------------------
load("data/ExploringPriors_invWishartDensities.RData")

## ----eval = FALSE-------------------------------------------------------------
# ex.fit.iw <- extract(fit.iw)
# plot(density(ex.fit.iw$Sigma[,1,1]),main="",xlab=expression(lambda),xlim=c(0,2))

## ----plotDensity, echo = FALSE------------------------------------------------
plot(invWishartDensity,main="",xlab=expression(lambda),xlim=c(0,2))

## ----eval = FALSE-------------------------------------------------------------
# sd.2050 <- sqrt(33*ex.fit.iw$Sigma[,1,1])
# plot(density(sqrt(33*ex.fit.iw$Sigma[,1,1])),main="",
#      xlab="standard deviation in 2050",xlim=c(0,5))

## ----plotSDDensity, echo = FALSE----------------------------------------------
plot(density(sd.2050),main="",xlab="standard deviation in 2050",xlim=c(0,5))

## -----------------------------------------------------------------------------
x <- seq(5,20,length.out=1000)
plot(x,colMeans(sapply(x, function(xs){dnorm(xs,tail(SSB_obs$Cod,n=1),sd.2050)})),
     type="l",axes=F,ylab="density",xlab="Cod in 2050 (1000 tonnes)")
labsx <- log(signif(exp(seq(5,20,length.out=7)),1))
axis(1,at=labsx,labels=c(exp(labsx)/1000))
axis(2)
abline(v=tail(SSB_obs$Cod,n=1))
abline(v=range(SSB_obs$Cod),lty=2)
box()

## -----------------------------------------------------------------------------
xs <- seq(0,5,0.001)
plot(xs,dgamma(xs,1,1),xlab=expression(pi[gamma,i]^2),ylab="density",type="l")

## -----------------------------------------------------------------------------
pis <- rgamma(1e5,1,1)
plot(density(sqrt(2 * pis)),xlab="Standard deviation",ylab="Density",main="")

## -----------------------------------------------------------------------------
xs <- seq(0,5,length.out=1000)
plot(xs,2*colMeans(sapply(xs, function(x){dnorm(x,0,sqrt(2*pis))})),
     type="l",axes=T,ylab="density",xlab="Difference of two models")

## ----eval=FALSE---------------------------------------------------------------
# cor_pri_st <- stan_model(model_code = " functions{
#   real priors_cor_hierarchical_beta(matrix ind_st_cor, int N, matrix M){
#     real log_prior = 0;
#     for (i in 1:(N-1)){
#       for (j in (i+1):N){
#         log_prior += beta_lpdf(0.5*(ind_st_cor[i, j] + 1)| M[i, j], M[j, i] );
#       }
#     }
#     return log_prior;
#   }
# }
# data {
# 
#   vector[4] cor_p;
# }
# 
# parameters {
#   matrix <lower=0>[5,5] beta_pars;
#   corr_matrix[5] rho[4];
# }
# model {
# 
#   for (i in 1:4){
#     for (j in (i+1):5){
#       beta_pars[i,j] ~ gamma(cor_p[1],cor_p[2]);
#       beta_pars[j,i] ~ gamma(cor_p[3],cor_p[4]);
#     }
#   }
#   for(i in 1:4){
#     target += priors_cor_hierarchical_beta(rho[i],5,beta_pars);
#     diagonal(beta_pars) ~ std_normal();
#   }
# }
# ")
# fit_cor <- sampling(cor_pri_st,data=list(cor_p=0.1 * c(1,1,1,1)),chains=4)
# ex.fit <- extract(fit_cor)
# def.par <- par(no.readonly=TRUE) #Old pars to reset afterwards
# par(mfrow=c(2,2))
# plot(density(ex.fit$beta_pars[,1,2]),xlab=expression(alpha[rho]),main="")
# plot(density(log(ex.fit$beta_pars[,1,2]/ex.fit$beta_pars[,2,1])),main="",
#      xlab="Expected log odds")
# plot(density(ex.fit$rho[,1,1,2]),xlab=expression(rho),main="")
# xs <- seq(-1,1,length.out=100)
# lines(xs,dbeta((xs+1)/2,2,2)/2,col="red")
# plot(density(apply(ex.fit$rho[,,1,2],1,var)),xlim=c(0,0.35),main="",
#      xlab=expression(var(rho)))
# par(def.par)

## ----echo=FALSE, fig.dim = c(7,6)---------------------------------------------
def.par <- par(no.readonly=TRUE) #old pars
load("data/ExploringPriors_HierBetaDensities.RData")
par(mfrow=c(2,2))
plot(beta_marginal,xlab=expression(alpha[rho]),main="")
plot(expected_log_odds,main="",xlab="Expected log odds")
plot(rho_density,xlab=expression(rho),main="")
xs <- seq(-1,1,length.out=100)
lines(xs,dbeta((xs+1)/2,2,2)/2,col="red")
plot(rho_var_density,xlim=c(0,0.35),main="",xlab=expression(var(rho)))

## ----eval=FALSE---------------------------------------------------------------
# st_mod1 <- stan_model(model_code = "data {
#   vector[4] gam_p;
# }
# parameters {
#   vector [5] log_sha_st_var[4];
#   vector[5] gamma_mean;
#   vector<lower=0> [5] gamma_var;
# }
# model {
#   gamma_mean ~ normal(gam_p[1],gam_p[2]);
#   gamma_var ~ inv_gamma(gam_p[3],gam_p[4]);
#   for (i in 1:4){
#     log_sha_st_var[i] ~ normal(gamma_mean,sqrt(gamma_var));
# 
#   }
# }
# generated quantities{
#   vector[5] sha_st_var[4];
#   for (i in 1:4){
#     sha_st_var[i] = exp(log_sha_st_var[i]);
#   }
# }
# ")
# test.fit_norm <- sampling(st_mod1,data=list(gam_p=c(-3,1,8,4)),chains=4)
# ex.fit <- extract(test.fit_norm)
# def.par <- par(no.readonly=TRUE) #old pars
# par(mfrow=c(2,2))
# plot(density(ex.fit$gamma_mean[,1]),main="",xlab=expression(mu[pi]))
# plot(density(ex.fit$gamma_var[,1]),xlim=c(0,1),main="",xlab=expression(sigma[pi]^2))
# plot(density(ex.fit$sha_st_var[,1,1]),xlim=c(0,1),main="",xlab=expression(pi["k,i"]^2))
# plot(density(apply(ex.fit$sha_st_var[,,1],1,var)),xlim=c(0,0.2),
#      xlab=expression(var(pi["k,i"]^2)) ,main="")
# par(def.par)

## ----echo = FALSE,fig.dim=c(7, 6)---------------------------------------------
load("data/ExploringPriors_HierGammaDensities.RData")
par(mfrow=c(2,2))
plot(gamma_mean,main="",xlab=expression(mu[pi]))
plot(gamma_var,xlim=c(0,1),main="",xlab=expression(sigma[pi]^2))
plot(sha_st_var,xlim=c(0,1),main="",xlab=expression(pi["k,i"]^2))
plot(sha_st_var_var,xlim=c(0,0.2),xlab=expression(var(pi["k,i"]^2)) ,main="")

## -----------------------------------------------------------------------------
hist(rbeta(1e5,2,2)*2 - 1,probability = T,xlab=expression(r["k,i"]),main="")

## -----------------------------------------------------------------------------
plot(density(rgamma(1e5,1,10)),xlim=c(0,0.5), main="",xlab=expression(pi[eta]^2))

## -----------------------------------------------------------------------------
plot(density(sqrt(rgamma(1e5,1,10))),xlim=c(0,0.5), main="",xlab=expression(pi[eta]))

## -----------------------------------------------------------------------------
hist(rbeta(1e5,2,2)*2 - 1,probability = T,xlab=expression(r["k,i"]),main="")

## ----eval=FALSE,fig.dim=c(7,6)------------------------------------------------
# priors <- EnsemblePrior(4,
#                         ind_st_params =IndSTPrior("hierarchical",list(-3,1,8,4),
#                                                   list(0.1,0.1,0.1,0.1),AR_params=c(2,2)),
#                         ind_lt_params = IndLTPrior("lkj",list(1,1),1),
#                         sha_st_params = ShaSTPrior("lkj",list(1,10),1,AR_params=c(2,2)),
#                         sha_lt_params = 5
# )
# prior_density <- prior_ensemble_model(priors, M = 4)
# samples <- sample_prior(observations = list(SSB_obs, Sigma_obs),
#              simulators = list(list(SSB_ewe, Sigma_ewe,"EwE"),
#                    list(SSB_fs ,  Sigma_fs,"FishSums"),
#                    list(SSB_lm ,  Sigma_lm,"LeMans"),
#                    list(SSB_miz, Sigma_miz,"mizer")),
#              priors=priors,
#              sam_priors = prior_density)
# plot(samples)

## ----echo =FALSE,fig.dim=c(7,6)-----------------------------------------------
knitr::include_graphics("data/p_priors.png")

#' A constructor for the `EnsembleSample` object
#'
#'
#' A constructor for the `EnsembleSample` class. These objects are generated automatically using the `generate_sample` function.
#'
#'
#' @param ensemble_fit An `EnsembleFit` object containing the fitted ensemble model.
#' @param mle An `array` of dimension \eqn{T \times (M + 2)\times N_{sample}} containing MLE point estimates from the `ensemble_fit` object, where \eqn{T} is the total time, \eqn{M} is the number of simulators and \eqn{N_{sample}} is the number of samples. For each time step, the `t`th element of the array is a `matrix` where each column is a sample and the rows are the variables:
#' \deqn{\left( y^{(t)}, \eta^{(t)}, z_1^{(t)}, z_2^{(t)}, \ldots, z_M^{(t)}\right)'}
#' where \eqn{y^{(t)}} is the ensemble model's prediction of the latent truth value at time \eqn{t},
#' \eqn{\eta^{(t)}} is the shared short-term discrepancy at time \eqn{t},
#' \eqn{z_i^{(t)}} is the individual short-term discrepancy of simulator \eqn{i} at time \eqn{t}.

#' @param samples An `array` of dimension \eqn{T \times (M + 2)\times N_{sample}} containing samples from the `ensemble_fit` object, where \eqn{T} is the total time, \eqn{M} is the number of simulators and \eqn{N_{sample}} is the number of samples. For each time step, the `t`th element of the array is a `matrix` where each column is a sample and the rows are the variables:
#' \deqn{\left( y^{(t)}, \eta^{(t)}, z_1^{(t)}, z_2^{(t)}, \ldots, z_M^{(t)}\right)'}
#' where \eqn{y^{(t)}} is the ensemble model's prediction of the latent truth value at time \eqn{t},
#' \eqn{\eta^{(t)}} is the shared short-term discrepancy at time \eqn{t},
#' \eqn{z_i^{(t)}} is the individual short-term discrepancy of simulator \eqn{i} at time \eqn{t}.
#'@return An object of class `EnsembleSample`
#' @seealso \code{\link{EnsembleSample}}, \code{\link{generate_sample}}
#' @export
EnsembleSample <- function(ensemble_fit, mle, samples) {
  ensemble_sample <- new('EnsembleSample',
                         ensemble_fit = ensemble_fit,
                         mle = mle,
                         samples = samples)
  return(ensemble_sample)
}

#### Class definition ####
#' A class to hold  samples of the ensemble model
#'
#'
#' `EnsembleSample` objects are generated using the `generate_sample` function.
#'
#'
#' @slot ensemble_fit An `EnsembleFit` object containing the fitted ensemble model.
#' @slot mle An `array` of dimension \eqn{T \times (M + 2)\times N_{sample}} containing MLE point estimates from the `ensemble_fit` object, where \eqn{T} is the total time, \eqn{M} is the number of simulators and \eqn{N_{sample}} is the number of samples. For each time step, the `t`th element of the array is a `matrix` where each column is a sample and the rows are the variables:
#' \deqn{\left( y^{(t)}, \eta^{(t)}, z_1^{(t)}, z_2^{(t)}, \ldots, z_M^{(t)}\right)'}
#' where \eqn{y^{(t)}} is the ensemble model's prediction of the latent truth value at time \eqn{t},
#' \eqn{\eta^{(t)}} is the shared short-term discrepancy at time \eqn{t},
#' \eqn{z_i^{(t)}} is the individual short-term discrepancy of simulator \eqn{i} at time \eqn{t}.

#' @slot samples An `array` of dimension \eqn{T \times (M + 2)\times N_{sample}} containing samples from the `ensemble_fit` object, where \eqn{T} is the total time, \eqn{M} is the number of simulators and \eqn{N_{sample}} is the number of samples. For each time step, the `t`th element of the array is a `matrix` where each column is a sample and the rows are the variables:
#' \deqn{\left( y^{(t)}, \eta^{(t)}, z_1^{(t)}, z_2^{(t)}, \ldots, z_M^{(t)}\right)'}
#' where \eqn{y^{(t)}} is the ensemble model's prediction of the latent truth value at time \eqn{t},
#' \eqn{\eta^{(t)}} is the shared short-term discrepancy at time \eqn{t},
#' \eqn{z_i^{(t)}} is the individual short-term discrepancy of simulator \eqn{i} at time \eqn{t}.
#' @seealso \code{\link{EnsembleSample}}, \code{\link{generate_sample}}
#' @export
setClass(
  "EnsembleSample",
  slots = c(
    ensemble_fit = "EnsembleFit",
    mle = "array",
    samples = "array"
  )
)

generate_simulator_stan_data_dri <- function(observations, simulators, MMod){
  N <- ncol(observations[[1]])
  M <- length(simulators)
  if (!missing(MMod)){
    MM <- max(unique(unlist(MMod)))
    n_dri <- sapply(MMod, length)
  }
  else {
    MM <- length(simulators[[1]][[1]])
    MMod <- replicate(M, 1:MM, simplify = F)
    n_dri <- rep(MM, M)
  }
  times <- rownames(observations[[1]])
  model_num_species <- c()
  Ms <- matrix(NA, nrow = 0, ncol = N)
  model_covariances <- c()
  observed_species <- colnames(observations[[1]])
  mod_to_dri <- matrix(0, nrow = M, ncol = MM)
  for (i in 1:M) {
    sim <- simulators[[i]]
    for (j in 1:n_dri[i]) {
      model_output <- sim[[1]][[j]]
      times <- unique(append(times, rownames(model_output)))
      mod_num_spec <- ncol(model_output)
      model_num_species <- c(model_num_species, mod_num_spec)
      sim_species <- colnames(model_output)
      Mij <- matrix(0, nrow = mod_num_spec, ncol = N)
      for (k in 1:mod_num_spec){
        for (l in 1:N){
          Mij[k,l] <- sim_species[k] == observed_species[l]
        }
      }
      Ms <- rbind(Ms, Mij)
      ## covariances
      if (is.matrix(sim[[2]][[j]])){
        model_covariances <- c(model_covariances, as.vector(sim[[2]][[j]]))
      }else{
        model_covariances <- c(model_covariances, as.numeric(sim[[2]][[j]]))
      }
      dri_index <- MMod[[i]][j]
      mod_to_dri[i, dri_index] <- dri_index
    }
  }

  times <- sort(as.integer(times))

  #Things that can't be done on the first pass
  observation_times <- matrix(NA, nrow=length(times), ncol=0)
  observation_times <- cbind(observation_times, times %in% rownames(observations[[1]]))
  for (i in 1:M) {
    sim <- simulators[[i]]
    model_output <- sim[[1]][[1]]
    present_data <- as.numeric(times %in% rownames(model_output))
    observation_times <- cbind(observation_times, present_data)
  }

  model_outputs <- matrix(NA, nrow=length(times), ncol=0)
  for (i in 1:M) {
    sim <- simulators[[i]]
    for (j in 1:n_dri[i]){
      model_output <- sim[[1]][[j]]
      y_i <- matrix(0, nrow = length(times), ncol=ncol(model_output))
      for (k in 1:length(times)){
        year = times[k]
        dat_for_year <- model_output[year == rownames(model_output), ]
        if (nrow(dat_for_year) == 1 || length(dat_for_year) == 1){
          y_i[k, ] <- unlist(dat_for_year)
        }
      }
      model_outputs <- cbind(model_outputs, y_i)
    }
  }

  obs_data <- observations[[1]]
  obs_data_all <- matrix(0, nrow = length(times), ncol=N)
  for (k in 1:length(times)){
    year = times[k]
    obs_for_year <- obs_data[year == rownames(obs_data), ]
    if (nrow(obs_for_year) == 1 || length(obs_for_year) == 1){
      obs_data_all[k, ] <- unlist(obs_for_year)
    }
  }
  obs_covariances <- observations[[2]]

  return(list(N = N,
              time = length(times),
              M = M,
              MM=MM,
              n_dri = n_dri,
              mod_to_dri = mod_to_dri,
              model_num_species = model_num_species,
              Ms = Ms,
              observation_times = observation_times,
              model_outputs = model_outputs,
              model_covariances = model_covariances,
              observations = obs_data_all,
              obs_covariances = obs_covariances))
}

#'Return the compiled ensemble model Stan object.
#'
#'Gets the unfit, compiled `stanmodel` object encoding the ensemble model. This allows for
#'manual fitting of the ensemble model directly using \code{rstan::sampling}.
#'
#'@param priors An `EnsemblePrior` object specifying the prior distributions for the ensemble for which the compiled `stanmodel` object will be obtained.
#'@param likelihood A `logical` that returns the compiled `stanmodel` object including the likelihood (the Kalman filter) for given priors if `TRUE`. If `FALSE` returns the compiled `stanmodel` object without the likelihood for sampling from the prior.
#'@param drivers A `logical` indicating whether drivers have been used in combination with simulators. Default value is FALSE.
#'@param sampler A character string selecting which state-space sampler to
#'  use. Either `explicit` (the default), which samples latent states
#'  directly in the Stan program, or `kalman` which retains the Kalman
#'  filter implementation.
#'@return The `stanmodel` object encoding the ensemble model.
#'@export
#'@examples
#'priors <- EnsemblePrior(4)
#'mod <- get_mcmc_ensemble_model(priors)
#'
#'ensemble_data <- EnsembleData(observations = list(SSB_obs, Sigma_obs),
#'                              simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
#'                                            list(SSB_fs,  Sigma_fs, "FishSUMS"),
#'                                            list(SSB_lm,  Sigma_lm, "LeMans"),
#'                                            list(SSB_miz, Sigma_miz, "mizer")),
#'                               priors = priors)
#'\donttest{
#'out <- rstan::sampling(mod, ensemble_data@@stan_input, chains = 1)
#'}

get_mcmc_ensemble_model <- function(priors, likelihood = TRUE, drivers = FALSE,
                                    sampler = c("explicit", "kalman")){

  sampler <- match.arg(sampler)
  use_explicit <- identical(sampler, "explicit")

  st_pf <- priors@ind_st_params@parametrisation_form
  if (st_pf == "hierarchical") {
    if (likelihood) {
      if (!drivers){
        if (use_explicit) {
          return(stanmodels$ensemble_model_hierarchical_explicit)
        }
        return(stanmodels$ensemble_model_hierarchical)
      } else {
        if (use_explicit) {
          return(stanmodels$ensemble_model_hierarchical_withdrivers_explicit)
        }
        return(stanmodels$ensemble_model_hierarchical_withdrivers)
      }
    } else {
      if (!drivers) {
        return(stanmodels$ensemble_prior_hierarchical)
      } else {
        return(stanmodels$ensemble_prior_hierarchical_withdrivers)
      }
    }
  } else {
    if (likelihood) {
      if (!drivers) {
        if (use_explicit) {
          return(stanmodels$ensemble_model_explicit)
        }
        return(stanmodels$ensemble_model)
      } else {
        if (use_explicit) {
          return(stanmodels$ensemble_model_withdrivers_explicit)
        }
        return(stanmodels$ensemble_model_withdrivers)
      }
    } else {
      if (!drivers) {
        return(stanmodels$ensemble_prior)
      } else {
        return(stanmodels$ensemble_prior_withdrivers)
      }
    }
  }
}

#'Fits the ensemble model
#'
#'`fit_ensemble_model` runs an MCMC of the ensemble model. This process can take a long time depending on the size of the datasets.
#'
#'@inheritParams EnsembleData
#'@param full_sample A `logical` that runs a full sampling of the posterior density of the ensemble model if `TRUE`. If `FALSE`, returns the point estimate which maximises the posterior density of the ensemble model.
#'@param control If creating a full sample, this is a named `list` of paramaters to control Stan's sampling behaviour. See the documentation of the `stan()` function in the `rstan` package for details. The default value is `list(adapt_delta = 0.95)`. If optimizing, this value is ignored.
#'@param sampler A character string choosing between the "explicit" latent
#'  sampler (default) and the legacy "kalman" implementation.
#'@param ... Additional arguments passed to the function \code{rstan::sampling} or  \code{rstan::optimizing}.
#'@inherit EnsembleData details
#'@return An `EnsembleFit` object.
#'@references Stan Development Team (2020). RStan: the R interface to Stan. R package version 2.21.2. https://mc-stan.org
#'@seealso \code{\linkS4class{EnsembleFit}}, \code{\link{EnsembleSample}}
#'@export
#'@examples
#'\donttest{
#'
#' fit <- fit_ensemble_model(observations = list(SSB_obs, Sigma_obs),
#'                simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
#'                                  list(SSB_fs,  Sigma_fs, "FishSUMS"),
#'                                  list(SSB_lm,  Sigma_lm, "LeMans"),
#'                                  list(SSB_miz, Sigma_miz, "Mizer")),
#'                priors = EnsemblePrior(4,
#'                ind_st_params = IndSTPrior(parametrisation_form = "lkj",
#'                var_params= list(1,1), cor_params = 10, AR_params = c(2, 2))),
#'                full_sample = FALSE) #Only optimise in this case
#'}
fit_ensemble_model <- function(observations, simulators, priors,
                               full_sample = TRUE, control = list(adapt_delta = 0.95),
                               drivers = FALSE, sampler = c("explicit", "kalman"),
                               MMod, ...){

  sampler <- match.arg(sampler)

  if (drivers == TRUE) {
    return(fit_ensemble_model_dri(observations, simulators, priors,
                                  full_sample = full_sample,
                                  control = control,
                                  sampler = sampler,
                                  MMod, ...))
  }
  else {
    ens_data <- EnsembleData(observations, simulators, priors)
    stan_input <- ens_data@stan_input

    #Using hierarchical priors uses a different model. This speeds up the sampling enormously
    if (sampler == "explicit"){
    mod <- stanmodels$ensemble_model_explicit
    } else {
    mod <- stanmodels$ensemble_model
    }

    if(stan_input$form_prior_ind_st == 3 || stan_input$form_prior_ind_st == 4){
      if (sampler == "explicit"){
        mod <- stanmodels$ensemble_model_hierarchical_explicit
      } else {
        mod <- stanmodels$ensemble_model_hierarchical
      }
      if(!full_sample){
        stop("It is possible to generate a point estimate for the prior if the individual short-term discrepancy prior follows a hierarchical parameterisation. Please generate a full sample using 'full_sample=TRUE'.")
      }
    }


    samples <- NULL; point_estimate <- NULL
    if(full_sample){
      samples <- stan_sampling_with_filter(mod, data = stan_input,
                                           control = control,
                                           ...)
    }else{
      point_estimate <- rstan::optimizing(mod, data=stan_input,as_vector=FALSE, ...)
    }

    return(EnsembleFit(ens_data, samples, point_estimate))
  }
}

test_that("prior combinations",{
  d<- 2
  M <- 2
  Time <- 10
  Times_obs <- round(Time * 0.8)
  obs <- matrix(c(0.57,0.75,0.9,1,1.07,1.09,1.04,0.94,0.99,0.54,0.57,1.09,1.67,2.01,2.17,2.33),ncol=d)
  obs.cov <- matrix(c(0.07,-0.02,-0.02,0.01),ncol=d)
  model.cov <- array(0,dim=c(M,d,d))
  models_output <- array(NA,dim=c(M,Time,d))
  models_output[1,,] <- c(-0.14,0.3,0.78,1.19,1.38,1.27,1.04,0.97,1.27,1.85,3.75,3.18,3.01,3.33,3.91,4.41,4.71,4.97,5.42,6.08)
  models_output[2,,] <- c(-0.94,-0.55,-0.25,-0.15,-0.26,-0.49,-0.7,-0.73,-0.52,-0.17,1.48,0.93,0.72,0.98,1.56,2.14,2.54,2.78,2.99,3.25)
  model.cov[1,,] <- c(0.06,0,0,0.03)
  model.cov[2,,] <- c(0.04,0.01,0.01,0.02)

  val_obs <- data.frame(obs); cov_obs <- obs.cov
  val_model_1 <- data.frame(models_output[1,,]); cov_model_1 <- model.cov[1,,]
  val_model_2 <- data.frame(models_output[2,,]); cov_model_2 <- model.cov[2,,]


  #Set the dimnames to ensure EcoEnsemble can identify the information.
  SPECIES_NAMES <- c("Species 1", "Species 2")
  dimnames(val_obs) <- list(paste(1:Times_obs), SPECIES_NAMES)
  dimnames(val_model_1) <- list(paste(1:Time), SPECIES_NAMES)
  dimnames(val_model_2) <- list(paste(1:Time), SPECIES_NAMES)


  dimnames(cov_obs) <- list(SPECIES_NAMES, SPECIES_NAMES)
  dimnames(cov_model_1) <- list(SPECIES_NAMES, SPECIES_NAMES)
  dimnames(cov_model_2) <- list(SPECIES_NAMES, SPECIES_NAMES)


  priors1 <- EnsemblePrior(2,
                           ind_st_params = IndSTPrior(parametrisation_form = "inv_wishart", var_params= list(1,1), cor_params = list(10,diag(2)), AR_params = c(2, 2)),
                           ind_lt_params = IndLTPrior("beta",list(10,5),list(matrix(5, 2, 2),matrix(2, 2, 2))
                           ),
                           sha_st_params = ShaSTPrior("inv_wishart",list(2, 1/3),list(10, diag(2))))

  set.seed(1234)
  samples <- sample_prior(observations = list(val_obs, cov_obs),
                          simulators = list(list(val_model_1, cov_model_1, "Model 1"),
                                            list(val_model_2, cov_model_2, "Model 2")
                          ),
                          priors = priors1,
                          full_sample = FALSE)
  fit <- fit_ensemble_model(observations = list(val_obs, cov_obs),
                            simulators = list(list(val_model_1, cov_model_1, "Model 1"),
                                              list(val_model_2, cov_model_2, "Model 2")
                            ),
                            priors = priors1,
                            full_sample = FALSE)


  priors2 <- EnsemblePrior(2,
                           ind_st_params = IndSTPrior(parametrisation_form = "beta", var_params= list(c(1,2),c(1,1)), cor_params = list(matrix(50, 2, 2),matrix(50, 2, 2))
                                                      , AR_params = c(2, 2)),
                           ind_lt_params = IndLTPrior("inv_wishart",list(c(1,2),c(1,1)),list(10,diag(2))
                           ),
                           sha_st_params = ShaSTPrior("beta",list(c(1,2),c(1,1)),list(matrix(5, 2, 2),matrix(2, 2, 2))
                           ))

  for (sampler in c("kalman", "explicit")){

    samples <- sample_prior(observations = list(val_obs, cov_obs),
                            simulators = list(list(val_model_1, cov_model_1, "Model 1"),
                                              list(val_model_2, cov_model_2, "Model 2")
                            ),
                            priors = priors2,
                            full_sample = FALSE)
    fit <- fit_ensemble_model(observations = list(val_obs, cov_obs),
                              simulators = list(list(val_model_1, cov_model_1, "Model 1"),
                                                list(val_model_2, cov_model_2, "Model 2")
                              ),
                              priors = priors2, sampler = sampler,
                              full_sample = FALSE)
    priors3 <- EnsemblePrior(2,ind_st_params = IndSTPrior(parametrisation_form = "lkj", var_params= list(1,1), cor_params = 10, AR_params = c(2, 2)))
    fit <- fit_ensemble_model(observations = list(val_obs, cov_obs),
                              simulators = list(list(val_model_1, cov_model_1, "Model 1"),
                                                list(val_model_2, cov_model_2, "Model 2")
                              ),
                              priors = priors3, sampler = sampler,
                              full_sample = FALSE)

    #Hierarchical priors
    priors4 <- EnsemblePrior(2,ind_st_params = IndSTPrior(parametrisation_form = "hierarchical", var_params= list(7,4,6,5), cor_params = list(0.4, 0.2, 0.7, 0.1), AR_params = c(2, 2)))
    suppressWarnings(fit <- fit_ensemble_model(observations = list(val_obs, cov_obs),
                                               simulators = list(list(val_model_1, cov_model_1, "Model 1"),
                                                                 list(val_model_2, cov_model_2, "Model 2")
                                               ),
                                               priors = priors4,sampler = sampler,
                                               control = list(adapt_delta = 0.9), full_sample = TRUE, chains = 1, iter = 4))
    priors5 <- EnsemblePrior(2,ind_st_params = IndSTPrior(parametrisation_form = "hierarchical_beta_conjugate", var_params= list(7,4,6,5), cor_params = list(0.9, 0.9, 0.5), AR_params = c(2, 2)))
    suppressWarnings(fit <- fit_ensemble_model(observations = list(val_obs, cov_obs),
                                               simulators = list(list(val_model_1, cov_model_1, "Model 1"),
                                                                 list(val_model_2, cov_model_2, "Model 2")
                                               ),
                                               priors = priors5, sampler = sampler,
                                               control = list(adapt_delta = 0.9), full_sample = TRUE, chains = 1, iter = 4))


    #Errors
    expect_error(EnsemblePrior(2,ind_st_params = IndSTPrior(parametrisation_form = "beta", var_params= list(1,1), cor_params = list(10,diag(2)), AR_params = c(2, 2))),"Invalid beta parameters for individual short-term correlation matrix priors. These should be square, symmetric matrices with the same dimension as the number of model outputs.")
    expect_error (EnsemblePrior(2,ind_st_params = IndSTPrior(parametrisation_form = "inv_wishart", var_params= list(1,1), cor_params = list(10), AR_params = c(2, 2))),"Invalid inverse Wishart parameters for individual short-term correlation matrix prior. This should be a numeric giving the degrees of freedom and a d x d scale matrix, with degrees of freedom > d-1.")
    expect_error (EnsemblePrior(2,ind_st_params = IndSTPrior(parametrisation_form = "lkj", var_params= list(1,1), cor_params = list(10,2), AR_params = c(2, 2))),"Invalid lkj parameter for individual short-term correlation matrix prior. This should be a list with exactly one real number.")
    expect_error (EnsemblePrior(2,ind_st_params = IndSTPrior(parametrisation_form = "lkj", var_params= list(1,1), cor_params = list(10,2), AR_params = c(2, 2))),"Invalid lkj parameter for individual short-term correlation matrix prior. This should be a list with exactly one real number.")
    expect_error (EnsemblePrior(2,ind_st_params = IndSTPrior(parametrisation_form = "hierarchical", var_params= list(1,1), cor_params = list(10,2), AR_params = c(2, 2))),"Invalid parameters for individual short-term correlation matrix priors. This should be a list of length 4.")
    expect_error (EnsemblePrior(2,ind_st_params = IndSTPrior(parametrisation_form = "hierarchical_beta_conjugate", var_params= list(1,1), cor_params = list(10,2), AR_params = c(2, 2))),"Invalid parameters for individual short-term correlation matrix priors. This should be a list of length 3.")
    expect_error (EnsemblePrior(2,ind_st_params = IndSTPrior(parametrisation_form = "boro", var_params= list(1,1), cor_params = list(10,2), AR_params = c(2, 2))),"Invalid parametrisation choice for priors. Prior parametrisation forms should be one of 'lkj', 'inv_wishart', 'beta', 'hierarchical' or 'hierarchical_beta_conjugate'. The 'hierarchical' and 'hierarchical_beta_conjugate' options are only available for individual short-term discrepancies. Prior choice: boro")
    expect_error (EnsemblePrior(2,ind_st_params = IndSTPrior(parametrisation_form = "lkj", var_params= list(1,1), cor_params = 10, AR_params = c(2, 2,2))),"Invalid autoregressive parameters specified. This should be a numeric of length 2 giving the shape parameters of the Beta distribution used as a prior for the AR parameters.")
    expect_error (EcoEnsemble:::validate_data(observations = list(val_obs, cov_obs),
                                              simulators = list(val_model_1, cov_model_1,"Model 1"
                                              ),
                                              priors = priors1))
    val_model_1a <- val_model_1
    names(val_model_1a)[2] <- "Car"
    expect_error(EcoEnsemble:::validate_data(observations = list(val_obs, cov_obs),
                                             simulators = list(list(val_model_1a,cov_model_1,"Model 1"
                                             )),
                                             priors = priors1))
    #Sampling for non-hierarchical options
    priors1 <- EnsemblePrior(2,
                             ind_lt_params = IndLTPrior("beta",list(10,5),list(matrix(5, 2, 2),matrix(2, 2, 2))
                             ),
                             sha_st_params = ShaSTPrior("inv_wishart",list(2, 1/3),list(10, diag(2))))
    suppressWarnings(fit <- fit_ensemble_model(observations = list(val_obs, cov_obs),
                                               simulators = list(list(val_model_1, cov_model_1, "Model 1"),
                                                                 list(val_model_2, cov_model_2, "Model 2")
                                               ),
                                               priors = priors1, sampler = sampler,
                                               control = list(adapt_delta = 0.9),chains=1,iter=4))
    priors1 <- EnsemblePrior(2,
                             ind_lt_params = IndLTPrior("inv_wishart",list(c(1,2),c(1,1)),list(10,diag(2))
                             ),
                             sha_st_params = ShaSTPrior("beta",list(c(1,2),c(1,1)),list(matrix(5, 2, 2),matrix(2, 2, 2))))
    suppressWarnings(fit <- fit_ensemble_model(observations = list(val_obs, cov_obs),
                                               simulators = list(list(val_model_1, cov_model_1, "Model 1"),
                                                                 list(val_model_2, cov_model_2, "Model 2")
                                               ),
                                               priors = priors1,sampler = sampler,
                                               control = list(adapt_delta = 0.9),chains=1,iter=4))
  }
})



test_that("prior combinations with drivers",{
  d<- 2
  M <- 2
  MM <- 2
  Time <- 10
  Times_obs <- round(Time * 0.8)
  obs <- matrix(c(0.57,0.75,0.9,1,1.07,1.09,1.04,0.94,0.99,0.54,0.57,1.09,1.67,2.01,2.17,2.33),ncol=d)
  obs.cov <- matrix(c(0.07,-0.02,-0.02,0.01),ncol=d)
  model.cov <- array(0,dim=c(M * MM,d,d))
  models_output <- array(NA,dim=c(M * MM,Time,d))
  models_output[1,,] <- c(-0.14,0.3,0.78,1.19,1.38,1.27,1.04,0.97,1.27,1.85,3.75,3.18,3.01,3.33,3.91,4.41,4.71,4.97,5.42,6.08)
  models_output[2,,] <- c(-0.94,-0.55,-0.25,-0.15,-0.26,-0.49,-0.7,-0.73,-0.52,-0.17,1.48,0.93,0.72,0.98,1.56,2.14,2.54,2.78,2.99,3.25)
  models_output[3,,] <- c(-0.05, 0.35, 0.84, 1.15, 1.18, 1.17, 1.08, 1.16, 1.94, 2.39, 3.93, 3.19, 2.88, 3.17, 3.69, 3.89, 3.99, 4.85, 5.6, 6.2)
  models_output[4,,] <- c(-1.16,-0.75,-0.65,-0.56,-0.66,-0.79,-0.9,-0.43,-0.02, 0.17,1.18,0.27,0.12,0.38,0.46,0.54,0.94,2.08,2.71,3.35)
  model.cov[1,,] <- c(0.06,0,0,0.03)
  model.cov[2,,] <- c(0.04,0.01,0.01,0.02)
  model.cov[3,,] <- c(0.617, 0.011, 0.011, 0.343)
  model.cov[4,,] <- c(0.19, 0.072, 0.072, 0.263)

  val_obs <- data.frame(obs); cov_obs <- obs.cov
  val_model_11 <- data.frame(models_output[1,,]); cov_model_11 <- model.cov[1,,]
  val_model_21 <- data.frame(models_output[2,,]); cov_model_21 <- model.cov[2,,]
  val_model_12 <- data.frame(models_output[3,,]); cov_model_12 <- model.cov[3,,]
  val_model_22 <- data.frame(models_output[4,,]); cov_model_22 <- model.cov[4,,]

  #Set the dimnames to ensure EcoEnsemble can identify the information.
  SPECIES_NAMES <- c("Species 1", "Species 2")
  dimnames(val_obs) <- list(paste(1:Times_obs), SPECIES_NAMES)
  dimnames(val_model_11) <- list(paste(1:Time), SPECIES_NAMES)
  dimnames(val_model_21) <- list(paste(1:Time), SPECIES_NAMES)
  dimnames(val_model_12) <- list(paste(1:Time), SPECIES_NAMES)
  dimnames(val_model_22) <- list(paste(1:Time), SPECIES_NAMES)

  dimnames(cov_obs) <- list(SPECIES_NAMES, SPECIES_NAMES)
  dimnames(cov_model_11) <- list(SPECIES_NAMES, SPECIES_NAMES)
  dimnames(cov_model_21) <- list(SPECIES_NAMES, SPECIES_NAMES)
  dimnames(cov_model_12) <- list(SPECIES_NAMES, SPECIES_NAMES)
  dimnames(cov_model_22) <- list(SPECIES_NAMES, SPECIES_NAMES)

  val_model_1 <- list(val_model_11, val_model_12)
  val_model_2 <- list(val_model_21, val_model_22)
  cov_model_1 <- list(cov_model_11, cov_model_12)
  cov_model_2 <- list(cov_model_21, cov_model_22)


  for (sampler in c("explicit", "kalman")){

    priors1 <- EnsemblePrior(2,
                             ind_st_params = IndSTPrior(parametrisation_form = "inv_wishart", var_params= list(1,1), cor_params = list(10,diag(2)), AR_params = c(2, 2)),
                             ind_lt_params = IndLTPrior("beta",list(10,5),list(matrix(5, 2, 2),matrix(2, 2, 2))
                             ),
                             sha_st_params = ShaSTPrior("inv_wishart",list(2, 1/3),list(10, diag(2))))

    set.seed(5678)
    samples <- sample_prior(observations = list(val_obs, cov_obs),
                            simulators = list(list(val_model_1, cov_model_1, "Simulator 1", c("Driver 1", "Driver 2")),
                                              list(val_model_2, cov_model_2, "Simulator 2", c("Driver 1", "Driver 2"))
                            ),
                            priors = priors1,
                            full_sample = FALSE, drivers = TRUE)

    fit <- fit_ensemble_model(observations = list(val_obs, cov_obs),
                              simulators = list(list(val_model_1, cov_model_1, "Simulator 1", c("Driver 1", "Driver 2")),
                                                list(val_model_2, cov_model_2, "Simulator 2", c("Driver 1", "Driver 2"))
                              ),
                              priors = priors1,sampler = sampler,
                              full_sample = FALSE, drivers = TRUE)


    priors2 <- EnsemblePrior(2,
                             ind_st_params = IndSTPrior(parametrisation_form = "beta", var_params= list(c(1,2),c(1,1)), cor_params = list(matrix(50, 2, 2),matrix(50, 2, 2))
                                                        , AR_params = c(2, 2)),
                             ind_lt_params = IndLTPrior("inv_wishart",list(c(1,2),c(1,1)),list(10,diag(2))
                             ),
                             sha_st_params = ShaSTPrior("beta",list(c(1,2),c(1,1)),list(matrix(5, 2, 2),matrix(2, 2, 2))
                             ))

    samples <- sample_prior(observations = list(val_obs, cov_obs),
                            simulators = list(list(val_model_1, cov_model_1, "Simulator 1", c("Driver 1", "Driver 2")),
                                              list(val_model_2, cov_model_2, "Simulator 2", c("Driver 1", "Driver 2"))
                            ),
                            priors = priors2,
                            full_sample = FALSE, drivers = TRUE)
    fit <- fit_ensemble_model(observations = list(val_obs, cov_obs),
                              simulators = list(list(val_model_1, cov_model_1, "Simulator 1", c("Driver 1", "Driver 2")),
                                                list(val_model_2, cov_model_2, "Simulator 2", c("Driver 1", "Driver 2"))
                              ),
                              priors = priors2,sampler = sampler,
                              full_sample = FALSE, drivers = TRUE)
    priors3 <- EnsemblePrior(2,ind_st_params = IndSTPrior(parametrisation_form = "lkj", var_params= list(1,1), cor_params = 10, AR_params = c(2, 2)))
    samples <- sample_prior(observations = list(val_obs, cov_obs),
                            simulators = list(list(val_model_1, cov_model_1, "Simulator 1", c("Driver 1", "Driver 2")),
                                              list(val_model_2, cov_model_2, "Simulator 2", c("Driver 1", "Driver 2"))
                            ),
                            priors = priors3,
                            full_sample = FALSE, drivers = TRUE)
    fit <- fit_ensemble_model(observations = list(val_obs, cov_obs),
                              simulators = list(list(val_model_1, cov_model_1, "Simulator 1", c("Driver 1", "Driver 2")),
                                                list(val_model_2, cov_model_2, "Simulator 2", c("Driver 1", "Driver 2"))
                              ),
                              priors = priors3,sampler = sampler,
                              full_sample = FALSE, drivers = TRUE)

    #Hierarchical priors
    priors4 <- EnsemblePrior(2,ind_st_params = IndSTPrior(parametrisation_form = "hierarchical", var_params= list(7,4,6,5), cor_params = list(0.4, 0.2, 0.7, 0.1), AR_params = c(2, 2)))
    suppressWarnings(fit <- fit_ensemble_model(observations = list(val_obs, cov_obs),
                                               simulators = list(list(val_model_1, cov_model_1, "Simulator 1", c("Driver 1", "Driver 2")),
                                                                 list(val_model_2, cov_model_2, "Simulator 2", c("Driver 1", "Driver 2"))
                                               ),
                                               priors = priors4,sampler = sampler,
                                               control = list(adapt_delta = 0.9), full_sample = TRUE, drivers = TRUE, chains = 1, iter = 4))
    priors5 <- EnsemblePrior(2,ind_st_params = IndSTPrior(parametrisation_form = "hierarchical_beta_conjugate", var_params= list(7,4,6,5), cor_params = list(0.9, 0.9, 0.5), AR_params = c(2, 2)))
    suppressWarnings(fit <- fit_ensemble_model(observations = list(val_obs, cov_obs),
                                               simulators = list(list(val_model_1, cov_model_1, "Simulator 1", c("Driver 1", "Driver 2")),
                                                                 list(val_model_2, cov_model_2, "Simulator 2", c("Driver 1", "Driver 2"))
                                               ),
                                               priors = priors5,sampler = sampler,
                                               control = list(adapt_delta = 0.9), full_sample = TRUE, drivers = TRUE, chains = 1, iter = 4))


    #Errors
    expect_error(EnsemblePrior(2,ind_st_params = IndSTPrior(parametrisation_form = "beta", var_params= list(1,1), cor_params = list(10,diag(2)), AR_params = c(2, 2))),"Invalid beta parameters for individual short-term correlation matrix priors. These should be square, symmetric matrices with the same dimension as the number of model outputs.")
    expect_error (EnsemblePrior(2,ind_st_params = IndSTPrior(parametrisation_form = "inv_wishart", var_params= list(1,1), cor_params = list(10), AR_params = c(2, 2))),"Invalid inverse Wishart parameters for individual short-term correlation matrix prior. This should be a numeric giving the degrees of freedom and a d x d scale matrix, with degrees of freedom > d-1.")
    expect_error (EnsemblePrior(2,ind_st_params = IndSTPrior(parametrisation_form = "lkj", var_params= list(1,1), cor_params = list(10,2), AR_params = c(2, 2))),"Invalid lkj parameter for individual short-term correlation matrix prior. This should be a list with exactly one real number.")
    expect_error (EnsemblePrior(2,ind_st_params = IndSTPrior(parametrisation_form = "lkj", var_params= list(1,1), cor_params = list(10,2), AR_params = c(2, 2))),"Invalid lkj parameter for individual short-term correlation matrix prior. This should be a list with exactly one real number.")
    expect_error (EnsemblePrior(2,ind_st_params = IndSTPrior(parametrisation_form = "hierarchical", var_params= list(1,1), cor_params = list(10,2), AR_params = c(2, 2))),"Invalid parameters for individual short-term correlation matrix priors. This should be a list of length 4.")
    expect_error (EnsemblePrior(2,ind_st_params = IndSTPrior(parametrisation_form = "hierarchical_beta_conjugate", var_params= list(1,1), cor_params = list(10,2), AR_params = c(2, 2))),"Invalid parameters for individual short-term correlation matrix priors. This should be a list of length 3.")
    expect_error (EnsemblePrior(2,ind_st_params = IndSTPrior(parametrisation_form = "boro", var_params= list(1,1), cor_params = list(10,2), AR_params = c(2, 2))),"Invalid parametrisation choice for priors. Prior parametrisation forms should be one of 'lkj', 'inv_wishart', 'beta', 'hierarchical' or 'hierarchical_beta_conjugate'. The 'hierarchical' and 'hierarchical_beta_conjugate' options are only available for individual short-term discrepancies. Prior choice: boro")
    expect_error (EnsemblePrior(2,ind_st_params = IndSTPrior(parametrisation_form = "lkj", var_params= list(1,1), cor_params = 10, AR_params = c(2, 2,2))),"Invalid autoregressive parameters specified. This should be a numeric of length 2 giving the shape parameters of the Beta distribution used as a prior for the AR parameters.")
    expect_error (EcoEnsemble:::validate_data(observations = list(val_obs, cov_obs),
                                              simulators = list(val_model_1, cov_model_1, "Simulator 1", c("Driver 1", "Driver 2")
                                              ),
                                              priors = priors1))
    val_model_1a <- val_model_1
    names(val_model_1a[[1]])[2] <- "Car"
    expect_error(EcoEnsemble:::validate_data(observations = list(val_obs, cov_obs),
                                             simulators = list(list(val_model_1a,cov_model_1,"Simulator 1", c("Driver 1", "Driver 2")
                                             )),
                                             priors = priors1))
    #Sampling for non-hierarchical options
    priors1 <- EnsemblePrior(2,
                             ind_lt_params = IndLTPrior("beta",list(10,5),list(matrix(5, 2, 2),matrix(2, 2, 2))
                             ),
                             sha_st_params = ShaSTPrior("inv_wishart",list(2, 1/3),list(10, diag(2))))
    suppressWarnings(fit <- fit_ensemble_model(observations = list(val_obs, cov_obs),
                                               simulators = list(list(val_model_1, cov_model_1, "Simulator 1", c("Driver 1", "Driver 2")),
                                                                 list(val_model_2, cov_model_2, "Simulator 2", c("Driver 1", "Driver 2"))
                                               ),
                                               priors = priors1,sampler = sampler,
                                               control = list(adapt_delta = 0.9),chains=1,iter=4,drivers=TRUE))
    priors1 <- EnsemblePrior(2,
                             ind_lt_params = IndLTPrior("inv_wishart",list(c(1,2),c(1,1)),list(10,diag(2))
                             ),
                             sha_st_params = ShaSTPrior("beta",list(c(1,2),c(1,1)),list(matrix(5, 2, 2),matrix(2, 2, 2))))
    suppressWarnings(fit <- fit_ensemble_model(observations = list(val_obs, cov_obs),
                                               simulators = list(list(val_model_1, cov_model_1, "Simulator 1", c("Driver 1", "Driver 2")),
                                                                 list(val_model_2, cov_model_2, "Simulator 2", c("Driver 1", "Driver 2"))
                                               ),
                                               priors = priors1,sampler = sampler,
                                               control = list(adapt_delta = 0.9),chains=1,iter=4,drivers=TRUE))
  }
})

test_that("get ensemble model works", {

  priors_hier <- EnsemblePrior(4)
  priors_lkj <- EnsemblePrior(4, ind_st_params = IndSTPrior("lkj", list(3, 2), 3, AR_params = c(2,4)))

  expect_equal(get_mcmc_ensemble_model(priors_hier, sampler = "explicit")@model_name, "ensemble_model_hierarchical_explicit")
  expect_equal(get_mcmc_ensemble_model(priors_hier, sampler = "kalman")@model_name, "ensemble_model_hierarchical")
  expect_equal(get_mcmc_ensemble_model(priors_hier, likelihood = FALSE, sampler = "explicit")@model_name, "ensemble_prior_hierarchical")
  expect_equal(get_mcmc_ensemble_model(priors_lkj, sampler = "explicit")@model_name, "ensemble_model_explicit")
  expect_equal(get_mcmc_ensemble_model(priors_lkj, sampler = "kalman")@model_name, "ensemble_model")
  expect_equal(get_mcmc_ensemble_model(priors_lkj, likelihood = FALSE)@model_name, "ensemble_prior")

  expect_equal(get_mcmc_ensemble_model(priors_hier, drivers = TRUE, sampler = "explicit")@model_name, "ensemble_model_hierarchical_withdrivers_explicit")
  expect_equal(get_mcmc_ensemble_model(priors_hier, drivers = TRUE, sampler = "kalman")@model_name, "ensemble_model_hierarchical_withdrivers")
  expect_equal(get_mcmc_ensemble_model(priors_hier, likelihood = FALSE, drivers = TRUE)@model_name, "ensemble_prior_hierarchical_withdrivers")
  expect_equal(get_mcmc_ensemble_model(priors_lkj, drivers = TRUE, sampler = "explicit")@model_name, "ensemble_model_withdrivers_explicit")
  expect_equal(get_mcmc_ensemble_model(priors_lkj, drivers = TRUE, sampler = "kalman")@model_name, "ensemble_model_withdrivers")
  expect_equal(get_mcmc_ensemble_model(priors_lkj, likelihood = FALSE, drivers = TRUE)@model_name, "ensemble_prior_withdrivers")
})

#' The constructor for the `EnsembleFit` object
#'
#' The constructor for an `EnsembleFit` object. This function need not be called as an `EnsembleFit` object is constructed automatically by the `fit_ensemble_model` function.
#' The `samples` slot contains the samples from the MCMC if a full sampling was completed, otherwise the `point_estimate` slot contains information about a point estimate.
#'
#' @param ensemble_data An `EnsembleData` object encapsulating the data used to fit the ensemble model.
#' @param samples A `stanfit` object containing the samples drawn from the fitted model. The default value is `NULL`.
#' @param point_estimate A `list` output of the optimised model. The default value is `NULL`.
#'@return An object of class `EnsembleFit`
#' @references Stan Development Team (2020). RStan: the R interface to Stan. R package version 2.21.2. https://mc-stan.org
#' @seealso \code{\linkS4class{EnsembleFit}}, \code{\link{fit_ensemble_model}},
#' @export
EnsembleFit <- function(ensemble_data, samples = NULL, point_estimate = NULL) {
  ensemble_data <- new('EnsembleFit',
                       ensemble_data = ensemble_data,
                       samples = samples,
                       point_estimate = point_estimate)
  return(ensemble_data)
}


setClassUnion("stanfit_or_null", c("stanfit", "NULL"))
setClassUnion("list_or_null", c("list", "NULL"))


#### Class definition ####
#' A class to hold the samples or point estimates from the ensemble model.
#'
#' An `EnsembleFit` object is returned by the `fit_ensemble_model` function.
#' The object contains a slot for the `EnsembleData` object originally used to
#' fit the ensemble model. The `samples` slot contains the samples from the MCMC
#' if a full sampling was completed, otherwise the `point_estimate` slot contains
#' information about a point estimate.
#'
#'
#' @slot ensemble_data An `EnsembleData` object encapsulating the data used to fit the ensemble model.
#' @slot samples A `stanfit` object containing the samples drawn from the fitted model. The default value is `NULL`.
#' @slot point_estimate A `list` output of the optimised model. The default value is `NULL`.
#'
#' @references Stan Development Team (2020). RStan: the R interface to Stan. R package version 2.21.2. https://mc-stan.org
#' @seealso \code{\link{EnsembleFit}}, \code{\link{fit_ensemble_model}}, \code{\link{generate_sample}}
#' @importClassesFrom rstan stanfit
#' @export
setClass(
  "EnsembleFit",
  slots = c(
    ensemble_data = "EnsembleData",
    samples = "stanfit_or_null",
    point_estimate = "list_or_null"
  )
)

construct_plot_dataframe_dri <- function(samples, variable, quantiles){

  fit <- samples@ensemble_fit
  ensemble_data <- fit@ensemble_data
  observations <- ensemble_data@observations[[1]]
  simulators <- ensemble_data@simulators
  stan_input <- ensemble_data@stan_input

  #Observations
  df <- tibble::rownames_to_column(observations)[, c("rowname", variable)]
  colnames(df) <- c("Year", "Observations")

  #Simulators
  for (i in 1:length(simulators)) {
    simulator <- simulators[[i]]
      for (j in 1:length(simulator[[1]])){
        driver <- simulator[[1]][[j]]
        #Skip simulators that don't have the VoI
        if (!(variable %in% colnames(driver))){
          next
        }

        df_sim <- tibble::rownames_to_column(driver, var = "Year")[, c("Year", variable)]
        #Use the name if available
        colnames(df_sim)[2] <- paste0("Simulator ", i, "+", "Driver ", j)
        if (length(simulator) >= 3){
          if (length(simulator) == 4){
            colnames(df_sim)[2] <- paste0(simulator[[3]], "+", simulator[[4]][[j]])
          } else {
            colnames(df_sim)[2] <- paste0(simulator[[3]], "+", "Driver ", j)
          }
        }
        df <- dplyr::full_join(df, df_sim, by = "Year")
      }
    }

  #Ensemble
  var_index = which(colnames(observations) == variable)
  df$Year <- as.numeric(df$Year)
  times <- sort(unique(df$Year))
  if(!is.null(samples@samples)){

    # The ensemble outputs need to be the first columns to ensure they are coloured consistently when some variables are missing.
    df_ensemble <-cbind(times, apply(samples@samples[, var_index, ], 1, median, na.rm = TRUE),
                        apply(samples@samples[, var_index, ], 1, quantile, min(quantiles), na.rm = TRUE),
                        apply(samples@samples[, var_index, ], 1, quantile, max(quantiles), na.rm = TRUE))%>%
      data.frame()


    colnames(df_ensemble) <- c("Year","Ensemble Model Prediction", "Lower", "Upper")
    df <- merge(df_ensemble,df)

    df <-  reshape2::melt(df, id.vars=c("Year", "Lower", "Upper"), variable.name="Simulator")

    #We have zero-width ribbons for the models and observations to avoid clutterling the scene
    df[df$Simulator != "Ensemble Model Prediction", c("Lower", "Upper")] <- df[df$Simulator != "Ensemble Model Prediction", "value"]

  }else{
    df_ensemble <-data.frame(times, samples@mle[, var_index])

    colnames(df_ensemble) <- c("Year","Ensemble Model Prediction")
    df <- merge(df_ensemble,df)

    df <-  reshape2::melt(df, id.vars=c("Year"), variable.name="Simulator")

  }
  return(df)
}

plot_single_dri <- function(samples, variable=1, quantiles=c(0.05, 0.95), ...){

  fit <- samples@ensemble_fit
  ensemble_data <- fit@ensemble_data
  observations <- ensemble_data@observations[[1]]
  simulators <- ensemble_data@simulators
  stan_input <- ensemble_data@stan_input

  variable <- get_variable(variable, observations)

  df <- construct_plot_dataframe_dri(samples, variable, quantiles)

  var_index = which(colnames(observations) == variable)
  if(!is.null(samples@samples)){
    return(plot_values_sample_gg(df, variable))
  }
  return(plot_values_optimised_gg(df, variable))
}

validate_compatibility <- function(name, pair){
  covar <- as.matrix(pair[[2]])

  if (!matrixcalc::is.square.matrix(covar)
      || !is.positive.definite(covar)){
    msg <- paste0(name, " covariance matrix is not a positive definite square matrix.")
    stop(msg)
  }

  df <- as.matrix(pair[[1]])
  if (ncol(df) != ncol(covar)){
    msg <- paste0(name, " data has ", ncol(df), " columns but the covariance matrix has ", ncol(covar), " columns. Variables for the covariance matrix should match the observed data.")
    stop(msg)
  }


  data_variables <- colnames(df)
  cova_variables <- colnames(covar)

  if (length(data_variables) == 0){

    msg <- paste0(name, " data frame variables are not named. Each column of the data frame and the covariance matrix should be named appropriately and match each other.")
    stop(msg)
  }

  if (length(cova_variables) == 0){
    msg <- paste0(name ," covariance matrix variables are not named. Each column of the data frame and the covariance matrix should be named appropriately and match each other.")
    stop(msg)
  }

  if (!all(cova_variables == data_variables)){
    msg <- paste(paste0(name, " covariance matrix variables do not match the associated data frame. Each column of the data frame and covariance matrix should match each other and be in the same order."),
                 "Data variables:",
                 paste(data_variables, sep="", collapse=', '),
                 "Covariance matrix variables: ",
                 paste(cova_variables, sep="", collapse=', '), sep =" ")
    stop(msg)
  }
}

validate_input_data <- function(name, input_data){
  if(!is.list(input_data)||
     (length(input_data) != 2 && length(input_data) != 3) ||
     !(is.matrix(input_data[[1]]) || is.data.frame(input_data[[1]])) ||
     !(is.matrix(input_data[[2]]) || is.data.frame(input_data[[2]]))){
    msg <- paste0(name, " data is not in the correct form. This data should be ",
                  "passed through as a list, the first element being a matrix or data ",
                  "frame containing the data, the second element being the covariance ",
                  "matrix as a matrix or data frame. The name can also be passed through",
                  "as an optional 3rd element.")
    stop(msg)

  }
}

validate_observations <- function(observations){
  validate_input_data("Observation", observations)
  validate_compatibility("Observation", observations)
}

validate_simulator <- function(simulator, observation_variables, name){
  #Check that the data is at least a list
  validate_input_data(name, simulator)
  # Check that it's internally compatible
  validate_compatibility(name, simulator)

  # Check also that it's compatible with the observations
  df <- simulator[[1]]
  if (!all(colnames(df) %in% observation_variables)){
    msg <- paste(name, "contains variables that are not present in the observed data. Model outputs",
                 " should match the observed data. Model variables:",
                 paste(colnames(df), sep="", collapse=', '), ". Observation variables:",
                 paste(observation_variables, sep="", collapse=', '), sep="")
    stop(msg)
  }

}

validate_data <- function(observations, simulators, priors){
  validate_observations(observations)

  if(!is.list(simulators)){
   stop(paste0("Simulator data should be passed through as a list, with each element containing a list of the data for a given simulator."))
  }

  for(i in 1:length(simulators)){
    simulator <- simulators[[i]]
    sim_name <- paste0("One of your simulator's [[", i, "]]")
    if(length(simulator) == 3){
      sim_name <- simulator[[3]]
    }
    validate_simulator(simulator, colnames(observations[[1]]), sim_name)
  }
}

test_that("Validation for initial input data works", {
  error_message <- paste0("TEST data is not in the correct form. This data should be passed through as a list, the first element being a matrix or data frame containing the data, the second element being the covariance matrix as a matrix or data frame.")
  expect_error(validate_input_data("TEST", NULL), error_message)
  expect_error(validate_input_data("TEST", list(x=1)), error_message)
  expect_error(validate_input_data("TEST", list(x=1, y=2, z=3, t =4)), error_message)
  expect_error(validate_input_data("TEST", list(x=1, y=2)), error_message)
  expect_error(validate_input_data("TEST", list(x=as.matrix(1), y=2)), error_message)
  expect_error(validate_input_data("TEST", list(x=1, y=as.matrix(2))), error_message)
  expect_null(validate_input_data("TEST", list(x=as.matrix(1), y=as.matrix(2))))
  expect_null(validate_input_data("TEST", list(x=as.matrix(1), y=as.matrix(2), "TEST")))
})


test_that("Validation for data/covariance compatibility works", {

  data_matrix <- matrix(0, 40, 9)
  covariance_matrix <- matrix(0,8,9)
  expect_error(validate_compatibility("TEST", list(data_matrix, covariance_matrix)),
               "TEST covariance matrix is not a positive definite square matrix.")

  covariance_matrix <- matrix(0,8,8)
  expect_error(validate_compatibility("TEST", list(data_matrix, covariance_matrix)),
               "TEST covariance matrix is not a positive definite square matrix.")

  covariance_matrix <- diag(8)
  expect_error(validate_compatibility("TEST", list(data_matrix, covariance_matrix)),
               "TEST data has 9 columns but the covariance matrix has 8 columns. Variables for the covariance matrix should match the observed data.")

  covariance_matrix <- diag(9)
  expect_error(validate_compatibility("TEST", list(data_matrix, covariance_matrix)),
               paste0("TEST data frame variables are not named. Each column of the data frame",
                     " and the covariance matrix should be named appropriately and match each other."))

  colnames(data_matrix) <- c("A","B","C","D","E","F","G","H","I")
  expect_error(validate_compatibility("TEST", list(data_matrix, covariance_matrix)),
               paste0("TEST covariance matrix variables are not named. Each column of the data frame",
                      " and the covariance matrix should be named appropriately and match each other."))

  colnames(covariance_matrix) <- paste(1:9)
  expect_error(validate_compatibility("TEST", list(data_matrix, covariance_matrix)),
               paste0("TEST covariance matrix variables do not match the associated data frame. Each ",
                      "column of the data frame and covariance matrix should match each other and be ",
                      "in the same order. Data variables: A, B, C, D, E, F, G, H, I Covariance matrix",
                      " variables:  1, 2, 3, 4, 5, 6, 7, 8, 9"))

  colnames(covariance_matrix) <- colnames(data_matrix)
  expect_null(validate_compatibility("TEST", list(data_matrix, covariance_matrix)))
})

#' @param parametrisation_form The parametrisation by which the covariance matrix of the noise of the AR process (in the case of `IndSTPrior` and `ShaSTPrior` objects or the covariance of the distribution of long-term discrepancies for `IndLTPrior` objects) is decomposed. The default is `hierarchical` for `IndSTPrior` objects, and `lkj` otherwise. See details.
#' @param var_params The parameters characterising the variance of the AR process (in the case of `IndSTPrior` and `ShaSTPrior` objects or the variance of the distribution of long-term discrepancies for `IndLTPrior` objects) on the discrepancy. The default value is `list(-3, 1, 8, 4)` for `IndSTPrior` objects, `list(1, 1)` for `IndLTPrior` objects, and `list(1, 10)` for `ShaSTPrior` objects. See details.
#' @param cor_params The parameters characterising the correlations of the AR process (or the distribution of long-term discrepancies) on the short-term discrepancies. The default value in this case is to use `list(0.1,0.1,0.1,0.1)` for `IndSTPrior` objects, and `1` for `IndLTPrior` and `ShaSTPrior` objects. See details.
#' @param AR_params The parameters giving the beta parameters for the prior distribution on the autoregressive parameter of the AR(1) process. The default is `c(2,2)`. See details.
#' @examples
#' ##### Different forms of the individual long term discrepancy priors
#' #LKJ(10) priors on correlation matrices and gamma(5, 3) priors on the variances
#' ist_lkj <- IndSTPrior("lkj", list(5, 3), 10)#
#'
#' #Same as above but with an additional beta(2, 4) prior on
#' #the autoregressive parameter of the AR process.
#' ist_lkj <- IndSTPrior("lkj", list(5, 3), 10, AR_params = c(2, 4))
#'
#' #Same as above but with different variance priors for 5 different variables of interest.
#' #This encodes that there is a gamma(1, 1) prior on the variance of the first variable,
#' #a gamma(23, 1) on the second variable etc...
#' ist_lkj <- IndSTPrior("lkj", list(c(1,23,24,6,87), c(1,1,1,1,5)), 10, AR_params = c(2, 4))
#'
#' #Hierarchical priors with gamma(1,2) and gamma(10, 1) on the variance hyperparameters and
#' #gamma(3,4), gamma(5,6) on the correlation hyperparameters
#' ist_hie <- IndSTPrior("hierarchical", list(1,2,10,1), list(3,4,5,6))
#'
#' #Hierarchical priors with gamma(1,2) and gamma(10, 1) on the variance hyperparameters and
#' #the beta conjugate prior with parameters (p = 0.75, q = 0.75, k = 0.2) on the
#' #correlation hyperparameters
#' ist_hie_beta_conj <- IndSTPrior("hierarchical_beta_conjugate",
#' list(1,2,10,1), list(0.75,0.75,0.2))
#'
#' #Inverse Wishart correlation priors. Gamma(2, 1/3) priors are on the variances and
#' #inv-Wishart(5, diag(5)) on the correlation matrices.
#' ist_inW <- IndSTPrior("inv_wishart", list(2, 1/3),list(5, diag(5)))
#' @rdname PriorConstructorFunctions
#' @export

IndSTPrior <- function(parametrisation_form = "hierarchical", var_params= list(-3, 1, 8, 4), cor_params = list(0.1, 0.1, 0.1, 0.1), AR_params = c(2, 2)){

  validate_parametrisation_form(parametrisation_form, valid_forms = c(CORRELATIONS_PRIOR_LKJ,
                                                                      CORRELATIONS_PRIOR_INV_WISHART,
                                                                      CORRELATIONS_PRIOR_BETA,
                                                                      CORRELATIONS_PRIOR_HIERARCHICAL,
                                                                      CORRELATIONS_PRIOR_BETA_CONJUGATE))
  validate_prior_AR_params(AR_params)

  # if(is.numeric(cor_params))
  #   cor_params <- list(cor_params)

  ret <- new('IndSTPrior',
             AR_param = AR_params,
             parametrisation_form = parametrisation_form,
             var_params = var_params,
             cor_params = cor_params)
  return(ret)
}



#### Class definition ####
#' A class to hold the priors for the ensemble model.
#'
#' An `IndSTPrior` object encapsulates the prior information for the short-term discrepancies of the individual simulators within the ensemble model.
#' @slot AR_param The parameters giving the beta parameters for the autoregressive parameter of the AR(1) process.
#' @slot parametrisation_form The parametrisation by which the covariance matrix of the noise of the AR process is decomposed.
#' @slot var_params The parameters characterising the variance of the AR process on the individual short-term discrepancy.
#' @slot cor_params The parameters characterising the correlations of the AR process on the individual short-term discrepancy. .
#' @seealso \code{\linkS4class{IndLTPrior}}, \code{\linkS4class{ShaSTPrior}}, \code{\linkS4class{TruthPrior}},
#' @details
#' Individual short-term discrepancies \eqn{z_k^{(t)}} are modelled as an AR(1) process so that \deqn{z_k^{(t+1)} \sim N(R_k z_k^{(t)}, \Lambda_k).} Accepted parametrisation forms for this discrepancy are `lkj`, `beta`, `inv_wishart`, `hierarchical` or `hierarchical_beta_conjugate`. See details of the `EnsemblePrior()` constructor for more details.
#' @export
setClass(
  "IndSTPrior",
  slots = c(AR_param = "numeric",
            parametrisation_form = "character",
            var_params = "listOrNumeric",
            cor_params = "listOrNumeric")
)




generate_priors_stan_input_ind_st <- function(d, x){

  if(!is(x, "IndSTPrior")){
    stop("Invalid object for individual short-term priors. This should be a IndSTPrior object.")
  }

  cor_form   <- correlation_form_prior(x@parametrisation_form)
  cor_params <- correlation_prior(x@cor_params, cor_form, "prior_ind_st_cor", TRUE)

  var_priors <- list(
    prior_ind_st_var_a = numeric(0),
    prior_ind_st_var_b = numeric(0),
    prior_ind_st_var_hierarchical_hyperparams = numeric(0)
  )
  if(cor_form == CORRELATIONS_PRIOR_HIERARCHICAL || cor_form == CORRELATIONS_PRIOR_BETA_CONJUGATE){
    var_priors$prior_ind_st_var_hierarchical_hyperparams <- unlist(x@var_params)
  }else{
    repeated_var_params <- repeat_variance_priors(d, x)
    var_priors$prior_ind_st_var_a <- repeated_var_params[[1]]
    var_priors$prior_ind_st_var_b <- repeated_var_params[[2]]
  }





  ret <- append(var_priors, cor_params)
  ret <- append(ret,
                list(form_prior_ind_st = cor_form,
                     prior_ind_st_ar_alpha = x@AR_param[1],
                     prior_ind_st_ar_beta = x@AR_param[2]))
  return(ret)

}

#Returns the compiled ensemble model Stan object in the case with drivers.


fit_ensemble_model_dri <- function(observations, simulators, priors,
                               full_sample = TRUE, control = list(adapt_delta = 0.95),
                               sampler = c("explicit", "kalman"), MMod, ...){

  sampler <- match.arg(sampler)


  if (!missing(MMod)) {
    #ens_data <- EnsembleDataDriver(observations, simulators, priors, MMod)
    ens_data <- EnsembleData(observations, simulators, priors, drivers = TRUE, MMod)
  }
  else {
    #ens_data <- EnsembleDataDriver(observations, simulators, priors)
    ens_data <- EnsembleData(observations, simulators, priors, drivers = TRUE)
  }
  stan_input <- ens_data@stan_input

  #Using hierarchical priors uses a different model. This speeds up the sampling enormously
  if(sampler == "explicit"){
  mod <-  stanmodels$ensemble_model_withdrivers_explicit
  } else {
    mod <-  stanmodels$ensemble_model_withdrivers
  }
  if(stan_input$form_prior_ind_st == 3 || stan_input$form_prior_ind_st == 4){
    if(sampler == "explicit"){
    mod <- stanmodels$ensemble_model_hierarchical_withdrivers_explicit
    } else {
      mod <- stanmodels$ensemble_model_hierarchical_withdrivers
    }
    if(!full_sample){
      stop("It is possible to generate a point estimate for the prior if the individual short-term discrepancy prior follows a hierarchical parameterisation. Please generate a full sample using 'full_sample=TRUE'.")
    }
  }


  samples <- NULL; point_estimate <- NULL
  if(full_sample){
    samples <- stan_sampling_with_filter(mod, data = stan_input,
                                         control = control,
                                         ...)
  }else{
    point_estimate <- rstan::optimizing(mod, data=stan_input,as_vector=FALSE, ...)
  }

  return(EnsembleFit(ens_data, samples, point_estimate))
}

#' @rdname PriorConstructorFunctions
#' @export
IndLTPrior <- function(parametrisation_form = "lkj", var_params = list(1, 1), cor_params = 1){
  # validate_parametrisation_form(parametrisation_form)
  # if(is.numeric(cor_params))
  #   cor_params <- list(cor_params)

  ret <- new('IndLTPrior',
             parametrisation_form = parametrisation_form,
             var_params = var_params,
             cor_params = cor_params)
return(ret)
}

setClassUnion(name = "listOrNumeric", members=c("list", "numeric"))

#### Class definition ####
#' A class to hold the priors for the ensemble model.
#'
#' An `IndLTPrior` object encapsulates the prior information for the long-term discrepancies of the individual simulators within the ensemble model.
#' @slot parametrisation_form The parametrisation by which the covariance matrix of the noise of the AR process is decomposed.
#' @slot var_params The parameters characterising the variance of the distribution of the individual long-term discrepancy.
#' @slot cor_params The parameters characterising the correlations of the distribution of the individual long-term discrepancy.
#' @seealso \code{\linkS4class{IndSTPrior}}, \code{\linkS4class{ShaSTPrior}}, \code{\linkS4class{TruthPrior}},
#' @details
#' Individual long-term discrepancies \eqn{\gamma_k} are drawn from a distribtion \deqn{\gamma_k \sim N(0, C_\gamma).} Accepted parametrisation forms for this discrepancy are `lkj`, `beta`, or `inv_wishart`. See details of the `EnsemblePrior()` constructor for more details.
#' @export
setClass(
  "IndLTPrior",
  slots = c(parametrisation_form = "character",
            var_params = "list",
            cor_params = "listOrNumeric")
)



generate_priors_stan_input_ind_lt <- function(d, x){

  if(!is(x, "IndLTPrior")){
    stop("Invalid object for individual long-term priors. This should be a IndLTPrior object.")
  }

  var_priors <- repeat_variance_priors(d, x)

  cor_form   <- correlation_form_prior(x@parametrisation_form)
  cor_params <- correlation_prior(x@cor_params, cor_form, "prior_ind_lt_cor", FALSE)

  ret <- append(list(
      prior_ind_lt_var_a = var_priors[[1]],
      prior_ind_lt_var_b = var_priors[[2]],
      form_prior_ind_lt = cor_form),
    cor_params)

  return(ret)
}

#' Constructor for the `EnsemblePrior` class
#'
#' Constructors for the `EnsemblePrior` class and related classes. These functions are used to encode prior information for the ensemble model. The `IndSTPrior`, `IndLTPrior`, `ShaSTPrior`, and `TruthPrior` constructors encapsulate prior information.
#' @param d A `numeric` specifying the number of variables of interest in the ensemble.
#' @param ind_st_params An `IndSTPrior` object specifying priors for the individual short-term discrepancies \eqn{z_k^{(t)}}. The default value is `IndSTPrior("hierarchical", list(-3, 1, 8, 4), list(0.1, 0.1, 0.1, 0.1), c(2, 2))`.
#' @param ind_lt_params An `IndLTPrior` object specifying priors for the individual long-term discrepancies \eqn{\gamma_k}. The default value is `IndLTPrior("lkj",list(1, 1), 1)`.
#' @param sha_st_params A `ShaSTPrior` object specifying priors for the shared short-term discrepancies \eqn{\eta^{(t)}}. The default value is `ShaSTPrior("lkj",list(1, 10), 1, c(2,2))`.
#' @param sha_lt_params A `numeric` of length `d` or `1` containing the standard deviations for the normal prior used on the shared short-term discrepancy \eqn{\mu}. If a single value is supplied,  this is repeated for each variable of interest. The default value is `5`.
#' @param truth_params A `TruthPrior` object specifying priors for the processes on the truth \eqn{y^{(t)}}. The default value is `TruthPrior(d)`.
#' @details
#'`IndSTPrior` and `ShaSTPrior` discrepancy prior parameter objects contain 4 slots corresponding to:
#' 1. `parametrisation_form` - A `character` specifying how the priors are parametrised. Currently supported priors are `'lkj'`, `'inv_wishart'`, `'beta'`, `'hierarchical'`, or `'hierarchical_beta_conjugate'` (`'hierarchical'` and `'hierarchical_beta_conjugate'` are only supported for `IndSTPrior` objects).
#' 2. `var_params` - The prior parameters for the discrepancy variances, either a `list` of length `2` or a `numeric` of length `4`. See below.
#' 3. `cor_params` - The correlation matrix parameters, either a `list` of length `2`, a `numeric` of length `3` or a `numeric` of length `4`. See below.
#' 4. `AR_params` - Parameters for the autoregressive parameter as a `numeric` of length `2`.
#'
#' `IndLTPrior` discrepancy prior parameter objects contain the slots `parametrisation_form`, `var_params`, and `cor_params`.
#'
#' There are currently five supported prior distributions on covariance matrices. As in Spence et. al. (2018), the individual and shared short-term discrepancy covariances, \eqn{\Lambda_k} and \eqn{\Lambda_\eta}, as well as the individual long-term discrepancy covariance, \eqn{\Lambda_\gamma},  are decomposed into a vector of variances and a correlation matrix \deqn{\Lambda = \sqrt{\mathrm{diag}(\pi)}  P \sqrt{\mathrm{diag}(\pi)},} where \eqn{\pi} is the vector of variances for each variable of interest (VoI), and \eqn{P} is the correlation matrix.
#'
#'   Selecting `'lkj'`, `'inv_wishart'`, `'beta'`, `'hierarchical'` or `'hierarchical_beta_conjugate'` refers to setting LKJ, inverse Wishart, beta or hierarchical (with gamma-distributed hyperparameters or beta-conjugate-distributed hyperparameters) prior distributions on the covariance matrix respectively. The variance parameters should be passed through as the `var_params` slot of the object and the correlation parameters should be passed through as the `cor_params`. For `'lkj'`, `'inv_wishart'`, and `'beta'` selections, variances are parameterised by gamma distributions, so the `var_params` slot should be a `list` of length two, where each element gives the shape and rate parameters for each VoI (either as a single value which is the same for each VoI or a `numeric` with the same length as the number of VoI). For example, setting `var_params  = list(c(5,6,7,8), c(4,3,2,1))` would correspond to a `Gamma(5, 4)` prior on the variance of the first VoI, a `Gamma(6, 3)` prior on the variance of the second VoI, etc...
#'   The correlations should be in the following form:
#'  * If `'lkj'` is selected, then `cor_params` should be a `numeric` \eqn{\eta} giving the LKJ shape parameter, such  that the probability density is given by  (Lewandowski et. al. 2009) \deqn{f(\Sigma | \eta)\propto \mathrm{det} (\Sigma)^{\eta - 1}.} Variances are parameterised by gamma distributions.
#'  * If `'inv_wishart'` is selected, then  `cor_params` should be a `list` containing a scalar value \eqn{\eta} (giving the degrees of freedom) and a symmetric, positive definite matrix \eqn{S} (giving the scale matrix). The dimensions of \eqn{S} should be the same as the correlation matrix it produces (i.e \eqn{d \times d} where \eqn{d} is the number of VoI). The density of an inverse Wishart is given by  \deqn{f(W|\eta, S) = \frac{1}{2^{\eta d/2} \Gamma_N \left( \frac{\eta}{2} \right)} |S|^{\eta/2} |W|^{-(\eta + d + 1)/2}  \exp \left(- \frac{1}{2} \mathrm{tr}\left(SW^{-1} \right) \right),} where \eqn{\Gamma_N} is the multivariate gamma function and \eqn{\mathrm{tr \left(X \right)}} is the trace of \eqn{X}.  Note that inverse Wishart distributions act over the space of all covariance matrices. When used for a correlation  matrix, only the subset of valid covariance matrices that are also valid correlation matrices are considered. Variances are parameterised by gamma distributions.
#'  * If `'beta'` is selected, then  `cor_params` should be a  `list` containing two symmetric `d`\eqn{\times}`d` matrices \eqn{A} and \eqn{B} giving the prior success parameters and prior failure parameters respectively. The correlation between the `i`th and `j`th VoI is \eqn{\rho_{i, j}} with \deqn{\frac{1}{\pi} \tan^{-1} \frac{\rho_{i, j}}{\sqrt{1-\rho_{i, j}^2}} + \frac{1}{2} \sim \mathrm{beta}(A_{i, j}, B_{i, j}).} Variances are parameterised by gamma distributions.
#'  * If `'hierarchical'` or `'hierarchical_beta_conjugate'` is selected, then variances are parameterised by log-normal distributions:
#'  \deqn{\log \pi_{k, i} \sim \mathrm{N}(\mu_i, \sigma^2_i)} with priors
#'  \deqn{\mu_i \sim \mathrm{N}(\alpha_\pi, \beta_\pi),}
#'  \deqn{\sigma^2_i \sim \mathrm{InvGamma}(\gamma_\pi, \delta_\pi).}
#'  The `var_params` slot should then be a `numeric` of length 4, giving the \eqn{\alpha_\pi, \beta_\pi, \gamma_\pi, \delta_\pi} hyperparameters respectively. Correlations (\eqn{\rho_{k, i, j}} where \eqn{\rho_{k, i, j}} is the correlation between VoI \eqn{i} and \eqn{j} for the \eqn{k}th simulator) are parameterised by hierarchical beta distributions.
#'  \deqn{\frac{\rho_{k, i, j} + 1}{2} \sim \mathrm{beta}(c_{k, i, j}, d_{k, i, j})} with priors
#'  \deqn{c_{k, i, j} \sim \mathrm{gamma}(\alpha_\rho, \beta_\rho),}
#'  \deqn{d_{k, i, j} \sim \mathrm{gamma}(\gamma_\rho, \delta_\rho).}
#'  NOTE: These options is only supported for the individual short-term discrepancy terms.
#'  * If `'hierarchical'` is selected, then the `cor_params` slot should be a `numeric` of length 4 giving the \eqn{\alpha_\rho, \beta_\rho, \gamma_\rho, \delta_\rho} hyperparameters. respectively. NOTE: This option is only supported for the individual short-term discrepancy terms.
#'  * If `'hierarchical_beta_conjugate'` is selected, then the `cor_params` slot should be a `numeric` of length 3. Denoting the values by \eqn{r,s,k}, they map to the hyperparameters \eqn{p, q, k} of the beta conjugate distribution via \eqn{k = k}, \eqn{p^{-1/k} = (1+e^{-s})(1+e^{-r})} and \eqn{q^{1/k} = e^{-r}(1+e^{-s})^{-1}(1+e^{-r})^{-1}}. The density of the beta conjugate distribution is defined up to a constant of proportionality by
#'  \deqn{p(\alpha_\rho, \beta_\rho\,|\,p, q, k) \propto \frac{\Gamma(\alpha_\rho + \beta_\rho)^{k}p^{\alpha_\rho}q^{\beta_\rho}}{\Gamma(\alpha_\rho)^{k}\Gamma(\beta_\rho)^{k}}\,.}
#'  NOTE: This option is only supported for the individual short-term discrepancy terms.
#'Priors may also be specified for the autoregressive parameters for discrepancies modelled using autoregressive processes (i.e. for `IndSTPrior` and `ShaSTPrior` objects). These are parametrised via beta distributions such that the autoregressive parameter \eqn{R \in (-1,1)} satisfies \deqn{\frac{R+1}{2} \sim \mathrm{Beta}(\alpha, \beta)}.
#'
#'  In addition to priors on the discrepancy terms, it is also possible to add prior information on the truth. We require priors on the truth at \eqn{t=0}. By default, a \eqn{N(0, 10)} prior is used on the initial values., however this can be configured by the `truth_params` argument. The covariance matrix of the random walk of the truth \eqn{\Lambda_y} can be configured using an inverse-Wishart prior. The `truth_params` argument should be a `TruthPrior` object.
#' @references Spence et. al. (2018). A general framework for combining ecosystem models. \emph{Fish and Fisheries}, 19(6):1031-1042.
#'
#' @examples
#' #The default priors for an ensemble with 8 variables of interest
#' priors <- EnsemblePrior(8)
#'
#' #With 4 variables of interest.
#' priors <- EnsemblePrior(4)
#'
#' #Defining custom priors for a model with 4 species.
#' num_species <- 5
#' priors <- EnsemblePrior(
#'   d = num_species,
#'   ind_st_params = IndSTPrior("lkj",  list(3, 2), 3, AR_params = c(2,4)),
#'   ind_lt_params = IndLTPrior(
#'     "beta",
#'     list(c(10,4,8, 7,6),c(2,3,1, 4,4)),
#'     list(matrix(5, num_species, num_species),
#'          matrix(0.5, num_species, num_species))
#'   ),
#'   sha_st_params = ShaSTPrior("inv_wishart",list(2, 1/3),list(5, diag(num_species))),
#'   sha_lt_params = 5,
#'   truth_params = TruthPrior(d = num_species, initial_mean = 5, initial_var = 10,
#'                             rw_covariance = list(10, diag(10)))
#' )
#'
#' @return `EnsemblePrior` returns an object of class `EnsemblePrior`.
#' `IndSTPrior` returns an object of class `IndSTPrior`.
#' `IndLTPrior` returns an object of class `IndLTPrior`.
#' `ShaSTPrior` returns an object of class `ShaSTPrior`.
#' `TruthPrior` returns an object of class `TruthPrior`.
#' @rdname PriorConstructorFunctions
#' @export
EnsemblePrior <- function(d, ind_st_params = IndSTPrior(), ind_lt_params = IndLTPrior(), sha_st_params = ShaSTPrior(), sha_lt_params = 5, truth_params = TruthPrior(d)){

  validate_prior(d, ind_st_params, ind_lt_params, sha_st_params, sha_lt_params, truth_params)

  priors_stan_input <- create_prior_stan_input(d, ind_st_params, ind_lt_params, sha_st_params, sha_lt_params, truth_params)
  ret <- new('EnsemblePrior',
             d = d,
             ind_st_params = ind_st_params,
             ind_lt_params = ind_lt_params,
             sha_st_params = sha_st_params,
             sha_lt_params = sha_lt_params,
             truth_params = truth_params,
             priors_stan_input = priors_stan_input)
  return(ret)
}



#### Class definition ####
#' A class to hold the priors for the ensemble model.
#'
#' An `EnsemblePrior` object encapsulates the prior information for the ensemble model.
#'
#' @slot d A `numeric` specifying the number of variables of interest in the ensemble.
#' @slot ind_st_params A `list` containing a prior specification for the individual short-term discrepancies \eqn{z_k^{(t)}}. See details of the `EnsemblePrior()` constructor.
#' @slot ind_lt_params A `list` containing a prior specification for the individual long-term discrepancies \eqn{\gamma_k}. See details of the `EnsemblePrior()` constructor.
#' @slot sha_st_params A `list` containing a prior specification for the shared short-term discrepancies \eqn{\eta^{(t)}}. See details of the `EnsemblePrior()` constructor.
#' @slot sha_lt_params A `numeric` containing the standard deviations for the normal prior used on the shared short-term discrepancy \eqn{\mu}. If a single value is supplied,  this is repeated for each variable
#' @slot truth_params A `list` containing a prior specification for the processes on the truth \eqn{y^{(t)}}. See details of the `EnsemblePrior()` constructor. The default value is `TruthPrior(d)`.
#' @slot priors_stan_input A `list` containing the prior data in the correct form to fit the model in Stan. This information is automatically generated by the constructor.
#'
#' @references Stan Development Team (2020). RStan: the R interface to Stan. R package version 2.21.2. https://mc-stan.org
#' @references Lewandowski, Daniel, Dorota Kurowicka, and Harry Joe. 2009. "Generating Random Correlation Matrices Based on Vines and Extended Onion Method." Journal of Multivariate Analysis 100: 1989--2001.
#' @export
setClass(
  "EnsemblePrior",
  slots = c(d = "numeric",
            ind_st_params = "IndSTPrior",
            ind_lt_params = "IndLTPrior",
            sha_st_params = "ShaSTPrior",
            sha_lt_params = "numeric",
            truth_params = "TruthPrior",
            priors_stan_input = "list"
  )
)



create_prior_stan_input <- function(d, ind_st_params, ind_lt_params, sha_st_params, sha_lt_params, truth_params){

  if(is.numeric(sha_lt_params) && length(sha_lt_params) == 1)
    sha_lt_params <- rep(sha_lt_params, d)

  dat <- generate_priors_stan_input_ind_st(d, ind_st_params) %>%
    append(generate_priors_stan_input_ind_lt(d, ind_lt_params)) %>%
    append(generate_priors_stan_input_sha_st(d, sha_st_params)) %>%
    append(generate_priors_stan_input_truth(d, truth_params)) %>%
    append(list(prior_sha_lt_sd = sha_lt_params))
  return(dat)
}



validate_prior <- function(d, ind_st_params, ind_lt_params, sha_st_params, sha_lt_params, truth_params){
  if(!is.numeric(d) || length(d) != 1)
    stop(paste0("The number of variables of interest (d) should be an integer value, current value: ", d))


  if(!is.numeric(sha_lt_params) || !(length(sha_lt_params) %in% c(1, d)))
    stop("The shared long-term discrepancy parameters should be a numeric of length 1 or d.")


  validate_prior_compatibility("individual short-term",ind_st_params,  d)
  validate_prior_compatibility("individual long-term", ind_lt_params, d)
  validate_prior_compatibility("shared short-term", sha_st_params, d)

}

## ----setup_lies,warning=FALSE-------------------------------------------------
library(EcoEnsemble)
set.seed(1234)

## ----eval = FALSE, message=FALSE, warning=FALSE, results='hide', silent=TRUE----
# cor_pri_st <- rstan::stan_model(model_code = " functions{
#   real priors_cor_hierarchical_beta(matrix ind_st_cor, int N, matrix M){
#     real log_prior = 0;
#     for (i in 1:(N-1)){
#       for (j in (i+1):N){
#         log_prior += beta_lpdf(0.5*(ind_st_cor[i, j] + 1)| M[i, j], M[j, i] );
#       }
#     }
#     return log_prior;
#   }
# 
#   real beta_conj_prior(real alpha, real betas, real r, real s, real k){
#     real rl = 1/(1 + exp(-r));
#     real sl = 1/(1 + exp(-s));
#     real p = (sl * rl)^k;
#     real q = (sl * (1 - rl))^k;
#     real ret = alpha * log(p) + betas * log(q) - k * lbeta(alpha,betas);
#     return ret;
#   }
# 
# }
# 
# data {
#   vector[3] cor_p;
# }
# 
# parameters {
#   matrix <lower=0>[5,5] beta_pars;
#   corr_matrix[5] rho[4];
# }
# 
# model {
#   for (i in 1:4){
#     for (j in (i+1):5){
#       target += beta_conj_prior(beta_pars[i,j], beta_pars[j,i], cor_p[1], cor_p[2], cor_p[3]);
#     }
#   }
# 
#   for (i in 1:4){
#     target += priors_cor_hierarchical_beta(rho[i],5,beta_pars);
#     diagonal(beta_pars) ~ std_normal();
#   }
# 
# }
# 
# generated quantities {
# 
#   matrix [5,5] rhovars;
#   for (i in 1:4){
#     for (j in (i+1):5){
#       rhovars[i,j] = 4 * (beta_pars[i,j] * beta_pars[j,i])/(square(beta_pars[i,j] + beta_pars[j,i]) * (beta_pars[i,j] + beta_pars[j,i] + 1));
#       rhovars[j,i] = (2 * beta_pars[i,j]/(beta_pars[i,j] + beta_pars[j,i])) - 1;
#     }
#   }
# 
#   for (i in 1:5){
#     rhovars[i,i] = 4;
#   }
# 
# }
# ")
# library(ggplot2)
# rhoplots <- list()
# kvals <- c(0.05, 5)
# parvals <- do.call(expand.grid, c(rep(list(c(0.25, 3)), 2), list(kvals)))
# #Sampling and gathering outputs for plotting
# for (i in 1:nrow(parvals)){
#   fit_cor <- rstan::sampling(cor_pri_st, data = list(cor_p=as.numeric(parvals[i,])), iter = 2000, chains=4)
#   ex.fit <- rstan::extract(fit_cor)
#   rho_density <- density(ex.fit$rho[,1,1,2], from = -1, to = 1)
#   rhoplot_data <- data.frame(rho_density$x, rho_density$y)
#   names(rhoplot_data) <- c("rho", "Density")
#   rhoplot_data <- cbind(rhoplot_data, rep(parvals[i,1], nrow(rhoplot_data)), rep(parvals[i,2], nrow(rhoplot_data)), rep(parvals[i,3], nrow(rhoplot_data)))
#   names(rhoplot_data)[3:5] <- c("r", "s", "k")
#   rhoplots[[i]] <- rhoplot_data
# }
# #Construct plots
# rhoplot_data <- do.call(rbind, rhoplots)
# rhoplot_data <- cbind(rhoplot_data, rep("Beta conjugate prior", nrow(rhoplot_data)))
# names(rhoplot_data)[6] <- "Prior"
# unif_range <- seq(-1, 1, length.out = nrow(rhoplots[[1]]))
# unif_data <- data.frame(cbind(rep(unif_range, nrow(parvals)), rep(dbeta((unif_range+1)/2,5/2,5/2)/2, nrow(parvals)), rhoplot_data[,3:5], rep("Uniform prior", nrow(rhoplot_data))))
# names(unif_data) <- names(rhoplot_data)
# rhoplot_data <- rbind(rhoplot_data, unif_data)
# rhoplot1 <- ggplot(rhoplot_data[which(rhoplot_data$k == kvals[1]),]) + geom_area(aes(x = rho, y = Density, color = Prior, fill = Prior), alpha = 0.3, position = "identity") + scale_x_continuous(bquote(rho), sec.axis = sec_axis(~., name = "r", breaks = NULL, labels = NULL)) + scale_y_continuous(sec.axis = sec_axis(~., name = "s", breaks = NULL, labels = NULL)) + theme(aspect.ratio = 1) + facet_grid(rows = vars(r), cols = vars(s)) + scale_color_manual(values = c("blue", "green")) + scale_fill_manual(values = c("blue", "green"))
# rhoplot2 <- ggplot(rhoplot_data[which(rhoplot_data$k == kvals[2]),], aes(x = rho, y = Density)) + geom_area(aes(color = Prior, fill = Prior), alpha = 0.3, position = "identity") + scale_x_continuous(bquote(rho), sec.axis = sec_axis(~., name = "r", breaks = NULL, labels = NULL)) + scale_y_continuous(sec.axis = sec_axis(~., name = "s", breaks = NULL, labels = NULL)) + theme(aspect.ratio = 1) + facet_grid(rows = vars(r), cols = vars(s)) + scale_color_manual(values = c("blue", "green")) + scale_fill_manual(values = c("blue", "green"))

## ----echo = FALSE, out.width="700px", out.height="700px"----------------------
knitr::include_graphics("data/rhoplot1.png")

## ----eval = FALSE, fig.dim = c(7,6), echo = FALSE-----------------------------
# rhoplot1

## ----echo = FALSE, out.width="700px", out.height="700px"----------------------
knitr::include_graphics("data/rhoplot2.png")

## ----eval = FALSE, fig.dim = c(7,6), echo = FALSE-----------------------------
# rhoplot2

## ----message=FALSE, warning=FALSE,eval=FALSE, results='hide', silent=TRUE-----
# kvals <- c(0.1, 0.2, 0.4, 0.8, 1.6, 3.2)
# #parvals <- cbind(rep(0.3, 6), rep(-1, 6), kvals)
# parvals <- cbind(rep(-0.3, 6), rep(3, 6), kvals)
# rhovarplots <- list()
# rhomeanplots <- list()
# for (i in 1:6){
#   fit_cor <- rstan::sampling(cor_pri_st, data = list(cor_p=parvals[i,]), iter = 2000, chains=4)
#   ex.fit <- rstan::extract(fit_cor)
#   rhovar_density <- density(ex.fit$rhovars[,1,2])
#   rhomean_density <- density(ex.fit$rhovars[,2,1])
#   rhovar_data <- data.frame(cbind(rhovar_density$x, rhovar_density$y, rep(kvals[i], length(rhovar_density$x))))
#   rhomean_data <- data.frame(cbind(rhomean_density$x, rhomean_density$y, rep(kvals[i], length(rhomean_density$x))))
#   names(rhovar_data) <- c("Variance", "Density", "k")
#   names(rhomean_data) <- c("Expectation", "Density", "k")
#   rhovarplots[[i]] <- rhovar_data
#   rhomeanplots[[i]] <- rhomean_data
# }
# rhovarplot_data <- do.call(rbind, rhovarplots)
# rhomeanplot_data <- do.call(rbind, rhomeanplots)
# rhovarplot <- ggplot(rhovarplot_data) + geom_area(aes(x = Variance, y = Density), color = "blue", fill = "blue", alpha = 0.3, position = "identity") + geom_vline(xintercept = 0.2, color = "green", linetype = "dashed", linewidth = 0.8) + facet_wrap(vars(k), nrow = 2, ncol = 3) + scale_x_continuous(sec.axis = sec_axis(~., name = "k", breaks = NULL, labels = NULL))
# rhomeanplot <- ggplot(rhomeanplot_data) + geom_area(aes(x = Expectation, y = Density), color = "blue", fill = "blue", alpha = 0.3, position = "identity") + geom_vline(xintercept = 0, color = "green", linetype = "dashed", linewidth = 0.8) + facet_wrap(vars(k), nrow = 2, ncol = 3) + scale_x_continuous(sec.axis = sec_axis(~., name = "k", breaks = NULL, labels = NULL))

## ----echo = FALSE, out.width="600px", out.height = "500px"--------------------
knitr::include_graphics("data/rhovarplot.png")

## ----eval = FALSE, echo = FALSE-----------------------------------------------
# rhovarplot

## ----echo = FALSE, out.width = "600px", out.height = "500px"------------------
knitr::include_graphics("data/rhomeanplot.png")

## ----eval=FALSE, fig.dim = c(8,5), echo = FALSE-------------------------------
# rhomeanplot

## ----eval = FALSE, message = FALSE, warning = FALSE, silent = TRUE, results = 'hide'----
# fit_cor <- rstan::sampling(cor_pri_st, data = list(cor_p=c(0.25, 3, 4), iter = 2000, chains=4))
# ex.fit <- rstan::extract(fit_cor)
# rhoplot_data  <- data.frame(ex.fit$rho[,1,1,2])
# names(rhoplot_data) <- "rho"
# rhoplot <- ggplot(rhoplot_data) + geom_histogram(aes(x = rho), color = "blue", fill = "blue", alpha = 0.3, binwidth = 0.05) + scale_x_continuous(bquote(rho))

## ----echo = FALSE, out.width = "400px", out.height = "400px"------------------
knitr::include_graphics("data/rhoplot.png")

## ----eval = FALSE, echo = FALSE, warning = FALSE, fig.dim = c(4,3)------------
# rhoplot

## ----eval=FALSE,message=FALSE, warning = FALSE, results = 'hide',silent=TRUE----
# priors <- EnsemblePrior(4,
#                         ind_st_params =IndSTPrior("hierarchical_beta_conjugate",list(-3,1,8,4),
#                                                   list(0.25,3,4),AR_params=c(2,2)),
#                         ind_lt_params = IndLTPrior("lkj",list(1,1),1),
#                         sha_st_params = ShaSTPrior("lkj",list(1,10),1,AR_params=c(2,2)),
#                         sha_lt_params = 5
# )
# prior_density <- prior_ensemble_model(priors, M = 4)
# samples <- sample_prior(observations = list(SSB_obs, Sigma_obs),
#              simulators = list(list(SSB_ewe, Sigma_ewe,"EwE"),
#                    list(SSB_fs ,  Sigma_fs,"FishSums"),
#                    list(SSB_lm ,  Sigma_lm,"LeMans"),
#                    list(SSB_miz, Sigma_miz,"mizer")),
#              priors=priors,
#              sam_priors = prior_density)

## ----eval = FALSE, fig.dim = c(7, 6)------------------------------------------
# plot(samples)

## ----echo = FALSE, out.height="500px", out.width="800px"----------------------
knitr::include_graphics("data/p_priors_beta_conj.png")

#' @rdname PriorConstructorFunctions
#' @export
ShaSTPrior <- function(parametrisation_form = "lkj", var_params = list(1, 10), cor_params = 1, AR_params = c(2, 2)){

  validate_parametrisation_form(parametrisation_form)
  validate_prior_AR_params(AR_params)

  # if(is.numeric(cor_params))
  #   cor_params <- list(cor_params)


  ret <- new('ShaSTPrior',
             AR_param = AR_params,
             parametrisation_form = parametrisation_form,
             var_params = var_params,
             cor_params = cor_params)
  return(ret)
}



#### Class definition ####
#' A class to hold the priors for the ensemble model.
#'
#' An `ShaSTPrior` object encapsulates the prior information for the short-term discrepancies of the shared discrepancy of the ensemble model.
#' @slot AR_param The parameters giving the beta parameters for the main parameter of the AR(1) process.
#' @slot parametrisation_form The parametrisation by which the covariance matrix of the noise of the AR process is decomposed.
#' @slot var_params The parameters characterising the variance of the AR process on the shared short-term discrepancy.
#' @slot cor_params The parameters characterising the correlations of the AR process on the shared short-term discrepancy.
#' @details
#' Shared short-term discrepancies \eqn{\mathbf{\eta}^{(t)}} are modelled as an AR(1) process so that \deqn{\mathbf{\eta}^{(t+1)} \sim N(R_\eta \mathbf{\eta}, \Lambda_\eta).} Accepted parametrisation forms for this discrepancy are `lkj`, `beta`, or `inv_wishart`. See details of the `EnsemblePrior()` constructor for more details.
#' @export
setClass(
  "ShaSTPrior",
  slots = c(AR_param = "numeric",
            parametrisation_form = "character",
            var_params = "list",
            cor_params = "listOrNumeric")
)



generate_priors_stan_input_sha_st <- function(d, x){
  if(!is(x, "ShaSTPrior")){
    stop("Invalid object for shared short-term priors. This should be a ShaSTPrior object.")
  }

  var_priors <- repeat_variance_priors(d, x)

  cor_form   <- correlation_form_prior(x@parametrisation_form)
  cor_params <- correlation_prior(x@cor_params, cor_form, "prior_sha_st_cor", FALSE)

  ret <- append(list(
    prior_sha_st_var_a = var_priors[[1]],
    prior_sha_st_var_b = var_priors[[2]],
    prior_sha_st_ar_alpha = x@AR_param[1],
    prior_sha_st_ar_beta = x@AR_param[2],
    form_prior_sha_st = cor_form),
    cor_params)

  return(ret)

}

validate_prior_compatibility <- function(type_str, params, d) {

  var_params <- params@var_params
  cor_params <- params@cor_params
  form <- tolower(params@parametrisation_form)

  msg <- ""
  if(form == "lkj" &
     length(cor_params) != 1){
    stop("Invalid lkj parameter for ", type_str, " correlation matrix prior. This should be a list with exactly one real number.")
  }else if (form == "inv_wishart"){
    if(length(cor_params) != 2 || !is.numeric(cor_params[[1]]) || !is.matrix(cor_params[[2]]) || !is.square.matrix(cor_params[[2]]) || cor_params[[1]] <= d-1){
      stop("Invalid inverse Wishart parameters for ", type_str, " correlation matrix prior. This should be a numeric giving the degrees of freedom and a d x d scale matrix, with degrees of freedom > d-1.")
    }
  }else if (form == "beta" &&
            (length(cor_params) !=2 ||
             !is.matrix(cor_params[[1]]) ||
             !is.matrix(cor_params[[2]]) ||
             !is.square.matrix(cor_params[[1]]) ||
             !is.square.matrix(cor_params[[2]]) ||
             !is.symmetric.matrix(cor_params[[1]]) ||
             !is.symmetric.matrix(cor_params[[2]]) ||
             !(dim(cor_params[[1]])[1] == d) ||
             !(dim(cor_params[[2]])[1] == d))){
    stop("Invalid beta parameters for ", type_str, " correlation matrix priors. These should be square, symmetric matrices with the same dimension as the number of model outputs.")
  }else if (form == "hierarchical"& length(cor_params) != 4){
    stop("Invalid parameters for ", type_str, " correlation matrix priors. This should be a list of length 4.")
  }else if (form == "hierarchical_beta_conjugate" & length(cor_params) != 3){
    stop("Invalid parameters for ", type_str, " correlation matrix priors. This should be a list of length 3.")
  }
}



validate_parametrisation_form <- function(form, valid_forms = c(CORRELATIONS_PRIOR_LKJ,CORRELATIONS_PRIOR_INV_WISHART,CORRELATIONS_PRIOR_BETA,CORRELATIONS_PRIOR_HIERARCHICAL,CORRELATIONS_PRIOR_BETA_CONJUGATE)){
  if(!isTRUE(correlation_form_prior(form) %in% valid_forms)){
    stop("Invalid parametrisation choice for priors. Prior parametrisation forms should be one of 'lkj', 'inv_wishart', 'beta', 'hierarchical' or 'hierarchical_beta_conjugate'. The 'hierarchical' and 'hierarchical_beta_conjugate' options are only available for individual short-term discrepancies. Prior choice: ", form)
  }

}


validate_prior_AR_params <- function(AR_params){
  if(!is.numeric(AR_params) || !(length(AR_params) == 2)){
    stop("Invalid autoregressive parameters specified. This should be a numeric of length 2 giving the shape parameters of the Beta distribution used as a prior for the AR parameters.")
  }
}

#'Generate samples of parameters from prior distribution
#'
#'Methods to generates samples of the parameters from the prior distribution of the ensemble model.
#'
#'@param priors An `EnsemblePrior` object specifying the prior distributions for the ensemble.
#'@param M A `numeric` that represents the number of simulators. The default is 1.
#'@param MM A `numeric` that represents the number of drivers. The default is `NULL`.
#'@param full_sample A `logical` that runs a full sampling of the prior density of the ensemble model if `TRUE`. If `FALSE`, returns the point estimate which maximises the prior density of the ensemble model.
#'@param control If creating a full sample, this is a named `list` of parameters to control Stan's sampling behaviour. See the documentation of the `stan()` function in the `rstan` package for details. The default value is `list(adapt_delta = 0.95)`. If optimizing, this value is ignored.
#'@param ... Additional arguments passed to the function \code{rstan::sampling} or  \code{rstan::optimizing}.
#'@return A `list` containing two items named `samples` and `point_estimate`. If `full_sample==TRUE`, `samples` is a `stanfit` and `point_estimate` is a `NULL` object, else `samples` is a `NULL` and `point_estimate` is a `list` object. It is possible to generate a point estimate for the prior if the individual short-term discrepancy prior follows a hierarchical parameterisation.
#'@references Stan Development Team (2020). RStan: the R interface to Stan. R package version 2.21.2. https://mc-stan.org
#'@seealso \code{\linkS4class{EnsembleFit}}
#'@export
#'@examples
#'\donttest{
#' priors <- EnsemblePrior(4)
#' prior_density <- prior_ensemble_model(priors, M = 4)
#' }
prior_ensemble_model <- function(priors, M = 1, MM = NULL,
                                    full_sample = TRUE, control = list(adapt_delta = 0.95), ...){
  stan_input <- priors@priors_stan_input
  stan_input$M <- M
  stan_input$N <- priors@d

  #Using hierarchical priors uses a different model. This speeds up the sampling enormously

  if (!is.null(MM)){
    stan_input$MM <- MM
    mod <- stanmodels$ensemble_prior_withdrivers
    if(stan_input$form_prior_ind_st == 3 || stan_input$form_prior_ind_st == 4){
      if(full_sample==FALSE){
        stop("It is possible to generate a point estimate for the prior if the individual short-term discrepancy prior follows a hierarchical parameterisation. Please generate a full sample using 'full_sample=TRUE'.")
      }
      mod <- stanmodels$ensemble_prior_hierarchical_withdrivers
    }
  }else{
    mod <- stanmodels$ensemble_prior
    if(stan_input$form_prior_ind_st == 3 || stan_input$form_prior_ind_st == 4){
      if(full_sample==FALSE){
        stop("It is possible to generate a point estimate for the prior if the individual short-term discrepancy prior follows a hierarchical parameterisation. Please generate a full sample using 'full_sample=TRUE'.")
      }
      mod <- stanmodels$ensemble_prior_hierarchical
    }
  }

  samples <- NULL; point_estimate <- NULL
  if(full_sample){
    samples <- rstan::sampling(mod, data=stan_input, control = control, ...)
  }else{
    point_estimate <- rstan::optimizing(mod, data=stan_input,as_vector=FALSE, ...)
  }
  return(list(samples=samples, point_estimate=point_estimate))
}




#'Generate samples of latent variables from prior predictive distribution
#'
#'Methods to generates samples of the latent variables from the prior predictive distribution of the ensemble model.
#'
#'@inheritParams EnsembleData
#'@param full_sample A `logical` that runs a full sampling of the prior density of the ensemble model if `TRUE`. If `FALSE`, returns the point estimate which maximises the prior density of the ensemble model.
#'@param sam_priors A `list` containing two items named `samples` and `point_estimate`. `samples` is either a `NULL` or a `stanfit` object containing the samples drawn from the prior distribution of the ensemble model and `point_estimate` is either a `NULL` or a `list` object containing the optimised prior distribution of the ensemble model. If this object is `missing` then `sample_prior` generates it.
#'@param num_samples A `numeric` specifying the number of samples to be generated. The default is 1.
#'@param ... Additional arguments passed to the function \code{rstan::sampling} or  \code{rstan::optimizing}.
#'@inherit generate_sample details
#'@return An `EnsembleSample` object.
#'@references Durbin, J. and Koopman, S. J. (2002). "A simple and efficient simulation smoother for state space time series analysis." Biometrika, 89(3), 603--616.
#'@references Chris M.Strickland, Ian. W.Turner, RobertDenhamb, Kerrie L.Mengersena. Efficient Bayesian estimation of multivariate state space models Computational Statistics & Data Analysis Volume 53, Issue 12, 1 October 2009, Pages 4116-4125
#'@seealso \code{\linkS4class{EnsembleFit}}, \code{\link{EnsembleSample}}, \code{\link{generate_sample}}, \code{\link{prior_ensemble_model}}
#'@export
#'@examples
#'\donttest{
#' priors <- EnsemblePrior(4)
#' prior_density <- prior_ensemble_model(priors, M = 4)
#' samples <- sample_prior(observations = list(SSB_obs, Sigma_obs),
#'              simulators = list(list(SSB_miz, Sigma_miz),
#'                                list(SSB_ewe, Sigma_ewe),
#'                                list(SSB_fs, Sigma_fs),
#'                                list(SSB_lm, Sigma_lm)),
#'              priors = priors,
#'              sam_priors = prior_density)
#' plot(samples) #Plot the prior predictive density.
#' }
sample_prior <- function(observations, simulators, priors, sam_priors, num_samples = 1, full_sample = TRUE, drivers = FALSE,...){
  if(missing(sam_priors)){
    if (drivers == FALSE) {
      sam_priors <- prior_ensemble_model(priors, M=length(simulators),
                                         full_sample = full_sample, ...)
    }
    else {
      sam_priors <- prior_ensemble_model(priors, M=length(simulators), MM = max(sapply(simulators, function(x) {length(x[[1]])})),
                                         full_sample = full_sample, ...)
    }
  }
  if (drivers == FALSE) {
    ens_data <- EnsembleData(observations, simulators, priors)
  }
  else {
    ens_data <- EnsembleData(observations, simulators, priors, drivers = TRUE)
  }
  fit_prior <- EnsembleFit(ens_data, sam_priors$samples, sam_priors$point_estimate)
  return(generate_sample(fit_prior,num_samples = num_samples))
}

test_that("prior test",{
	testthat::skip_on_cran()
  priors1 <- EnsemblePrior(4,ind_st_params = IndSTPrior(parametrisation_form = "lkj", var_params= list(1,1), cor_params = 10, AR_params = c(2, 2)))
  set.seed(1234)
  prior_density <- prior_ensemble_model(priors1, M = 4)
  samples <- sample_prior(observations = list(SSB_obs, Sigma_obs),
                          simulators = list(list(SSB_miz, Sigma_miz),
                                            list(SSB_ewe, Sigma_ewe),
                                            list(SSB_fs, Sigma_fs),
                                            list(SSB_lm, Sigma_lm)),
                          priors = priors1,
                          sam_priors = prior_density)
  plot(samples)
  priors2 <- EnsemblePrior(4)
  prior_density1 <- prior_ensemble_model(priors1, M = 4,full_sample = FALSE)
  prior_density2 <- prior_ensemble_model(priors2, M = 4,full_sample = TRUE)
  expect_error(prior_ensemble_model(priors2, M = 4,full_sample = FALSE),"It is possible to generate a point estimate for the prior if the individual short-term discrepancy prior follows a hierarchical parameterisation. Please generate a full sample using 'full_sample=TRUE'.")
})

test_that("Ensemble prior test",{
  d <-4
  prior1 <- EnsemblePrior(d=d)
  iLT <- IndLTPrior("lkj",list(1,1),1)
  iST <- IndSTPrior(parametrisation_form = "hierarchical", var_params= list(-3, 1, 8, 4), cor_params = list(0.1, 0.1, 0.1, 0.1), AR_params = c(2, 2))
  sST <- ShaSTPrior(parametrisation_form = "lkj", var_params = list(1, 10), cor_params = 1, AR_params = c(2, 2))
  tp <- TruthPrior(d, initial_mean = 0, initial_var = 100, rw_covariance = list(2* d, diag(d)))
  prior2 <- EnsemblePrior(d=d,ind_st_params = iST,ind_lt_params = iLT,sha_st_params = sST,truth_params = tp)
  expect_equal(prior1,prior2)
})

#' A general framework for combining ecosystem models
#'
#' @description The `EcoEnsemble` package implements the framework for combining ecosystem models laid out in Spence et al (2018).
#'
#' @details
#' The ensemble model can be implemented in three main stages:
#' 1. Eliciting priors on discrepancy terms: This is done by using the `EnsemblePrior` constructor.
#' 2. Fitting the ensemble model: Using `fit_ensemble_model` with simulator outputs, observations and prior information. The ensemble model can be fit, obtaining either the point estimate, which maximises the posterior density, or running Markov chain Monte Carlo to generate a sample from the posterior denisty of the ensemble model.
#' 3. Sampling the latent variables from the fitted model: Using `generate_sample` with the fitted ensemble object, the discrepancy terms and the ensemble's best guess of the truth can be generated. Similarly to `fit_ensemble_model`, this can either be a point estimate or a full sample.
#'
#' @useDynLib EcoEnsemble, .registration = TRUE
#' @import Rcpp
#' @import methods
#' @import dplyr
#' @import ggplot2
#' @import reshape2
#' @import tibble
#' @import cowplot
#' @import matrixcalc
#' @importFrom rstan sampling optimizing
#'
#' @references Stan Development Team (2020). RStan: the R interface to Stan. R package version 2.21.2. [https://mc-stan.org](https://mc-stan.org)
#' @references Spence, M. A., J. L. Blanchard, A. G. Rossberg, M. R. Heath, J. J. Heymans, S. Mackinson, N. Serpetti, D. C. Speirs, R. B. Thorpe, and P. G. Blackwell. 2018. "A General Framework for Combining Ecosystem Models." Fish and Fisheries 19: 1013-42. [https://onlinelibrary.wiley.com/doi/abs/10.1111/faf.12310](https://onlinelibrary.wiley.com/doi/abs/10.1111/faf.12310)
"_PACKAGE"

which_fun_subset <- function(i, j, mod_to_esm){
  ret <- 0
  num <- 0
  while (num < j){
    ret <- ret + 1
    if (mod_to_esm[i,ret] == ret){
      num <- num + 1
    }
  }
  return(ret)
}


#'@rdname get_stan_outputs
#'@export
get_transformed_data_dri <- function(fit){
  stan_input <- fit@ensemble_data@stan_input

  N <- stan_input$N
  M <- stan_input$M
  MM <- stan_input$MM
  model_num_species <- stan_input$model_num_species
  obs_covariances <- stan_input$obs_covariances
  Ms <- stan_input$Ms
  model_covariances <- stan_input$model_covariances
  observation_times <- stan_input$observation_times
  time <- stan_input$time
  observations <- stan_input$observations
  model_outputs <- stan_input$model_outputs
  n_dri <- stan_input$n_dri
  mod_to_dri <- stan_input$mod_to_dri

  bigM <- matrix(0, N + sum(model_num_species), (M + MM + 2) * N)
  all_eigenvalues_cov <- rep(0, N + sum(model_num_species))
  all_eigenvectors_cov = matrix(0, N + sum(model_num_species), N + sum(model_num_species))

  new_data <- observation_available <- matrix(0, time, N + sum(model_num_species))

  bigM[1:N,1:N] = diag(N)

  tmp_eigen <- eigen(obs_covariances,symmetric=T)
  all_eigenvalues_cov[1:N] = tmp_eigen$values
  all_eigenvectors_cov[1:N,1:N] = tmp_eigen$vectors
  if (M > 0) {
    if (MM > 0) {
      bigM[(N + 1):(N + model_num_species[1]), 1:N] <- Ms[1:model_num_species[1],]
      bigM[(N + 1):(N + model_num_species[1]), (N + 1):(2*N)] <- Ms[1:model_num_species[1],]
      bigM[(N + 1):(N + model_num_species[1]), (2*N + 1):(3*N)] <- Ms[1:model_num_species[1],]
      bigM[(N + 1):(N + model_num_species[1]), ((M + 2)*N + (which_fun_subset(1, 1, mod_to_dri)-1)*N + 1):((M + 2)*N + which_fun_subset(1, 1, mod_to_dri)*N)] <- Ms[1:model_num_species[1],]

      tmp_eigen <- eigen(matrix(model_covariances[1:(model_num_species[1]^2)],model_num_species[1],model_num_species[1]),symmetric=T)
      all_eigenvalues_cov[(N + 1):(N + model_num_species[1])] <- tmp_eigen$values
      all_eigenvectors_cov[(N + 1):(N + model_num_species[1]),(N + 1):(N + model_num_species[1])] <- tmp_eigen$vectors

      if (MM > 1) {
        for (j in 2:n_dri[1]) {
          start_index <- N + sum(model_num_species[1:(j-1)]) + 1
          end_index <-  N + sum(model_num_species[1:j])
          Ms_transformation <- Ms[(start_index - N):(end_index - N),]
          bigM[start_index:end_index, 1:N] <- Ms_transformation
          bigM[start_index:end_index, (N + 1):(2*N)] <- Ms_transformation
          bigM[start_index:end_index, (2*N + 1):(3*N)] <- Ms_transformation
          bigM[start_index:end_index, ((M + 2)*N + (which_fun_subset(1, j, mod_to_dri)-1)*N + 1):((M + 2)*N + which_fun_subset(1, j, mod_to_dri)*N)] <- Ms_transformation
          tmp_eigen <- eigen(matrix(model_covariances[(sum(model_num_species[1:(j-1)]^2) + 1):sum(model_num_species[1:j]^2)],model_num_species[j],model_num_species[j]),symmetric=T)
          all_eigenvalues_cov[start_index:end_index] <- tmp_eigen$values
          all_eigenvectors_cov[start_index:end_index, start_index:end_index] <- tmp_eigen$vectors
        }
      }
    }
    else {
      bigM[(N + 1):(N + model_num_species[1]), 1:N] <- Ms[1:model_num_species[1],]
      bigM[(N + 1):(N + model_num_species[1]), (N + 1):(2*N)] <- Ms[1:model_num_species[1],]
      bigM[(N + 1):(N + model_num_species[1]), (2*N + 1):(3*N)] <- Ms[1:model_num_species[1],]
      tmp_eigen <- eigen(matrix(model_covariances[1:(model_num_species[1]^2)],model_num_species[1],model_num_species[1]),symmetric=T)
      all_eigenvalues_cov[(N + 1):(N + model_num_species[1])] <- tmp_eigen$values
      all_eigenvectors_cov[(N + 1):(N + model_num_species[1]),(N + 1):(N + model_num_species[1])] <- tmp_eigen$vectors
    }
    if (M > 1) {
      if (MM > 0) {
        for (i in 2:M) {
          start_index <-  N + sum(model_num_species[1:sum(n_dri[1:(i-1)])]) + 1
          end_index <- N + sum(model_num_species[1:(sum(n_dri[1:(i-1)]) + 1)])
          Ms_transformation <- Ms[(start_index - N):(end_index - N),]
          bigM[start_index:end_index,1:N] <- Ms_transformation;
          bigM[start_index:end_index,(N+1):(2*N)] <- Ms_transformation;
          bigM[start_index:end_index,((i + 1)*N + 1):((i + 2) * N)] <- Ms_transformation
          bigM[start_index:end_index, ((M + 1)*N + (which_fun_subset(i, 1, mod_to_dri)-1)*N + 1):((M + 1)*N + which_fun_subset(i, 1, mod_to_dri)*N)] <- Ms_transformation
          tmp_eigen <- eigen(matrix(model_covariances[(sum(model_num_species[1:sum(n_dri[1:(i-1)])]^2) + 1):sum(model_num_species[1:(sum(n_dri[1:(i-1)]) + 1)]^2)], model_num_species[sum(n_dri[1:(i-1)]) + 1], model_num_species[sum(n_dri[1:(i-1)]) + 1]),symmetric=T)
          all_eigenvalues_cov[start_index:end_index] <- tmp_eigen$values
          all_eigenvectors_cov[start_index:end_index, start_index:end_index] <- tmp_eigen$vectors
        }
        if (MM > 1) {
          for (i in 2:M) {
            for (j in 2:n_dri[i]) {
              start_index <-  N + sum(model_num_species[1:(sum(n_dri[1:(i-1)]) + (j-1))]) + 1
              end_index <- N + sum(model_num_species[1:(sum(n_dri[1:(i-1)]) + j)])
              Ms_transformation <- Ms[(start_index - N):(end_index - N),]
              bigM[start_index:end_index,1:N] <- Ms_transformation
              bigM[start_index:end_index,(N+1):(2*N)] <- Ms_transformation
              bigM[start_index:end_index,((i + 1)*N + 1):((i + 2) * N)] = Ms_transformation
              bigM[start_index:end_index, ((M + 2)*N + (which_fun_subset(i, j, mod_to_dri)-1)*N + 1):((M + 2)*N + which_fun_subset(i, j, mod_to_dri)*N)] <- Ms_transformation
              tmp_eigen <- eigen(matrix(model_covariances[(sum(model_num_species[1:(sum(n_dri[1:(i-1)]) + (j-1))]^2) + 1):sum(model_num_species[1:(sum(n_dri[1:(i-1)]) + j)]^2)], model_num_species[sum(n_dri[1:(i-1)]) + j], model_num_species[sum(n_dri[1:(i-1)]) + j]),symmetric=T)
              all_eigenvalues_cov[start_index:end_index] <- tmp_eigen$values
              all_eigenvectors_cov[start_index:end_index, start_index:end_index] <- tmp_eigen$vectors
            }
          }
        }
      }
      else {
        for (i in 2:M) {
          start_index <-  N + sum(model_num_species[1:(i-1)]) + 1
          end_index <- N + sum(model_num_species[1:i])
          Ms_transformation <- Ms[(sum(model_num_species[1:(i-1)]) + 1):sum(model_num_species[1:i]),]
          bigM[start_index:end_index,1:N] <- Ms_transformation
          bigM[start_index:end_index,(N+1):(2*N)] <- Ms_transformation
          bigM[start_index:end_index,((i + 1)*N + 1):((i + 2) * N)] <- Ms_transformation
          tmp_eigen <- eigen(matrix(model_covariances[(sum(model_num_species[1:(i-1)]^2) + 1):sum(model_num_species[1:i]^2)],model_num_species[i],model_num_species[i]),symmetric=T)
          all_eigenvalues_cov[start_index:end_index] <- tmp_eigen$values
          all_eigenvectors_cov[start_index:end_index, start_index:end_index] <- tmp_eigen$vectors
        }
      }
    }
  }

  for (i in 1:time){
    observation_available[i,1:N] <- rep(observation_times[i,1],N)
    if (M > 0){
      observation_available[i,(N+1):(N + model_num_species[1])] <- rep(observation_times[i,2],model_num_species[1])
      for (j in 2:M){
        observation_available[i,(N + sum(model_num_species[1:sum(n_dri[1:(j-1)])]) + 1):(N + sum(model_num_species[1:sum(n_dri[1:j])]))] <- rep(observation_times[i,j + 1],sum(model_num_species[(sum(n_dri[1:(j-1)]) + 1):sum(n_dri[1:j])]))
      }
    }
    new_data[i,] <- t(all_eigenvectors_cov) %*% matrix(c(observations[i,],model_outputs[i,]),ncol=1)
  }


  bigM = t(all_eigenvectors_cov) %*% bigM;

  return(
    list(bigM=bigM,
         all_eigenvalues_cov=all_eigenvalues_cov,
         new_data=new_data,
         observation_available=observation_available)
  )
}

#' Ecopath with EcoSim SSB
#'
#' A dataset containing the predictions for spawning stock biomass of Norway Pout, Herring, Cod, and Sole in the North Sea between 1991-2050 under an MSY fishing scenario from EcoPath with EcoSim.
#'
#' @format A `data.frame` with 60 rows and 4 variables:
#' \describe{
#'   \item{`N.pout`}{Spawning stock biomass of Norway Pout, in log tonnes.}
#'   \item{`Herring`}{Spawning stock biomass of Herring, in log tonnes.}
#'   \item{`Cod`}{Spawning stock biomass of Cod, in log tonnes.}
#'   \item{`Sole`}{Spawning stock biomass of Sole, in log tonnes.}
#' }
#' @references ICES (2016). Working Group on Multispecies Assessment Methods (WGSAM). Technical report, International Council for Exploration of the Seas.
"SSB_ewe"

#' FishSUMS SSB
#'
#' A `data frame` containing the predictions for spawning stock biomass of Norway Pout, Herring, and Cod
#' in the North Sea between 1984-2050 under an MSY fishing scenario from FishSUMS. Note that FishSUMS does not
#' produce outputs for Sole.
#'
#' @format A `data.frame` with 67 rows and 3 variables:
#' \describe{
#'   \item{`N.pout`}{Spawning stock biomass of Norway Pout, in log tonnes.}
#'   \item{`Herring`}{Spawning stock biomass of Herring, in log tonnes.}
#'   \item{`Cod`}{Spawning stock biomass of Cod, in log tonnes.}
#' }
#' @references Speirs, D., Greenstreet, S., and Heath, M. (2016). Modelling the effects of fishing on the North Sea fish community size composition. Ecological Modelling, 321, 35–45
"SSB_fs"

#' LeMans SSB
#'
#' A `data.frame` containing the predictions for spawning stock biomass of Norway Pout, Herring, Cod, and Sole
#' in the North Sea between 1986-2050 under an MSY fishing scenario from the LeMaRns package.
#'
#' @format A `data.frame` with 65 rows and 4 variables:
#' \describe{
#'   \item{`N.pout`}{Spawning stock biomass of Norway Pout, in log tonnes.}
#'   \item{`Herring`}{Spawning stock biomass of Herring, in log tonnes.}
#'   \item{`Cod`}{Spawning stock biomass of Cod, in log tonnes.}
#'   \item{`Sole`}{Spawning stock biomass of Sole, in log tonnes.}
#' }
#' @references Thorpe, R. B., Le Quesne, W. J. F., Luxford, F., Collie, J. S., and Jennings, S. (2015). Evaluation and management implications of uncertainty in a multispecies size-structured model of population and community responses to fishing. Methods in Ecology and Evolution, 6(1), 49–58.
"SSB_lm"


#' mizer SSB
#'
#' A `data.frame` containing the predictions for spawning stock biomass of Norway Pout, Herring, Cod, and Sole
#' in the North Sea between 1984-2050 under an MSY fishing scenario from mizer.
#'
#' @format A data frame with 67 rows and 4 variables:
#' \describe{
#'   \item{`N.pout`}{Spawning stock biomass of Norway Pout, in log tonnes.}
#'   \item{`Herring`}{Spawning stock biomass of Herring, in log tonnes.}
#'   \item{`Cod`}{Spawning stock biomass of Cod, in log tonnes.}
#'   \item{`Sole`}{Spawning stock biomass of Sole, in log tonnes.}
#' }
#' @references Blanchard, J. L., Andersen, K. H., Scott, F., Hintzen, N. T., Piet, G., and Jennings, S. (2014). Evaluating targets and trade-offs among fisheries and conservation objectives using a multispecies size spectrum model. Journal of Applied Ecology, 51(3), 612–622
"SSB_miz"

#' Stock assessment SSB
#'
#' A `data.frame` containing the single species stock assessment estimates of spawning stock
#' biomass of Norway Pout, Herring, Cod, and Sole in the North Sea between 1984-2017.
#'
#' @format A data frame with 34 rows and 4 variables:
#' \describe{
#'   \item{`N.pout`}{Spawning stock biomass of Norway Pout, in log tonnes.}
#'   \item{`Herring`}{Spawning stock biomass of Herring, in log tonnes.}
#'   \item{`Cod`}{Spawning stock biomass of Cod, in log tonnes.}
#'   \item{`Sole`}{Spawning stock biomass of Sole, in log tonnes.}
#' }
#' @references Herring Assessment Working Group for the Area South of 62 N (HAWG)(2020).Technical report,
#' ICES Scientific Reports. ACOM:07. 960 pp, ICES, Copenhagen.
#' @references Report of the Working Group on the Assessment of Demersal Stocks in the North Sea and
#' Skagerrak (2020). Technical report, ICES Scientific Reports. ACOM:22.pp, ICES, Copenhagen.
"SSB_obs"


#' Ecopath with EcoSim Sigma
#'
#' A 4x4 covariance matrix quantifying the parameter uncertainty of Ecopath with EcoSim
#' @format A 4x4 `matrix`.
#' @references Mackinson, S., Platts, M., Garcia, C., and Lynam, C. (2018). Evaluating the fishery and ecological consequences of the proposed North Sea multi-annual plan. PLOS ONE, 13(1), 1–23.
"Sigma_ewe"

#' FishSUMS Sigma
#'
#' A 3x3 covariance matrix quantifying the parameter uncertainty of FishSUMS
#' @format A 3x3 `matrix`.
#' @references Spence, M. A., Blanchard, J. L., Rossberg, A. G., Heath, M. R., Heymans, J. J., Mackinson, S., Serpetti, N., Speirs, D. C., Thorpe, R. B., and Blackwell, P. G. (2018). A general framework for combining ecosystem models. Fish and Fisheries, 19(6), 1031–1042.
"Sigma_fs"

#' LeMans Sigma
#'
#' @format A 4x4 `matrix`.
#' A 4x4 covariance matrix quantifying the parameter uncertainty of LeMans
#' @references   Thorpe, R. B., Le Quesne, W. J. F., Luxford, F., Collie, J. S., and Jennings, S. (2015). Evaluation and management implications of uncertainty in a multispecies size-structured model of population and community responses to fishing. Methods in Ecology and Evolution, 6(1), 49–58.
"Sigma_lm"

#' mizer Sigma
#'
#' @format A 4x4 `matrix`.
#' A 4x4 covariance matrix quantifying the parameter uncertainty of mizer
#' @references Spence, M. A., Blackwell, P. G., and Blanchard, J. L. (2016). Parameter uncertainty of a dynamic multispecies size spectrum model. Canadian Journal of Fisheries and Aquatic Sciences, 73(4), 589–59

"Sigma_miz"

#' Stock assessment Sigma
#'
#' A 4x4 covariance matrix quantifying the covariances of the stock assessment estimates of biomass.
#' @format A 4x4 `matrix`.
#' @references Herring Assessment Working Group for the Area South of 62 N (HAWG).Technical report,
#' ICES Scientific Reports. ACOM:07. 960 pp, ICES, Copenhagen.
#' @references Report of the Working Group on the Assessment of Demersal Stocks inthe North Sea and
#' Skagerrak. Technical report, ICES Scientific Reports. ACOM:22.pp, ICES, Copenhagen.

"Sigma_obs"

stan_sampling_with_filter <- function(mod, data, control = NULL, ...) {
  warn_buf <- character(0)

  fit <- withCallingHandlers(
    rstan::sampling(mod, data = data, control = control, ...),
    warning = function(w) {
      warn_buf <<- c(warn_buf, conditionMessage(w))
      invokeRestart("muffleWarning")
    }
  )

  if (length(warn_buf) > 0) {
    # preserve order, drop duplicates
    msgs <- warn_buf[!duplicated(warn_buf)]
    emit_msgs <- msgs
    # robust match for Stan treedepth warnings
    is_treedepth <- grepl(
      "exceeded the maximum treedepth|maximum\\s+treedepth(.*exceeded)?",
      msgs, ignore.case = TRUE, perl = TRUE
    )
    is_pairs <- grepl(
      "examine\\s+the\\s+pairs\\(\\)\\s+plot",
      msgs, ignore.case = TRUE, perl = TRUE
    )
    is_ess <- grepl(
      "effective\\s+sample\\s+size|\\bESS\\b.*too\\s+low|bulk\\s+effective\\s+sample\\s+size|tail\\s+effective\\s+sample\\s+size",
      msgs, ignore.case = TRUE, perl = TRUE
    )
    emit_msgs[is_ess] <- paste0(
      emit_msgs[is_ess],
      ' Run - vignette("ESS_vignette", package = "EcoEnsemble").'
    )

    suppressible <- is_treedepth | is_pairs

    if (!all(suppressible)) {
      for (msg in emit_msgs) {
        warning(msg, call. = FALSE)
      }
    }
  }

  return(fit)
}

# tests/testthat/test-warning-filter.R

capture_warnings <- function(expr) {
  msgs <- character(0)
  withCallingHandlers(
    expr,
    warning = function(w) {
      msgs <<- c(msgs, conditionMessage(w))
      invokeRestart("muffleWarning")
    }
  )
  msgs
}

test_that("policy: only max-treedepth warnings are suppressed", {
  fake_sampling <- function(...) {
    warning(
      "There were 5 transitions after warmup that exceeded the maximum treedepth. Increase max_treedepth above 10.",
      call. = FALSE
    )
    structure(list(sampler = "fake"))
  }

  local_mocked_bindings(
    sampling = fake_sampling,
    .package = "rstan"
  )

  msgs <- capture_warnings({
    fit <- EcoEnsemble:::stan_sampling_with_filter(
      mod = NULL,
      data = list(),
      control = list()
    )
    expect_type(fit, "list")
  })

  expect_length(msgs, 0)
})

test_that("policy: mixed warnings (treedepth + others) re-emit all, in order, deduped", {
  fake_sampling <- function(...) {
    warning(
      "There were 5 transitions after warmup that exceeded the maximum treedepth. Increase max_treedepth above 10.",
      call. = FALSE
    )
    warning(
      "Bulk Effective Samples Size (ESS) is too low, indicating posterior means and medians may be unreliable.",
      call. = FALSE
    )
    warning(
      "Tail Effective Samples Size (ESS) is too low, indicating posterior variances and tail quantiles may be unreliable.",
      call. = FALSE
    )
    warning( # duplicate, should be deduped
      "There were 5 transitions after warmup that exceeded the maximum treedepth. Increase max_treedepth above 10.",
      call. = FALSE
    )
    structure(list(sampler = "fake"))
  }

  local_mocked_bindings(
    sampling = fake_sampling,
    .package = "rstan"
  )

  msgs <- capture_warnings({
    fit <- EcoEnsemble:::stan_sampling_with_filter(
      mod = NULL,
      data = list(),
      control = list()
    )
    expect_type(fit, "list")
  })

  expect_gte(length(msgs), 3L)
  expect_equal(sum(grepl("maximum treedepth", msgs, ignore.case = TRUE)), 1L)
  expect_true(any(grepl("maximum treedepth", msgs[1], ignore.case = TRUE)))
  expect_true(any(grepl("^Bulk Effective Samples Size", msgs[2])))
  expect_true(any(grepl("^Tail Effective Samples Size", msgs[3])))
})

test_that("policy: non-treedepth warnings are emitted", {
  fake_sampling <- function(...) {
    warning("The largest R-hat is NA, indicating chains have not mixed.", call. = FALSE)
    structure(list(sampler = "fake"))
  }

  local_mocked_bindings(
    sampling = fake_sampling,
    .package = "rstan"
  )

  msgs <- capture_warnings({
    fit <- EcoEnsemble:::stan_sampling_with_filter(
      mod = NULL,
      data = list(),
      control = list()
    )
    expect_type(fit, "list")
  })

  expect_length(msgs, 1)
  expect_match(msgs[[1]], "^The largest R-hat is NA")
})

# These are the currently accepted priors for correlation matrices. If you add anything here, you will also need to change the validate_parametrisation_form function in validation_prior_functions.R
CORRELATIONS_PRIOR_LKJ <- 0
CORRELATIONS_PRIOR_INV_WISHART <- 1
CORRELATIONS_PRIOR_BETA <- 2
CORRELATIONS_PRIOR_HIERARCHICAL <- 3
CORRELATIONS_PRIOR_BETA_CONJUGATE <- 4

correlation_form_prior <- function(prior_choice){
  return(
    switch(EXPR = tolower(prior_choice),
           "lkj" = CORRELATIONS_PRIOR_LKJ,
           "inv_wishart" = CORRELATIONS_PRIOR_INV_WISHART,
           "beta" = CORRELATIONS_PRIOR_BETA,
           "hierarchical" = CORRELATIONS_PRIOR_HIERARCHICAL,
           "hierarchical_beta_conjugate" = CORRELATIONS_PRIOR_BETA_CONJUGATE)
  )
}


correlation_prior <- function(params, form, type, allow_hierarchical){
  retv <- rep(NA, 5)
  names(retv) <- c(paste0(type, "_lkj"), paste0(type, "_wish_nu"),
                   paste0(type, "_wish_sigma"),paste0(type, "_beta_1"),
                   paste0(type, "_beta_2"))


  if(allow_hierarchical){
    retv <- append(retv, NA)
    names(retv)[6] <- paste0(type, "_hierarchical_beta_hyper_params")
  }

  ret <- as.list(retv)

  ret[[paste0(type, "_lkj")]] <- numeric(0)
  ret[[paste0(type, "_wish_nu")]] <- numeric(0)
  ret[[paste0(type, "_wish_sigma")]] <- matrix(0,0,0)
  ret[[paste0(type, "_beta_1")]] <- matrix(0,0,0)
  ret[[paste0(type, "_beta_2")]] <- matrix(0,0,0)

  if(allow_hierarchical){
    ret[[paste0(type, "_hierarchical_beta_hyper_params")]] <- numeric(0)
  }

  if (form == CORRELATIONS_PRIOR_LKJ){
    ret[[paste0(type, "_lkj")]] <- array(params, 1)
  }else if(form == CORRELATIONS_PRIOR_INV_WISHART){
    ret[[paste0(type, "_wish_nu")]] <-array(params[[1]], 1)
    ret[[paste0(type, "_wish_sigma")]] <- params[[2]]
  }else if(form == CORRELATIONS_PRIOR_BETA){
    ret[[paste0(type, "_beta_1")]] <- params[[1]]
    ret[[paste0(type, "_beta_2")]] <- params[[2]]
  }else if(form == CORRELATIONS_PRIOR_HIERARCHICAL
           && allow_hierarchical){
    ret[[paste0(type, "_hierarchical_beta_hyper_params")]] <- unlist(params)
  }else if(form == CORRELATIONS_PRIOR_BETA_CONJUGATE
           && allow_hierarchical){
    ret[[paste0(type, "_hierarchical_beta_hyper_params")]] <- unlist(params)
  }
  return(ret)
}

#If the user only specifies one variance parameter common for each species, then we should repeat this for each species so that Stan can understand.
repeat_variance_priors <- function(d, priors){

  prior_var_params <- priors@var_params

  ret <- prior_var_params

  if(length(prior_var_params[[1]]) == 1 && length(prior_var_params[[2]] == 1)){
    if(d == 1){
      ret[[1]] <- array(prior_var_params[[1]], dim = 1)
      ret[[2]] <- array(prior_var_params[[2]], dim = 1)
    }else{
      ret[[1]] <- rep(prior_var_params[[1]], d)
      ret[[2]] <- rep(prior_var_params[[2]], d)
    }
  }

  return(ret)
}

test_that("Correlation prior strings are identified", {
  expect_equal(correlation_form_prior("LKJ"), 0);
  expect_equal(correlation_form_prior("inv_wishart"), 1);
  expect_equal(correlation_form_prior("beta"), 2);
  expect_equal(correlation_form_prior("hierarchical"), 3);
  expect_equal(correlation_form_prior("hierarchical_beta_conjugate"), 4);
  expect_equal(correlation_form_prior("fdhfdhgfdhgf"), NULL);
})

test_that("STAN correlation priors are generated", {
  lkj_param <- 10
  inv_wish_params <- list(2, diag(9))
  beta_params <- list(matrix(5,9,9), matrix(2,9,9))

  STAN_data <-list("TEST_lkj" = array(lkj_param,1),
                   "TEST_wish_nu" = numeric(0),
                   "TEST_wish_sigma" =  matrix(0,0,0),
                   "TEST_beta_1" =  matrix(0,0,0),
                   "TEST_beta_2" =  matrix(0,0,0))
  expect_equal(correlation_prior(params = lkj_param,form=0,type="TEST", allow_hierarchical = FALSE),STAN_data)

  STAN_data[[1]] <- numeric(0)
  STAN_data[[2]] <- array(inv_wish_params[[1]],1)
  STAN_data[[3]] <- inv_wish_params[[2]]

  expect_equal(correlation_prior(params = inv_wish_params,form=1,type="TEST", allow_hierarchical = FALSE),STAN_data)

  STAN_data[[2]] <- numeric(0)
  STAN_data[[3]] <- matrix(0,0,0)
  STAN_data[[4]] <- beta_params[[1]]
  STAN_data[[5]] <- beta_params[[2]]

  expect_equal(correlation_prior(params = beta_params,form=2,type="TEST", allow_hierarchical = FALSE),STAN_data)
})

test_that("STAN correlation priors are generated for hierachical types", {
  #Hierarchical priors
  hierarchical_params <- list(0.8, 0.6, 0.2, 0.7)
  hierarchical_beta_conjugate_params <- list(0.8, 0.8, 0.5)
  STAN_data <-list("TEST_lkj" = numeric(0),
                   "TEST_wish_nu" = numeric(0),
                   "TEST_wish_sigma" =  matrix(0,0,0),
                   "TEST_beta_1" =  matrix(0,0,0),
                   "TEST_beta_2" =  matrix(0,0,0),
                   "TEST_hierarchical_beta_hyper_params" = unlist(hierarchical_params))

  expect_equal(correlation_prior(params = hierarchical_params,form=3,type="TEST", allow_hierarchical = TRUE),STAN_data)

  STAN_data[[6]] <- unlist(hierarchical_beta_conjugate_params)

  expect_equal(correlation_prior(params = hierarchical_beta_conjugate_params,form=4,type="TEST", allow_hierarchical = TRUE),STAN_data)
})

## ----include=FALSE------------------------------------------------------------
#Load in the data and load the library
library(EcoEnsemble)
#load("data/vignette_plots.Rdata")

## ----label="Observations"-----------------------------------------------------
SSB_obs
Sigma_obs

## ----label="EwESigma"---------------------------------------------------------
Sigma_ewe

## ----label="lmSigma"----------------------------------------------------------
Sigma_lm

## ----label="mizSigma"---------------------------------------------------------
Sigma_miz

## ----label="fsSigma"----------------------------------------------------------
Sigma_fs

## ----model_outputs, warning=FALSE, fig.dim = c(7, 3), message = FALSE, echo = FALSE----
library(tibble)
library(dplyr)
library(reshape2)
library(ggplot2)

#Inlcude years in the data frames
SSB_obs_tmp <- rownames_to_column(SSB_obs, var = "Year")
SSB_ewe_tmp <- rownames_to_column(SSB_ewe, var = "Year")
SSB_miz_tmp <- rownames_to_column(SSB_miz, var = "Year")
SSB_lm_tmp <- rownames_to_column(SSB_lm, var = "Year")
SSB_fs_tmp <- rownames_to_column(SSB_fs, var = "Year")

#Join dataframes together
df_all <- SSB_obs_tmp %>% 
    full_join(SSB_ewe_tmp, by = "Year", suffix = c("","_ewe")) %>%
    full_join(SSB_miz_tmp, by = "Year", suffix = c("","_miz")) %>%
    full_join(SSB_lm_tmp, by = "Year", suffix = c("","_lm")) %>%
    full_join(SSB_fs_tmp, by = "Year", suffix = c("","_fs"))

#Melt into long data format for ggplot
df_all <- melt(df_all, id.vars = "Year")
colnames(df_all) <- c("Year", "Simulator", "logSSB")
df_all$Year <- as.numeric(df_all$Year)

#Finally create the plots
df_n_pout <- df_all[grepl("N.pout", df_all$Simulator), ]
p1 <- ggplot(data=df_n_pout, aes(x=`Year`, y=`logSSB`, na.rm = TRUE)) + 
  geom_line(aes(group=`Simulator`,colour=`Simulator`)) +
  ggtitle("Norway pout")

df_herring <- df_all[grepl("Herring", df_all$Simulator), ]
p2 <- ggplot(data=df_herring, aes(x=`Year`, y=`logSSB`, na.rm = TRUE)) +    
  geom_line(aes(group=`Simulator`,colour=`Simulator`)) + 
  ggtitle("Herring")

df_cod <- df_all[grepl("Cod", df_all$Simulator), ]
p3 <- ggplot(data=df_cod, aes(x=`Year`, y=`logSSB`, na.rm = TRUE)) +    
  geom_line(aes(group=`Simulator`,colour=`Simulator`))+
  ggtitle("Cod")

df_sole <- df_all[grepl("Sole", df_all$Simulator), ]
p4 <- ggplot(data=df_sole, aes(x=`Year`, y=`logSSB`, na.rm = TRUE)) +    
  geom_line(aes(group=`Simulator`,colour=`Simulator`)) +
  ggtitle("Sole")

cowplot::plot_grid(p1, p2, p3, p4, ncol = 2, nrow = 2)


## -----------------------------------------------------------------------------
#Setting priors for an ensemble model with 4 variables of interest.
priors <- EnsemblePrior(4)

## -----------------------------------------------------------------------------
priors_custom <- EnsemblePrior(
                        d = 4,
                        ind_st_params = IndSTPrior("lkj", list(25, 0.25), 30),
                        ind_lt_params = IndLTPrior(
                          "beta",
                          list(c(25, 25, 25, 10),c(0.25, 0.25, 0.25, 0.1)),
                          list(matrix(40, 4, 4), matrix(40, 4, 4))
                          ),
                        sha_st_params = ShaSTPrior("lkj", list(25, 0.25), 30),
                        sha_lt_params = 3)

## ----label = "fit", eval=FALSE------------------------------------------------
# fit_sample <- fit_ensemble_model(observations = list(SSB_obs, Sigma_obs),
#                                  simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
#                                                    list(SSB_lm,  Sigma_lm,  "LeMans"),
#                                                    list(SSB_miz, Sigma_miz, "mizer"),
#                                                    list(SSB_fs,  Sigma_fs,  "FishSUMS")),
#                                  priors = priors)
# samples <- generate_sample(fit_sample)

## ----label = "Initial_plots", eval=FALSE--------------------------------------
# plot(samples)

## ----fig.dim = c(7, 4), echo=FALSE, warning = FALSE---------------------------
knitr::include_graphics("data/plot_initial_outputs.png")

## ----label = "Initial_plots_changed", fig.dim = c(7, 4), eval=FALSE-----------
# plot(samples, variable = "Cod", quantiles = c(0.25, 0.75)) +
#     ggplot2::theme_classic() +
#     ggplot2::scale_color_brewer(palette="Set2")  +
#     ggplot2::ylab("SSB (log tonnes)")

## ----fig.dim = c(7, 4), echo=FALSE, warning = FALSE---------------------------
knitr::include_graphics("data/plot_initial_customised.png")

## ----visualise_priors, eval= FALSE--------------------------------------------
# prior_model <- prior_ensemble_model(priors = priors, M = 4)
# prior_samples <- sample_prior(observations = list(SSB_obs, Sigma_obs),
#                                  simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
#                                                    list(SSB_lm,  Sigma_lm,  "LeMans"),
#                                                    list(SSB_miz, Sigma_miz, "mizer"),
#                                                    list(SSB_fs,  Sigma_fs,  "FishSUMS")),
#                                  priors = priors, prior_model)
# 

## ----plot_priors, fig.dim = c(7, 4), eval = FALSE-----------------------------
# plot(prior_samples)

## ----fig.dim = c(7, 6), echo = FALSE, warning = FALSE-------------------------
knitr::include_graphics("data/plot_priors_only.png")

## ----label = "create_data_manual"---------------------------------------------
ens_data <- EnsembleData(observations = list(SSB_obs, Sigma_obs),
                          simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
                                            list(SSB_lm,  Sigma_lm,  "LeMans"),
                                            list(SSB_miz, Sigma_miz, "mizer"),
                                            list(SSB_fs,  Sigma_fs,  "FishSUMS")),
                          priors = priors)


## ----label = "fitting_ensemble_DIY", eval=FALSE-------------------------------
# mod <- get_mcmc_ensemble_model(priors)
# # or request the Kalman filter variant explicitly
# kalman_mod <- get_mcmc_ensemble_model(priors, sampler = "kalman")
# samples <- rstan::sampling(mod, data = ens_data@stan_input)
# fit <- EnsembleFit(ens_data, samples = samples)

## ----eval = FALSE-------------------------------------------------------------
# # Generating samples using DIY functions
# transf_data <- get_transformed_data(fit)
# mle_sample <- gen_sample(1, ex.fit = rstan::extract(samples), transformed_data = transf_data,
#                          time = ens_data@stan_input$time)

## ----eval=FALSE---------------------------------------------------------------
# fit <- fit_ensemble_model(observations = observations, simulators = simulators,
#                           priors = priors)
# # or use the Kalman filter sampler
# fit_kalman <- fit_ensemble_model(observations = observations, simulators = simulators,
#                                  priors = priors,
#                                  sampler = "kalman")
# samples_tmp <- generate_sample(fit)
# samples <- samples_tmp@samples

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' @title Backwards Kalman filter
#' @description Finds the most likely path through a dynamical linear model.
#' @param rhos A \code{numeric} containing the diagonal elements of the transition matrix of the evolution equation.
#' @param dee A \code{numeric} of the same length as \code{rho} containing the discrepancies or biases in the observation process.
#' @param R A \code{numeric} representing the variances of the observation process.
#' @param Q A \code{matrix} of dimensions \code{length(rho)} and \code{length(rho)} representing the covariance of the evolution process.
#' @param C A a \code{matrix} of dimensions \code{length(rho)} and \code{length(R)} representing the observation operator of the observation process.
#' @param P A \code{matrix} of dimensions \code{length(rho)} and \code{length(rho)} representing the covariance matrix of the first state of the evolution process.
#' @param xhat A \code{numeric} of the same length as \code{rho} representing the expectation of the first state of the evolution process.
#' @param Time A numeric The length of time of the dynamical linear model.
#' @param y A \code{matrix} of dimensions \code{Time} and \code{length(R)} of observations of the observation process.
#' @param obs A \code{matrix} of dimensions \code{Time} and \code{length(R)}. \code{1} means in the \code{i},\code{j}th element means that the \code{j}th output is observed at tim \code{i}.
#' @details For the model with the evolution process \deqn{x_{t+1}\sim{}N(\rho{}\cdot{}x_{t},Q)} and observation process \deqn{y_{t}\sim{}N(\rho{}(x_{t} + \delta),diag(R))}.
#' @details Using the sequential Kalman filter, the function gives the mostly path of \eqn{x_{t}} for all \eqn{t}.
#' @return A \code{matrix} with dimensions \code{nrow(time)} and \code{length(xhat)} representing the most likely values of the latent variables.
#' @references Chui, C.K. & Chen, G. (2009) Kalman Filtering with Real-Time Applications. Springer, Berlin, Heidelberg, Fourth Edtion.
#' @references Kalman, R. E. (1960) A new approach to linear filtering and prediction problems. Trans. ASME, J. Basic Eng., 82, pp. 35-45.
#' @examples
#'\donttest{
#' fit <- fit_ensemble_model(observations = list(SSB_obs, Sigma_obs),
#'                simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
#'                                  list(SSB_fs,  Sigma_fs, "FishSUMS"),
#'                                  list(SSB_lm,  Sigma_lm, "LeMans"),
#'                                  list(SSB_miz, Sigma_miz, "Mizer")),
#'                priors = EnsemblePrior(4,
#'                ind_st_params = IndSTPrior(parametrisation_form = "lkj",
#'                var_params= list(1,1), cor_params = 10, AR_params = c(2, 2))),
#'                full_sample = FALSE) #Only optimise in this case
#' transformed_data <- get_transformed_data(fit)
#' ex.fit <- fit@point_estimate$par
#' params <- get_parameters(ex.fit)
#' ret <- KalmanFilter_back(params$AR_params, params$lt_discrepancies,
#'                           transformed_data$all_eigenvalues_cov,params$SIGMA,
#'                           transformed_data$bigM, params$SIGMA_init, params$x_hat,
#'                           fit@ensemble_data@stan_input$time,transformed_data$new_data,
#'                           transformed_data$observation_available)
#'}
#' @export
KalmanFilter_back <- function(rhos, dee, R, Q, C, P, xhat, Time, y, obs) {
    .Call(`_EcoEnsemble_KalmanFilter_back`, rhos, dee, R, Q, C, P, xhat, Time, y, obs)
}

#' Constructor for the `EnsembleData` class
#'
#' A constructor for the `EnsembleData` class. This is used to convert input data into the required form for `fit_ensemble_model`.
#'@param observations A `list` of length 2 containing observations and a covariance matrix. The first element is a `data.frame` or `matrix` with each column giving observations of each output of interest and each row a time. Rows should be named with the times and columns should be named the variables. The second element is is a \eqn{d \times d} `matrix` where \eqn{d} is the number of columns of the observations data frame / matrix. This matrix is the covariance matrix of the observations.
#'@param simulators A `list` with length equal to the number of simulators. For each simulator, there is a `list` of 2 objects containing the simulator output and covariance matrix. The first element is a `data.frame` or `matrix` with each column giving a simulator outputs of interest and each row a time. Rows should be named with the times and columns should be named the variables. The second element is a \eqn{n_k \times n_k} `matrix` where \eqn{n_k} is the number of columns of the simulators output data frame / matrix. This matrix is the covariance matrix of the simulator outputs.
#'@param  priors An `EnsemblePrior` object specifying the prior distributions for the ensemble.
#'@param drivers A `logical` indicating whether drivers have been used in combination with simulators. Default value is FALSE.
#'@param MMod Not currently implemented.
#'@return An object of class \code{EnsembleData}
#'@seealso \code{\linkS4class{EnsembleData}}, \code{\link{EnsemblePrior}}, \code{\link{fit_ensemble_model}}
#'@export
#'@include EnsemblePrior-class.R
#'@examples
#'ensemble_data <- EnsembleData(observations = list(SSB_obs, Sigma_obs),
#'                              simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
#'                                            list(SSB_fs,  Sigma_fs, "FishSUMS"),
#'                                            list(SSB_lm,  Sigma_lm, "LeMans"),
#'                                            list(SSB_miz, Sigma_miz, "mizer")),
#'                               priors = EnsemblePrior(4))
EnsembleData <- function(observations, simulators, priors, drivers = FALSE, MMod) {
  # Checks that the input data can be converted into the form required for Stan.

  if (drivers == FALSE){
    # Checks that the input data can be converted into the form required for Stan.
    validate_data(observations, simulators, priors)

    stan_data_sims <- generate_simulator_stan_data(observations, simulators)
  }else{
    # Checks that the input data can be converted into the form required for Stan.
    if (!missing(MMod)){
      validate_data_dri(observations, simulators, priors, MMod)

      stan_data_sims <- generate_simulator_stan_data_dri(observations, simulators, MMod)
    }else{
      validate_data_dri(observations, simulators, priors)

      stan_data_sims <- generate_simulator_stan_data_dri(observations, simulators)
    }
  }
  stan_input <- append(stan_data_sims, priors@priors_stan_input)


  ensemble_data <- new('EnsembleData', stan_input = stan_input,
                       observations = observations,
                       simulators = simulators,
                       priors = priors)
  return(ensemble_data)
}

#### Class definition ####
#' A class to hold the Ensemble data
#'
#' A class that holds the observation data, simulator outputs, and prior information to convert into the required form for `fit_ensemble_model`.
#'
#' @slot stan_input A `list` of parameters in the correct form to fit the ensemble model in Stan.
#' @slot observations A `list` of length 2 containing observations and a covariance matrix. The first element is a `data.frame` or `matrix` with each column giving observations of each output of interest and each row a time. Rows should be named with the times and columns should be named the variables. The second element is the covariance matrix of the observations.
#' @slot simulators A `list` with length equal to the number of simulators. For each simulator, there is a `list` of 2 objects containing the simulator output and covariance matrix. The first element is a `data.frame` or `matrix` with each column giving a simulator outputs of interest and each row a time. Rows should be named with the times and columns should be named the variables. The second element is the covariance matrix of the simulator outputs.
#' @slot priors An `EnsemblePrior` object specifying the prior distributions for the ensemble.
#' @seealso \code{\link{EnsembleData}}, \code{\link{EnsemblePrior}}, \code{\link{fit_ensemble_model}}
#' @export
setClass(
  "EnsembleData",
  slots = c(
    stan_input = "list",
    observations = "list",
    simulators = "list",
    priors = "EnsemblePrior"
  )
)

generate_simulator_stan_data <- function(observations, simulators){
  N <- ncol(observations[[1]])
  M <- length(simulators)
  times <- rownames(observations[[1]])
  model_num_species <- rep(NA, M)
  Ms  <- matrix(NA, nrow=0, ncol=N)
  model_covariances <- c()
  for (i in 1:M) {
    sim <- simulators[[i]]
    model_ouput <- sim[[1]]
    times <- unique(append(times, rownames(model_ouput)))
    model_num_species[i] <- ncol(model_ouput)

    Mi <- matrix(0, nrow=model_num_species[i], ncol=N)
    observed_species <- colnames(observations[[1]])
    sim_species      <- colnames(model_ouput)
    for(k in 1:model_num_species[i]){
      for(l in 1:N){
        Mi[k,l] <- sim_species[k] == observed_species[l]
      }
    }
    Ms <- rbind(Ms, Mi)

    model_covariances <- append(model_covariances, as.numeric(sim[[2]]))

  }
  times <- sort(as.integer(times))

  #Things that can't be done on the first pass
  observation_times <- matrix(NA, nrow=length(times), ncol=0)
  model_outputs <- matrix(NA, nrow=length(times), ncol=0)
  observation_times <- cbind(observation_times, times %in% rownames(observations[[1]]))
  for (i in 1:M) {
    sim <- simulators[[i]]
    model_ouput <- sim[[1]]
    present_data <- as.numeric(times %in% rownames(model_ouput))
    observation_times <- cbind(observation_times, present_data )


    y_i<- matrix(0, nrow = length(times), ncol=ncol(model_ouput))
    for(k in 1:length(times)){
      year = times[k]
      dat_for_year <- model_ouput[year == rownames(model_ouput), ]
      if(nrow(dat_for_year) == 1 ||
         length(dat_for_year) == 1){
        y_i[k, ] <- unlist(dat_for_year)
      }
    }
    model_outputs <- cbind(model_outputs, y_i)

  }

  if(M == 1){
    model_num_species = array(model_num_species, 1)
  }

  obs_data <- observations[[1]]
  obs_data_all <- matrix(0, nrow = length(times), ncol=N)
  for(k in 1:length(times)){
    year = times[k]
    obs_for_year <- obs_data[year == rownames(obs_data), ]
    if(nrow(obs_for_year) == 1 ||
       length(obs_for_year) == 1){
      obs_data_all[k, ] <- unlist(obs_for_year)
    }
  }
  obs_covariances <- observations[[2]]
  return(list(N = N,
              time = length(times),
              M = M,
              model_num_species = model_num_species,
              Ms = Ms,
              observation_times = observation_times,
              model_outputs = model_outputs,
              model_covariances = model_covariances,
              observations = obs_data_all,
              obs_covariances = obs_covariances))
  }

validate_data_dri <- function(observations, simulators, priors, MMod){
  validate_observations(observations)

  if(!is.list(simulators)){
    stop(paste0("Simulator data should be passed through as a list, with each element also a list corresponding to each model. For each sublist corresponding to a model, the first element should be a further list containing the data for each of the allowed driver combinations with the model, and the second element a list containing the corresponding covariances. The third element element is the optional model name and the fourth element is a list of optional driver names (for each allowed combination)."))
  }

  M <- length(simulators)
  if (!missing(MMod)){
    MM <- max(unique(unlist(MMod)))
    n_dri <- sapply(MMod, length)
  }
  else {
    MM <- length(simulators[[1]][[1]])
    MMod <- replicate(M, 1:MM, simplify = F)
    n_dri <- rep(MM, M)
  }

  for(i in 1:M){
    simulator <- simulators[[i]]
    sim_name <- paste0("One of your simulators [[", i, "]]")
    if(length(simulator) >= 3){
      sim_name <- simulator[[3]]
    }
    if(!is.list(simulator)){
      stop(paste0(sim_name, "is not a list. Each simulator should be passed as a list, the first element of which should be a further list containing the data for each of the allowed driver combinations with the simulator, and the second element should be a list containing the corresponding covariances. The (optional) third element element is the simulator name and the (optional) fourth element is a list driver names (for each allowed combination with the given simulator)."))
    }
    #Check supplied model/driver combinations are compatible
    if (!missing(MMod)){
      if (length(simulator[[1]]) != n_dri[i] || length(simulator[[2]]) != n_dri[[i]]){
        stop(paste0(sim_name), "has a number of driver combinations incompatible with user-supplied number of combinations:", n_dri[i])
      }
    }
    else {
      if (length(simulator[[1]]) != MM || length(simulator[[2]]) != MM){
        stop(paste0(sim_name), "does not have data for all driven combinations. Data for all", MM,"drivers should be included.")
      }
    }
    for (j in 1:n_dri[i]){
      driver <- simulator[[1]][[j]]
      driver_sigma <- simulator[[2]][[j]]
      sim_name <- paste0("One of your simulator/driver combinations [[", i, "]][[", j,"]]")
      dri_name <- paste0("[[", j,"]]")
      if(length(simulator) >= 3){
        sim_name <- simulator[[3]]
        dri_name <- paste0(" + ","driver [[", j,"]]")
        if (length(simulator) == 4){
          dri_name <- paste0(" + ", simulator[[4]][[j]])
        }
        sim_name <- paste0(sim_name, dri_name)
      }
      sim_name <- paste0(sim_name, dri_name)
      validate_simulator(list(driver, driver_sigma), colnames(observations[[1]]), sim_name)
    }
  }
}

#'Generate samples from a fitted ensemble model to get MCMC effective sample size diagnostics
#'
#'Methods to generates samples of the latent variables from a fitted ensemble model in the same order as the MCMC and to calculate diagnostics effective sample size for the MCMC.
#'@param fit An `EnsembleFit` object.
#'@param ex.fit A named `numeric`.
#'@param transformed_data A `list` of transformed input data.
#'@param time A `numeric` specifying the time for which the ensemble model was run.
#'@param arr A `named numeric`
#'@param stan_name A `character` object
#'@param only_voi A `logical` indicating whether only the effective sample size of the variables of interest should be returned. Default value is `TRUE`.
#'@param sammy An `array` of size the number of parameters \eqn{\times} `num_samples` \eqn{\times} `nchains`.
#'
#'@details  Effective sample size is calculated from these samples, only for the quantity of interest if desired.
#'
#'@return
#'`generate_sample_array` gives an array of size  `time * (MM + M + 2)*N` \eqn{\times} `num_samples` \eqn{\times} `nchains` where `M` is the number of simulators, `MM` is the number of drivers (usually 1), `N` is the number of variables of interest `num_samples` is the number of samples from the ensemble model for each chain and `nchains` is the number of chains in the `EnsembleFit` object.
#'
#'`get_parameters_array` gives a `list` of ensemble parameters from the requested sample.
#'
#'`get_mle_array` gives an array of size `time * (MM + M + 2)*N` \eqn{\times} `num_samples` \eqn{\times} `nchains`.
#'
#'`gen_sample_array` gives a `list` of two arrays of dimensions `time * (MM + M + 2)*N` \eqn{\times} `num_samples` \eqn{\times} `nchains`
#'
#'`get_ESS_diag` and `calc_ess` gives a list of length two containing `ESS_bulk` and `ESS_tail` which has the bulk effective sample size and the tail effective sample size. Each list is of dimensions `time` \eqn{\times} `(MM + M + 2)*N`.
#'
#'@importFrom posterior ess_bulk
#'@importFrom posterior ess_tail
#'
#'@rdname get_ESS_func
#'@seealso See [generate_sample()] for information about the sampling and \code{\link[posterior]{ess_bulk}} and \code{\link[posterior]{ess_tail}} for information about the effective sample size calculations.
#'@export
#'@examples
#'\donttest{
#' fit <- fit_ensemble_model(observations = list(SSB_obs, Sigma_obs),
#'                simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
#'                                  list(SSB_fs,  Sigma_fs, "FishSUMS"),
#'                                  list(SSB_lm,  Sigma_lm, "LeMans"),
#'                                  list(SSB_miz, Sigma_miz, "Mizer")),
#'                priors = EnsemblePrior(4,
#'                ind_st_params = IndSTPrior(parametrisation_form = "lkj",
#'                var_params= list(1,1), cor_params = 10, AR_params = c(2, 2))))
#' get_ESS_diag(fit)
#'}
generate_sample_array <- function(fit) {
  stan_input <- fit@ensemble_data@stan_input

  # If we have samples, then a full MCMC was run
  full_sample <- !is.null(fit@samples)

  if ("MM" %in% names(stan_input)) {
    transformed_data <- get_transformed_data_dri(fit)
  } else {
    transformed_data <- get_transformed_data(fit)
  }

  if (!full_sample) {
    stop("`fit@samples` is NULL. A full MCMC fit is required.", call. = FALSE)
  }

  ex.fit <- rstan::extract(fit@samples, permuted = FALSE)


  mle <- apply(
    ex.fit,
    c(1, 2),
    get_mle_array,
    transformed_data = transformed_data,
    time = stan_input$time
  )

  sammy <- apply(
    ex.fit,
    c(1, 2),
    gen_sample_array,
    transformed_data = transformed_data,
    time = stan_input$time
  )

  hat <- array(unlist(lapply(sammy, "[[", "sam_x_hat")), dim = dim(mle))
  x <- array(unlist(lapply(sammy, "[[", "sam_x")), dim = dim(mle))

  sample_ret <- mle - hat + x

  return(sample_ret)
}

#'@rdname get_ESS_func
#'@export
get_mle_array <- function(ex.fit, transformed_data, time) {
  params <- get_parameters_array(ex.fit)

  ret <- KalmanFilter_back(
    params$AR_params,
    params$lt_discrepancies,
    transformed_data$all_eigenvalues_cov,
    params$SIGMA,
    transformed_data$bigM,
    params$SIGMA_init,
    params$x_hat,
    time,
    transformed_data$new_data,
    transformed_data$observation_available
  )

  return(ret)
}

#'@rdname get_ESS_func
#'@export
get_parameters_array <- function(ex.fit) {
  ret <- list()

  tmp <- ex.fit[get_param_idx(ex.fit, "x_hat")]
  dimmy <- length(tmp)

  ret$x_hat <- tmp
  ret$SIGMA_init <- matrix(
    ex.fit[get_param_idx(ex.fit, "SIGMA_init")],
    dimmy,
    dimmy
  )
  ret$AR_params <- ex.fit[get_param_idx(ex.fit, "AR_params")]
  ret$SIGMA <- matrix(
    ex.fit[get_param_idx(ex.fit, "SIGMA")],
    dimmy,
    dimmy
  )
  ret$lt_discrepancies <- ex.fit[get_param_idx(ex.fit, "lt_discrepancies")]

  return(ret)
}

#'@rdname get_ESS_func
#'@export
get_param_idx <- function(arr, stan_name) {
  pnames <- names(arr)
  grep(paste0("^", stan_name, "(\\[|$)"), pnames)
}

#'@rdname get_ESS_func
#'@export
gen_sample_array <- function(ex.fit, transformed_data, time) {
  params <- get_parameters_array(ex.fit)

  bigM <- transformed_data$bigM
  all_eigenvalues_cov <- transformed_data$all_eigenvalues_cov
  observation_available <- transformed_data$observation_available

  sam_y <- matrix(0, time, nrow(bigM))
  sam_x <- matrix(0, time, length(params$x_hat))

  sam_x[1, ] <- as.matrix(params$x_hat) +
    t(chol(params$SIGMA_init)) %*% rnorm(length(params$x_hat))
  sam_y[1, ] <- bigM %*% (sam_x[1, ] + params$lt_discrepancies) +
    rnorm(nrow(bigM), 0, sqrt(all_eigenvalues_cov))

  ch_sigma <- t(chol(params$SIGMA))

  for (i in 2:time) {
    sam_x[i, ] <- as.matrix(params$AR_params * sam_x[i - 1, ]) +
      ch_sigma %*% rnorm(length(params$x_hat))
    sam_y[i, ] <- bigM %*% (sam_x[i, ] + params$lt_discrepancies) +
      rnorm(nrow(bigM), 0, sqrt(all_eigenvalues_cov))
  }

  sam_x_hat <- KalmanFilter_back(
    params$AR_params,
    params$lt_discrepancies,
    all_eigenvalues_cov,
    params$SIGMA,
    bigM,
    params$SIGMA_init,
    params$x_hat,
    time,
    sam_y,
    observation_available
  )

  return(list(sam_x = sam_x, sam_x_hat = sam_x_hat))
}

#'@rdname get_ESS_func
#'@export
get_ESS_diag <- function(fit,only_voi=TRUE){
  sammy <- generate_sample_array(fit)
  ####
  if (only_voi==TRUE){
    idx <- 1:(fit@ensemble_data@stan_input$time * fit@ensemble_data@stan_input$N)
    #sammy_save <- sammy
    sammy <- sammy[idx,,]
  }
  ESS_tmp <- calc_ess(sammy)
  ret <- list(
    ESS_bulk = matrix(ESS_tmp$ESS_bulk,nrow=fit@ensemble_data@stan_input$time),
    ESS_tail = matrix(ESS_tmp$ESS_tail,nrow=fit@ensemble_data@stan_input$time)
  )
  return(ret)
}

#'@rdname get_ESS_func
#'@export
calc_ess <- function(sammy){
  ret <- list(
    ESS_bulk = apply(sammy,1,posterior::ess_bulk),
    ESS_tail = apply(sammy,1,posterior::ess_tail)
  )
  return(ret)
}

#' Plot the ensemble output
#'
#'Plots the latent variables predicted by the ensemble model, along with simulator outputs and observations.
#'@param x An `EnsembleSample` object.
#'@param variable The name of the variable to plot. This can either be a `character` string in the same form as the observation variable, or an index for the column in the observations data frame.
#'@param quantiles A `numeric` vector of length 2 giving the quantiles for which to plot ribbons if doing a full sampling of the ensemble model. The default is `c(0.05,0.95)`.
#'@param ... Other arguments passed on to methods. Not currently used.
#'@return The `ggplot` object.
#'@importFrom stats median
#'@importFrom stats quantile
#'@importFrom stats rnorm
#'@importFrom cowplot plot_grid
#'@export
#'@examples
#'\donttest{
#' priors <- EnsemblePrior(4)
#' prior_density <- prior_ensemble_model(priors, M = 4)
#' samples <- sample_prior(observations = list(SSB_obs, Sigma_obs),
#'              simulators = list(list(SSB_miz, Sigma_miz),
#'                                list(SSB_ewe, Sigma_ewe),
#'                                list(SSB_fs, Sigma_fs),
#'                                list(SSB_lm, Sigma_lm)),
#'              priors = priors,
#'              sam_priors = prior_density)
#' plot(samples) #Plot the prior predictive density.
#' plot(samples, variable="Herring")
#'}
plot.EnsembleSample <- function(x, variable = NULL, quantiles=c(0.05, 0.95), ...){
  stan_input <- x@ensemble_fit@ensemble_data@stan_input
  if ("MM" %in% names(stan_input)){
    if(!is.null(variable)){
      return(plot_single_dri(x, variable, quantiles, ...))
    }

    d <- x@ensemble_fit@ensemble_data@priors@d
    plots_all <- lapply(1:d, function(i){plot_single_dri(x, i, quantiles, ...) + ggplot2::theme(legend.position = "none")})
    legend <- cowplot::get_plot_component(plot_single_dri(x, 1), "guide-box-right")
    plots_all <- append(plots_all, list(legend))
  }else{
    if(!is.null(variable)){
      return(plot_single(x, variable, quantiles, ...))
    }

    d <- x@ensemble_fit@ensemble_data@priors@d
    plots_all <- lapply(1:d, function(i){plot_single(x, i, quantiles, ...) + ggplot2::theme(legend.position = "none")})
    legend <- cowplot::get_plot_component(plot_single(x, 1), "guide-box-right")
    plots_all <- append(plots_all, list(legend))
  }
  return(
    do.call(cowplot::plot_grid, plots_all)
  )
}

get_variable <- function(variable, observations){
  ret <- variable
  if(is.double(variable) || is.integer(variable)){
    ret <- colnames(observations)[variable]
    if (abs(variable - round(variable)) > .Machine$double.eps^0.5){
      warning("Non-integer variable specified. variable will be taken as floor(variable). This is done by R, strange isn't it?!")
    }
  }

  if (is.na(ret)
      || !(ret %in% colnames(observations) )){
    stop(paste0("Invalid variable. This should be the name of a variable or an index less than ",
                ncol(observations) + 1))
  }

  return(ret)

}

construct_plot_dataframe <- function(samples, variable, quantiles){

  fit <- samples@ensemble_fit
  ensemble_data <- fit@ensemble_data
  observations <- ensemble_data@observations[[1]]
  simulators <- ensemble_data@simulators
  stan_input <- ensemble_data@stan_input

  #Observations
  df <- tibble::rownames_to_column(observations)[, c("rowname", variable)]
  colnames(df) <- c("Year", "Observations")

  #Simulators
  for (i in 1:length(simulators)) {
    simulator <- simulators[[i]]

    #Skip simulators that dont have the VoI
    if (!(variable %in% colnames(simulator[[1]]))){
      next
    }

    df_sim <- tibble::rownames_to_column(simulator[[1]], var = "Year")[, c("Year", variable)]
    #Use the name if available
    colnames(df_sim)[2] <- paste0("Simulator ", i)
    if (length(simulator) == 3){
      colnames(df_sim)[2] <- simulator[[3]]
    }


    df <- dplyr::full_join(df, df_sim, by = "Year")

  }

  #Ensemble
  var_index = which(colnames(observations) == variable)
  df$Year <- as.numeric(df$Year)
  times <- sort(unique(df$Year))
  if(!is.null(samples@samples)){

    # The ensemble outputs need to be the first columns to ensure they are coloured consistently when some variables are missing.
    df_ensemble <-cbind(times, apply(samples@samples[, var_index, ], 1, median, na.rm = TRUE),
                        apply(samples@samples[, var_index, ], 1, quantile, min(quantiles), na.rm = TRUE),
                        apply(samples@samples[, var_index, ], 1, quantile, max(quantiles), na.rm = TRUE))%>%
                        data.frame()


    colnames(df_ensemble) <- c("Year","Ensemble Model Prediction", "Lower", "Upper")
    df <- merge(df_ensemble,df)

    df <-  reshape2::melt(df, id.vars=c("Year", "Lower", "Upper"), variable.name="Simulator")

    #We have zero-width ribbons for the models and observations to avoid clutterling the scene
    df[df$Simulator != "Ensemble Model Prediction", c("Lower", "Upper")] <- df[df$Simulator != "Ensemble Model Prediction", "value"]

  }else{
    df_ensemble <-data.frame(times, samples@mle[, var_index])

    colnames(df_ensemble) <- c("Year","Ensemble Model Prediction")
    df <- merge(df_ensemble,df)

    df <-  reshape2::melt(df, id.vars=c("Year"), variable.name="Simulator")

  }
  return(df)
}

# Fudge to get past "no visible binding for global variable" in R-CMD check
utils::globalVariables(c("Year", "Lower", "Upper", "value", "Simulator"))

plot_single <- function(samples, variable=1, quantiles=c(0.05, 0.95), ...){

  fit <- samples@ensemble_fit
  ensemble_data <- fit@ensemble_data
  observations <- ensemble_data@observations[[1]]
  simulators <- ensemble_data@simulators
  stan_input <- ensemble_data@stan_input

  variable <- get_variable(variable, observations)

  df <- construct_plot_dataframe(samples, variable, quantiles)

  var_index = which(colnames(observations) == variable)
  if(!is.null(samples@samples)){
    return(plot_values_sample_gg(df, variable))
  }
  return(plot_values_optimised_gg(df, variable))
}

plot_values_optimised_gg<- function(df, title, ...){
  p <- ggplot2::ggplot(data=df, ggplot2::aes(x=`Year`, y=`value`, na.rm = TRUE), ...) + ggplot2::geom_line(ggplot2::aes(group=`Simulator`,colour=`Simulator`), na.rm = TRUE) + ggplot2::ggtitle(title)
  return(p)
}

plot_values_sample_gg<- function(df, title, ...){
    return(plot_values_optimised_gg(df, title, ...) +
             ggplot2::geom_ribbon(ggplot2::aes(ymin=`Lower`, ymax =`Upper`, fill = `Simulator`), alpha=0.2,na.rm = TRUE)) +
    ggplot2::theme(legend.position = "right")
}

#'Generate samples from a fitted ensemble model.
#'
#'Methods to generates samples of the latent variables from a fitted ensemble model.
#'@param fit An `EnsembleFit` object.
#'@param num_samples A `numeric` specifying the number of samples to be generated. The default is 1.
#'@param time A `numeric` specifying the time for which the ensemble model was run.
#'@param ex.fit A `list` containing the samples / point estimate from the `EnsembleFit` object.
#'@param x A `numeric` specifying which sample from the posterior to use. The default is 1.
#'@param transformed_data A `list` of transformed input data.
#'
#'@details The samples are created using the methods described in Strickland et. al. (2009) and Durbin and Koopman (2002).
#'
#'@return
#'`generate_sample` gives a `list` of length 2, the first element being the MLE of latent variables and the second element being a set of samples of the latent variables.
#'
#'* If `fit` is a sampling of the ensemble model parameters, then:
#'    + `mle` is a `time`\eqn{\times (3M + 2) \times} `num_samples` `array`, where \eqn{M} is the number of simulators and `num_samples` is the number of samples from the ensemble model, giving the MLE of the latent variables for each available sample from the ensemble model.
#'    + `sample` is a `time`\eqn{\times (3M + 2) \times} `num_samples` `array`, giving a sample of the latent variables for each available sample of the ensemble model.
#'
#'* If `fit` is a point estimate of the ensemble model parameters, then:
#'    + `mle` is a `time`\eqn{\times (3M + 2) \times} 1 `array` giving the MLE of the latent variables for the point estimate of the ensemble model.
#'    + `sample` is a `time`\eqn{\times (3M + 2) \times} `num_samples` `array`, giving `num_samples` samples of the latent variables for the single point estimate of the ensemble model.
#'
#'`get_transformed_data` gives a `list` of transformed input data.
#'
#'`get_parameters` gives a `list` of ensemble parameters from the requested sample.
#'
#'`get_mle` If `fit` is a sampling of the ensemble model parameters, then this is a `time`\eqn{\times (3M + 2) \times} `num_samples` `array`. If `fit` is a point estimate of the ensemble model parameters, then this is a `time`\eqn{\times (3M + 2) \times} 1 `array` giving the MLE of the latent variables for the point estimate of the ensemble model.
#'
#'`gen_sample` If `fit` is a sampling of the ensemble model parameters, then this is a `time`\eqn{\times (3M + 2) \times} `num_samples` `array`, giving a sample of the latent variables for each available sample of the ensemble model. If `fit` is a point estimate of the ensemble model parameters, then this is a `time`\eqn{\times (3M + 2) \times} `num_samples` `array`.
#'
#'
#'@rdname get_stan_outputs
#' @references Durbin, J. and Koopman, S. J. (2002). "A simple and efficient simulation smoother for state space time series analysis." Biometrika, 89(3), 603--616.
#' @references Chris M.Strickland, Ian. W.Turner, Robert Denhamb, Kerrie L.Mengersena. Efficient Bayesian estimation of multivariate state space models Computational Statistics & Data Analysis Volume 53, Issue 12, 1 October 2009, Pages 4116-4125
#'@export
#'@examples
#'\donttest{
#' fit <- fit_ensemble_model(observations = list(SSB_obs, Sigma_obs),
#'                simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
#'                                  list(SSB_fs,  Sigma_fs, "FishSUMS"),
#'                                  list(SSB_lm,  Sigma_lm, "LeMans"),
#'                                  list(SSB_miz, Sigma_miz, "Mizer")),
#'                priors = EnsemblePrior(4,
#'                ind_st_params = IndSTPrior(parametrisation_form = "lkj",
#'                var_params= list(1,1), cor_params = 10, AR_params = c(2, 2))),
#'                full_sample = FALSE) #Only optimise in this case
#' samples <- generate_sample(fit, num_samples = 2000)
#'}
generate_sample <- function(fit, num_samples = 1)
{

  stan_input <- fit@ensemble_data@stan_input

  # If we have samples, then a full MCMC was run
  full_sample <- !is.null(fit@samples)

  if ("MM" %in% names(stan_input)) {
    transformed_data <- get_transformed_data_dri(fit)
  }
  else {
    transformed_data <- get_transformed_data(fit)
  }

  if (full_sample){
    ex.fit <- rstan::extract(fit@samples)
    num_samples <- nrow(ex.fit$x_hat)
    mle <- sapply(1:num_samples, get_mle, ex.fit = ex.fit, transformed_data = transformed_data,
                      time = stan_input$time, simplify = F)
  }else {
    ex.fit <- fit@point_estimate$par
    mle <- get_mle(x=1, ex.fit, transformed_data = transformed_data,
                       time = stan_input$time)
  }

  sammy <- sapply(1:num_samples, gen_sample, ex.fit = ex.fit,
                  transformed_data = transformed_data,
                  time = stan_input$time, simplify = F)


  if (full_sample){
    tmp <- mapply(function(x,y){
      y - x$sam_x_hat + x$sam_x
    }, y = mle,
    x = sammy)
    sample_ret <-array(as.numeric(unlist(tmp)),dim=c(nrow(mle[[1]]),ncol(mle[[1]]),num_samples))
    mle <-array(as.numeric(unlist(mle)),dim=c(nrow(mle[[1]]),ncol(mle[[1]]),num_samples))
  }else{
    tmp<-lapply(sammy,function(x,y){
      y - x$sam_x_hat + x$sam_x
    }, y = mle)
    sample_ret <-array(as.numeric(unlist(tmp)),dim=c(nrow(mle), ncol(mle), num_samples))
  }


  #Clean up the Stan functions
  #rm("As", "KalmanFilter_back", "KalmanFilter_seq_em", "priors_cor_beta", "priors_cor_hierarchical_beta", "sq_int")

  return(EnsembleSample(fit, mle, sample_ret))
}



#'@rdname get_stan_outputs
#'@export
get_transformed_data <- function(fit){
  stan_input <- fit@ensemble_data@stan_input

  N <- stan_input$N
  M <- stan_input$M
  model_num_species <- stan_input$model_num_species
  obs_covariances <- stan_input$obs_covariances
  Ms <- stan_input$Ms
  model_covariances <- stan_input$model_covariances
  observation_times <- stan_input$observation_times
  time <- stan_input$time
  observations <- stan_input$observations
  model_outputs <- stan_input$model_outputs

  bigM <- matrix(0,N + sum(model_num_species) , (M+2) * N )
  all_eigenvalues_cov <- rep(0,N + sum(model_num_species) )
  all_eigenvectors_cov = matrix(0,N + sum(model_num_species) , N + sum(model_num_species) )
  new_data <- observation_available <- matrix(0,time , N + sum(model_num_species))
  bigM[1:N,1:N] = diag(N)

  tmp_eigen <- eigen(obs_covariances,symmetric=T)
  all_eigenvalues_cov[1:N] <- tmp_eigen$values
  all_eigenvectors_cov[1:N,1:N] <- tmp_eigen$vectors
  if (M > 0){
    bigM[(N + 1):(N + model_num_species[1] ),1:N] <- Ms[1:model_num_species[1],]
    bigM[(N + 1):(N + model_num_species[1] ),(N+1):(2*N)] <-  Ms[1:model_num_species[1],]
    bigM[(N + 1):(N + model_num_species[1] ),(2*N + 1):(3 * N)] <- Ms[1:model_num_species[1],]

    tmp_eigen <- eigen(matrix(model_covariances[1:(model_num_species[1]^2)],model_num_species[1],model_num_species[1]),symmetric=T)
    all_eigenvalues_cov[(N + 1):(N + model_num_species[1] )] <- tmp_eigen$values
    all_eigenvectors_cov[(N + 1):(N + model_num_species[1]),(N + 1):(N + model_num_species[1])] = tmp_eigen$vectors;

    if(M > 1){
      for(i in 2:M){
        start_index <-  N + sum(model_num_species[1:(i-1)]) + 1
        end_index <- N + sum(model_num_species[1:i])
        Ms_transformation <- Ms[(sum(model_num_species[1:(i-1)]) + 1):sum(model_num_species[1:i]),]
        bigM[start_index:end_index,1:N] <- Ms_transformation
        bigM[start_index:end_index,(N+1):(2*N)] <- Ms_transformation
        bigM[start_index:end_index,((i + 1)*N + 1):((i + 2) * N)] <- Ms_transformation

        tmp_eigen <- eigen(matrix(model_covariances[(cumsum(model_num_species^2)[i-1] + 1):(cumsum(model_num_species^2)[i])],model_num_species[i],model_num_species[i]),symmetric=T)
        all_eigenvalues_cov[start_index:end_index] <- tmp_eigen$values
        all_eigenvectors_cov[start_index:end_index, start_index:end_index] <- tmp_eigen$vectors
      }
    }
  }

  for (i in 1:time){
    observation_available[i,1:N] = rep(observation_times[i,1],N)
    if (M > 0){
      observation_available[i,(N+1):(N + model_num_species[1])] <- rep(observation_times[i,2],model_num_species[1])
      if(M > 1){
        for (j in 2:M){
          observation_available[i,(N + sum(model_num_species[1:(j-1)]) + 1):(N + sum(model_num_species[1:j]))] <- rep(observation_times[i,j + 1],model_num_species[j])
        }
      }
    }

    new_data[i,] <- t(all_eigenvectors_cov) %*% matrix(c(observations[i,],model_outputs[i,]),ncol=1)
  }

  bigM <- t(all_eigenvectors_cov) %*% bigM

  return(
    list(bigM=bigM,
         all_eigenvalues_cov=all_eigenvalues_cov,
         new_data=new_data,
         observation_available=observation_available)
  )
}

#'@rdname get_stan_outputs
#'@export
get_parameters <- function(ex.fit, x = 1){
  ret <- list()
  is_sample<- is(ex.fit$x_hat,"matrix")

  if (is_sample){
    ret$x_hat <- ex.fit$x_hat[x,]
    ret$SIGMA_init <- ex.fit$SIGMA_init[x,,]
    ret$AR_params <- ex.fit$AR_params[x,]
    ret$new_x <- ex.fit$x_hat[x,]
    ret$SIGMA <- ex.fit$SIGMA[x,,]
    ret$lt_discrepancies <- ex.fit$lt_discrepancies[x,]
  }else{
    ret$x_hat <- ex.fit$x_hat
    ret$SIGMA_init <- ex.fit$SIGMA_init
    ret$AR_params <- ex.fit$AR_params
    ret$new_x <- ex.fit$x_hat
    ret$SIGMA <- ex.fit$SIGMA
    ret$lt_discrepancies <- ex.fit$lt_discrepancies
  }
  return(ret)
}


#'@rdname get_stan_outputs
#'@export
get_mle <- function(x=1, ex.fit, transformed_data, time) ## get the MLE from the fit
{
  params <- get_parameters(ex.fit,x)

  ret <- KalmanFilter_back(params$AR_params, params$lt_discrepancies, transformed_data$all_eigenvalues_cov,
                           params$SIGMA, transformed_data$bigM, params$SIGMA_init, params$x_hat,time,
                           transformed_data$new_data, transformed_data$observation_available)
  #if(sum(is.na(ret)) > 0){
  #  browser()
  #}
  return(ret)
}


#'@rdname get_stan_outputs
#'@export
gen_sample <- function(x=1, ex.fit, transformed_data, time)
{
  params <- get_parameters(ex.fit, x)

  bigM = transformed_data$bigM
  all_eigenvalues_cov = transformed_data$all_eigenvalues_cov
  observation_available = transformed_data$observation_available

  sam_y <- matrix(0,time,nrow(bigM))
  sam_x <- matrix(0,time,length(params$x_hat))

  sam_x[1,] <- as.matrix(params$x_hat) + t(chol(params$SIGMA_init))%*%rnorm(length(params$x_hat))
  sam_y[1,] <- bigM %*% (sam_x[1,] + params$lt_discrepancies) + rnorm(nrow(bigM),0,sqrt(all_eigenvalues_cov))
  ch_sigma <- t(chol(params$SIGMA))
  for (i in 2:time){
    sam_x[i,] <- as.matrix(params$AR_params * sam_x[i-1,]) + ch_sigma%*%rnorm(length(params$x_hat))
    sam_y[i,] <- bigM %*% (sam_x[i,] + params$lt_discrepancies) + rnorm(nrow(bigM),0,sqrt(all_eigenvalues_cov))
  }
  sam_x_hat <- KalmanFilter_back(params$AR_params, params$lt_discrepancies,
                                 all_eigenvalues_cov, params$SIGMA,
                                 bigM,
                                 params$SIGMA_init,
                                 params$x_hat, time, sam_y,
                                 observation_available)
  #if(sum(is.na(sam_x_hat)) > 0){
  #  browser()
  #}
  return(list(sam_x=sam_x,sam_x_hat=sam_x_hat))
}

test_that("test including drivers, with simulated data set",{
  d<- 2
  M <- 2
  MM <- 3
  Time <- 10
  Times_obs <- round(Time * 0.8)
  obs <- matrix(c(0.57,0.75,0.9,1,1.07,1.09,1.04,0.94,0.99,0.54,0.57,1.09,1.67,2.01,2.17,2.33),ncol=d)
  obs.cov <- matrix(c(0.07,-0.02,-0.02,0.01),ncol=d)
  model.cov <- array(0,dim=c(M * MM,d,d))
  models_output <- array(NA,dim=c(M * MM,Time,d))
  models_output[1,,] <- c(-0.14,0.3,0.78,1.19,1.38,1.27,1.04,0.97,1.27,1.85,3.75,3.18,3.01,3.33,3.91,4.41,4.71,4.97,5.42,6.08)
  models_output[2,,] <- c(-0.94,-0.55,-0.25,-0.15,-0.26,-0.49,-0.7,-0.73,-0.52,-0.17,1.48,0.93,0.72,0.98,1.56,2.14,2.54,2.78,2.99,3.25)
  models_output[3,,] <- c(-0.05, 0.35, 0.84, 1.15, 1.18, 1.17, 1.08, 1.16, 1.94, 2.39, 3.93, 3.19, 2.88, 3.17, 3.69, 3.89, 3.99, 4.85, 5.6, 6.2)
  models_output[4,,] <- c(-1.16,-0.75,-0.65,-0.56,-0.66,-0.79,-0.9,-0.43,-0.02, 0.17,1.18,0.27,0.12,0.38,0.46,0.54,0.94,2.08,2.71,3.35)
  models_output[5,,] <- c(-1.14, 1.3, 1.78, 2.19, 2.38, 2.27, 2.04, 1.97, 2.27, 2.85, 4.75, 4.18, 4.01, 4.33, 4.91, 5.41, 5.71, 5.97, 6.42, 7.08)
  models_output[6,,] <- c(0.06, 0.45, 0.75, 0.85, 0.74, 0.51, 0.3, 0.27, 0.48, 0.83, 2.84, 1.93, 1.72, 1.98, 2.56, 3.14, 3.54, 3.78, 3.99, 4.25)

  testplts <- lapply(1:4, function(i) {cbind(reshape2::melt(data.frame(models_output[i,,])), rep(seq(1,10),2), rep(i,20))})
  testdf <- do.call(rbind, testplts)
  names(testdf)[3:4] <- c("year", "combination")
  testdf$combination <- as.factor(testdf$combination)
  testplt <- ggplot2::ggplot(testdf) + geom_line(aes(x = year, y = value, color = variable)) + facet_wrap(vars(combination))
  testplt

  obsdat <- cbind(reshape2::melt(data.frame(obs)), rep(seq(1,8),2))
  names(obsdat)[3] <- "year"
  obsplt <- ggplot2::ggplot(obsdat) + geom_line(aes(x = year, y = value, color = variable))
  obsplt

  model.cov[1,,] <- c(0.06,0,0,0.03)
  model.cov[2,,] <- c(0.04,0.01,0.01,0.02)
  model.cov[3,,] <- c(0.05, 0, 0, 0.02)
  model.cov[4,,] <- c(0.05, -0.03, -0.03, 0.06)
  model.cov[5,,] <- c(0.07,-0.01, -0.01, 0.08)
  model.cov[6,,] <- c(0.06, 0.03, 0.03, 0.05)

  val_obs <- data.frame(obs); cov_obs <- obs.cov
  val_model_11 <- data.frame(models_output[1,,]); cov_model_11 <- model.cov[1,,]
  val_model_21 <- data.frame(models_output[2,,]); cov_model_21 <- model.cov[2,,]
  val_model_12 <- data.frame(models_output[3,,]); cov_model_12 <- model.cov[3,,]
  val_model_22 <- data.frame(models_output[4,,]); cov_model_22 <- model.cov[4,,]
  val_model_13 <- data.frame(models_output[5,,]); cov_model_13 <- model.cov[5,,]
  val_model_23 <- data.frame(models_output[6,,]); cov_model_23 <- model.cov[6,,]

  #Set the dimnames to ensure EcoEnsemble can identify the information.
  SPECIES_NAMES <- c("Species 1", "Species 2")
  dimnames(val_obs) <- list(paste(1:Times_obs), SPECIES_NAMES)
  dimnames(val_model_11) <- list(paste(1:Time), SPECIES_NAMES)
  dimnames(val_model_21) <- list(paste(1:Time), SPECIES_NAMES)
  dimnames(val_model_12) <- list(paste(1:Time), SPECIES_NAMES)
  dimnames(val_model_22) <- list(paste(1:Time), SPECIES_NAMES)
  dimnames(val_model_13) <- list(paste(1:Time), SPECIES_NAMES)
  dimnames(val_model_23) <- list(paste(1:Time), SPECIES_NAMES)

  dimnames(cov_obs) <- list(SPECIES_NAMES, SPECIES_NAMES)
  dimnames(cov_model_11) <- list(SPECIES_NAMES, SPECIES_NAMES)
  dimnames(cov_model_21) <- list(SPECIES_NAMES, SPECIES_NAMES)
  dimnames(cov_model_12) <- list(SPECIES_NAMES, SPECIES_NAMES)
  dimnames(cov_model_22) <- list(SPECIES_NAMES, SPECIES_NAMES)
  dimnames(cov_model_13) <- list(SPECIES_NAMES, SPECIES_NAMES)
  dimnames(cov_model_23) <- list(SPECIES_NAMES, SPECIES_NAMES)

  val_model_1 <- list(val_model_11, val_model_12, val_model_13)
  val_model_2 <- list(val_model_21, val_model_22, val_model_23)
  cov_model_1 <- list(cov_model_11, cov_model_12, cov_model_13)
  cov_model_2 <- list(cov_model_21, cov_model_22, cov_model_23)

  for (sampler in c("kalman", "explicit")){
  suppressWarnings(fit <- fit_ensemble_model(observations = list(val_obs, cov_obs),
                                             simulators = list(list(val_model_1, cov_model_1, "Simulator 1", c("Driver 1", "Driver 2", "Driver 3")),
                                                               list(val_model_2, cov_model_2, "Simulator 2", c("Driver 1", "Driver 2", "Driver 3"))
                                             ),
                                             priors = EnsemblePrior(d), sampler = sampler,
                                             control = list(adapt_delta = 0.9),chains=1,iter=4,drivers=T))

  priors2 <- EnsemblePrior(2,
                           ind_st_params = IndSTPrior(parametrisation_form = "beta", var_params= list(c(1,2),c(1,1)), cor_params = list(matrix(50, 2, 2),matrix(50, 2, 2))
                                                      , AR_params = c(2, 2)),
                           ind_lt_params = IndLTPrior("inv_wishart",list(c(1,2),c(1,1)),list(10,diag(2))
                           ),
                           sha_st_params = ShaSTPrior("beta",list(c(1,2),c(1,1)),list(matrix(5, 2, 2),matrix(2, 2, 2))
                           ))
  suppressWarnings(fit.opt <- fit_ensemble_model(observations = list(val_obs, cov_obs),
                                                 simulators = list(list(val_model_1, cov_model_1, "Simulator 1", c("Driver 1", "Driver 2", "Driver 3")),
                                                                   list(val_model_2, cov_model_2, "Simulator 2", c("Driver 1", "Driver 2", "Driver 3"))
                                                 ), sampler = sampler,
                                                 priors = priors2,full_sample = F, drivers = T))

  }
  samples <- generate_sample(fit)
  expect_true(ggplot2::is_ggplot(plot(samples,quantiles = c(0.05, 0.95))))
  samples.opt <- generate_sample(fit.opt,num_samples = 500)
  expect_true(ggplot2::is_ggplot(plot(samples.opt,quantiles = c(0.05, 0.95))))
  err_mess <- paste0("Invalid variable. This should be the name of a variable or an index less than ",
                     d + 1)
  expect_error(plot(samples.opt,quantiles = c(0.05, 0.95),variable = 4),err_mess)
  ### test get_transformed_data

  stan_input <- fit@ensemble_data@stan_input

  N <- stan_input$N
  M <- stan_input$M
  MM <- stan_input$MM
  model_num_species <- stan_input$model_num_species
  obs_covariances <- stan_input$obs_covariances
  Ms <- stan_input$Ms
  model_covariances <- stan_input$model_covariances
  observation_times <- stan_input$observation_times
  time <- stan_input$time
  observations <- stan_input$observations
  model_outputs <- stan_input$model_outputs
  n_dri <- stan_input$n_dri
  mod_to_dri <- stan_input$mod_to_dri

  bigM <- matrix(0, N + sum(model_num_species), (M + MM + 2) * N)
  all_eigenvalues_cov <- rep(0, N + sum(model_num_species))
  all_eigenvectors_cov = matrix(0, N + sum(model_num_species), N + sum(model_num_species))
  new_data <- observation_available <- matrix(0, time, N + sum(model_num_species))
  bigM[1:N,1:N] = diag(N)

  tmp_eigen <- eigen(obs_covariances,symmetric=T)
  all_eigenvalues_cov[1:N] <- tmp_eigen$values
  all_eigenvectors_cov[1:N,1:N] <- tmp_eigen$vectors

  bigM[(N + 1):(N + model_num_species[1] ),1:N] <- Ms[1:model_num_species[1],]
  bigM[(N + 1):(N + model_num_species[1] ),(N+1):(2*N)] <-  Ms[1:model_num_species[1],]
  bigM[(N + 1):(N + model_num_species[1] ),(2*N + 1):(3 * N)] <- Ms[1:model_num_species[1],]
  bigM[(N + 1):(N + model_num_species[1]), ((M + 2)*N + (which_fun_subset(1, 1, mod_to_dri)-1)*N + 1):((M + 2)*N + which_fun_subset(1, 1, mod_to_dri)*N)] <- Ms[1:model_num_species[1],]

  tmp_eigen <- eigen(matrix(model_covariances[1:(model_num_species[1]^2)],model_num_species[1],model_num_species[1]),symmetric=T)
  all_eigenvalues_cov[(N + 1):(N + model_num_species[1] )] <- tmp_eigen$values
  all_eigenvectors_cov[(N + 1):(N + model_num_species[1]),(N + 1):(N + model_num_species[1])] = tmp_eigen$vectors;

  j <- 2

  start_index <- N + sum(model_num_species[1:(j-1)]) + 1
  end_index <-  N + sum(model_num_species[1:j])
  Ms_transformation <- Ms[(start_index - N):(end_index - N),]
  bigM[start_index:end_index, 1:N] <- Ms_transformation
  bigM[start_index:end_index, (N + 1):(2*N)] <- Ms_transformation
  bigM[start_index:end_index, (2*N + 1):(3*N)] <- Ms_transformation
  bigM[start_index:end_index, ((M + 2)*N + (which_fun_subset(1, j, mod_to_dri)-1)*N + 1):((M + 2)*N + which_fun_subset(1, j, mod_to_dri)*N)] <- Ms_transformation

  tmp_eigen <- eigen(matrix(model_covariances[(sum(model_num_species[1:(j-1)]^2) + 1):sum(model_num_species[1:j]^2)],model_num_species[j],model_num_species[j]),symmetric=T)
  all_eigenvalues_cov[start_index:end_index] <- tmp_eigen$values
  all_eigenvectors_cov[start_index:end_index, start_index:end_index] <- tmp_eigen$vectors

  j <- 3

  start_index <- N + sum(model_num_species[1:(j-1)]) + 1
  end_index <-  N + sum(model_num_species[1:j])
  Ms_transformation <- Ms[(start_index - N):(end_index - N),]
  bigM[start_index:end_index, 1:N] <- Ms_transformation
  bigM[start_index:end_index, (N + 1):(2*N)] <- Ms_transformation
  bigM[start_index:end_index, (2*N + 1):(3*N)] <- Ms_transformation
  bigM[start_index:end_index, ((M + 2)*N + (which_fun_subset(1, j, mod_to_dri)-1)*N + 1):((M + 2)*N + which_fun_subset(1, j, mod_to_dri)*N)] <- Ms_transformation

  tmp_eigen <- eigen(matrix(model_covariances[(sum(model_num_species[1:(j-1)]^2) + 1):sum(model_num_species[1:j]^2)],model_num_species[j],model_num_species[j]),symmetric=T)
  all_eigenvalues_cov[start_index:end_index] <- tmp_eigen$values
  all_eigenvectors_cov[start_index:end_index, start_index:end_index] <- tmp_eigen$vectors

  i <- 2
  j <- 1

  start_index <-  N + sum(model_num_species[1:sum(n_dri[1:(i-1)])]) + 1
  end_index <- N + sum(model_num_species[1:(sum(n_dri[1:(i-1)]) + 1)])
  Ms_transformation <- Ms[(start_index - N):(end_index - N),]
  bigM[start_index:end_index,1:N] <- Ms_transformation;
  bigM[start_index:end_index,(N+1):(2*N)] <- Ms_transformation;
  bigM[start_index:end_index,((i + 1)*N + 1):((i + 2) * N)] <- Ms_transformation;
  bigM[start_index:end_index, ((M + 1)*N + (which_fun_subset(i, 1, mod_to_dri)-1)*N + 1):((M + 1)*N + which_fun_subset(i, 1, mod_to_dri)*N)] <- Ms_transformation

  tmp_eigen <- eigen(matrix(model_covariances[(sum(model_num_species[1:sum(n_dri[1:(i-1)])]^2) + 1):sum(model_num_species[1:(sum(n_dri[1:(i-1)]) + 1)]^2)], model_num_species[sum(n_dri[1:(i-1)]) + 1], model_num_species[sum(n_dri[1:(i-1)]) + 1]),symmetric=T)
  all_eigenvalues_cov[start_index:end_index] <- tmp_eigen$values
  all_eigenvectors_cov[start_index:end_index, start_index:end_index] <- tmp_eigen$vectors

  j <- 2

  start_index <-  N + sum(model_num_species[1:(sum(n_dri[1:(i-1)]) + (j-1))]) + 1
  end_index <- N + sum(model_num_species[1:(sum(n_dri[1:(i-1)]) + j)])
  Ms_transformation <- Ms[(start_index - N):(end_index - N),]
  bigM[start_index:end_index,1:N] <- Ms_transformation
  bigM[start_index:end_index,(N+1):(2*N)] <- Ms_transformation
  bigM[start_index:end_index,((i + 1)*N + 1):((i + 2) * N)] = Ms_transformation
  bigM[start_index:end_index, ((M + 2)*N + (which_fun_subset(i, j, mod_to_dri)-1)*N + 1):((M + 2)*N + which_fun_subset(i, j, mod_to_dri)*N)] <- Ms_transformation

  tmp_eigen <- eigen(matrix(model_covariances[(sum(model_num_species[1:(sum(n_dri[1:(i-1)]) + (j-1))]^2) + 1):sum(model_num_species[1:(sum(n_dri[1:(i-1)]) + j)]^2)], model_num_species[sum(n_dri[1:(i-1)]) + j], model_num_species[sum(n_dri[1:(i-1)]) + j]),symmetric=T)
  all_eigenvalues_cov[start_index:end_index] <- tmp_eigen$values
  all_eigenvectors_cov[start_index:end_index, start_index:end_index] <- tmp_eigen$vectors

  j <- 3

  start_index <-  N + sum(model_num_species[1:(sum(n_dri[1:(i-1)]) + (j-1))]) + 1
  end_index <- N + sum(model_num_species[1:(sum(n_dri[1:(i-1)]) + j)])
  Ms_transformation <- Ms[(start_index - N):(end_index - N),]
  bigM[start_index:end_index,1:N] <- Ms_transformation
  bigM[start_index:end_index,(N+1):(2*N)] <- Ms_transformation
  bigM[start_index:end_index,((i + 1)*N + 1):((i + 2) * N)] = Ms_transformation
  bigM[start_index:end_index, ((M + 2)*N + (which_fun_subset(i, j, mod_to_dri)-1)*N + 1):((M + 2)*N + which_fun_subset(i, j, mod_to_dri)*N)] <- Ms_transformation

  tmp_eigen <- eigen(matrix(model_covariances[(sum(model_num_species[1:(sum(n_dri[1:(i-1)]) + (j-1))]^2) + 1):sum(model_num_species[1:(sum(n_dri[1:(i-1)]) + j)]^2)], model_num_species[sum(n_dri[1:(i-1)]) + j], model_num_species[sum(n_dri[1:(i-1)]) + j]),symmetric=T)
  all_eigenvalues_cov[start_index:end_index] <- tmp_eigen$values
  all_eigenvectors_cov[start_index:end_index, start_index:end_index] <- tmp_eigen$vectors

  for (i in 1:time){
    observation_available[i,1:N] <- rep(observation_times[i,1],N)
    if (M > 0){
      observation_available[i,(N+1):(N + model_num_species[1])] <- rep(observation_times[i,2],model_num_species[1])
      for (j in 2:M){
        observation_available[i,(N + sum(model_num_species[1:sum(n_dri[1:(j-1)])]) + 1):(N + sum(model_num_species[1:sum(n_dri[1:j])]))] <- rep(observation_times[i,j + 1],sum(model_num_species[(sum(n_dri[1:(j-1)]) + 1):sum(n_dri[1:j])]))
      }
    }
    new_data[i,] <- t(all_eigenvectors_cov) %*% matrix(c(observations[i,],model_outputs[i,]),ncol=1)
  }

  bigM <- t(all_eigenvectors_cov) %*% bigM
  ret <- list(bigM=bigM,
              all_eigenvalues_cov=all_eigenvalues_cov,
              new_data=new_data,
              observation_available=observation_available)
  expect_equal(get_transformed_data_dri(fit),ret)

  #test get_parameters
  ex.fit <- rstan::extract(fit@samples)
  x <- 1
  ret <- list()
  ret$x_hat <- ex.fit$x_hat[x,]
  ret$SIGMA_init <- ex.fit$SIGMA_init[x,,]
  ret$AR_params <- ex.fit$AR_params[x,]
  ret$new_x <- ex.fit$x_hat[x,]
  ret$SIGMA <- ex.fit$SIGMA[x,,]
  ret$lt_discrepancies <- ex.fit$lt_discrepancies[x,]
  expect_equal(get_parameters(ex.fit),ret)

  ### test get_mle
  ex.fit <- rstan::extract(fit@samples)
  transformed_data <- get_transformed_data_dri(fit)
  time <- fit@ensemble_data@stan_input$time

  params <- get_parameters(ex.fit,x=1)

  ret <- KalmanFilter_back(params$AR_params, params$lt_discrepancies, transformed_data$all_eigenvalues_cov,
                           params$SIGMA, transformed_data$bigM, params$SIGMA_init, params$x_hat,time,
                           transformed_data$new_data, transformed_data$observation_available)

  expect_equal(get_mle(x=1, ex.fit, transformed_data = transformed_data,
                       time = time),ret)
  ### test kalman filer
  back_kalmanFilter_R <- function(rhos, dee, R, Q, C, P, xhat, Time, y, obs){
    ## initials
    A <- as.matrix(rhos) %*% t(as.matrix(rhos))
    Gs <- array(0,dim=c(length(rhos),ncol(y),Time))
    Ps <- array(0,dim=c(length(rhos),length(rhos),Time))
    xhats <- matrix(0,length(rhos),Time)
    er <- Qinv <- matrix(0,Time,ncol(y))

    for (i in 1:Time)
    {
      P <-  P * A + Q
      xhat <- rhos * xhat
      Ps[,,i] <- P
      xhats[,i] <- xhat
      I <- diag(length(xhat))
      for (j in 1:ncol(y))
      {
        if(obs[i,j]==1.0){
          g <- P %*%  C[j,]
          Qinv[i,j] <- as.numeric(1/(t(C[j,]) %*% g + R[j]))
          er[i,j] <- y[i,j] - sum((xhat + dee) * C[j,])
          Gs[,j,i] <- Qinv[i,j] * g
          xhat <- xhat + Gs[,j,i] * er[i,j]
          P <- P - Gs[,j,i] %*% t(g)
        }
      }
    }
    ret <- matrix(0,Time,length(xhat))
    r <- rep(0,length(rhos))
    for (i in Time:1){
      for(j in ncol(y):1){
        if(obs[i,j]==1.0){
          L <- I - Gs[,j,i] %*% t(C[j,])
          r <- C[j,] * Qinv[i,j] * er[i,j] + t(L) %*% r
        }
      }
      ret[i,] <- xhats[,i] + t(Ps[,,i] %*% r)
      r <- rhos * as.numeric(r)
    }
    return(ret)
  }


  ### test kalman filter test
  time <- fit.opt@ensemble_data@stan_input$time
  transformed_data <- get_transformed_data_dri(fit.opt)
  params <- get_parameters(fit.opt@point_estimate$par)

  ret1 <- KalmanFilter_back(params$AR_params, params$lt_discrepancies, transformed_data$all_eigenvalues_cov,
                            params$SIGMA, transformed_data$bigM, params$SIGMA_init, params$x_hat,time,
                            transformed_data$new_data, transformed_data$observation_available)

  expect_equal(back_kalmanFilter_R(params$AR_params, params$lt_discrepancies, transformed_data$all_eigenvalues_cov,
                                   params$SIGMA, transformed_data$bigM, params$SIGMA_init, params$x_hat,time,
                                   transformed_data$new_data, transformed_data$observation_available),ret1)
})

test_that("test with simulated data set",{
  d<- 2
  M <- 2
  Time <- 10
  Times_obs <- round(Time * 0.8)
  obs <- matrix(c(0.57,0.75,0.9,1,1.07,1.09,1.04,0.94,0.99,0.54,0.57,1.09,1.67,2.01,2.17,2.33),ncol=d)
  obs.cov <- matrix(c(0.07,-0.02,-0.02,0.01),ncol=d)
  model.cov <- array(0,dim=c(M,d,d))
  models_output <- array(NA,dim=c(M,Time,d))
  models_output[1,,] <- c(-0.14,0.3,0.78,1.19,1.38,1.27,1.04,0.97,1.27,1.85,3.75,3.18,3.01,3.33,3.91,4.41,4.71,4.97,5.42,6.08)
  models_output[2,,] <- c(-0.94,-0.55,-0.25,-0.15,-0.26,-0.49,-0.7,-0.73,-0.52,-0.17,1.48,0.93,0.72,0.98,1.56,2.14,2.54,2.78,2.99,3.25)
  model.cov[1,,] <- c(0.06,0,0,0.03)
  model.cov[2,,] <- c(0.04,0.01,0.01,0.02)

  val_obs <- data.frame(obs); cov_obs <- obs.cov
  val_model_1 <- data.frame(models_output[1,,]); cov_model_1 <- model.cov[1,,]
  val_model_2 <- data.frame(models_output[2,,]); cov_model_2 <- model.cov[2,,]


  #Set the dimnames to ensure EcoEnsemble can identify the information.
  SPECIES_NAMES <- c("Species 1", "Species 2")
  dimnames(val_obs) <- list(paste(1:Times_obs), SPECIES_NAMES)
  dimnames(val_model_1) <- list(paste(1:Time), SPECIES_NAMES)
  dimnames(val_model_2) <- list(paste(1:Time), SPECIES_NAMES)


  dimnames(cov_obs) <- list(SPECIES_NAMES, SPECIES_NAMES)
  dimnames(cov_model_1) <- list(SPECIES_NAMES, SPECIES_NAMES)
  dimnames(cov_model_2) <- list(SPECIES_NAMES, SPECIES_NAMES)

  for (sampler in c("kalman", "explicit")){
  suppressWarnings(fit <- fit_ensemble_model(observations = list(val_obs, cov_obs),
                                           simulators = list(list(val_model_1, cov_model_1, "Model 1"),
                                                             list(val_model_2, cov_model_2, "Model 2")
                                           ), sampler = sampler,
                                           priors = EnsemblePrior(d),
                                           control = list(adapt_delta = 0.9),chains=1,iter=4))
  priors2 <- EnsemblePrior(2,
	ind_st_params = IndSTPrior(parametrisation_form = "beta", var_params= list(c(1,2),c(1,1)), cor_params = list(matrix(50, 2, 2),matrix(50, 2, 2))
  , AR_params = c(2, 2)),
	ind_lt_params = IndLTPrior("inv_wishart",list(c(1,2),c(1,1)),list(10,diag(2))
  ),
	sha_st_params = ShaSTPrior("beta",list(c(1,2),c(1,1)),list(matrix(5, 2, 2),matrix(2, 2, 2))
  ))
  suppressWarnings(fit.opt <- fit_ensemble_model(observations = list(val_obs, cov_obs),
                                               simulators = list(list(val_model_1, cov_model_1, "Model 1"),
                                                                 list(val_model_2, cov_model_2, "Model 2")
                                               ), sampler = sampler,
                                               priors = priors2,full_sample = F))


  samples <- generate_sample(fit)
  expect_true(ggplot2::is_ggplot(plot(samples,quantiles = c(0.05, 0.95))))
  samples.opt <- generate_sample(fit.opt,num_samples = 500)
  expect_true(ggplot2::is_ggplot(plot(samples.opt,quantiles = c(0.05, 0.95))))
  err_mess <- paste0("Invalid variable. This should be the name of a variable or an index less than ",
                     d + 1)
  expect_error(plot(samples.opt,quantiles = c(0.05, 0.95),variable = 4),err_mess)
  ### test get_transformed_data

  stan_input <- fit@ensemble_data@stan_input

  N <- stan_input$N
  M <- stan_input$M
  model_num_species <- stan_input$model_num_species
  obs_covariances <- stan_input$obs_covariances
  Ms <- stan_input$Ms
  model_covariances <- stan_input$model_covariances
  observation_times <- stan_input$observation_times
  time <- stan_input$time
  observations <- stan_input$observations
  model_outputs <- stan_input$model_outputs

  bigM <- matrix(0,N + sum(model_num_species) , (M+2) * N )
  all_eigenvalues_cov <- rep(0,N + sum(model_num_species) )
  all_eigenvectors_cov = matrix(0,N + sum(model_num_species) , N + sum(model_num_species) )
  new_data <- observation_available <- matrix(0,time , N + sum(model_num_species))
  bigM[1:N,1:N] = diag(N)

  tmp_eigen <- eigen(obs_covariances,symmetric=T)
  all_eigenvalues_cov[1:N] <- tmp_eigen$values
  all_eigenvectors_cov[1:N,1:N] <- tmp_eigen$vectors

  bigM[(N + 1):(N + model_num_species[1] ),1:N] <- Ms[1:model_num_species[1],]
  bigM[(N + 1):(N + model_num_species[1] ),(N+1):(2*N)] <-  Ms[1:model_num_species[1],]
  bigM[(N + 1):(N + model_num_species[1] ),(2*N + 1):(3 * N)] <- Ms[1:model_num_species[1],]

  tmp_eigen <- eigen(matrix(model_covariances[1:(model_num_species[1]^2)],model_num_species[1],model_num_species[1]),symmetric=T)
  all_eigenvalues_cov[(N + 1):(N + model_num_species[1] )] <- tmp_eigen$values
  all_eigenvectors_cov[(N + 1):(N + model_num_species[1]),(N + 1):(N + model_num_species[1])] = tmp_eigen$vectors;

  i <- 2

  start_index <-  N + sum(model_num_species[1:(i-1)]) + 1
  end_index <- N + sum(model_num_species[1:i])
  Ms_transformation <- Ms[(sum(model_num_species[1:(i-1)]) + 1):sum(model_num_species[1:i]),]
  bigM[start_index:end_index,1:N] <- Ms_transformation
  bigM[start_index:end_index,(N+1):(2*N)] <- Ms_transformation
  bigM[start_index:end_index,((i + 1)*N + 1):((i + 2) * N)] <- Ms_transformation

  tmp_eigen <- eigen(matrix(model_covariances[(cumsum(model_num_species^2)[i-1] + 1):(cumsum(model_num_species^2)[i])],model_num_species[i],model_num_species[i]),symmetric=T)
  all_eigenvalues_cov[start_index:end_index] <- tmp_eigen$values
  all_eigenvectors_cov[start_index:end_index, start_index:end_index] <- tmp_eigen$vectors

  for (i in 1:time){
    observation_available[i,1:N] = rep(observation_times[i,1],N)
    if (M > 0){
      observation_available[i,(N+1):(N + model_num_species[1])] <- rep(observation_times[i,2],model_num_species[1])
      if(M > 1){
        for (j in 2:M){
          observation_available[i,(N + sum(model_num_species[1:(j-1)]) + 1):(N + sum(model_num_species[1:j]))] <- rep(observation_times[i,j + 1],model_num_species[j])
        }
      }
    }

    new_data[i,] <- t(all_eigenvectors_cov) %*% matrix(c(observations[i,],model_outputs[i,]),ncol=1)
  }

  bigM <- t(all_eigenvectors_cov) %*% bigM
  ret <- list(bigM=bigM,
              all_eigenvalues_cov=all_eigenvalues_cov,
              new_data=new_data,
              observation_available=observation_available)
  expect_equal(get_transformed_data(fit),ret)


  ex.fit <- rstan::extract(fit@samples)
  x <- 1
  ret <- list()
  ret$x_hat <- ex.fit$x_hat[x,]
  ret$SIGMA_init <- ex.fit$SIGMA_init[x,,]
  ret$AR_params <- ex.fit$AR_params[x,]
  ret$new_x <- ex.fit$x_hat[x,]
  ret$SIGMA <- ex.fit$SIGMA[x,,]
  ret$lt_discrepancies <- ex.fit$lt_discrepancies[x,]
  expect_equal(get_parameters(ex.fit),ret)

  ### test get_mle
  ex.fit <- rstan::extract(fit@samples)
  transformed_data <- get_transformed_data(fit)
  time <- fit@ensemble_data@stan_input$time

  params <- get_parameters(ex.fit,x=1)

  ret <- KalmanFilter_back(params$AR_params, params$lt_discrepancies, transformed_data$all_eigenvalues_cov,
                           params$SIGMA, transformed_data$bigM, params$SIGMA_init, params$x_hat,time,
                           transformed_data$new_data, transformed_data$observation_available)

  expect_equal(get_mle(x=1, ex.fit, transformed_data = transformed_data,
                       time = time),ret)
  ### test kalman filer
  back_kalmanFilter_R <- function(rhos, dee, R, Q, C, P, xhat, Time, y, obs){
    ## initials
    A <- as.matrix(rhos) %*% t(as.matrix(rhos))
    Gs <- array(0,dim=c(length(rhos),ncol(y),Time))
    Ps <- array(0,dim=c(length(rhos),length(rhos),Time))
    xhats <- matrix(0,length(rhos),Time)
    er <- Qinv <- matrix(0,Time,ncol(y))

    for (i in 1:Time)
    {
      P <-  P * A + Q
      xhat <- rhos * xhat
      Ps[,,i] <- P
      xhats[,i] <- xhat
      I <- diag(length(xhat))
      for (j in 1:ncol(y))
      {
        if(obs[i,j]==1.0){
          g <- P %*%  C[j,]
          Qinv[i,j] <- as.numeric(1/(t(C[j,]) %*% g + R[j]))
          er[i,j] <- y[i,j] - sum((xhat + dee) * C[j,])
          Gs[,j,i] <- Qinv[i,j] * g
          xhat <- xhat + Gs[,j,i] * er[i,j]
          P <- P - Gs[,j,i] %*% t(g)
        }
      }
    }
    ret <- matrix(0,Time,length(xhat))
    r <- rep(0,length(rhos))
    for (i in Time:1){
      for(j in ncol(y):1){
        if(obs[i,j]==1.0){
          L <- I - Gs[,j,i] %*% t(C[j,])
          r <- C[j,] * Qinv[i,j] * er[i,j] + t(L) %*% r
        }
      }
      ret[i,] <- xhats[,i] + t(Ps[,,i] %*% r)
      r <- rhos * as.numeric(r)
    }
    return(ret)
  }


  ### test kalman filter test
  time <- fit.opt@ensemble_data@stan_input$time
  transformed_data <- get_transformed_data(fit.opt)
  params <- get_parameters(fit.opt@point_estimate$par)

  ret1 <- KalmanFilter_back(params$AR_params, params$lt_discrepancies, transformed_data$all_eigenvalues_cov,
                            params$SIGMA, transformed_data$bigM, params$SIGMA_init, params$x_hat,time,
                            transformed_data$new_data, transformed_data$observation_available)

  expect_equal(back_kalmanFilter_R(params$AR_params, params$lt_discrepancies, transformed_data$all_eigenvalues_cov,
                                   params$SIGMA, transformed_data$bigM, params$SIGMA_init, params$x_hat,time,
                                   transformed_data$new_data, transformed_data$observation_available),ret1)
}})

#'@param d A `numeric` giving the number of variables of interest in the ensemble model.
#'@param initial_mean A `numeric` giving the mean of the normal distribution giving the prior on the initial value of the random walk. This is the same value for each variable Default value is `0`.
#'@param initial_var A `numeric` giving the variance of the normal distribution giving the prior on the initial value of the random walk. This is the same value for each variable Default value is `100`.
#'@param rw_covariance A `list` of length `2` containing the inverse-Wishart parameters for the covariance of the random walk of the truth. The default value is `list(2*d, diag(d))`.
#' @examples
#'
#' ##### TruthPrior
#' #Simple default truth prior with 7 variables of interest
#' truth_def <- TruthPrior(7)
#' # A more fine-tuned truth prior for an ensemble with 7 species.
#' truth_cus <- TruthPrior(7, initial_mean = 2, initial_var = 10, rw_covariance = list(10, diag(7)))
#'
#' @rdname PriorConstructorFunctions
#' @export
TruthPrior <- function(d, initial_mean = 0, initial_var = 100, rw_covariance = list(2*d, diag(d))){

  ret <- new('TruthPrior',
             d = d,
             initial_mean  = initial_mean,
             initial_var  = initial_var,
             rw_covariance = rw_covariance)
  return(ret)
}



#### Class definition ####
#' A class to hold the priors for the truth model in the ensemble framework
#'
#' An `TruthPrior` object encapsulates the prior information for the short-term discrepancies of the shared discrepancy of the ensemble model.
#' @slot d A `numeric` giving the number of variables of interest in the ensemble model.
#' @slot initial_mean A `numeric` giving the standard deviation of the normal prior on the initial mean value of the random walk. This is the same standard deviation for each variable of interest.
#' @slot initial_var A `list` of length `2` containing the shape and scale parameters (respectively) for the gamma priors on the variance of the initial value of the truth.
#' @slot rw_covariance A `list` of length `2` containing the inverse-Wishart parameters for the covariance of the random walk of the truth.
#' @details
#' The truth \eqn{\mathbf{y}^{(t)}} is modelled as a random walk such that \deqn{\mathbf{y}^{(t+1)} \sim N(\mathbf{y}^{(t)}, \Lambda_y).} The covariance matrix \eqn{\Lambda_y} is parameterised by an inverse Wishart distribution (contained in the `rw_covariance` slot) and the initial value is modelled as drawn from a normal distribution.
#' @export
setClass(
  "TruthPrior",
  slots = c(d = "numeric",
            initial_mean  = "numeric",
            initial_var  = "numeric",
            rw_covariance = "list")
)



generate_priors_stan_input_truth <- function(d, x){
  if(!is(x, "TruthPrior")){
    stop("Invalid object for priors on the truth. This should be a TruthPrior object.")
  }


  if(length(x@initial_mean) == 1)
    prior_y_init_mean <- rep(x@initial_mean, d)
  if(length(x@initial_var) == 1){
    prior_y_init_var = rep(x@initial_var, d)
  }

  ret <- list(
    prior_y_init_mean = prior_y_init_mean,
    prior_y_init_var = prior_y_init_var,
    prior_sigma_t_inv_wish_nu = x@rw_covariance[[1]],
    prior_sigma_t_inv_wish_sigma = x@rw_covariance[[2]]
  )
  return(ret)
}

---
title: "Case Study: Synthetic Data"
author: "James A. Martindale, Michael J. Thomson and Michael A. Spence"
date: "2022-09-23"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{SyntheticData}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```


```{r setup_lies, warning=FALSE}
library(EcoEnsemble)
library(mgcv)
library(ggplot2)
library(cowplot)
set.seed(1234)
```

## Introduction
In this document, we generate synthetic data for simulator outputs and observations, then fit the ensemble model using these data. Finally, we demonstrate the consistency of the ensemble framework and explore uncertainty levels for different numbers of simulators. We do this for an ensemble model of 4 simulators, each describing 3 variables of interest.

## Generating the data
We generate the synthetic data by sampling from an informative prior distribution of the ensemble model. We choose  priors with very strong correlations of individual long-term discrepancies ($\mathrm{Beta}(5,1)$ via the method of concordance) and high autocorrelations of the AR processes on individual and shared short-term discrepancies ($\frac{r_i + 1}{2}\sim \mathrm{Beta}(10,2)$). These are different to the vague priors that will later be used to fit the model. We can configure this prior using the `EnsemblePrior()` constructor function.
```{r configure_priors}
d <- 3 # The number of variables.

priors <- EnsemblePrior(d,
                        ind_st_params = IndSTPrior(AR_params=c(10,2)), 
                        ind_lt_params = IndLTPrior("beta",
                                                   cor_params = list(matrix(5, d, d), 
                                                                     matrix(1, d, d))),
                        sha_st_params = ShaSTPrior(AR_params=c(10,2)),
)
```

From this configuration, sampling the ensemble model parameters is done using the `prior_ensemble_model()` function. In this case we sample the prior of an ensemble model with $M = 4$ simulators.

```{r sampling priors, eval = FALSE}
M <- 4 # The number of models we are considering.
prior_density <- prior_ensemble_model(priors, M = M)
ex.fit <- rstan::extract(prior_density$samples) # Extracted samples
```

Of these samples, the data we choose to make up our synthetic data is the single sample which maximizes the likelihood. Recall that for  `EcoEnsemble`, the truth and the discrepancy terms follow a dynamic linear model with

\begin{equation}
\mathbf\theta^{(t)}=(\mathbf{y}^{(t)'},\mathbf\eta^{(t)'},\mathbf{z}_1^{(t)'},\ldots,\mathbf{z}_m^{(t)'})',
\end{equation}
so that

\begin{equation}
\mathbf\theta^{(t)}|\mathbf\theta^{(t-1)}\sim{}N(A\mathbf\theta^{(t)},\Lambda),
\end{equation}

with
\begin{equation}
A= 
\begin{pmatrix}
I_d & 0 & 0 & 0 & \ldots & 0\\
0 & R_\eta & 0 & 0 & \ldots & 0\\
0  & 0 & R_1  & 0 & \ldots & 0\\
0  & 0 & 0 & R_2  & \ldots & 0\\
\vdots & \vdots & \vdots & \vdots & \ddots & \vdots \\
0  &  0 & 0 & 0 & \ldots & R_m
\end{pmatrix},
\end{equation}
with $I_d$ being a $d$ dimensional indicator matrix,
and
\begin{equation}
\Lambda=
\begin{pmatrix}
\Lambda_y & 0 & 0 & 0 & \ldots & 0\\
0 & \Lambda_\eta & 0 & 0 & \ldots & 0\\
0  & 0 & \Lambda_1  & 0 & \ldots & 0\\
0  & 0 & 0 & \Lambda_2  & \ldots & 0\\
\vdots & \vdots & \vdots & \vdots & \ddots & \vdots \\
0  &  0 & 0 & 0 & \ldots & \Lambda_m
\end{pmatrix}.
\end{equation}

See the `EcoEnsemble` vignette for further details. The samples from the prior distribution provide us with $A$ and $\Lambda$ values. To get from these ensemble model parameters to simulated model outputs requires using the above transformations. Values of the truth and discrepancy terms are generated using these values and sums of discrepancy terms to give the "best guesses" of simulator outputs.


```{r truth_best_guesses, eval = FALSE}
Time <- 50

true_par <-which.max(ex.fit$lp__)

latent <- matrix(NA, Time, (M+2)*d)

#Priors on initial values
latent[1, 1:d] <- rnorm(d, 0, 1)
latent[1, -(1:d)] <- t(chol(ex.fit$SIGMA_init[true_par,-(1:d) ,-(1:d)]))
                                          %*% rnorm((M+1)*d, 0, 1)

#Find all the latent values of the dynamic linear model
SIGMA <- ex.fit$SIGMA[true_par,,]
SIGMA_chol <- t(chol(SIGMA))
for (t in 2:Time) {
  latent[t,] <- ex.fit$AR_params[true_par,] * latent[t-1,] + SIGMA_chol 
                                          %*% rnorm((M+2)*d, 0, 1)
}

#The truth is simply the first d values
truth_latent <- latent[,1:d]

#The best guesses are sums of the truth and discrepancies
simulator_best_guesses <- array(NA,dim=c(Time,d,M))
for(i in 1:M){
  simulator_best_guesses[,,i] <- t(
    t(latent[,1:d] + latent[,(d+1):(2*d)] + latent[,(1+i) * d + (1:d)]) + 
      ex.fit$ind_lt[true_par,i,] + ex.fit$sha_lt[true_par,] )
}

```

We can see how the truth relates to simulator best guesses by plotting these outputs, for example for the first variable.
```{r load_datasets, echo = FALSE}
#Here I will load in the data
M <- 4
Time <- 50
load("data/SyntheticData_PriorSamples.RData")
```

```{r plot_data, fig.dim = c(7, 4)}
plot(truth_latent[,1], ylim=c(-2.5, 3), pch = 19)
lines(simulator_best_guesses[,1,1], col = 2)
lines(simulator_best_guesses[,1,2], col = 3)
lines(simulator_best_guesses[,1,3], col = 4)
lines(simulator_best_guesses[,1,4], col = 5)

```

In `EcoEnsemble`, we do not assume that we have access to the truth, nor the model best guesses. Instead, `EcoEnsemble` assumes that model outputs are noisy representations of the best guesses, and that observations are noisy representations of the truth. In order to produce the final synthetic dataset, we add noise to the simulated truth and best guesses. For simulators, this noise represents parameter uncertainty.

We use gams from the `mgcv` package to achieve this, and we overparametrise the model (by setting $k$ - the dimension of the basis used to represent the smooth term - to be something very large) to ensure that the outputs are sufficiently smooth.

In addition, we assume that we only have access to the first 40 years of observations.


```{r add_noise}
Times_obs <- round(Time * 0.8)
obs <- matrix(NA,Times_obs,d)
for(i in 1:d){
  g1 <- gam(y~s(x,k = 15),data=data.frame(x=1:Times_obs,y = truth_latent[1:Times_obs,i]))
  obs[,i] <- predict(g1)
}

obs.cov <- cov(obs - truth_latent[1:Times_obs,])


model.cov <- array(0,dim=c(M,d,d))
models_output <- array(NA,dim=c(M,Time,d))
for (j in 1:M){
  for(i in 1:d){
    g1 <- gam(y~s(x, k = 15),data=data.frame(x=1:Time,y=simulator_best_guesses[,i,j]))
    models_output[j,,i] <- predict(g1)
  }
  model.cov[j,,] <- cov(models_output[j,,] - simulator_best_guesses[,,j])
}

```

We can see how this data compares to the best guesses by again plotting the first variable of interest.
```{r plot_obs, fig.dim = c(7, 6)}
#Observations and model outputs
plot(obs[,1], ylim=c(-2.5, 3), xlim = c(1,Time), pch = 19)
lines(models_output[1,,1], col=2, lwd = 2)
lines(models_output[2,,1], col=3, lwd = 2)
lines(models_output[3,,1], col=4, lwd = 2)
lines(models_output[4,,1], col=5, lwd = 2)
```
In the above graph, dashed lines / empty points represent the best guesses / truth from the ensemble model, while solid lines / filled point represent the simulator outputs / observations.

## Fitting to Synthetic Data
We now need to convert the observations and simulator outputs into the required form for `EcoEnsemble`. This requires that input columns and rows are named consistently with years and species so that data from different models can be matched to each other, and missing data accounted for. We create `data frames` for each simulator and assign names.


```{r}
#Create the data frames that we'll use for EcoEnsemble
val_obs <- data.frame(obs); cov_obs <- obs.cov
val_model_1 <- data.frame(models_output[1,,]); cov_model_1 <- model.cov[1,,]
val_model_2 <- data.frame(models_output[2,,]); cov_model_2 <- model.cov[2,,]
val_model_3 <- data.frame(models_output[3,,]); cov_model_3 <- model.cov[3,,]
val_model_4 <- data.frame(models_output[4,,]); cov_model_4 <- model.cov[4,,]

#Set the dimnames to ensure EcoEnsemble can identify the information.
SPECIES_NAMES <- c("Species 1", "Species 2", "Species 3")
dimnames(val_obs) <- list(paste(1:Times_obs), SPECIES_NAMES)
dimnames(val_model_1) <- list(paste(1:Time), SPECIES_NAMES)
dimnames(val_model_2) <- list(paste(1:Time), SPECIES_NAMES)
dimnames(val_model_3) <- list(paste(1:Time), SPECIES_NAMES)
dimnames(val_model_4) <- list(paste(1:Time), SPECIES_NAMES)

dimnames(cov_obs) <- list(SPECIES_NAMES, SPECIES_NAMES)
dimnames(cov_model_1) <- list(SPECIES_NAMES, SPECIES_NAMES)
dimnames(cov_model_2) <- list(SPECIES_NAMES, SPECIES_NAMES)
dimnames(cov_model_3) <- list(SPECIES_NAMES, SPECIES_NAMES)
dimnames(cov_model_4) <- list(SPECIES_NAMES, SPECIES_NAMES)

```

Now we can fit the ensemble model, this time using vague default priors.
```{r fitting, eval = FALSE}
fit <- fit_ensemble_model(observations = list(val_obs, cov_obs),
                          simulators = list(list(val_model_1, cov_model_1, "Model 1"),
                                            list(val_model_2, cov_model_2, "Model 2"),
                                            list(val_model_3, cov_model_3, "Model 3"),
                                            list(val_model_4, cov_model_4, "Model 4")),
                          priors = EnsemblePrior(d),
                          control = list(adapt_delta = 0.9))
samples <- generate_sample(fit)
```
## Results

We can compare the truth as predicted by the ensemble model with the original truth value.
```{r results, eval = FALSE}
var_index <- 1

df <- data.frame("Year" = paste(1:Time),
                 "Ensemble" = apply(samples@mle[, var_index, ], 1, median),
                 "Lower"  = apply(samples@samples[, var_index, ], 1, quantile, 0.1),
                 "Upper"  = apply(samples@samples[, var_index, ], 1, quantile, 0.9),
                 "Actual" = truth_latent[,var_index])
df$Year <- as.numeric(df$Year)
df <-  reshape2::melt(df, id.vars=c("Year", "Lower", "Upper"), variable.name="Simulator")

# We only want uncertainty bands for the Ensemble values, else we set zero width bands 
df[df$Simulator == "Actual", c("Lower", "Upper")] <- df[df$Simulator != "Actual", "value"]

ggplot(df, aes(x=`Year`, y=`value`)) +
  geom_line(aes(group=`Simulator`,colour=`Simulator`)) +
  geom_ribbon(aes(ymin=`Lower`, ymax =`Upper`, fill = `Simulator`), alpha=0.2)

```

```{r plot_truth, echo = FALSE, fig.dim = c(7, 4)}
knitr::include_graphics("data/p_truth.png")
```

Here we see that the ensemble model is able to effectively capture the true value. 

## Effect of number of simulators
Unlike other model averaging methods, the ensemble framework allows for uncertainty to decrease as the number of models increases. We can observe this by refitting the ensemble on a smaller number of simulators. We refit the ensemble with $M = 1, 2, 3$.

```{r eval = FALSE}
fit_M1 <- fit_ensemble_model(observations = list(val_obs, cov_obs),
                             simulators = list(list(val_model_1, cov_model_1, "Model 1")),
                             priors = EnsemblePrior(d),
                             control = list(adapt_delta = 0.9))
samples_M1 <- generate_sample(fit_M1)

fit_M2 <- fit_ensemble_model(observations = list(val_obs, cov_obs),
                             simulators = list(list(val_model_1, cov_model_1, "Model 1"),
                                            list(val_model_2, cov_model_2, "Model 2")),
                          priors = EnsemblePrior(d),
                          control = list(adapt_delta = 0.9))
samples_M2 <- generate_sample(fit_M2)

fit_M3 <- fit_ensemble_model(observations = list(val_obs, cov_obs),
                          simulators = list(list(val_model_1, cov_model_1, "Model 1"),
                                            list(val_model_2, cov_model_2, "Model 2"),
                                            list(val_model_3, cov_model_3, "Model 3")),
                          priors = EnsemblePrior(d),
                          control = list(adapt_delta = 0.9))
samples_M3 <- generate_sample(fit_M3)

plot_grid(
  plot(samples_M1, variable=1) + ggtitle("1 Model")  + theme(legend.position = "none"),
  plot(samples_M2, variable=1) + ggtitle("2 Models") + theme(legend.position = "none"),
  plot(samples_M3, variable=1) + ggtitle("3 Models") + theme(legend.position = "none"),
  plot(samples   , variable=1) + ggtitle("4 Models") + theme(legend.position = "none"))
```
```{r model_comparison, echo = FALSE, fig.dim = c(7, 6)}
knitr::include_graphics("data/p_NumberOfModels.png")
```

To quantify this effect, we plot how the variance of predictions beyond the range of observation data changes over time.
```{r quantify_variance, eval = FALSE}
def.par <- par(no.readonly=TRUE) #old pars
par(mfrow = c(d, 1))
legend_ys <- c(0.4, 0.18, 0.12)
for(i in 1:d){
  plot.ts(apply(samples_M1@samples[40:50,i,],1, var), 
          main = paste0("Species ", i), ylab="Variance" )
  lines(apply(samples_M2@samples[40:50,i,],1, var), col = 2)
  lines(apply(samples_M3@samples[40:50,i,],1, var), col = 3)
  lines(apply(samples@samples[40:50,i,],1, var), col = 4)
  legend(1, legend_ys[i], legend = c("1 Model", "2 Models", "3 Models", "4 Models"),
          col = 1:4, lty = 1)
}
par(def.par)
```

```{r plot_variances, echo=FALSE, fig.dim = c(7,6)}
load("data/SyntheticData_AddingModelsData.RData")
def.par <- par(no.readonly=TRUE) #old pars
par(mfrow = c(d, 1), mar = c(4, 4, 1.4, 1.4))
legend_ys <- c(0.4, 0.18, 0.12)
for(i in 1:d){
  plot.ts(var_m1[, i], 
          main = paste0("Species ", i), ylab="Variance" )
  lines(var_m2[, i], col = 2)
  lines(var_m3[, i], col = 3)
  lines(var_m4[, i], col = 4)
  legend(1, legend_ys[i], legend = c("1 Model", "2 Models", "3 Models", "4 Models"),
          col = 1:4, lty = 1)
}
par(def.par)
```

As shown in the graphs, there is a marked reduction in the variance of predictions when more simulators are used in the ensemble.

---
title: "Including drivers"
author: "Thomas I. J. Bartos and Michael A. Spence"
date: "2024-06-04"
vignette: >
  %\VignetteIndexEntry{Including drivers}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}#
references:
- id: spence_ff
  title: A general framework for combining ecosystem models.
  author:
  - family: Spence
    given: Michael A.
  - family: Blanchard
    given: Julia L.
  - family: Rossberg
    given: Axel G.
  - family: Heath
    given: Michael R.
  - family: Heymans
    given: J. J.
  - family: Mackinson
    given: Steven
  - family: Serpetti
    given: Natalia
  - family: Speirs
    given: Douglas C.
  - family: Thorpe
    given: Robert B.
  - family: Blackwell
    given: Paul G.
  container-title: Fish and Fisheries
  volume: 19
  page: 1013-1042
  type: article-journal
  issued:
    year: 2018
- id: fishsums
  title: "Modelling the effects of fishing on the North Sea fish community size composition"
  author:
  - family: Speirs
    given: Douglas C. 
  - family: Greenstreet
    given: Simon P.R.
  - family: Heath
    given: Michael R.
  container-title: Ecological Modelling
  volume: 321
  page: 35--45
  type: article-journal 
  issued: 
    year: 2016
- id: kennedy_ohagan
  title: Bayesian calibration of computer models
  author: 
   - family: Kennedy
     given: Marc C.
   - family: "O'Hagan"
     given: Anthony
  container-title: "Journal of the Royal Statistical Society: Series B (Statistical Methodology)"
  volume: 63
  page: 425-464
  type: article-journal 
  issued: 
    year: 2001
- id: ices_hawg
  title: "Herring Assessment Working Group for the Area South of 62 N (HAWG)}"
  author: 
    - family: ICES
  issued:
    year: 2020
  type: report
  container-title: "ICES Scientific Reports. 2:60. 1151 pp, ICES, Copenhagen"
  doi: "10.17895/ices.pub.6105"
- id: ices_ns
  title: "Report of the Working Group on the Assessment of Demersal Stocks in the North Sea and Skagerrak"
  author: 
    - family: ICES
  issued:
    year: 2020
  type: report
  container-title: "ICES Scientific Reports. 2:61. 1353, ICES, Copenhagen"
  doi: "10.17895/ices.pub.6092"
- id: ices_ewe
  title: "Working Group on Multispecies Assessment Methods (WGSAM)"
  author: 
    - family: ICES
  issued:
    year: 2016
  type: report
- id: ggplot
  author:
   - family: Wickham
     given: Hadley
  title: "ggplot2: Elegant Graphics for Data Analysis"
  publisher: Springer-Verlag New York
  issued: 
    year: 2016
  type: book
  isbn: "978-3-319-24277-4"
  url: "https://ggplot2.tidyverse.org"
  
- id: rstan
  author:
  - family: "Stan Development Team"
  title: "RStan: the R interface to Stan"
  issued: 
    year: 2020
  url: "https://mc-stan.org/"
  type: article
---

```{r setup_lies, warning=FALSE, message=FALSE, silent = TRUE}
library(EcoEnsemble)
library(mgcv)
library(ggplot2)
library(cowplot)
set.seed(5678)
```

# Introduction
Sometimes a set of simulators can be jointly affected by a process driving the dynamics. For example, the outputs of a set of ecosystem models (simulators) predicting the biomass of various species could be affected by a set of earth-system models (drivers) giving a set of combined ecosystem - earth system model outputs. `EcoEnsemble` allows fitting of an ensemble model to inputs resulting from such scenarios. These inputs should take the form of a set of simulator models and a set of driver models with the data being the output of simulator-driver pairs. Input data must be supplied for all simulator-driver combinations as fitting specific combinations is not currently supported.

# Model description

At time $t$, the $d$ variables of interest (VoI), $\mathbf{y}^{(t)}=(y_1^{(t)},\ldots,y_d^{(t)})'$, are described by $m$ simulators and $p$ drivers, which for $k=1,\ldots,m$ and $j=1,\ldots,p$ give $n_k$ outputs $\mathbf{\hat{x}}_{k,j}^{(t)}=(\hat{x}_{k,j,1}^{(t)},\ldots,\hat{x}_{k,j,n_{k}}^{(t)})'$ each relating to one of the variables of interest. We also have noisy observations of the variables of interest, $\hat{\mathbf{y}}^{(t)}=(\hat{y}_1^{(t)},\ldots,\hat{y}_d^{(t)})'$.

Not all of the simulator-driver pairs output all the variables over the whole time period. 
To accommodate these differences, we introduce a latent variable known as the "best guess", $\mathbf{x}_{k,j}^{(t)}=({x}_{1}^{(t)},\ldots,{x}_{d}^{(t)})'$, which represents the output if simulator $k$ and driver $j$ described all $d$ variables at time $t$ with no parameter uncertainty. In `EcoEnsemble`, variables of interest are either present or absent in each simulator-driver pair, therefore if the $k$th simulator and $j$th driver were evaluated at time $t$, their combined output is
$$
\mathbf{\hat{x}}_{k,j}^{(t)}\sim{}N(M_{k,j}\mathbf{x}_{k,j}^{(t)},\Sigma_{k,j}),
$$
where for each $j = 1,\ldots,p$, $M_{k,j}^{(t)}$ is an $n_k\times{}d$ matrix and $\Sigma_{k,j}$ reflects the combined parameter uncertainty of the $k$th simulator and the $j$th driver.

The variables of interest at time $t$ are equal to the "best guess" of simulator $k$ and driver $j$, $\mathbf{x}_{k,j}^{(t)}$, plus a discrepancy term, $\mathbf{\zeta}^{(t)}_{k,j}$ (@kennedy_ohagan), i.e. 
$$
\mathbf{y}^{(t)} =  \mathbf{x}_{k,j}^{(t)} + \mathbf{\zeta}^{(t)}_{k,j}.
$$
The discrepancy term, $\mathbf{\zeta}^{(t)}_{k,j}$, is split between discrepancies that are shared between all of the simulators/driver combinations, and discrepancies that were specific to each simulator/driver combination. The shared discrepancy is further split into long-term shared discrepancy $\delta$ and the short-term shared discrepancy $\mathbf\eta^{(t)}$. The discrepancies specific to each combination are also split into a discrepancy arising from the simulator and a discrepancy arising from the driver. These two discrepancies are further split into long-term individual discrepancies $\mathbf\gamma_k$ (simulator) and $\mathbf\iota_j$ (driver), and short-term individual discrepancies $\mathbf z_{k}^{(t)}$ (simulator) and $\mathbf\alpha_{j}^{(t)}$ (driver). Written together we have
$$
\mathbf{\zeta}^{(t)}_{k,j} = \mathbf\delta + \mathbf\eta^{(t)} + \mathbf\gamma_k + \mathbf{z}_k^{(t)} + \mathbf\iota_{j} + \mathbf\alpha_{j}^{(t)}.
$$
The long-term individual discrepancies for the $k$th simulator and $j$th driver are
$$
    \mathbf{\gamma}_k\sim{}N(\mathbf{0},C_{\gamma}) \hspace{10pt},\hspace{10pt} \mathbf{\iota}_j\sim{}N(\mathbf{0},C_{\iota}).
$$
The short-term discrepancy terms, $\mathbf\eta^{(t)}$, $\mathbf{z}_k^{(t)}$ and $\mathbf{\alpha}_j^{(t)}$, follow auto-regressive processes of order one, 
$$
\mathbf{\eta}^{(t)}\sim{}N(R_{\eta}\mathbf{\eta}^{(t-1)},\Lambda_{\eta})
$$
and
$$
\mathbf{z}_{k}^{(t)}\sim{}N(R_{k}\mathbf{z}_{k}^{(t-1)},\Lambda_{k}) \hspace{10pt},\hspace{10pt} \mathbf{\alpha}_{j}^{(t)}\sim{}N(Q_{j}\mathbf{\alpha}_{j}^{(t-1)},\Gamma_{j})
$$
respectively, where the diagonal elements of $R_i,  (i = \eta, 1, 2, \ldots, m )$ and $Q_j, (j = 1,2,\ldots,p)$ are between $-1$ and $1$, and the off-diagonal elements are $0$.


In the absence of any other information, the variables of interest evolve according to a random walk,
$$
\mathbf{y}^{(t)}\sim{}N(\mathbf{y}^{(t-1)},\Lambda_y),
$$
with noisy observations,
$$
\mathbf{\hat{y}}^{(t)}\sim{}N(\mathbf{y}^{(t)},\Sigma_y),
$$
when there are observations at time $t$.

## Dynamic linear model
As in the case without drivers, the ensemble model can be written as a dynamical linear model, by letting
\begin{equation}
\mathbf\theta^{(t)}=(\mathbf{y}^{(t)'},\mathbf\eta^{(t)'},\mathbf{z}_1^{(t)'},\ldots,\mathbf{z}_m^{(t)'},\mathbf{\alpha}_1^{(t)'},\ldots,\mathbf{\alpha}_p^{(t)'})',
\end{equation}
so that
\begin{equation}
\mathbf\theta^{(t)}|\mathbf\theta^{(t-1)}\sim{}N(A\mathbf\theta^{(t)},\Lambda),
\end{equation}
with
\begin{equation}
A= 
\begin{pmatrix}
I_d & 0 & 0 & 0 & \ldots & 0 & 0 & 0 & \ldots & 0\\
0 & R_\eta & 0 & 0 & \ldots & 0 & 0 & 0 & \ldots & 0\\
0  & 0 & R_1  & 0 & \ldots & 0 & 0 & 0 & \ldots & 0\\
0  & 0 & 0 & R_2  & \ldots & 0 & 0 & 0 & \ldots & 0\\
\vdots & \vdots & \vdots & \vdots & \ddots & \vdots & \vdots & \vdots & \ldots & \vdots\\
0  &  0 & 0 & 0 & \ldots & R_m & 0 & 0 & \ldots & 0\\
0  &  0 & 0 & 0 & \ldots & 0 & Q_1 & 0 & \ldots & 0\\
0  &  0 & 0 & 0 & \ldots & 0 & 0 & Q_2 & \ldots & 0\\
\vdots & \vdots & \vdots & \vdots & \vdots & \vdots & \vdots & \vdots & \ddots & \vdots\\
0  &  0 & 0 & 0 & \ldots & 0 & 0 & 0 & \ldots & Q_p\\
\end{pmatrix},
\end{equation}
with $I_d$ being a $d$ dimensional indicator matrix,
and
\begin{equation}
\Lambda=
\begin{pmatrix}
\Lambda_y & 0 & 0 & 0 & \ldots & 0 & 0 & 0 & \ldots & 0\\
0 & \Lambda_\eta & 0 & 0 & \ldots & 0 & 0 & 0 & \ldots & 0\\
0  & 0 & \Lambda_1  & 0 & \ldots & 0 & 0 & 0 & \ldots & 0\\
0  & 0 & 0 & \Lambda_2  & \ldots & 0 & 0 & 0 & \ldots & 0\\
\vdots & \vdots & \vdots & \vdots & \ddots & \vdots & \vdots & \vdots & \ldots & \vdots\\
0  &  0 & 0 & 0 & \ldots & \Lambda_m & 0 & 0 & \ldots & 0\\
0  &  0 & 0 & 0 & \ldots & 0 & \Gamma_1 & 0 & \ldots & 0\\
0  &  0 & 0 & 0 & \ldots & 0 & 0 & \Gamma_2 & \ldots & 0\\
\vdots & \vdots & \vdots & \vdots & \vdots & \vdots & \vdots & \vdots & \ddots & \vdots\\
0  &  0 & 0 & 0 & \ldots & 0 & 0 & 0 & \ldots & \Gamma_p\\
\end{pmatrix}.
\end{equation}
The observations
\begin{equation}
\mathbf{w}^{(t)}=(\mathbf{\hat{y}}^{(t)'},\mathbf{\hat{x}}^{(t)'}_{1,1},\ldots,\mathbf{\hat{x}}^{(t)'}_{1,p},\ldots,\mathbf{\hat{x}}^{(t)'}_{m,1},\ldots,\mathbf{\hat{x}}^{(t)'}_{m,p})'
\end{equation}
are 
\begin{equation}
S^{(t)}\mathbf{w}^{(t)}\sim{}N(S^{(t)}B(\mathbf{\theta}^{(t)} + \mathbf\zeta),S^{(t)}\Sigma S^{(t)'})
\label{eq:DLM_obs}
\end{equation}
where
\begin{equation}
B= 
\begin{pmatrix}
I_d & 0 & 0 & 0 & \ldots & 0 & 0 & 0 & \ldots & 0\\
M_{1,1} & M_{1,1} & M_{1,1} & 0  & \ldots & 0 & M_{1,1} & 0 &  \ldots & 0\\
M_{1,2}  & M_{1,2} & M_{1,2} & 0 & \ldots & 0 & 0 & M_{1,2} & \ldots & 0\\
\vdots & \vdots & \vdots & \vdots & \ddots & 0 & \vdots & \vdots & \ddots & \vdots\\
M_{1,p} & M_{1,p} & M_{1,p} & 0 & \ldots & 0 & 0 & 0 & \ldots & M_{1,p}\\
M_{2,1} & M_{2,1} & 0  & M_{2,1} & \ldots & 0 & M_{2,1} & 0 &  \ldots & 0\\
M_{2,2}  & M_{2,2} & 0 & M_{2,2} & \ldots & 0 & 0 & M_{2,2} & \ldots & 0\\
\vdots & \vdots & \vdots & \vdots & \ddots & 0 & \vdots & \vdots & \ddots & \vdots\\
M_{2,p} & M_{2,p} & 0 & M_{2,p} & \ldots & 0 & 0 & 0 & \ldots & M_{2,p}\\
\vdots & \vdots & \vdots & \vdots & \ddots & \vdots & \vdots & \vdots & \ddots & \vdots\\
M_{m,1} & M_{m,1} & 0 & 0 & \ldots & M_{m,1} & M_{m,1} & 0 & \ldots & 0\\
M_{m,2} & M_{m,2} & 0 & 0 & \ldots & M_{m,2} & 0 & M_{m,2} & \ldots & 0\\
\vdots & \vdots & \vdots & \vdots & \ddots & \vdots & \vdots & \vdots & \ddots & \vdots\\
M_{m,p} & M_{m,p} & 0 & 0 & \ldots & M_{m,p} & 0 & 0 & \ldots & M_{m,p}\\
\end{pmatrix},
\end{equation}
\begin{equation}
\mathbf\zeta=(\mathbf{0}',\mathbf{\delta}',\mathbf\gamma'_1,\ldots,\mathbf\gamma'_m,\mathbf\iota'_1,\ldots,\mathbf\iota'_p)',
\end{equation}
with
$\mathbf{0}$ being a $d$ dimensional vector of 0s,
\begin{equation}
\Sigma=
\begin{pmatrix}
\Sigma_y & 0 &\ldots & 0 & 0 & \ldots & 0\\
0 & \Sigma_{1,1} & \ldots & 0 & 0 & \ldots & 0\\
\vdots & \vdots  & \ddots & \vdots & \vdots & \ddots & \vdots \\
0  &  0  & \ldots & \Sigma_{1,p} & 0 & \ldots & 0 \\
0 & 0 & \ldots & 0 & \Sigma_{2,1} & \ldots & 0\\
\vdots & \vdots & \ddots & \vdots  & \vdots & \ddots & \vdots \\
0 & 0 & \ldots & 0 & 0 & \ldots & \Sigma_{m,p}
\end{pmatrix}
\end{equation}
and $S^{(t)}$ is a matrix that is used to describe which variables are observed at time $t$. The $S^{(t)}$ matrix is an $r(t) \times \left(d + \sum_{k=1}^m pn_k\right)$ matrix, where $r(t)$ is the total number of observations and simulator/driver outputs available at time $t$, with each element given by 
$$
S^{(t)}_{i, j} = 
\begin{cases}
  1 & \mathrm{if\ the \ }i^{th} \mathrm{\ available\ observation\ /\ output\ is\ the\ }j^{th}\mathrm{\ of\ all\ possible\ observation\ /\ outputs.} \\
  0 & \mathrm{otherwise}
\end{cases}
$$

For example, if all observations and the simulators/drivers give an output at time $t$ then $S^{(t)}=I_{d+\sum_{k=1}^{m}pn_k}$. If all simulators/drivers output the variables of interest but there are no observations then $S^{(t)}$ is a  $\left(\sum_{k=1}^m pn_k\right) \times \left(d + \sum_{k=1}^m pn_k\right)$ matrix, equal to $I_{d+\sum_{k=1}^{m}pn_k}$ with the first $d$ rows removed.

## Demonstration on synthetic data
Here we generate synthetic data for simulator/driver outputs and observations, then fit the ensemble model using these data. We do this for an ensemble model of 3 simulators and 2 drivers, each describing 3 variables of interest (which we imagine to be species biomass).

## Generating the data
We generate the synthetic data by sampling from an informative prior distribution of the ensemble model. We choose  priors with strong correlations of individual long-term discrepancies ($\mathrm{Beta}(5,1)$ via the method of concordance) and high autocorrelations of the AR processes on individual and shared short-term discrepancies ($\frac{r_i + 1}{2}\sim \mathrm{Beta}(10,2)$). These are different to the vague priors that will later be used to fit the model. We can configure this prior using the `EnsemblePrior()` constructor function.
```{r configure_priors, eval = FALSE}
d <- 3 # The number of variables.

priors <- EnsemblePrior(d,
                        ind_st_params = IndSTPrior(AR_params=c(10,2)), 
                        ind_lt_params = IndLTPrior("beta",
                                                   cor_params = list(matrix(5, d, d), 
                                                                     matrix(1, d, d)),var_params = list(1,0.5)),
                        sha_st_params = ShaSTPrior(AR_params=c(10,2)),
)
```

From this configuration, sampling the ensemble model parameters is done using the `prior_ensemble_model()` function. In this case we sample the prior of an ensemble model with $m_{1} = 3$ simulators and $m_{2} = 2$ drivers.

```{r sampling priors, eval = FALSE}
M <- 3 # The number of simulators we are considering.
MM <- 2 # The number of drivers we are considering.
prior_density <- prior_ensemble_model(priors, M = M, MM = MM)
ex.fit <- rstan::extract(prior_density$samples) # Extracted samples
```

Of these samples, the data we choose to make up our synthetic data is the single sample which maximizes the likelihood (see above section). The samples from the prior distribution provide us with $A$ and $\Lambda$ values. To get from these ensemble model parameters to simulated model outputs requires using the above transformations. Values of the truth and discrepancy terms are generated using these values and sums of discrepancy terms to give the "best guesses" of simulator/driver combined outputs.


```{r truth_best_guesses, eval = FALSE}
Time <- 50

true_par <-which.max(ex.fit$lp__)

latent <- matrix(NA, Time, (M+MM+2)*d)

#Priors on initial values
latent[1, 1:d] <- rnorm(d, 0, 1)
latent[1, -(1:d)] <- t(chol(ex.fit$SIGMA_init[true_par,-(1:d) ,-(1:d)])) %*% rnorm((M+MM+1)*d, 0, 1)

#Find all the latent values of the dynamic linear model
SIGMA <- ex.fit$SIGMA[true_par,,]
SIGMA_chol <- t(chol(SIGMA))
for (t in 2:Time) {
  latent[t,] <- ex.fit$AR_params[true_par,] * latent[t-1,] + SIGMA_chol %*% rnorm((M+MM+2)*d, 0, 1)
}

#The truth is simply the first d values
truth_latent <- latent[,1:d]

#The best guesses are sums of the truth and discrepancies
simulator_best_guesses <- array(NA,dim=c(Time,d,M*MM))
for(i in 1:M){
  for (j in 1:MM){
    simulator_best_guesses[,,MM*(i-1)+j] <- t(
      t(latent[,1:d] + latent[,(d+1):(2*d)] + latent[,(i + 1)*d + (1:d)] + latent[,(M + j + 1)*d + (1:d)]) + 
        ex.fit$ind_lt[true_par,i,] + ex.fit$ind_lt_dri[true_par,j,] + ex.fit$sha_lt[true_par,])
  }
}
```

We can see how the truth relates to simulator/driver best guesses by plotting these outputs, for example for the first variable.

```{r plot_data, eval = FALSE}
plotlist <- list()
for (sim in 1:M){
  for (dri in 1:MM){
    plotlist[[MM*(sim-1)+dri]] <- cbind(c(1:Time),simulator_best_guesses[,1,MM*(sim-1)+dri],rep(sim,Time),rep(dri,Time),rep("sim",Time))
  }
}
plot_df <- data.frame(rbind(do.call(rbind, plotlist), cbind(c(1:Time), truth_latent[,1], rep(0,Time), rep(0,Time), rep("truth",Time))))
names(plot_df) <- c("Year", "Value", "Simulator", "Driver", "Type")
plot_df$Year <- as.numeric(plot_df$Year)
plot_df$Value <- as.numeric(plot_df$Value)
bgplot <- ggplot2::ggplot(plot_df) + geom_line(data = plot_df[which(plot_df$Type == "sim"),], aes(x = Year, y = Value, color = Simulator, linetype = Driver), linewidth = 0.8) + geom_line(data = plot_df[which(plot_df$Type == "truth"),], aes(x = Year, y = Value), linewidth = 1, color = "purple")
```

```{r, echo = FALSE, out.width="500px", out.height="500px"}
knitr::include_graphics("data/bgplot.png")
```

Note in the above that the purple line represents the truth. In `EcoEnsemble`, we do not assume that we have access to the truth, nor the best guesses of each simulator/driver combination. Instead, `EcoEnsemble` assumes that simulator/driver outputs are noisy representations of the best guesses, and that observations are noisy representations of the truth. In order to produce the final synthetic dataset, we add noise to the simulated truth and best guesses. For simulators/drivers, this noise represents parameter uncertainty.

We use gams from the `mgcv` package to achieve this, and we overparametrise the model (by setting $k$ - the dimension of the basis used to represent the smooth term - to be something very large) to ensure that the outputs are sufficiently smooth.

In addition, we assume that we only have access to the first 40 years of observations.

```{r add_noise, eval = FALSE}
Times_obs <- round(Time * 0.8)
obs <- matrix(NA,Times_obs,d)
for(i in 1:d){
  g1 <- gam(y~s(x,k = 15),data=data.frame(x=1:Times_obs,y = truth_latent[1:Times_obs,i]))
  obs[,i] <- predict(g1)
}

obs.cov <- cov(obs - truth_latent[1:Times_obs,])


model.cov <- array(0,dim=c(M*MM,d,d))
models_output <- array(NA,dim=c(M*MM,Time,d))
for (j in 1:(M*MM)){
  for(i in 1:d){
    g1 <- gam(y~s(x, k = 15),data=data.frame(x=1:Time,y=simulator_best_guesses[,i,j]))
    models_output[j,,i] <- predict(g1)
  }
  model.cov[j,,] <- cov(models_output[j,,] - simulator_best_guesses[,,j])
}
```

We can see how this data compares to the best guesses by again plotting the first variable of interest.
```{r plot_obs, eval = FALSE}
#Observations and model outputs
plotlist <- list()
for (sim in 1:M){
  for (dri in 1:MM){
    plotlist[[MM*(sim-1)+dri]] <- cbind(c(1:Time),models_output[MM*(sim-1)+dri,,1],rep(sim,Time),rep(dri,Time),rep("sim",Time),rep("observed", Time))
  }
}
plot_df <- data.frame(rbind(do.call(rbind, plotlist), cbind(c(1:Time), c(obs[,1],rep(0, Time - max(Times_obs))), rep(0,Time), rep(0,Time), rep("truth",Time), c(rep("observed", max(Times_obs)),rep("unobserved", Time - max(Times_obs))))))
names(plot_df) <- c("Year", "Value", "Simulator", "Driver", "Type", "Obs_Status")
plot_df$Year <- as.numeric(plot_df$Year)
plot_df$Value <- as.numeric(plot_df$Value)
obsplot <- ggplot(plot_df) + geom_line(data=plot_df[which(plot_df$Type == "sim"),], aes(x = Year, y = Value, color = Simulator, linetype = Driver), linewidth = 0.8) + geom_point(data = plot_df[intersect(which(plot_df$Type == "truth"),which(plot_df$Obs_Status == "observed")),], aes(x = Year, y = Value), color = "purple") 
```

```{r, echo = FALSE, out.width="500px", out.height="500px"}
knitr::include_graphics("data/obsplot.png")
```

In the above graph, the purple points represent the observations, while the coloured and solid/dashed lines represent the simulator and driver outputs respectively.

## Fitting to Synthetic Data
We now need to convert the observations and simulator/driver outputs into the required form for `EcoEnsemble`. This requires that input columns and rows are named consistently with years and species so that data from different models can be matched to each other, and missing data accounted for. We create data frames for each simulator/driver combination and assign names.


```{r, eval = FALSE}
#Create the data frames that we'll use for EcoEnsemble
val_obs <- data.frame(obs); cov_obs <- obs.cov
val_model_11 <- data.frame(models_output[1,,]); cov_model_11 <- model.cov[1,,]
val_model_12 <- data.frame(models_output[2,,]); cov_model_12 <- model.cov[2,,]
val_model_21 <- data.frame(models_output[3,,]); cov_model_21 <- model.cov[3,,]
val_model_22 <- data.frame(models_output[4,,]); cov_model_22 <- model.cov[4,,]
val_model_31 <- data.frame(models_output[5,,]); cov_model_31 <- model.cov[5,,]
val_model_32 <- data.frame(models_output[6,,]); cov_model_32 <- model.cov[6,,]

#Set the dimnames to ensure EcoEnsemble can identify the information.
SPECIES_NAMES <- c("Species 1", "Species 2", "Species 3")
dimnames(val_obs) <- list(paste(1:Times_obs), SPECIES_NAMES)
dimnames(val_model_11) <- list(paste(1:Time), SPECIES_NAMES)
dimnames(val_model_21) <- list(paste(1:Time), SPECIES_NAMES)
dimnames(val_model_12) <- list(paste(1:Time), SPECIES_NAMES)
dimnames(val_model_22) <- list(paste(1:Time), SPECIES_NAMES)
dimnames(val_model_31) <- list(paste(1:Time), SPECIES_NAMES)
dimnames(val_model_32) <- list(paste(1:Time), SPECIES_NAMES)

dimnames(cov_obs) <- list(SPECIES_NAMES, SPECIES_NAMES)
dimnames(cov_model_11) <- list(SPECIES_NAMES, SPECIES_NAMES)
dimnames(cov_model_21) <- list(SPECIES_NAMES, SPECIES_NAMES)
dimnames(cov_model_12) <- list(SPECIES_NAMES, SPECIES_NAMES)
dimnames(cov_model_22) <- list(SPECIES_NAMES, SPECIES_NAMES)
dimnames(cov_model_31) <- list(SPECIES_NAMES, SPECIES_NAMES)
dimnames(cov_model_32) <- list(SPECIES_NAMES, SPECIES_NAMES)

val_model_1 <- list(val_model_11, val_model_12)
val_model_2 <- list(val_model_21, val_model_22)
val_model_3 <- list(val_model_31, val_model_32)
cov_model_1 <- list(cov_model_11, cov_model_12)
cov_model_2 <- list(cov_model_21, cov_model_22)
cov_model_3 <- list(cov_model_31, cov_model_32)
```

Now we can fit the ensemble model, this time using vague default priors.
```{r fitting, eval = FALSE}
fit <- fit_ensemble_model(observations = list(val_obs, cov_obs),
                                             simulators = list(list(val_model_1, cov_model_1, "Simulator 1", c("Driver 1", "Driver 2")),
                                                               list(val_model_2, cov_model_2, "Simulator 2", c("Driver 1", "Driver 2")),
                                                               list(val_model_3, cov_model_3, "Simulator 3", c("Driver 1", "Driver 2"))
                                             ),
                                             priors = EnsemblePrior(d),
                                             control = list(adapt_delta = 0.9),drivers=T)
# Switch to the Kalman filter sampler with `sampler = "kalman"` if desired
fit_kalman <- fit_ensemble_model(observations = list(val_obs, cov_obs),
                                 simulators = list(list(val_model_1, cov_model_1, "Simulator 1", c("Driver 1", "Driver 2")),
                                                   list(val_model_2, cov_model_2, "Simulator 2", c("Driver 1", "Driver 2")),
                                                   list(val_model_3, cov_model_3, "Simulator 3", c("Driver 1", "Driver 2"))
                                 ),
                                 priors = EnsemblePrior(d),
                                 control = list(adapt_delta = 0.9),
                                 drivers = TRUE,
                                 sampler = "kalman")
samples <- generate_sample(fit)
```
## Results

We can compare the truth as predicted by the ensemble model with the original truth value.
```{r results, eval = FALSE}
df_list <- list()
for (var_index in 1:3){
  df <- data.frame("Year" = paste(1:Time),
                   "Ensemble" = apply(samples@mle[, var_index, ], 1, median),
                   "Lower"  = apply(samples@samples[, var_index, ], 1, quantile, 0.1),
                   "Upper"  = apply(samples@samples[, var_index, ], 1, quantile, 0.9),
                   "Actual" = truth_latent[,var_index])
  df$Year <- as.numeric(df$Year)
  df <-  reshape2::melt(df, id.vars=c("Year", "Lower", "Upper"), variable.name="Simulator")
  # We only want uncertainty bands for the Ensemble values, else we set zero width bands 
  df[df$Simulator == "Actual", c("Lower", "Upper")] <- df[df$Simulator != "Actual", "value"]
  df$Species = rep(SPECIES_NAMES[var_index], Time)
  df_list[[var_index]] <- ggplot(df, aes(x=`Year`, y=`value`)) +
  geom_line(aes(group=`Simulator`,colour=`Simulator`)) +
  geom_ribbon(aes(ymin=`Lower`, ymax =`Upper`, fill = `Simulator`), alpha=0.2) + ggplot2::ggtitle(SPECIES_NAMES[var_index]) + ggplot2::theme(legend.position = "right")
}

legend <- cowplot::get_plot_component(df_list[[1]], "guide-box-right")
df_list <- lapply(df_list, function(x) {x + ggplot2::theme(legend.position = "none")})
output_plot_nolegend <- cowplot::plot_grid(plotlist=df_list, nrow = 3, ncol = 1)
output_plot <- cowplot::plot_grid(output_plot_nolegend, legend, rel_widths = c(4,1), nrow = 1, ncol = 2)
```

```{r plot_truth, echo = FALSE, out.width="600px", out.height="600px"}
knitr::include_graphics("data/output_plot.png")
```

Here we see that, with the chosen priors, the ensemble model does a reasonable job at capturing the true value. 

## Checking the fit

To check that our ensemble estimates above are valid we can run `rstan`'s built-in MCMC diagnostics (@rstan). These include the $\hat{R}$ statistic and the bulk and tail effective sample size. The $\hat{R}$ diagnostic compares the variances of the between and within-chain estimates for model parameters. As these should be similar, an $\hat{R}$ value of below 1.05 is usually indicative that the chains have mixed well. Note that we remove `NA` values as these indicate non-moving chains, which is expected for some parameters such the diagonal entries of the correlation matrix `ind_st_cor` (identically equal to 1).

```{r, eval = FALSE}
samples_array <- as.array(fit@samples)
#Drop last element of third dimension as this is "lp__"
max(apply(samples_array[,,-dim(samples_array)[3]], 3, rstan::Rhat), na.rm = TRUE)
min(apply(samples_array[,,-dim(samples_array)[3]], 3, rstan::ess_bulk), na.rm = TRUE)
min(apply(samples_array[,,-dim(samples_array)[3]], 3, rstan::ess_tail), na.rm = TRUE)
```

```{r,echo=FALSE}
1.007311; 836.737; 559.6022
```

We also wish to check that `rstan`'s NUTS sampler has not encountered too many divergent transitions. This is achieved by running the `check_divergences` function.

```{r, eval = FALSE}
rstan::check_divergences(fit@samples)
```

```{r, echo = FALSE}
message(sprintf("%s of %s iterations ended with a divergence (%s%%).", 
            5, 4000, 100 * 5/4000), "\nTry increasing 'adapt_delta' to remove the divergences.")
```

This is a low number of divergences and does not indicate any issues with our posterior sample. We can also produce trace plots of some of the parameters to check mixing. For example, the plot below is the trace plot of the post burn-in iterations of `ind_st_sd[3,2]`, the short-term standard deviation of the 3rd simulator's best guess for the 2nd variable of interest.

```{r, eval = FALSE, fig.dim = c(7,4)}
rstan::traceplot(fit@samples, pars = "ind_st_sd[3,2]")
```

```{r, echo = FALSE, out.width = "500px", out.height = "300px"}
knitr::include_graphics("data/drivers_trace.png")
```

The four chains are visually mixing well for this parameter.

# References

---
title: "EcoEnsemble"
subtitle: "An R package for combining ecosystem models"
author: "James A. Martindale and Michael A. Spence"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{EcoEnsemble}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}#
references:
- id: spence_ff
  title: A general framework for combining ecosystem models.
  author:
  - family: Spence
    given: Michael A.
  - family: Blanchard
    given: Julia L.
  - family: Rossberg
    given: Axel G.
  - family: Heath
    given: Michael R.
  - family: Heymans
    given: J. J.
  - family: Mackinson
    given: Steven
  - family: Serpetti
    given: Natalia
  - family: Speirs
    given: Douglas C.
  - family: Thorpe
    given: Robert B.
  - family: Blackwell
    given: Paul G.
  container-title: Fish and Fisheries
  volume: 19
  page: 1013-1042
  type: article-journal
  issued:
    year: 2018
- id: chandler
  title: "Exploiting strength, discounting weakness: combining information from multiple climate simulators"
  author:
  - family: Chandler
    given: Richard E.
  container-title: "Philosophical Transactions of the Royal Society A: Mathematical, Physical and Engineering Sciences"
  volume: 371
  issue: 1991 
  type: article-journal
  issued: 
    year: 2013
- id: zhang
  title: Information Uncertainty and Stock Returns
  author:
  - family: Zhang
    given: X. Frank
  container-title: The Journal of Finance
  volume: 61
  page: 105-137 
  type: article-journal
  issued:
    year: 2006
- id: rougier_13
  title: "Model and data limitations: the sources and implications of epistemic uncertainty"
  author:
  - family: Rougier
    given: J. C.
  - family: Beven
    given: K. J.
  publisher: Cambridge University Press
  page: 40-63
  type: book
  issued:
    year: 2013
- id: Rougier_16
  title: Ensemble Averaging and Mean Squared Error
  author:
  - family: Rougier
    given: J. C. 
  container-title: Journal of Climate
  volume: 29
  page: 8865-8870
  type: article-journal
  issued:
    year: 2016
- id: banner_higgs
  title: Considerations for assessing model averaging of regression coefficients
  author:
  - family: Banner
    given: Katharine M.
  - family: Higgs
    given: Megan D.
  container-title: Ecological Applications
  volume: 27
  page: 78-93
  type: article-journal
  issued:
    year: 2017
- id: Dormann_18
  title: "Model averaging in ecology: a review of Bayesian, information-theoretic, and tactical approaches for predictive inference"
  author:
  - family: Dormann, 
    given: Carsten F.
  - family: Calabrese
    given: Justin M. 
  - family: Guillera-Arroita
    given: Gurutzeta 
  - family: Matechou
    given: Eleni 
  - family: Bahn
    given: Volker 
  - family: Barton
    given: Kamil 
  - family: Beale
    given: Colin M. 
  - family: Ciuti
    given: Simone 
  - family: Elith
    given: Jane 
  - family: Gerstner
    given: Katharina 
  - family: Guelat
    given: Jerome 
  - family: Keil
    given: Petr 
  - family: Lahoz-Monfort
    given: Jose J. 
  - family: Pollock
    given: Laura J. 
  - family: Reineking
    given: Björn 
  - family: Roberts
    given: David R. 
  - family: Schrøder
    given: Boris 
  - family: Thuiller
    given: Wilfried 
  - family: Warton
    given: David I. 
  - family: Wintle
    given: Brendan A. 
  - family: Wood
    given: Simon N. 
  - family: Wuest
    given: Rafael O. 
  - family: Hartig
    given: Florian
  container-title: Ecological Monographs
  volume: 88
  page: 485-504
  type: article-journal
  issued:
    year: 2018
- id: mackinson
  title: Evaluating the fishery and ecological consequences of the proposed North Sea multi-annual plan
  author:
  - family: Mackinson
    given: Steven 
  - family: Platts
    given: Mark 
  - family: Garcia
    given: Clement
  - family: Lynam
    given: Christopher
  container-title: PLOS ONE
  volume: 13
  page: 1-23
  type: article-journal
  issued:
    year: 2018
- id: thorpe15
  title: Evaluation and management implications of uncertainty in a multispecies size-structured model of population and community responses to fishing
  author:
  - family: Thorpe
    given:  Robert B. 
  - family: Le Quesne
    given:  Will J. F. 
  - family: Luxford
    given:  Fay 
  - family: Collie
    given:  Jeremy S. 
  - family: Jennings
    given:  Simon
  container-title: Methods in Ecology and Evolution
  volume: 6
  page: 49-58
  type: article-journal
  issued:
    year: 2015
- id: blanchard
  title: Evaluating targets and trade-offs among fisheries and conservation objectives using a multispecies size spectrum model
  author: 
   - family: Blanchard
     given: Julia L.
   - family: Andersen
     given: Ken H.
   - family: Scott
     given: Finlay
   - family: Hintzen
     given: Niels T.
   - family: Piet
     given: Gerjan
   - family: Jennings
     given: Simon

  container-title: Journal of Applied Ecology
  volume: 51
  page: 612-622
  type: article-journal 
  issued: 
    year: 2014
- id: Collie_16
  title: "Ecosystem models for fisheries management: finding the sweet spot"
  author: 
   - family: Collie
     given: Jeremy S
   - family: Botsford
     given: Louis W
   - family: Hastings
     given: Alan
   - family: Kaplan
     given: Isaac C
   - family: Largier
     given: John L
   - family: Livingston
     given: Patricia A
   - family: Plaganyi
     given: Eva
   - family: Rose
     given: Kenneth A
   - family: Wells
     given: Brian K
   - family: Werner
     given: Francisco E
  container-title: Fish and Fisheries
  page: 101-125
  type: article-journal 
  issued: 
   year: 2016
- id: Essington_13
  title: "Pitfalls and guidelines for recycling models for ecosystem-based fisheries management: evaluating model suitability for forage fish fisheries"
  author: 
   - family: Essington
     given: Timothy E.
   - family: Plaganyi
     given: Eva E.
  container-title: ICES Journal of Marine Science
  page: 118-127
  type: article-journal 
  issued: 
   year: 2013
- id: fulton_03
  title: Effect of Complexity of Marine Ecosystem Models
  author: 
   - family: Fulton
     given: Elizabeth
   - family: Smith
     given: A.
   - family: Johnson
     given: Craig
  container-title: Marine Ecology Progress Series
  page: 1-16
  type: article-journal 
  issued: 
    year: 2003
- id: Hart_fay
  title: Applying tree analysis to assess combinations of Ecosystem-Based Fisheries Management actions in Management Strategy Evaluation
  author: 
   - family: Hart
     given: Amanda R.
   - family: Fay
     given: Gavin
  container-title: Fisheries Research
  page: 105466
  type: article-journal 
  issued: 
    year: 2020
- id: Gaichas_08
  title: "A context for ecosystem-based fishery management: Developing concepts of ecosystems and sustainability"
  author: 
   - family: Gaichas
     given: Sarah K.
  container-title: Marine Policy
  page: 393 - 401
  type: article-journal 
  issued:
    year: 2008
- id: fishsums
  title: "Modelling the effects of fishing on the North Sea fish community size composition"
  author:
  - family: Speirs
    given: Douglas C. 
  - family: Greenstreet
    given: Simon P.R.
  - family: Heath
    given: Michael R.
  container-title: Ecological Modelling
  volume: 321
  page: 35--45
  type: article-journal 
  issued: 
    year: 2016
- id: kennedy_ohagan
  title: Bayesian calibration of computer models
  author: 
   - family: Kennedy
     given: Marc C.
   - family: "O'Hagan"
     given: Anthony
  container-title: "Journal of the Royal Statistical Society: Series B (Statistical Methodology)"
  volume: 63
  page: 425-464
  type: article-journal 
  issued: 
    year: 2001
- id: ices_hawg
  title: "Herring Assessment Working Group for the Area South of 62 N (HAWG)}"
  author: 
    - family: ICES
  issued:
    year: 2020
  type: report
  container-title: "ICES Scientific Reports. 2:60. 1151 pp, ICES, Copenhagen"
  doi: "10.17895/ices.pub.6105"
- id: ices_ns
  title: "Report of the Working Group on the Assessment of Demersal Stocks in the North Sea and Skagerrak"
  author: 
    - family: ICES
  issued:
    year: 2020
  type: report
  container-title: "ICES Scientific Reports. 2:61. 1353, ICES, Copenhagen"
  doi: "10.17895/ices.pub.6092"
- id: ices_ewe
  title: "Working Group on Multispecies Assessment Methods (WGSAM)"
  author: 
    - family: ICES
  issued:
    year: 2016
  type: report
- id: spence_ns
  title: Parameter uncertainty of a dynamic multispecies size spectrum model
  author: 
   - family: Spence
     given: Michael A.
   - family: Blackwell
     given: Paul G.
   - family: Blanchard
     given: Julia L.
  container-title: Canadian Journal of Fisheries and Aquatic Sciences
  volume: 73
  page: 589-597
  type: article-journal 
  issued: 
    year: 2016
- id: ggplot
  author:
   - family: Wickham
     given: Hadley
  title: "ggplot2: Elegant Graphics for Data Analysis"
  publisher: Springer-Verlag New York
  issued: 
    year: 2016
  type: book
  isbn: "978-3-319-24277-4"
  url: "https://ggplot2.tidyverse.org"
  
- id: rstan
  author:
  - family: "Stan Development Team"
  title: "RStan: the R interface to Stan"
  issued: 
    year: 2020
  url: "https://mc-stan.org/"
  type: article
---
```{r include=FALSE}
#Load in the data and load the library
library(EcoEnsemble)
#load("data/vignette_plots.Rdata")
```

# Introduction
When managing ecosystems, management decisions are often informed by outputs of complex ecosystem models (henceforth referred to as simulators). There are often several such simulators available, each providing different outputs with potentially different implications for management actions. While some simulators are better at capturing some aspects of the ecosystem than others, in general no simulator is uniformly better than the others (@chandler), and often the average of simulators outperforms all of the simulators individually (@Rougier_16). 

Management decisions on the basis of a single simulator are therefore very sensitive to the choice of simulator ( @Collie_16, @Essington_13, @fulton_03, @Hart_fay, @Gaichas_08) and by ignoring other available simulators, we limit the amount of information utilised and increase our uncertainty (@spence_ff), often to a greater extent than we formally recognise.

Instead of choosing one simulator, it is possible to combine them using an ensemble model. However, many such methods, such as weighting schemes (e.g. Bayesian model averaging, @banner_higgs, @Dormann_18), explicitly assume that one of the simulators correctly describes the truth (@chandler). Not only is this a strong assumption, but treating a suite of simulators as containing the true model tends to underestimate the uncertainty because alternative, possibly as yet undeveloped, simulators could give predictions outside of the current range (@chandler). Under this scheme, adding a new simulator to the suite could increase the uncertainty of the ensemble despite the additional information contained in the new model (@Dormann_18). For an ensemble model with a robust quantification of uncertainty, uncertainty should decrease as simulators are added (or at worst stay the same).


@spence_ff developed an ensemble model treating individual simulators as exchangeable and drawn from a population of possible simulators. They separated each individual simulator's discrepancy into a discrepancy shared between the simulators and discrepancy that is specific to the simulator. This scheme  allows simulator discrepancies that are not independent, which is expected since individual simulator are often similar, having similar processes, forcing inputs or fitted using similar data (@rougier_13). By modelling in this way, the ensemble model exploits the strengths whilst discounting the weaknesses of each individual simulator (@chandler), whilst robustly quantifying the uncertainty allowing for more accurate advice.

`EcoEnsemble` is an `R` package that implements the ensemble model of @spence_ff when the simulators either output the quantity of interest or not.

# Model description

At time $t$, the true quantity of interest for $d$ variables, $\mathbf{y}^{(t)}=(y_1^{(t)},\ldots,y_d^{(t)})'$, is described by $m$ simulators, $\mathbf{\hat{x}}_{k}^{(t)}=(\hat{x}_{k,1}^{(t)},\ldots,\hat{x}_{k,n_k}^{(t)})'$, with $n_k$ outputs each relating to one of the variables of interest, for $k=1,\ldots,m$, and noisy observations of the variables of interest, $\hat{\mathbf{y}}^{(t)}=(\hat{y}_1^{(t)},\ldots,\hat{y}_d^{(t)})'$.

Not all of the simulators output all the variables over the whole time period. 
To accommodate these differences, @spence_ff introduced a latent variable, known as the "best guess", $\mathbf{x}_{k}^{(t)}=({x}_{1}^{(t)},\ldots,{x}_{d}^{(t)})'$, which represents simulator $k$'s output if it described all $d$ variables at time $t$ with no parameter uncertainty. In `EcoEnsemble`, variables of interest are either present or absent in each simulator, therefore if the $k$th simulator was evaluated at time $t$, its output is
$$
\mathbf{\hat{x}}_{k}^{(t)}\sim{}N(M_k^{(t)}\mathbf{x}_{k}^{(t)},\Sigma_k),
$$
where $M_k^{(t)}$ is an $n_k\times{}d$ matrix and $\Sigma_k$ reflects the parameter uncertainty of the $k$th simulator.

The true values of the variables of interest at time $t$ is simulator $k$'s "best guess" plus a discrepancy term, $\mathbf{\zeta}^{(t)}_k$, @kennedy_ohagan, i.e. 
$$
\mathbf{y}^{(t)} =  \mathbf{x}_{k}^{(t)} + \mathbf{\zeta}^{(t)}_k.
$$
The discrepancy term, $\mathbf{\zeta}^{(t)}_k$, is split between discrepancies that are shared between all of the simulators, and discrepancies that were specific to the $k$th simulator. These two discrepancies are further split into fixed discrepancies, the long-term shared discrepancy, $\mathbf\delta$, and simulator $k$'s long-term individual discrepancy, $\mathbf\gamma_k$, and dynamic discrepancies, the short-term shared discrepancy, $\mathbf\eta^{(t)}$, and simulator $k$'s short-term individual discrepancy, $\mathbf{z}_k^{(t)}$, i.e.
$$
\mathbf{\zeta}^{(t)}_k = \mathbf\delta + \mathbf\eta^{(t)} + \mathbf\gamma_k + \mathbf{z}_k^{(t)}.
$$
The long-term individual discrepancy for the $k$th simulator is
$$
    \mathbf{\gamma}_k\sim{}N(\mathbf{0},C_{\gamma}).
$$
The short-term discrepancy terms, $\mathbf\eta^{(t)}$ and $\mathbf{z}_k^{(t)}$, follow an auto-regressive processes of order one,
$$
\mathbf{\eta}^{(t)}\sim{}N(R_{\eta}\mathbf{\eta}^{(t-1)},\Lambda_{\eta})
$$
and
$$
\mathbf{z}_{k}^{(t)}\sim{}N(R_{k}\mathbf{z}_{k}^{(t-1)},\Lambda_{k})
$$
respectively, where the diagonal elements of $R_i,  (i = \eta, 1, 2, \ldots, m )$ are between $-1$ and $1$, and the off-diagonal elements are $0$.


In the absence of any other information, the true variables evolve according to a random walk,
$$
\mathbf{y}^{(t)}\sim{}N(\mathbf{y}^{(t-1)},\Lambda_y),
$$
with noisy observations,
$$
\mathbf{\hat{y}}^{(t)}\sim{}N(\mathbf{y}^{(t)},\Sigma_y),
$$
when there are observations at time $t$.

## Dynamic linear model
The ensemble model can be written as a dynamical linear model, by letting
\begin{equation}
\mathbf\theta^{(t)}=(\mathbf{y}^{(t)'},\mathbf\eta^{(t)'},\mathbf{z}_1^{(t)'},\ldots,\mathbf{z}_m^{(t)'})',
\end{equation}
so that
\begin{equation}
\mathbf\theta^{(t)}|\mathbf\theta^{(t-1)}\sim{}N(A\mathbf\theta^{(t)},\Lambda),
\end{equation}
with
\begin{equation}
A= 
\begin{pmatrix}
I_d & 0 & 0 & 0 & \ldots & 0\\
0 & R_\eta & 0 & 0 & \ldots & 0\\
0  & 0 & R_1  & 0 & \ldots & 0\\
0  & 0 & 0 & R_2  & \ldots & 0\\
\vdots & \vdots & \vdots & \vdots & \ddots & \vdots \\
0  &  0 & 0 & 0 & \ldots & R_m
\end{pmatrix},
\end{equation}
with $I_d$ being a $d$ dimensional indicator matrix,
and
\begin{equation}
\Lambda=
\begin{pmatrix}
\Lambda_y & 0 & 0 & 0 & \ldots & 0\\
0 & \Lambda_\eta & 0 & 0 & \ldots & 0\\
0  & 0 & \Lambda_1  & 0 & \ldots & 0\\
0  & 0 & 0 & \Lambda_2  & \ldots & 0\\
\vdots & \vdots & \vdots & \vdots & \ddots & \vdots \\
0  &  0 & 0 & 0 & \ldots & \Lambda_m
\end{pmatrix}.
\end{equation}
The observations
\begin{equation}
\mathbf{w}^{(t)}=(\mathbf{\hat{y}}^{(t)'},\mathbf{\hat{x}}^{(t)'}_1,\ldots,\mathbf{\hat{x}}^{(t)'}_m)'
\end{equation}
are 
\begin{equation}
S^{(t)}\mathbf{w}^{(t)}\sim{}N(S^{(t)}B(\mathbf{\theta} + \mathbf\zeta),S^{(t)}\Sigma^{(t)}S^{(t)'})
\label{eq:DLM_obs}
\end{equation}
where
\begin{equation}
B= 
\begin{pmatrix}
I_d & 0 & 0 & 0 & \ldots & 0\\
M_1 & M_1 & M_1  & 0 & \ldots & 0\\
M_2  & M_2 & 0 & M_2  & \ldots & 0\\
\vdots & \vdots & \vdots & \vdots & \ddots & \vdots \\
M_m  &  M_m & 0 & 0 & \ldots & M_m
\end{pmatrix},
\end{equation}
\begin{equation}
\mathbf\zeta=(\mathbf{0}',\mathbf{\delta}',\mathbf\gamma'_1,\ldots,\mathbf\gamma'_m)',
\end{equation}
with
$\mathbf{0}$ being a $d$ dimensional vector of 0s,
\begin{equation}
\Sigma=
\begin{pmatrix}
\Sigma_y & 0  & \ldots & 0\\
0 & \Sigma_1 &   \ldots & 0\\
\vdots & \vdots  & \ddots & \vdots \\
0  &  0  & \ldots & \Sigma_m
\end{pmatrix}
\end{equation}
and $S^{(t)}$ is a matrix that is used to describe which variables are observed at time $t$. The $S^{(t)}$ matrix is an $r(t) \times \left(d + \sum_{k=1}^m n_k\right)$ matrix, where $r(t)$ is the total number of observations and simulator outputs available at time $t$, with each element given by 
$$
S^{(t)}_{i, j} = 
\begin{cases}
  1 & \mathrm{if\ the \ }i^{th} \mathrm{\ available\ observation\ /\ model\ output\ is\ the\ }j^{th}\mathrm{\ of\ all\ possible\ observation\ /\ model\ outputs.} \\
  0 & \mathrm{otherwise}
\end{cases}
$$

For example, if all observations and the simulators give an output at time $t$ then $S^{(t)}=I_{d+\sum_{k=1}^{m}n_k}$. If all   simulators output the variables of interest but there are no observations then $S^{(t)}$ is a  $\left(\sum_{k=1}^m n_k\right) \times \left(d + \sum_{k=1}^m n_k\right)$ matrix, equal to $I_{d+\sum_{k=1}^{m}n_k}$ with the first $d$ rows removed.


# Dataset for the North Sea
We demonstrate the ensemble model by investigating what would happen to the spawning stock biomass (SSB) of four species in the North Sea ($d=4$), Norway pout *(Trisopterus esmarkii)*, Atlantic herring *(Clupea harengus)*, Atlantic cod *(Gadus morhua)*, and Common sole *(Solea solea)*. We used four simulators and estimates of SSB from single-species assessments to predict the true SSB from 1984 to 2017, and the SSB from 2018 to 2050 under MSY.


## Observations
Noisy observations of the SSB were taken from single-species stock assessments (@ices_ns, @ices_hawg). The uncertainty is species specific so the off diagonal elements of $\Sigma_y$ are zero and the diagonal elements are calculated from the uncertainty in the stock assessments. These values, and the associated uncertainty are contained in the `SSB_obs` and `Sigma_obs` variables. The `SSB_obs` data frame contains a row for each year, with a column for each species observed. Note that for `EcoEnsemble`, the rows and columns of all observations and model outputs must be named appropriately in order for the data transformations to behave correctly.
```{r label="Observations"}
SSB_obs
Sigma_obs
```

## Simulators

**EcoPath with EcoSim** (EwE) is an ecosystem model with 60 functional groups for the North Sea (@ices_ewe). It outputs all four species from 1991 until 2050, in the variable `SSB_ewe`, meaning that $n_1=4$ and $M_1=I_4$.
The covariance of the EwE's output, $\Sigma_{1}$, was calculated in @mackinson and is stored in the variable `Sigma_ewe`,

```{r label="EwESigma"}
Sigma_ewe
```



**LeMans** is a discrete time length-based model that describes growth and predation (@thorpe15). It outputs all four species from 1986 until 2050, in the variable `SSB_lm`, meaning that $n_2=4$ and $M_2=I_4$.
The covariance of the LeMans' output, $\Sigma_{2}$, was calculated in @thorpe15 and is stored in the variable `Sigma_lm`,

```{r label="lmSigma"}
Sigma_lm
```


**mizer** is a size-based model that describes ontogenetic feeding and growth, mortality, and reproduction driven by size-dependent predation and maturation processes (@blanchard). It outputs all four species from 1984 until 2050, in the variable `SSB_miz`, meaning that $n_3=4$ and $M_3=I_4$.
The covariance of the mizer's output, $\Sigma_{3}$, was calculated in @spence_ns and is stored in the variable `Sigma_miz`,

```{r label="mizSigma"}
Sigma_miz
```



**FishSUMS** is a discrete time length-based model that describes growth, density-dependent mortality, and losses due to fishing and predation by explicitly modelled species, and seasonal reproduction (@fishsums). It outputs Norway pout, Atlantic Herring, and Atlantic Cod (i.e. it does not output Sole), from 1984 until 2050, in the variable `SSB_fs`, meaning that $n_4=3$ and
\begin{equation}
    M_4=
    \begin{pmatrix}
    1 & 0 &  0 & 0\\
    0 & 1  & 0 & 0\\
    0 & 0 & 1 & 0 
    \end{pmatrix}.
\end{equation}


The covariance of the FishSUMS' output, $\Sigma_{4}$, was calculated in @spence_ff and is stored in the variable `Sigma_fs`. Its value is


```{r label="fsSigma"}
Sigma_fs
```

The single-species stock assessments for the 4 species, along with the model outputs are plotted below. 


```{r model_outputs, warning=FALSE, fig.dim = c(7, 3), message = FALSE, echo = FALSE}
library(tibble)
library(dplyr)
library(reshape2)
library(ggplot2)

#Inlcude years in the data frames
SSB_obs_tmp <- rownames_to_column(SSB_obs, var = "Year")
SSB_ewe_tmp <- rownames_to_column(SSB_ewe, var = "Year")
SSB_miz_tmp <- rownames_to_column(SSB_miz, var = "Year")
SSB_lm_tmp <- rownames_to_column(SSB_lm, var = "Year")
SSB_fs_tmp <- rownames_to_column(SSB_fs, var = "Year")

#Join dataframes together
df_all <- SSB_obs_tmp %>% 
    full_join(SSB_ewe_tmp, by = "Year", suffix = c("","_ewe")) %>%
    full_join(SSB_miz_tmp, by = "Year", suffix = c("","_miz")) %>%
    full_join(SSB_lm_tmp, by = "Year", suffix = c("","_lm")) %>%
    full_join(SSB_fs_tmp, by = "Year", suffix = c("","_fs"))

#Melt into long data format for ggplot
df_all <- melt(df_all, id.vars = "Year")
colnames(df_all) <- c("Year", "Simulator", "logSSB")
df_all$Year <- as.numeric(df_all$Year)

#Finally create the plots
df_n_pout <- df_all[grepl("N.pout", df_all$Simulator), ]
p1 <- ggplot(data=df_n_pout, aes(x=`Year`, y=`logSSB`, na.rm = TRUE)) + 
  geom_line(aes(group=`Simulator`,colour=`Simulator`)) +
  ggtitle("Norway pout")

df_herring <- df_all[grepl("Herring", df_all$Simulator), ]
p2 <- ggplot(data=df_herring, aes(x=`Year`, y=`logSSB`, na.rm = TRUE)) +    
  geom_line(aes(group=`Simulator`,colour=`Simulator`)) + 
  ggtitle("Herring")

df_cod <- df_all[grepl("Cod", df_all$Simulator), ]
p3 <- ggplot(data=df_cod, aes(x=`Year`, y=`logSSB`, na.rm = TRUE)) +    
  geom_line(aes(group=`Simulator`,colour=`Simulator`))+
  ggtitle("Cod")

df_sole <- df_all[grepl("Sole", df_all$Simulator), ]
p4 <- ggplot(data=df_sole, aes(x=`Year`, y=`logSSB`, na.rm = TRUE)) +    
  geom_line(aes(group=`Simulator`,colour=`Simulator`)) +
  ggtitle("Sole")

cowplot::plot_grid(p1, p2, p3, p4, ncol = 2, nrow = 2)

```

# Fitting the model
## Prior configuration

In `EcoEnsemble`, short-term discrepancies ($\mathbf{z_k^{(t)}}, \mathbf{\eta^{(t)}}$) are modelled using auto-regressive processes of order one, so that 
\begin{align}
    \mathbf{z}_{k}^{(t)} & \sim{}N(R_{k}\mathbf{z}_{k}^{(t-1)},\Lambda_{k}), \\
    \mathbf{\eta}^{(t)}& \sim{}N(R_{\eta}\mathbf{\eta}^{(t-1)},\Lambda_{\eta}). 
\end{align}
Note also that the long-term individual discrepancies ($\mathbf{\gamma}_k$) are drawn from a normal distribution 
\begin{equation}
    \mathbf{\gamma}_k  \sim{}N(\mathbf{0},C_{\gamma}). \\
\end{equation}

To include prior beliefs in the ensemble model, we therefore need to place priors on $m+2$ different covariance matrices - $\Lambda_{1}, \Lambda_2, \ldots \Lambda_m, \Lambda_{\eta}, C_\gamma$. In `EcoEnsemble`, the choice of prior parametrisations is the same for all of these matrices, however it should be noted that the interpretation of each is different. In particular, the $\Lambda$ matrices are covariances of a dynamic auto-regressive process across time, while $C_\gamma$ is the covariance of the distribution of static long-term discrepancies across different simulators.

In `EcoEnsemble`, we follow the approach of @spence_ff and decompose the covariance matrices into correlation matrices and a diagonal matrix of variances, i.e.  

\begin{equation}
    \Lambda = \sqrt{\mathrm{diag}(\mathbf{\pi})}  P \sqrt{\mathrm{diag}(\mathbf{\pi})},
\end{equation}
where $\mathbf{\pi}$ is the vector of variances of each species, and $P$ is the correlation matrix. Here we write $\Lambda$ for any of $\Lambda_k, \Lambda_\eta, C_\gamma$.


There are three available prior parametrisations for covariance matrices. In each case, the variance terms are parametrised by inverse-gamma distributions, while the correlation matrices can be parametrised either by an LKJ distribution, an inverse-wishart distribution, or by Beta distributions.

Configuring these options in `EcoEnsemble` package is most easily done using the `EnsemblePrior` constructor. A set of default priors is available which produce weakly informative priors that hold for a wide range of different datasets. We can set

``` {r}
#Setting priors for an ensemble model with 4 variables of interest.
priors <- EnsemblePrior(4)
```

If desired, priors can be manually configured for finer control, for example we can set 

``` {r}
priors_custom <- EnsemblePrior(
                        d = 4,
                        ind_st_params = IndSTPrior("lkj", list(25, 0.25), 30),
                        ind_lt_params = IndLTPrior(
                          "beta",
                          list(c(25, 25, 25, 10),c(0.25, 0.25, 0.25, 0.1)),
                          list(matrix(40, 4, 4), matrix(40, 4, 4))
                          ),
                        sha_st_params = ShaSTPrior("lkj", list(25, 0.25), 30),
                        sha_lt_params = 3)
```

This is:

* `ind_st_params` - The individual short-term discrepancies $\mathbf{z}_{k}^{(t)}$. `IndSTPrior("lkj", list(25, 0.25), 30)` encodes that we use $\mathrm{\text{Gamma}(25, 0.25)}$ priors for the variances of all species and an $LKJ(30)$ prior on the correlation matrix.
* `ind_lt_params` - The individual long-term discrepancies $\mathbf{\gamma_k}$ use an $\mathrm{\text{Gamma}(25, 0.25)}$ prior for the variances of the first 3 species and an $\mathrm{\text{Gamma}(10, 0.1)}$ prior for the variance of the final species. The correlation matrix uses a $\mathrm{Beta}(40, 40)$ distribution for each of the off-diagonal elements.
* `sha_st_params` - The shared short-term discrepancies $\mathbf{\eta}^{(t)}$ use $\mathrm{\text{Gamma}(25, 0.25)}$ priors for variances of all species and an $LKJ(30)$ prior on the correlation matrix.
* `sha_lt_params` - Unlike the other discrepancies, the shared long-term discrepancy $\mathbf{\mu}$ is not decomposed into correlation matrices and variances. Instead, a Gaussian distribution is used as the prior on each of the species. `sha_lt_params` encodes that the standard deviation of this Gaussian prior is `3`.


## Model fitting

The ensemble model parameters are fit using the `fit_ensemble_model` function, which converts the data into the required form. Hamiltonian Monte Carlo then samples from the posterior distribution using the `rstan` package @rstan. A sample of the ensemble parameters can then be generated using the `generate_sample` function.

```{r, label = "fit", eval=FALSE}
fit_sample <- fit_ensemble_model(observations = list(SSB_obs, Sigma_obs),
                                 simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
                                                   list(SSB_lm,  Sigma_lm,  "LeMans"),
                                                   list(SSB_miz, Sigma_miz, "mizer"),
                                                   list(SSB_fs,  Sigma_fs,  "FishSUMS")),
                                 priors = priors)
samples <- generate_sample(fit_sample)
```

Note that for the simulators, we pass through the name of the simulators as the third argument. This is not required, however provides more accurate error messages and appropriately labeled plots when needed.


## Results

Results can be shown by calling `plot` on an `EnsembleSample` object. This plots the observations, simulator outputs, and the ensemble model prediction for the truth for each of the variables of interest. The ensemble model prediction also shows ribbons for the quantiles if a full sampling is performed (by default these are the 5% and 95% quantiles).
``` {r, label = "Initial_plots", eval=FALSE}
plot(samples)
```
```{r fig.dim = c(7, 4), echo=FALSE, warning = FALSE}
knitr::include_graphics("data/plot_initial_outputs.png")
```


We can change which variable we plot by passing through a `variable` option, and are free to change the aesthetics in the same way as other `ggplot2` plots, see the ggplot manual @ggplot. Different quantiles can also be specified with the quantiles option, e.g.

```{r, label = "Initial_plots_changed", fig.dim = c(7, 4), eval=FALSE}
plot(samples, variable = "Cod", quantiles = c(0.25, 0.75)) + 
    ggplot2::theme_classic() + 
    ggplot2::scale_color_brewer(palette="Set2")  +
    ggplot2::ylab("SSB (log tonnes)")
```
```{r fig.dim = c(7, 4), echo=FALSE, warning = FALSE}
knitr::include_graphics("data/plot_initial_customised.png")
```


# Case Studies

## Visualising priors

Priors can be visualised using the `prior_ensemble_model()` function and/or the `sample_prior()` function. These models are quick to evaluate and indicate what restrictions the priors place on the ensemble outputs, for example using the priors from the previous section.
```{r visualise_priors, eval= FALSE}
prior_model <- prior_ensemble_model(priors = priors, M = 4)
prior_samples <- sample_prior(observations = list(SSB_obs, Sigma_obs),
                                 simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
                                                   list(SSB_lm,  Sigma_lm,  "LeMans"),
                                                   list(SSB_miz, Sigma_miz, "mizer"),
                                                   list(SSB_fs,  Sigma_fs,  "FishSUMS")),
                                 priors = priors, prior_model)

```
The `sample_prior()` function produces an `EnsembleSample` object which can be plotted in the same way as other `EnsembleSample` objects.

```{r plot_priors, fig.dim = c(7, 4), eval = FALSE}
plot(prior_samples)
```
```{r fig.dim = c(7, 6), echo = FALSE, warning = FALSE}
knitr::include_graphics("data/plot_priors_only.png")
```


## Finer fitting control
Using the wrapper functions `fit_ensemble_model, generate_sample` is the easiest way to generate samples from the ensemble model. However, for finer control, the model can be fit directly with `rstan`. To create data in the correct form of the model, simply create an `EnsembleData` object using the constructor.

```{r, label = "create_data_manual"}
ens_data <- EnsembleData(observations = list(SSB_obs, Sigma_obs),
                          simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
                                            list(SSB_lm,  Sigma_lm,  "LeMans"),
                                            list(SSB_miz, Sigma_miz, "mizer"),
                                            list(SSB_fs,  Sigma_fs,  "FishSUMS")),
                          priors = priors)

```
The `EnsembleData` object has 4 slots: `observations`, `simulators`, `priors`, and `stan_input`. The first 3 slots are the data as passed through in the constructor. The `stan_input` slot is automatically generated by transforming the other data into the form required for the Stan implementation of the ensemble model. To fit the model, the compiled `rstan` object can be retrieved through the `get_mcmc_ensemble_model()` method.

```{r, label = "fitting_ensemble_DIY", eval=FALSE}
mod <- get_mcmc_ensemble_model()
samples <- rstan::sampling(mod, data = ens_data@stan_input)
fit <- EnsembleFit(ens_data, samples = samples)
```
To generate individual samples of the ensemble parameters with this output, now call the `get_transformed_data` and `gen_sample` functions:

``` {r, eval = FALSE}
# Generating samples using DIY functions
transf_data <- get_transformed_data(fit)
mle_sample <- gen_sample(1, ex.fit = rstan::extract(samples), transformed_data = transf_data,
                         time = ens_data@stan_input$time)
```

The equivalent code using wrapper functions:
```{r, eval=FALSE}
fit <- fit_ensemble_model(observations = observations, simulators = simulators,
                          priors = priors)
samples_tmp <- generate_sample(fit)
samples <- samples_tmp@samples
```


# References

---
title: "EcoEnsemble"
subtitle: "An R package for combining ecosystem models"
author: "James A. Martindale and Michael A. Spence"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{EcoEnsemble}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}#
references:
- id: spence_ff
  title: A general framework for combining ecosystem models.
  author:
  - family: Spence
    given: Michael A.
  - family: Blanchard
    given: Julia L.
  - family: Rossberg
    given: Axel G.
  - family: Heath
    given: Michael R.
  - family: Heymans
    given: J. J.
  - family: Mackinson
    given: Steven
  - family: Serpetti
    given: Natalia
  - family: Speirs
    given: Douglas C.
  - family: Thorpe
    given: Robert B.
  - family: Blackwell
    given: Paul G.
  container-title: Fish and Fisheries
  volume: 19
  page: 1013-1042
  type: article-journal
  issued:
    year: 2018
- id: chandler
  title: "Exploiting strength, discounting weakness: combining information from multiple climate simulators"
  author:
  - family: Chandler
    given: Richard E.
  container-title: "Philosophical Transactions of the Royal Society A: Mathematical, Physical and Engineering Sciences"
  volume: 371
  issue: 1991 
  type: article-journal
  issued: 
    year: 2013
- id: zhang
  title: Information Uncertainty and Stock Returns
  author:
  - family: Zhang
    given: X. Frank
  container-title: The Journal of Finance
  volume: 61
  page: 105-137 
  type: article-journal
  issued:
    year: 2006
- id: rougier_13
  title: "Model and data limitations: the sources and implications of epistemic uncertainty"
  author:
  - family: Rougier
    given: J. C.
  - family: Beven
    given: K. J.
  publisher: Cambridge University Press
  page: 40-63
  type: book
  issued:
    year: 2013
- id: Rougier_16
  title: Ensemble Averaging and Mean Squared Error
  author:
  - family: Rougier
    given: J. C. 
  container-title: Journal of Climate
  volume: 29
  page: 8865-8870
  type: article-journal
  issued:
    year: 2016
- id: banner_higgs
  title: Considerations for assessing model averaging of regression coefficients
  author:
  - family: Banner
    given: Katharine M.
  - family: Higgs
    given: Megan D.
  container-title: Ecological Applications
  volume: 27
  page: 78-93
  type: article-journal
  issued:
    year: 2017
- id: Dormann_18
  title: "Model averaging in ecology: a review of Bayesian, information-theoretic, and tactical approaches for predictive inference"
  author:
  - family: Dormann, 
    given: Carsten F.
  - family: Calabrese
    given: Justin M. 
  - family: Guillera-Arroita
    given: Gurutzeta 
  - family: Matechou
    given: Eleni 
  - family: Bahn
    given: Volker 
  - family: Barton
    given: Kamil 
  - family: Beale
    given: Colin M. 
  - family: Ciuti
    given: Simone 
  - family: Elith
    given: Jane 
  - family: Gerstner
    given: Katharina 
  - family: Guelat
    given: Jerome 
  - family: Keil
    given: Petr 
  - family: Lahoz-Monfort
    given: Jose J. 
  - family: Pollock
    given: Laura J. 
  - family: Reineking
    given: Björn 
  - family: Roberts
    given: David R. 
  - family: Schrøder
    given: Boris 
  - family: Thuiller
    given: Wilfried 
  - family: Warton
    given: David I. 
  - family: Wintle
    given: Brendan A. 
  - family: Wood
    given: Simon N. 
  - family: Wuest
    given: Rafael O. 
  - family: Hartig
    given: Florian
  container-title: Ecological Monographs
  volume: 88
  page: 485-504
  type: article-journal
  issued:
    year: 2018
- id: mackinson
  title: Evaluating the fishery and ecological consequences of the proposed North Sea multi-annual plan
  author:
  - family: Mackinson
    given: Steven 
  - family: Platts
    given: Mark 
  - family: Garcia
    given: Clement
  - family: Lynam
    given: Christopher
  container-title: PLOS ONE
  volume: 13
  page: 1-23
  type: article-journal
  issued:
    year: 2018
- id: thorpe15
  title: Evaluation and management implications of uncertainty in a multispecies size-structured model of population and community responses to fishing
  author:
  - family: Thorpe
    given:  Robert B. 
  - family: Le Quesne
    given:  Will J. F. 
  - family: Luxford
    given:  Fay 
  - family: Collie
    given:  Jeremy S. 
  - family: Jennings
    given:  Simon
  container-title: Methods in Ecology and Evolution
  volume: 6
  page: 49-58
  type: article-journal
  issued:
    year: 2015
- id: blanchard
  title: Evaluating targets and trade-offs among fisheries and conservation objectives using a multispecies size spectrum model
  author: 
   - family: Blanchard
     given: Julia L.
   - family: Andersen
     given: Ken H.
   - family: Scott
     given: Finlay
   - family: Hintzen
     given: Niels T.
   - family: Piet
     given: Gerjan
   - family: Jennings
     given: Simon

  container-title: Journal of Applied Ecology
  volume: 51
  page: 612-622
  type: article-journal 
  issued: 
    year: 2014
- id: Collie_16
  title: "Ecosystem models for fisheries management: finding the sweet spot"
  author: 
   - family: Collie
     given: Jeremy S
   - family: Botsford
     given: Louis W
   - family: Hastings
     given: Alan
   - family: Kaplan
     given: Isaac C
   - family: Largier
     given: John L
   - family: Livingston
     given: Patricia A
   - family: Plaganyi
     given: Eva
   - family: Rose
     given: Kenneth A
   - family: Wells
     given: Brian K
   - family: Werner
     given: Francisco E
  container-title: Fish and Fisheries
  page: 101-125
  type: article-journal 
  issued: 
   year: 2016
- id: Essington_13
  title: "Pitfalls and guidelines for recycling models for ecosystem-based fisheries management: evaluating model suitability for forage fish fisheries"
  author: 
   - family: Essington
     given: Timothy E.
   - family: Plaganyi
     given: Eva E.
  container-title: ICES Journal of Marine Science
  page: 118-127
  type: article-journal 
  issued: 
   year: 2013
- id: fulton_03
  title: Effect of Complexity of Marine Ecosystem Models
  author: 
   - family: Fulton
     given: Elizabeth
   - family: Smith
     given: A.
   - family: Johnson
     given: Craig
  container-title: Marine Ecology Progress Series
  page: 1-16
  type: article-journal 
  issued: 
    year: 2003
- id: Hart_fay
  title: Applying tree analysis to assess combinations of Ecosystem-Based Fisheries Management actions in Management Strategy Evaluation
  author: 
   - family: Hart
     given: Amanda R.
   - family: Fay
     given: Gavin
  container-title: Fisheries Research
  page: 105466
  type: article-journal 
  issued: 
    year: 2020
- id: Gaichas_08
  title: "A context for ecosystem-based fishery management: Developing concepts of ecosystems and sustainability"
  author: 
   - family: Gaichas
     given: Sarah K.
  container-title: Marine Policy
  page: 393 - 401
  type: article-journal 
  issued:
    year: 2008
- id: fishsums
  title: "Modelling the effects of fishing on the North Sea fish community size composition"
  author:
  - family: Speirs
    given: Douglas C. 
  - family: Greenstreet
    given: Simon P.R.
  - family: Heath
    given: Michael R.
  container-title: Ecological Modelling
  volume: 321
  page: 35--45
  type: article-journal 
  issued: 
    year: 2016
- id: kennedy_ohagan
  title: Bayesian calibration of computer models
  author: 
   - family: Kennedy
     given: Marc C.
   - family: "O'Hagan"
     given: Anthony
  container-title: "Journal of the Royal Statistical Society: Series B (Statistical Methodology)"
  volume: 63
  page: 425-464
  type: article-journal 
  issued: 
    year: 2001
- id: ices_hawg
  title: "Herring Assessment Working Group for the Area South of 62 N (HAWG)}"
  author: 
    - family: ICES
  issued:
    year: 2020
  type: report
  container-title: "ICES Scientific Reports. 2:60. 1151 pp, ICES, Copenhagen"
  doi: "10.17895/ices.pub.6105"
- id: ices_ns
  title: "Report of the Working Group on the Assessment of Demersal Stocks in the North Sea and Skagerrak"
  author: 
    - family: ICES
  issued:
    year: 2020
  type: report
  container-title: "ICES Scientific Reports. 2:61. 1353, ICES, Copenhagen"
  doi: "10.17895/ices.pub.6092"
- id: ices_ewe
  title: "Working Group on Multispecies Assessment Methods (WGSAM)"
  author: 
    - family: ICES
  issued:
    year: 2016
  type: report
- id: spence_ns
  title: Parameter uncertainty of a dynamic multispecies size spectrum model
  author: 
   - family: Spence
     given: Michael A.
   - family: Blackwell
     given: Paul G.
   - family: Blanchard
     given: Julia L.
  container-title: Canadian Journal of Fisheries and Aquatic Sciences
  volume: 73
  page: 589-597
  type: article-journal 
  issued: 
    year: 2016
- id: ggplot
  author:
   - family: Wickham
     given: Hadley
  title: "ggplot2: Elegant Graphics for Data Analysis"
  publisher: Springer-Verlag New York
  issued: 
    year: 2016
  type: book
  isbn: "978-3-319-24277-4"
  url: "https://ggplot2.tidyverse.org"
  
- id: rstan
  author:
  - family: "Stan Development Team"
  title: "RStan: the R interface to Stan"
  issued: 
    year: 2020
  url: "https://mc-stan.org/"
  type: article
---
```{r include=FALSE}
#Load in the data and load the library
library(EcoEnsemble)
#load("data/vignette_plots.Rdata")
```

# Introduction
When managing ecosystems, management decisions are often informed by outputs of complex ecosystem models (henceforth referred to as simulators). There are often several such simulators available, each providing different outputs with potentially different implications for management actions. While some simulators are better at capturing some aspects of the ecosystem than others, in general no simulator is uniformly better than the others (@chandler), and often the average of simulators outperforms all of the simulators individually (@Rougier_16). 

Management decisions on the basis of a single simulator are therefore very sensitive to the choice of simulator ( @Collie_16, @Essington_13, @fulton_03, @Hart_fay, @Gaichas_08) and by ignoring other available simulators, we limit the amount of information utilised and increase our uncertainty (@spence_ff), often to a greater extent than we formally recognise.

Instead of choosing one simulator, it is possible to combine them using an ensemble model. However, many such methods, such as weighting schemes (e.g. Bayesian model averaging, @banner_higgs, @Dormann_18), explicitly assume that one of the simulators correctly describes the truth (@chandler). Not only is this a strong assumption, but treating a suite of simulators as containing the true model tends to underestimate the uncertainty because alternative, possibly as yet undeveloped, simulators could give predictions outside of the current range (@chandler). Under this scheme, adding a new simulator to the suite could increase the uncertainty of the ensemble despite the additional information contained in the new model (@Dormann_18). For an ensemble model with a robust quantification of uncertainty, uncertainty should decrease as simulators are added (or at worst stay the same).


@spence_ff developed an ensemble model treating individual simulators as exchangeable and drawn from a population of possible simulators. They separated each individual simulator's discrepancy into a discrepancy shared between the simulators and discrepancy that is specific to the simulator. This scheme  allows simulator discrepancies that are not independent, which is expected since individual simulator are often similar, having similar processes, forcing inputs or fitted using similar data (@rougier_13). By modelling in this way, the ensemble model exploits the strengths whilst discounting the weaknesses of each individual simulator (@chandler), whilst robustly quantifying the uncertainty allowing for more accurate advice.

`EcoEnsemble` is an `R` package that implements the ensemble model of @spence_ff when the simulators either output the quantity of interest or not.

# Model description

At time $t$, the true quantity of interest for $d$ variables, $\mathbf{y}^{(t)}=(y_1^{(t)},\ldots,y_d^{(t)})'$, is described by $m$ simulators, $\mathbf{\hat{x}}_{k}^{(t)}=(\hat{x}_{k,1}^{(t)},\ldots,\hat{x}_{k,n_k}^{(t)})'$, with $n_k$ outputs each relating to one of the variables of interest, for $k=1,\ldots,m$, and noisy observations of the variables of interest, $\hat{\mathbf{y}}^{(t)}=(\hat{y}_1^{(t)},\ldots,\hat{y}_d^{(t)})'$.

Not all of the simulators output all the variables over the whole time period. 
To accommodate these differences, @spence_ff introduced a latent variable, known as the "best guess", $\mathbf{x}_{k}^{(t)}=({x}_{1}^{(t)},\ldots,{x}_{d}^{(t)})'$, which represents simulator $k$'s output if it described all $d$ variables at time $t$ with no parameter uncertainty. In `EcoEnsemble`, variables of interest are either present or absent in each simulator, therefore if the $k$th simulator was evaluated at time $t$, its output is
$$
\mathbf{\hat{x}}_{k}^{(t)}\sim{}N(M_k^{(t)}\mathbf{x}_{k}^{(t)},\Sigma_k),
$$
where $M_k^{(t)}$ is an $n_k\times{}d$ matrix and $\Sigma_k$ reflects the parameter uncertainty of the $k$th simulator.

The true values of the variables of interest at time $t$ is simulator $k$'s "best guess" plus a discrepancy term, $\mathbf{\zeta}^{(t)}_k$, @kennedy_ohagan, i.e. 
$$
\mathbf{y}^{(t)} =  \mathbf{x}_{k}^{(t)} + \mathbf{\zeta}^{(t)}_k.
$$
The discrepancy term, $\mathbf{\zeta}^{(t)}_k$, is split between discrepancies that are shared between all of the simulators, and discrepancies that were specific to the $k$th simulator. These two discrepancies are further split into fixed discrepancies, the long-term shared discrepancy, $\mathbf\delta$, and simulator $k$'s long-term individual discrepancy, $\mathbf\gamma_k$, and dynamic discrepancies, the short-term shared discrepancy, $\mathbf\eta^{(t)}$, and simulator $k$'s short-term individual discrepancy, $\mathbf{z}_k^{(t)}$, i.e.
$$
\mathbf{\zeta}^{(t)}_k = \mathbf\delta + \mathbf\eta^{(t)} + \mathbf\gamma_k + \mathbf{z}_k^{(t)}.
$$
The long-term individual discrepancy for the $k$th simulator is
$$
    \mathbf{\gamma}_k\sim{}N(\mathbf{0},C_{\gamma}).
$$
The short-term discrepancy terms, $\mathbf\eta^{(t)}$ and $\mathbf{z}_k^{(t)}$, follow an auto-regressive processes of order one,
$$
\mathbf{\eta}^{(t)}\sim{}N(R_{\eta}\mathbf{\eta}^{(t-1)},\Lambda_{\eta})
$$
and
$$
\mathbf{z}_{k}^{(t)}\sim{}N(R_{k}\mathbf{z}_{k}^{(t-1)},\Lambda_{k})
$$
respectively, where the diagonal elements of $R_i,  (i = \eta, 1, 2, \ldots, m )$ are between $-1$ and $1$, and the off-diagonal elements are $0$.


In the absence of any other information, the true variables evolve according to a random walk,
$$
\mathbf{y}^{(t)}\sim{}N(\mathbf{y}^{(t-1)},\Lambda_y),
$$
with noisy observations,
$$
\mathbf{\hat{y}}^{(t)}\sim{}N(\mathbf{y}^{(t)},\Sigma_y),
$$
when there are observations at time $t$.

## Dynamic linear model
The ensemble model can be written as a dynamical linear model, by letting
\begin{equation}
\mathbf\theta^{(t)}=(\mathbf{y}^{(t)'},\mathbf\eta^{(t)'},\mathbf{z}_1^{(t)'},\ldots,\mathbf{z}_m^{(t)'})',
\end{equation}
so that
\begin{equation}
\mathbf\theta^{(t)}|\mathbf\theta^{(t-1)}\sim{}N(A\mathbf\theta^{(t)},\Lambda),
\end{equation}
with
\begin{equation}
A= 
\begin{pmatrix}
I_d & 0 & 0 & 0 & \ldots & 0\\
0 & R_\eta & 0 & 0 & \ldots & 0\\
0  & 0 & R_1  & 0 & \ldots & 0\\
0  & 0 & 0 & R_2  & \ldots & 0\\
\vdots & \vdots & \vdots & \vdots & \ddots & \vdots \\
0  &  0 & 0 & 0 & \ldots & R_m
\end{pmatrix},
\end{equation}
with $I_d$ being a $d$ dimensional indicator matrix,
and
\begin{equation}
\Lambda=
\begin{pmatrix}
\Lambda_y & 0 & 0 & 0 & \ldots & 0\\
0 & \Lambda_\eta & 0 & 0 & \ldots & 0\\
0  & 0 & \Lambda_1  & 0 & \ldots & 0\\
0  & 0 & 0 & \Lambda_2  & \ldots & 0\\
\vdots & \vdots & \vdots & \vdots & \ddots & \vdots \\
0  &  0 & 0 & 0 & \ldots & \Lambda_m
\end{pmatrix}.
\end{equation}
The observations
\begin{equation}
\mathbf{w}^{(t)}=(\mathbf{\hat{y}}^{(t)'},\mathbf{\hat{x}}^{(t)'}_1,\ldots,\mathbf{\hat{x}}^{(t)'}_m)'
\end{equation}
are 
\begin{equation}
S^{(t)}\mathbf{w}^{(t)}\sim{}N(S^{(t)}B(\mathbf{\theta} + \mathbf\zeta),S^{(t)}\Sigma^{(t)}S^{(t)'})
\label{eq:DLM_obs}
\end{equation}
where
\begin{equation}
B= 
\begin{pmatrix}
I_d & 0 & 0 & 0 & \ldots & 0\\
M_1 & M_1 & M_1  & 0 & \ldots & 0\\
M_2  & M_2 & 0 & M_2  & \ldots & 0\\
\vdots & \vdots & \vdots & \vdots & \ddots & \vdots \\
M_m  &  M_m & 0 & 0 & \ldots & M_m
\end{pmatrix},
\end{equation}
\begin{equation}
\mathbf\zeta=(\mathbf{0}',\mathbf{\delta}',\mathbf\gamma'_1,\ldots,\mathbf\gamma'_m)',
\end{equation}
with
$\mathbf{0}$ being a $d$ dimensional vector of 0s,
\begin{equation}
\Sigma=
\begin{pmatrix}
\Sigma_y & 0  & \ldots & 0\\
0 & \Sigma_1 &   \ldots & 0\\
\vdots & \vdots  & \ddots & \vdots \\
0  &  0  & \ldots & \Sigma_m
\end{pmatrix}
\end{equation}
and $S^{(t)}$ is a matrix that is used to describe which variables are observed at time $t$. The $S^{(t)}$ matrix is an $r(t) \times \left(d + \sum_{k=1}^m n_k\right)$ matrix, where $r(t)$ is the total number of observations and simulator outputs available at time $t$, with each element given by 
$$
S^{(t)}_{i, j} = 
\begin{cases}
  1 & \mathrm{if\ the \ }i^{th} \mathrm{\ available\ observation\ /\ model\ output\ is\ the\ }j^{th}\mathrm{\ of\ all\ possible\ observation\ /\ model\ outputs.} \\
  0 & \mathrm{otherwise}
\end{cases}
$$

For example, if all observations and the simulators give an output at time $t$ then $S^{(t)}=I_{d+\sum_{k=1}^{m}n_k}$. If all   simulators output the variables of interest but there are no observations then $S^{(t)}$ is a  $\left(\sum_{k=1}^m n_k\right) \times \left(d + \sum_{k=1}^m n_k\right)$ matrix, equal to $I_{d+\sum_{k=1}^{m}n_k}$ with the first $d$ rows removed.


# Dataset for the North Sea
We demonstrate the ensemble model by investigating what would happen to the spawning stock biomass (SSB) of four species in the North Sea ($d=4$), Norway pout *(Trisopterus esmarkii)*, Atlantic herring *(Clupea harengus)*, Atlantic cod *(Gadus morhua)*, and Common sole *(Solea solea)*. We used four simulators and estimates of SSB from single-species assessments to predict the true SSB from 1984 to 2017, and the SSB from 2018 to 2050 under MSY.


## Observations
Noisy observations of the SSB were taken from single-species stock assessments (@ices_ns, @ices_hawg). The uncertainty is species specific so the off diagonal elements of $\Sigma_y$ are zero and the diagonal elements are calculated from the uncertainty in the stock assessments. These values, and the associated uncertainty are contained in the `SSB_obs` and `Sigma_obs` variables. The `SSB_obs` data frame contains a row for each year, with a column for each species observed. Note that for `EcoEnsemble`, the rows and columns of all observations and model outputs must be named appropriately in order for the data transformations to behave correctly.
```{r label="Observations"}
SSB_obs
Sigma_obs
```

## Simulators

**EcoPath with EcoSim** (EwE) is an ecosystem model with 60 functional groups for the North Sea (@ices_ewe). It outputs all four species from 1991 until 2050, in the variable `SSB_ewe`, meaning that $n_1=4$ and $M_1=I_4$.
The covariance of the EwE's output, $\Sigma_{1}$, was calculated in @mackinson and is stored in the variable `Sigma_ewe`,

```{r label="EwESigma"}
Sigma_ewe
```



**LeMans** is a discrete time length-based model that describes growth and predation (@thorpe15). It outputs all four species from 1986 until 2050, in the variable `SSB_lm`, meaning that $n_2=4$ and $M_2=I_4$.
The covariance of the LeMans' output, $\Sigma_{2}$, was calculated in @thorpe15 and is stored in the variable `Sigma_lm`,

```{r label="lmSigma"}
Sigma_lm
```


**mizer** is a size-based model that describes ontogenetic feeding and growth, mortality, and reproduction driven by size-dependent predation and maturation processes (@blanchard). It outputs all four species from 1984 until 2050, in the variable `SSB_miz`, meaning that $n_3=4$ and $M_3=I_4$.
The covariance of the mizer's output, $\Sigma_{3}$, was calculated in @spence_ns and is stored in the variable `Sigma_miz`,

```{r label="mizSigma"}
Sigma_miz
```



**FishSUMS** is a discrete time length-based model that describes growth, density-dependent mortality, and losses due to fishing and predation by explicitly modelled species, and seasonal reproduction (@fishsums). It outputs Norway pout, Atlantic Herring, and Atlantic Cod (i.e. it does not output Sole), from 1984 until 2050, in the variable `SSB_fs`, meaning that $n_4=3$ and
\begin{equation}
    M_4=
    \begin{pmatrix}
    1 & 0 &  0 & 0\\
    0 & 1  & 0 & 0\\
    0 & 0 & 1 & 0 
    \end{pmatrix}.
\end{equation}


The covariance of the FishSUMS' output, $\Sigma_{4}$, was calculated in @spence_ff and is stored in the variable `Sigma_fs`. Its value is


```{r label="fsSigma"}
Sigma_fs
```

The single-species stock assessments for the 4 species, along with the model outputs are plotted below. 


```{r model_outputs, warning=FALSE, fig.dim = c(7, 3), message = FALSE, echo = FALSE}
library(tibble)
library(dplyr)
library(reshape2)
library(ggplot2)

#Inlcude years in the data frames
SSB_obs_tmp <- rownames_to_column(SSB_obs, var = "Year")
SSB_ewe_tmp <- rownames_to_column(SSB_ewe, var = "Year")
SSB_miz_tmp <- rownames_to_column(SSB_miz, var = "Year")
SSB_lm_tmp <- rownames_to_column(SSB_lm, var = "Year")
SSB_fs_tmp <- rownames_to_column(SSB_fs, var = "Year")

#Join dataframes together
df_all <- SSB_obs_tmp %>% 
    full_join(SSB_ewe_tmp, by = "Year", suffix = c("","_ewe")) %>%
    full_join(SSB_miz_tmp, by = "Year", suffix = c("","_miz")) %>%
    full_join(SSB_lm_tmp, by = "Year", suffix = c("","_lm")) %>%
    full_join(SSB_fs_tmp, by = "Year", suffix = c("","_fs"))

#Melt into long data format for ggplot
df_all <- melt(df_all, id.vars = "Year")
colnames(df_all) <- c("Year", "Simulator", "logSSB")
df_all$Year <- as.numeric(df_all$Year)

#Finally create the plots
df_n_pout <- df_all[grepl("N.pout", df_all$Simulator), ]
p1 <- ggplot(data=df_n_pout, aes(x=`Year`, y=`logSSB`, na.rm = TRUE)) + 
  geom_line(aes(group=`Simulator`,colour=`Simulator`)) +
  ggtitle("Norway pout")

df_herring <- df_all[grepl("Herring", df_all$Simulator), ]
p2 <- ggplot(data=df_herring, aes(x=`Year`, y=`logSSB`, na.rm = TRUE)) +    
  geom_line(aes(group=`Simulator`,colour=`Simulator`)) + 
  ggtitle("Herring")

df_cod <- df_all[grepl("Cod", df_all$Simulator), ]
p3 <- ggplot(data=df_cod, aes(x=`Year`, y=`logSSB`, na.rm = TRUE)) +    
  geom_line(aes(group=`Simulator`,colour=`Simulator`))+
  ggtitle("Cod")

df_sole <- df_all[grepl("Sole", df_all$Simulator), ]
p4 <- ggplot(data=df_sole, aes(x=`Year`, y=`logSSB`, na.rm = TRUE)) +    
  geom_line(aes(group=`Simulator`,colour=`Simulator`)) +
  ggtitle("Sole")

cowplot::plot_grid(p1, p2, p3, p4, ncol = 2, nrow = 2)

```

# Fitting the model
## Prior configuration

In `EcoEnsemble`, short-term discrepancies ($\mathbf{z_k^{(t)}}, \mathbf{\eta^{(t)}}$) are modelled using auto-regressive processes of order one, so that 
\begin{align}
    \mathbf{z}_{k}^{(t)} & \sim{}N(R_{k}\mathbf{z}_{k}^{(t-1)},\Lambda_{k}), \\
    \mathbf{\eta}^{(t)}& \sim{}N(R_{\eta}\mathbf{\eta}^{(t-1)},\Lambda_{\eta}). 
\end{align}
Note also that the long-term individual discrepancies ($\mathbf{\gamma}_k$) are drawn from a normal distribution 
\begin{equation}
    \mathbf{\gamma}_k  \sim{}N(\mathbf{0},C_{\gamma}). \\
\end{equation}

To include prior beliefs in the ensemble model, we therefore need to place priors on $m+2$ different covariance matrices - $\Lambda_{1}, \Lambda_2, \ldots \Lambda_m, \Lambda_{\eta}, C_\gamma$. In `EcoEnsemble`, the choice of prior parametrisations is the same for all of these matrices, however it should be noted that the interpretation of each is different. In particular, the $\Lambda$ matrices are covariances of a dynamic auto-regressive process across time, while $C_\gamma$ is the covariance of the distribution of static long-term discrepancies across different simulators.

In `EcoEnsemble`, we follow the approach of @spence_ff and decompose the covariance matrices into correlation matrices and a diagonal matrix of variances, i.e.  

\begin{equation}
    \Lambda = \sqrt{\mathrm{diag}(\mathbf{\pi})}  P \sqrt{\mathrm{diag}(\mathbf{\pi})},
\end{equation}
where $\mathbf{\pi}$ is the vector of variances of each species, and $P$ is the correlation matrix. Here we write $\Lambda$ for any of $\Lambda_k, \Lambda_\eta, C_\gamma$.


There are three available prior parametrisations for covariance matrices. In each case, the variance terms are parametrised by inverse-gamma distributions, while the correlation matrices can be parametrised either by an LKJ distribution, an inverse-wishart distribution, or by Beta distributions.

Configuring these options in `EcoEnsemble` package is most easily done using the `EnsemblePrior` constructor. A set of default priors is available which produce weakly informative priors that hold for a wide range of different datasets. We can set

``` {r}
#Setting priors for an ensemble model with 4 variables of interest.
priors <- EnsemblePrior(4)
```

If desired, priors can be manually configured for finer control, for example we can set 

``` {r}
priors_custom <- EnsemblePrior(
                        d = 4,
                        ind_st_params = IndSTPrior("lkj", list(25, 0.25), 30),
                        ind_lt_params = IndLTPrior(
                          "beta",
                          list(c(25, 25, 25, 10),c(0.25, 0.25, 0.25, 0.1)),
                          list(matrix(40, 4, 4), matrix(40, 4, 4))
                          ),
                        sha_st_params = ShaSTPrior("lkj", list(25, 0.25), 30),
                        sha_lt_params = 3)
```

This is:

* `ind_st_params` - The individual short-term discrepancies $\mathbf{z}_{k}^{(t)}$. `IndSTPrior("lkj", list(25, 0.25), 30)` encodes that we use $\mathrm{\text{Gamma}(25, 0.25)}$ priors for the variances of all species and an $LKJ(30)$ prior on the correlation matrix.
* `ind_lt_params` - The individual long-term discrepancies $\mathbf{\gamma_k}$ use an $\mathrm{\text{Gamma}(25, 0.25)}$ prior for the variances of the first 3 species and an $\mathrm{\text{Gamma}(10, 0.1)}$ prior for the variance of the final species. The correlation matrix uses a $\mathrm{Beta}(40, 40)$ distribution for each of the off-diagonal elements.
* `sha_st_params` - The shared short-term discrepancies $\mathbf{\eta}^{(t)}$ use $\mathrm{\text{Gamma}(25, 0.25)}$ priors for variances of all species and an $LKJ(30)$ prior on the correlation matrix.
* `sha_lt_params` - Unlike the other discrepancies, the shared long-term discrepancy $\mathbf{\mu}$ is not decomposed into correlation matrices and variances. Instead, a Gaussian distribution is used as the prior on each of the species. `sha_lt_params` encodes that the standard deviation of this Gaussian prior is `3`.


## Model fitting

The ensemble model parameters are fit using the `fit_ensemble_model` function, which converts the data into the required form. Hamiltonian Monte Carlo then samples from the posterior distribution using the `rstan` package @rstan. A sample of the ensemble parameters can then be generated using the `generate_sample` function.

```{r, label = "fit", eval=FALSE}
fit_sample <- fit_ensemble_model(observations = list(SSB_obs, Sigma_obs),
                                 simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
                                                   list(SSB_lm,  Sigma_lm,  "LeMans"),
                                                   list(SSB_miz, Sigma_miz, "mizer"),
                                                   list(SSB_fs,  Sigma_fs,  "FishSUMS")),
                                 priors = priors)
samples <- generate_sample(fit_sample)
```

Note that for the simulators, we pass through the name of the simulators as the third argument. This is not required, however provides more accurate error messages and appropriately labeled plots when needed.


## Results

Results can be shown by calling `plot` on an `EnsembleSample` object. This plots the observations, simulator outputs, and the ensemble model prediction for the truth for each of the variables of interest. The ensemble model prediction also shows ribbons for the quantiles if a full sampling is performed (by default these are the 5% and 95% quantiles).
``` {r, label = "Initial_plots", eval=FALSE}
plot(samples)
```
```{r fig.dim = c(7, 4), echo=FALSE, warning = FALSE}
knitr::include_graphics("data/plot_initial_outputs.png")
```


We can change which variable we plot by passing through a `variable` option, and are free to change the aesthetics in the same way as other `ggplot2` plots, see the ggplot manual @ggplot. Different quantiles can also be specified with the quantiles option, e.g.

```{r, label = "Initial_plots_changed", fig.dim = c(7, 4), eval=FALSE}
plot(samples, variable = "Cod", quantiles = c(0.25, 0.75)) + 
    ggplot2::theme_classic() + 
    ggplot2::scale_color_brewer(palette="Set2")  +
    ggplot2::ylab("SSB (log tonnes)")
```
```{r fig.dim = c(7, 4), echo=FALSE, warning = FALSE}
knitr::include_graphics("data/plot_initial_customised.png")
```


# Case Studies

## Visualising priors

Priors can be visualised using the `prior_ensemble_model()` function and/or the `sample_prior()` function. These models are quick to evaluate and indicate what restrictions the priors place on the ensemble outputs, for example using the priors from the previous section.
```{r visualise_priors, eval= FALSE}
prior_model <- prior_ensemble_model(priors = priors, M = 4)
prior_samples <- sample_prior(observations = list(SSB_obs, Sigma_obs),
                                 simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
                                                   list(SSB_lm,  Sigma_lm,  "LeMans"),
                                                   list(SSB_miz, Sigma_miz, "mizer"),
                                                   list(SSB_fs,  Sigma_fs,  "FishSUMS")),
                                 priors = priors, prior_model)

```
The `sample_prior()` function produces an `EnsembleSample` object which can be plotted in the same way as other `EnsembleSample` objects.

```{r plot_priors, fig.dim = c(7, 4), eval = FALSE}
plot(prior_samples)
```
```{r fig.dim = c(7, 6), echo = FALSE, warning = FALSE}
knitr::include_graphics("data/plot_priors_only.png")
```


## Finer fitting control
Using the wrapper functions `fit_ensemble_model, generate_sample` is the easiest way to generate samples from the ensemble model. However, for finer control, the model can be fit directly with `rstan`. To create data in the correct form of the model, simply create an `EnsembleData` object using the constructor.

```{r, label = "create_data_manual"}
ens_data <- EnsembleData(observations = list(SSB_obs, Sigma_obs),
                          simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
                                            list(SSB_lm,  Sigma_lm,  "LeMans"),
                                            list(SSB_miz, Sigma_miz, "mizer"),
                                            list(SSB_fs,  Sigma_fs,  "FishSUMS")),
                          priors = priors)

```
The `EnsembleData` object has 4 slots: `observations`, `simulators`, `priors`, and `stan_input`. The first 3 slots are the data as passed through in the constructor. The `stan_input` slot is automatically generated by transforming the other data into the form required for the Stan implementation of the ensemble model. To fit the model, the compiled `rstan` object can be retrieved through the `get_mcmc_ensemble_model()` method. This takes an `EnsemblePrior` object as input and returns the compiled `stanmodel` corresponding to the form of the prior. By default the Stan program samples the latent states explicitly; to retain the legacy Kalman filter implementation call the function with `sampler = "kalman"`.

```{r, label = "fitting_ensemble_DIY", eval=FALSE}
mod <- get_mcmc_ensemble_model(priors)
# or request the Kalman filter variant explicitly
kalman_mod <- get_mcmc_ensemble_model(priors, sampler = "kalman")
samples <- rstan::sampling(mod, data = ens_data@stan_input)
fit <- EnsembleFit(ens_data, samples = samples)
```
To generate individual samples of the ensemble parameters with this output, now call the `get_transformed_data` and `gen_sample` functions:

``` {r, eval = FALSE}
# Generating samples using DIY functions
transf_data <- get_transformed_data(fit)
mle_sample <- gen_sample(1, ex.fit = rstan::extract(samples), transformed_data = transf_data,
                         time = ens_data@stan_input$time)
```

The equivalent code using wrapper functions:
```{r, eval=FALSE}
fit <- fit_ensemble_model(observations = observations, simulators = simulators,
                          priors = priors)
# or use the Kalman filter sampler
fit_kalman <- fit_ensemble_model(observations = observations, simulators = simulators,
                                 priors = priors,
                                 sampler = "kalman")
samples_tmp <- generate_sample(fit)
samples <- samples_tmp@samples
```


# References

---
title: "Exploring priors"
author: "Michael A. Spence, James A. Martindale and Michael J. Thomson"
date: "2023-04-24"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{ExploringPriors}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
references: 
- id: ices_ns
  title: "Report of the Working Group on the Assessment of Demersal Stocks in the North Sea and Skagerrak"
  author: 
    - family: ICES
  issued:
    year: 2020
  type: report
  container-title: "ICES Scientific Reports. 2:61. 1353, ICES, Copenhagen"
  doi: "10.17895/ices.pub.6092"

---

```{r, echo=FALSE,warning=FALSE,error=FALSE,message=FALSE}
library(rstan)
library(EcoEnsemble)
```

# Introduction

In this document we demonstrate the prior distributions and how to we chose the default priors, based on the example of four species in the North Sea.

# Truth
## Random walk parameter
The truth follows a multivariate random walk with covariance parameter $\Lambda_y$,
\begin{equation*}
\mathbf{y}_t\sim{}N(\mathbf{y}_{t-1},\Lambda_y).
\end{equation*}
In `EcoEnsemble`, the prior distribution for this parameter is the inverse Wishart distribution. The inverse Wishart distribution is descried by two parameters, the scale matrix $\Psi$ and the degrees of freedom $\nu$. \emph{apriori} we were unsure of correlations of the four species and therefore set the off diagonal elements of $\Psi$ to zero.

To get an idea of the diagonal elements were examined the marginal distribution of the diagonal elements of $\Lambda_y$. To do this we explored a univariate random walk,
\begin{equation*}
y_{t,i}\sim{}N(y_{t-1,i},\lambda^2).
\end{equation*}
If $y_{0,i}=0$, then the distribution after $T$ times is
\begin{equation*}
y_{T,i}\sim{}N(0,T\lambda^2).
\end{equation*}
The example in `EcoEnsemble` runs for 33 years without any observations from single-species models, and we can investigate what we believe the change in biomass is in 2050 relative to 2017, without any other information. To do this we will look sample from an inverse Wishart distribution, using `rstan`, with varying degrees of freedom.
```{r,eval=FALSE}
inv_wish_st_mod <- stan_model(model_code="data {
  int<lower=0> N;
  real nu;
  matrix[N,N] mat_cov;
}
parameters {
  cov_matrix[N] Sigma;
}
model {
  Sigma ~ inv_wishart(nu, mat_cov);
}
")
```

The lower the degrees of freedom, the less informative the prior. However, this comes at the expense of computational efficiency and $\nu > d-1$ is required. We therefore want $\nu$ to be as large as possible without excluding any plausible values. We look at 8 degrees of freedom:
```{r,eval=FALSE}
fit.iw <- sampling(inv_wish_st_mod,data=list(N=4,nu=2*4,mat_cov=diag(4)))
```

```{r echo=FALSE}
load("data/ExploringPriors_invWishartDensities.RData")
```

We look at the distribution of the first diagonal element
```{r eval = FALSE}
ex.fit.iw <- extract(fit.iw)
plot(density(ex.fit.iw$Sigma[,1,1]),main="",xlab=expression(lambda),xlim=c(0,2))
```
```{r plotDensity, echo = FALSE}
plot(invWishartDensity,main="",xlab=expression(lambda),xlim=c(0,2))
```

which in 2050 leads to
```{r eval = FALSE}
sd.2050 <- sqrt(33*ex.fit.iw$Sigma[,1,1])
plot(density(sqrt(33*ex.fit.iw$Sigma[,1,1])),main="",
     xlab="standard deviation in 2050",xlim=c(0,5))
```
```{r plotSDDensity, echo = FALSE}
plot(density(sd.2050),main="",xlab="standard deviation in 2050",xlim=c(0,5))
```

For example, this means that the SSB of cod in 2050, assuming in 2017 it is exactly the value from the assessment (@ices_ns), will be
```{r}
x <- seq(5,20,length.out=1000)
plot(x,colMeans(sapply(x, function(xs){dnorm(xs,tail(SSB_obs$Cod,n=1),sd.2050)})),
     type="l",axes=F,ylab="density",xlab="Cod in 2050 (1000 tonnes)")
labsx <- log(signif(exp(seq(5,20,length.out=7)),1))
axis(1,at=labsx,labels=c(exp(labsx)/1000))
axis(2)
abline(v=tail(SSB_obs$Cod,n=1))
abline(v=range(SSB_obs$Cod),lty=2)
box()
```

The solid vertical line is the assessment's SSB in 2017, with the dashed lines being the maximum and minimum SSB values from 1984 until 2017. Therefore we used the fairly uninformative prior
\begin{equation}
\lambda_y\sim\text{inv-Wishart}(I,8).
\end{equation}

## Initial distribution of SSB
We also require a prior on the distribution of SSB in 1983, that is
\begin{equation}
\mathbf{y}_{0}\sim{}N(0,1000).
\end{equation}


# Individual discrepancies

## Long-term Individual discrepancy
The long-term individual discrepancy for the $k$th model is
\begin{equation*}
\mathbf{\gamma}_k\sim{}N(0,C_{\gamma})
\end{equation*}
with
\begin{equation*}
C_{\gamma} = \Pi_{\gamma}  P_{\gamma} \Pi_{\gamma},
\end{equation*}
where $P_{\gamma}$ is a correlation matrix and $\Pi_{\gamma}=\text{diag}(\pi_{\gamma,1},\ldots,\pi_{\gamma,4})$. We put a uniform distribution on all correlation matrices,
\begin{equation*}
P_{\gamma} \sim{}LKJ(1),
\end{equation*}
and 
\begin{equation*}
\pi_{\gamma,i}^2 \sim{}\text{gamma}(1,1).
\end{equation*}
This prior gives more weight around zero but has an expectation of 1.
```{r}
xs <- seq(0,5,0.001)
plot(xs,dgamma(xs,1,1),xlab=expression(pi[gamma,i]^2),ylab="density",type="l")
```

The difference between two simulators,
\begin{equation*}
\gamma_{k,i} - \gamma_{l,i} \sim{}N(0,2\pi_{\gamma,i}^2),
\end{equation*}
for $k\neq{}l$.
The standard deviation of the difference of two models is
```{r}
pis <- rgamma(1e5,1,1)
plot(density(sqrt(2 * pis)),xlab="Standard deviation",ylab="Density",main="")
```

and the distribution of the absolute difference is
```{r}
xs <- seq(0,5,length.out=1000)
plot(xs,2*colMeans(sapply(xs, function(x){dnorm(x,0,sqrt(2*pis))})),
     type="l",axes=T,ylab="density",xlab="Difference of two models")
```

We do not expect the models to differ by much than one order of magnitude and therefore we used this prior.

## Short-term Individual discrepancy
The short-term individual discrepancy for the $k$th model follows an auto-regressive process,
\begin{equation*}
\mathbf{z}_{k}^{(t)}\sim{}N(R_{k}\mathbf{z}_{k}^{(t-1)},\Lambda_{k})
\end{equation*}
with $R_{k}=\text{diag}(r_{k,1},\ldots,r_{k,4})$
\begin{equation*}
\Lambda_{k} = \Pi_{k}  P_{k} \Pi_{k},
\end{equation*}
where $P_{k}$ is a correlation matrix and $\Pi_{k}=\text{diag}(\pi_{k,1},\ldots,\pi_{k,4})$.

The distribution was
\begin{equation}
    f(P_k | \alpha, \beta) \propto \begin{cases}
    \prod_{i = 1}^{N-1}\prod_{j = i+ 1}^{N} \frac{1}{\mathrm{B}(\alpha_{\rho,i,j}, \beta_{\rho,i, j})} s(P_{k,i,j})^{\alpha_{\rho,i, j} - 1}\left(1-s(P_{k,i, j})\right)^{\beta_{\rho,i, j} - 1}&\text{if }P\text{ is positive definite}\\
    0 &\text{otherwise}.
    \end{cases}
\end{equation}
with 
\begin{equation}
    s(\rho)=\frac{\rho+1}{2},
\end{equation}
and hyperpriors
\begin{equation}
    \alpha_{\rho,i,j}\sim{}\text{gamma}(0.1,0.1)
\end{equation}
and
\begin{equation}
    \beta_{\rho,i,j}\sim{}\text{gamma}(0.1,0.1).
\end{equation}
Sampling from this prior
```{r,eval=FALSE}
cor_pri_st <- stan_model(model_code = " functions{
  real priors_cor_hierarchical_beta(matrix ind_st_cor, int N, matrix M){
    real log_prior = 0;
    for (i in 1:(N-1)){
      for (j in (i+1):N){
        log_prior += beta_lpdf(0.5*(ind_st_cor[i, j] + 1)| M[i, j], M[j, i] );
      }
    }
    return log_prior;
  }
}  
data {

  vector[4] cor_p;
}

parameters {
  matrix <lower=0>[5,5] beta_pars;
  corr_matrix[5] rho[4];
}
model {
  
  for (i in 1:4){
    for (j in (i+1):5){
      beta_pars[i,j] ~ gamma(cor_p[1],cor_p[2]);
      beta_pars[j,i] ~ gamma(cor_p[3],cor_p[4]);
    }
  }
  for(i in 1:4){
    target += priors_cor_hierarchical_beta(rho[i],5,beta_pars);
    diagonal(beta_pars) ~ std_normal();
  }
}
")
fit_cor <- sampling(cor_pri_st,data=list(cor_p=0.1 * c(1,1,1,1)),chains=4)
ex.fit <- extract(fit_cor)
def.par <- par(no.readonly=TRUE) #Old pars to reset afterwards
par(mfrow=c(2,2))
plot(density(ex.fit$beta_pars[,1,2]),xlab=expression(alpha[rho]),main="")
plot(density(log(ex.fit$beta_pars[,1,2]/ex.fit$beta_pars[,2,1])),main="",
     xlab="Expected log odds")
plot(density(ex.fit$rho[,1,1,2]),xlab=expression(rho),main="")
xs <- seq(-1,1,length.out=100)
lines(xs,dbeta((xs+1)/2,2,2)/2,col="red")
plot(density(apply(ex.fit$rho[,,1,2],1,var)),xlim=c(0,0.35),main="",
     xlab=expression(var(rho)))
par(def.par)
```

```{r echo=FALSE, fig.dim = c(7,6)}
def.par <- par(no.readonly=TRUE) #old pars
load("data/ExploringPriors_HierBetaDensities.RData")
par(mfrow=c(2,2))
plot(beta_marginal,xlab=expression(alpha[rho]),main="")
plot(expected_log_odds,main="",xlab="Expected log odds")
plot(rho_density,xlab=expression(rho),main="")
xs <- seq(-1,1,length.out=100)
lines(xs,dbeta((xs+1)/2,2,2)/2,col="red")
plot(rho_var_density,xlim=c(0,0.35),main="",xlab=expression(var(rho)))
```

The above plot shows the different aspects of the emergent distribution of this prior. The top left plot shows the marginal distribution of parameter of the beta distribution with the top right shows the log odds of the expectation of the beta distribution. The bottom left plot shows the marginal distribution of $\rho_{k,i,i}$, with the red curve being the distribution that gives a uniform distribution over all correlation matrices. We gave slightly more weight to correlation matrices that are less correlated. The bottom right plot shows the variance of $\rho_{k,i,i}$ over $k$. The variance of a uniform distribution between -1 and 1 is $1/3$ and the variance of the marginal distribution of uniform correlation parameters is $1/5$. We allowed the prior to be fairly uninformative over this space, as we were unsure how $\rho_{k,i,i}$ and $\rho_{l,i,i}$ related to one another, however, we do believe they may be similar and therefore we increased our beliefs around 0.

The $i$th diagonal element of $\Pi_k$, $\pi_{k,i}$, was
\begin{equation}
    \ln(\pi_{k,i}^2)\sim{}N(\mu_{\pi,i},\sigma_{\pi,i}^2),
\end{equation}
with zeros on the off diagonals. The hyperpriors were
\begin{equation}
    \mu_{\pi,i}\sim{}N(-3,1^2)
\end{equation}
and
\begin{equation}
    \sigma_{\pi,i}^2\sim{}\text{inv-gamma}(8,4).
\end{equation}
Sampling from this leads to
```{r,eval=FALSE}
st_mod1 <- stan_model(model_code = "data {
  vector[4] gam_p;
}
parameters {
  vector [5] log_sha_st_var[4];
  vector[5] gamma_mean;
  vector<lower=0> [5] gamma_var;
}
model {
  gamma_mean ~ normal(gam_p[1],gam_p[2]);
  gamma_var ~ inv_gamma(gam_p[3],gam_p[4]);
  for (i in 1:4){
    log_sha_st_var[i] ~ normal(gamma_mean,sqrt(gamma_var));
    
  }
}
generated quantities{
  vector[5] sha_st_var[4];
  for (i in 1:4){
    sha_st_var[i] = exp(log_sha_st_var[i]);
  }
}
")
test.fit_norm <- sampling(st_mod1,data=list(gam_p=c(-3,1,8,4)),chains=4)
ex.fit <- extract(test.fit_norm)
def.par <- par(no.readonly=TRUE) #old pars
par(mfrow=c(2,2))
plot(density(ex.fit$gamma_mean[,1]),main="",xlab=expression(mu[pi]))
plot(density(ex.fit$gamma_var[,1]),xlim=c(0,1),main="",xlab=expression(sigma[pi]^2))
plot(density(ex.fit$sha_st_var[,1,1]),xlim=c(0,1),main="",xlab=expression(pi["k,i"]^2))
plot(density(apply(ex.fit$sha_st_var[,,1],1,var)),xlim=c(0,0.2),
     xlab=expression(var(pi["k,i"]^2)) ,main="")
par(def.par)
```
```{r echo = FALSE,fig.dim=c(7, 6)}
load("data/ExploringPriors_HierGammaDensities.RData")
par(mfrow=c(2,2))
plot(gamma_mean,main="",xlab=expression(mu[pi]))
plot(gamma_var,xlim=c(0,1),main="",xlab=expression(sigma[pi]^2))
plot(sha_st_var,xlim=c(0,1),main="",xlab=expression(pi["k,i"]^2))
plot(sha_st_var_var,xlim=c(0,0.2),xlab=expression(var(pi["k,i"]^2)) ,main="")
```

The top left plot shows the marginal distribution of $\mu_{\pi,i}$ and the top right plot being $\sigma^2_{\pi,i}$. The bottom left plot shows the prior distribution of $pi_{k,i}^2$. For the same reason that describes the variance of the random walk on the truth, we felt that $\sigma^2_{\pi,i}$ should not be larger than 0.2. For a similar reason to the correlation parameters, the variance of $\sigma^2_{\pi,i}$ will also be small.

For the auto regressive parameter, $r_{k,i}$, we set a boundary avoiding prior
\begin{equation*}
\frac{r_{k,i} + 1}{2} \sim{}\text{beta}(2,2).
\end{equation*}
```{r}
hist(rbeta(1e5,2,2)*2 - 1,probability = T,xlab=expression(r["k,i"]),main="")
```

# Shared discrepancies

## Long-term shared discrepancy

We set the prior on the long-term shared discrepancy to be
\begin{equation*}
\mathbf{\delta}\sim{}N(0,5^2I).
\end{equation*}
This is a fairly vague prior as we did not expect the simulators to differ by the truth by more than 5 degrees of magnitude.

## Short-term shared discrepancy
The shared short-term discrepancy for the $k$th model follows an auto-regressive process,
\begin{equation*}
\mathbf{\eta}^{(t)}\sim{}N(R_{\eta}\mathbf{\eta}^{(t-1)},\Lambda_{\eta})
\end{equation*}
with $R_{\eta}=\text{diag}(r_{\eta,1},\ldots,r_{\eta,4})$
\begin{equation*}
\Lambda_{k} = \Pi_{k}  P_{k} \Pi_{k},
\end{equation*}
where $P_{\eta}$ is a correlation matrix and $\Pi_{\eta}=\text{diag}(\pi_{\eta,1},\ldots,\pi_{\eta,4})$.

We put a uniform prior on all correlation matrices,
\begin{equation*}
P_{\eta}\sim{}LKJ(1),
\end{equation*}
and, for similar reasons for the variance of the random walk on the truth we set
\begin{equation*}
\pi_{\eta,i}^2\sim{}\text{gamma}(1,10),
\end{equation*}
which has an expectation of 0.1, and is
```{r}
plot(density(rgamma(1e5,1,10)),xlim=c(0,0.5), main="",xlab=expression(pi[eta]^2))
```

and the standard deviation of
```{r}
plot(density(sqrt(rgamma(1e5,1,10))),xlim=c(0,0.5), main="",xlab=expression(pi[eta]))
```

For the auto regressive parameter, $r_{k,i}$, we set a boundary avoiding prior
\begin{equation*}
\frac{r_{\eta,i} + 1}{2} \sim{}\text{beta}(2,2)
\end{equation*}
or
```{r}
hist(rbeta(1e5,2,2)*2 - 1,probability = T,xlab=expression(r["k,i"]),main="")
```

# Prior predictive distribution
The prior predictive distribution is then
```{r,eval=FALSE,fig.dim=c(7,6)}
priors <- EnsemblePrior(4,
                        ind_st_params =IndSTPrior("hierarchical",list(-3,1,8,4),
                                                  list(0.1,0.1,0.1,0.1),AR_params=c(2,2)),
                        ind_lt_params = IndLTPrior("lkj",list(1,1),1),
                        sha_st_params = ShaSTPrior("lkj",list(1,10),1,AR_params=c(2,2)),
                        sha_lt_params = 5
)
prior_density <- prior_ensemble_model(priors, M = 4)
samples <- sample_prior(observations = list(SSB_obs, Sigma_obs),
             simulators = list(list(SSB_ewe, Sigma_ewe,"EwE"),
                   list(SSB_fs ,  Sigma_fs,"FishSums"),
                   list(SSB_lm ,  Sigma_lm,"LeMans"),
                   list(SSB_miz, Sigma_miz,"mizer")),
             priors=priors,
             sam_priors = prior_density)
plot(samples)
```
```{r echo =FALSE,fig.dim=c(7,6)}
knitr::include_graphics("data/p_priors.png")
```

# References

---
title: "Beta Conjugate Prior"
author: "Thomas I. J. Bartos, Michael A. Spence, Michael J. Thomson"
date: "2024-06-5"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Beta Conjugate Prior}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
references: 
- id: ices_ns
  title: "Report of the Working Group on the Assessment of Demersal Stocks in the North Sea and Skagerrak"
  author: 
    - family: ICES
  issued:
    year: 2020
  type: report
  container-title: "ICES Scientific Reports. 2:61. 1353, ICES, Copenhagen"
  doi: "10.17895/ices.pub.6092"
- id: lkj
  title: "Generating random correlation matrices based on vines and extended onion method"
  author:
  - family: Lewandowski
    given: Daniel
  - family: Kuratowicka
    given: Dorota
  - family: Joe
    given: Harry
  container-title: Journal of Multivariate Analysis
  volume: 100
  page: 1989--2001
  type: article-journal 
  issued:
    month: October
    year: 2009
- id: conj_prior_wiki
  url: https://en.wikipedia.org/wiki/Conjugate_prior
---

```{r setup_lies,warning=FALSE}
library(EcoEnsemble)
set.seed(1234)
```

## Introduction

In this document we demonstrate the use of the beta conjugate prior distribution for the hyperparameters of the hierarchical prior for the correlation matrix entries governing the dependencies of the individual short-term discrepancy dynamics between variables of interest. We demonstrate the usage based on the built-in example of four fish species in the North Sea.

## Short-term individual discrepancy

Suppose we have $M$ simulators with up to $N$ variables of interest predicted by each simulator. The short-term individual discrepancy for the $m$th model follows an auto-regressive process,
\begin{equation*}
\mathbf{z}_{m}^{(t)}\sim{}N(R_{m}\mathbf{z}_{m}^{(t-1)},\Lambda_{m})
\end{equation*}
with $R_{m}=\text{diag}(r_{m,1},\ldots,r_{m,4})$
\begin{equation*}
\Lambda_{m} = \Pi_{m}  P_{m} \Pi_{m},
\end{equation*}
where $P_{m}$ is a correlation matrix and $\Pi_{m}=\text{diag}(\pi_{m,1},\ldots,\pi_{m,4})$.

The distribution is
\begin{equation}
    f(P_m | \alpha, \beta) \propto \begin{cases}
    \prod_{i = 1}^{N-1}\prod_{j = i+ 1}^{N} \frac{1}{\mathrm{B}(\alpha_{\rho,i,j}, \beta_{\rho,i, j})} s(P_{m,i,j})^{\alpha_{\rho,i, j} - 1}\left(1-s(P_{m,i, j})\right)^{\beta_{\rho,i, j} - 1}&\text{if }P_{m}\text{ is positive definite}\\
    0 &\text{otherwise}.
    \end{cases}
\end{equation}
with 
\begin{equation}
    s(\rho)=\frac{\rho+1}{2},
\end{equation}
and where we need to also specify hyperpriors on the hyperparameters $\alpha_{\rho, i, j}$ and $\beta_{\rho, i, j}$. Conditional on $P_m$ being positive definite, the marginal distribution of each off-diagonal entry of $P_m$ is a scaled beta distribution. Hence one option is to place a conjugate prior on the parameters of the derived un-scaled beta distribution, which means placing a conjugate prior on the joint distribution of $(\alpha_{\rho, i, j}, \beta_{\rho, i,j})$. It is known that up to proportionality, this prior is ([https://en.wikipedia.org/wiki/Conjugate_prior][@conj_prior_wiki]),
\begin{equation}
g(\alpha_{\rho, i, j}, \beta_{\rho, i,j}\,|\,p, q, k) \propto \frac{\Gamma(\alpha_{\rho, i, j} + \beta_{\rho, i, j})^{k}p^{\alpha_{\rho, i, j}}q^{\beta_{\rho, i, j}}}{\Gamma(\alpha_{\rho, i, j})^{k}\Gamma(\beta_{\rho, i, j})^{k}}\,,   
\end{equation}
for given positive real parameters $p$, $q$ and $k$. We refer to the distribution defined by the above density as the *Beta conjugate prior*. The interpretation of the prior in the case that $k$ is a positive integer and $0 < p\,,\,q < 1$ is that $\alpha_{\rho, i, j}$ and $\beta_{\rho, i, j}$ have been estimated from $k$ observations with product $p$ and product of complements $q$. The equivalent of an improper prior occurs when $k \approx 0$ and $p = q = 1$.

## Exploring the prior

### New parameterisation

We introduce a new parameterisation $(r, s, k)$ of the Beta conjugate prior, which restricts the original parameters $(p, q, k)$ to a subspace of reasonable choices available to the user. We keep $k > 0$ the same and assume we can express the original parameters as $p = z^{k}w^k$ and $q = (1-z)^{k}w^k$ where we require $0 \leq z, w \leq 1$. Then we set $r$ to be the logit transform of $z$, $r = \log\big(\frac{z}{1-z}\big) = \frac{1}{k}\log\big(\frac{p}{q}\big)$. The second parameter is a scale parameter $s$ controlling the order of magnitude of $p$ and $q$, which we define as the logit transform of $w$, $s = \log\big(\frac{w}{1-w}\big)$.
In the case that $k$ is an integer we can interpret the values $z$, $w$ and $k$ as follows: we form our prior using $k$ observations $x_{1}, \ldots, x_{k}$ of a Beta distribution where $w$ is the sum of the geometric means of the observations $x_i$ and observed complements $1-x_{i}$:
$$w = \bigg(\prod_{i = 1}^{k}x_{i}\bigg)^{\frac{1}{k}} + \bigg(\prod_{i = 1}^{k}(1-x_{i})\bigg)^{\frac{1}{k}}\,.$$
We interpret $z$ as the ratio of the geometric mean of the observations $x_i$ to $w$:
$$z = \frac{\bigg(\prod_{i = 1}^{k}x_{i}\bigg)^{\frac{1}{k}}}{\bigg(\prod_{i = 1}^{k}x_{i}\bigg)^{\frac{1}{k}} + \bigg(\prod_{i = 1}^{k}(1-x_{i})\bigg)^{\frac{1}{k}}}\,.$$
Note however, that $k$ need not be integer-valued in general. In the plots below, we show the marginal distribution of $\rho_{m,1,2}$ with $m = 5$ in a number of cases, with the green curve being the resulting marginal of the LKJ(1) distribution, which gives a uniform distribution over all correlation matrices (@lkj). Sampling from the prior with $k = 0.05$ we have
```{r eval = FALSE, message=FALSE, warning=FALSE, results='hide', silent=TRUE}
cor_pri_st <- rstan::stan_model(model_code = " functions{
  real priors_cor_hierarchical_beta(matrix ind_st_cor, int N, matrix M){
    real log_prior = 0;
    for (i in 1:(N-1)){
      for (j in (i+1):N){
        log_prior += beta_lpdf(0.5*(ind_st_cor[i, j] + 1)| M[i, j], M[j, i] );
      }
    }
    return log_prior;
  }

  real beta_conj_prior(real alpha, real betas, real r, real s, real k){
    real rl = 1/(1 + exp(-r));
    real sl = 1/(1 + exp(-s));
    real p = (sl * rl)^k;
    real q = (sl * (1 - rl))^k;
    real ret = alpha * log(p) + betas * log(q) - k * lbeta(alpha,betas);
    return ret;
  }

}

data {
  vector[3] cor_p;
}

parameters {
  matrix <lower=0>[5,5] beta_pars;
  corr_matrix[5] rho[4];
}

model {
  for (i in 1:4){
    for (j in (i+1):5){
      target += beta_conj_prior(beta_pars[i,j], beta_pars[j,i], cor_p[1], cor_p[2], cor_p[3]);
    }
  }

  for (i in 1:4){
    target += priors_cor_hierarchical_beta(rho[i],5,beta_pars);
    diagonal(beta_pars) ~ std_normal();
  }

}

generated quantities {

  matrix [5,5] rhovars;
  for (i in 1:4){
    for (j in (i+1):5){
      rhovars[i,j] = 4 * (beta_pars[i,j] * beta_pars[j,i])/(square(beta_pars[i,j] + beta_pars[j,i]) * (beta_pars[i,j] + beta_pars[j,i] + 1));
      rhovars[j,i] = (2 * beta_pars[i,j]/(beta_pars[i,j] + beta_pars[j,i])) - 1;
    }
  }
  
  for (i in 1:5){
    rhovars[i,i] = 4;
  }
  
}
")
library(ggplot2)
rhoplots <- list()
kvals <- c(0.05, 5)
parvals <- do.call(expand.grid, c(rep(list(c(0.25, 3)), 2), list(kvals)))
#Sampling and gathering outputs for plotting
for (i in 1:nrow(parvals)){
  fit_cor <- rstan::sampling(cor_pri_st, data = list(cor_p=as.numeric(parvals[i,])), iter = 2000, chains=4)
  ex.fit <- rstan::extract(fit_cor)
  rho_density <- density(ex.fit$rho[,1,1,2], from = -1, to = 1)
  rhoplot_data <- data.frame(rho_density$x, rho_density$y)
  names(rhoplot_data) <- c("rho", "Density")
  rhoplot_data <- cbind(rhoplot_data, rep(parvals[i,1], nrow(rhoplot_data)), rep(parvals[i,2], nrow(rhoplot_data)), rep(parvals[i,3], nrow(rhoplot_data)))
  names(rhoplot_data)[3:5] <- c("r", "s", "k")
  rhoplots[[i]] <- rhoplot_data
}
#Construct plots
rhoplot_data <- do.call(rbind, rhoplots)
rhoplot_data <- cbind(rhoplot_data, rep("Beta conjugate prior", nrow(rhoplot_data)))
names(rhoplot_data)[6] <- "Prior"
unif_range <- seq(-1, 1, length.out = nrow(rhoplots[[1]]))
unif_data <- data.frame(cbind(rep(unif_range, nrow(parvals)), rep(dbeta((unif_range+1)/2,5/2,5/2)/2, nrow(parvals)), rhoplot_data[,3:5], rep("Uniform prior", nrow(rhoplot_data))))
names(unif_data) <- names(rhoplot_data)
rhoplot_data <- rbind(rhoplot_data, unif_data)
rhoplot1 <- ggplot(rhoplot_data[which(rhoplot_data$k == kvals[1]),]) + geom_area(aes(x = rho, y = Density, color = Prior, fill = Prior), alpha = 0.3, position = "identity") + scale_x_continuous(bquote(rho), sec.axis = sec_axis(~., name = "r", breaks = NULL, labels = NULL)) + scale_y_continuous(sec.axis = sec_axis(~., name = "s", breaks = NULL, labels = NULL)) + theme(aspect.ratio = 1) + facet_grid(rows = vars(r), cols = vars(s)) + scale_color_manual(values = c("blue", "green")) + scale_fill_manual(values = c("blue", "green"))
rhoplot2 <- ggplot(rhoplot_data[which(rhoplot_data$k == kvals[2]),], aes(x = rho, y = Density)) + geom_area(aes(color = Prior, fill = Prior), alpha = 0.3, position = "identity") + scale_x_continuous(bquote(rho), sec.axis = sec_axis(~., name = "r", breaks = NULL, labels = NULL)) + scale_y_continuous(sec.axis = sec_axis(~., name = "s", breaks = NULL, labels = NULL)) + theme(aspect.ratio = 1) + facet_grid(rows = vars(r), cols = vars(s)) + scale_color_manual(values = c("blue", "green")) + scale_fill_manual(values = c("blue", "green"))
```

```{r, echo = FALSE, out.width="700px", out.height="700px"}
knitr::include_graphics("data/rhoplot1.png")
```

```{r, eval = FALSE, fig.dim = c(7,6), echo = FALSE}
rhoplot1
```

and for $k = 5$ we have

```{r, echo = FALSE, out.width="700px", out.height="700px"}
knitr::include_graphics("data/rhoplot2.png")
```

```{r, eval = FALSE, fig.dim = c(7,6), echo = FALSE}
rhoplot2
```
Note that in the case $k = 0.05$, we see that choosing $r$ large we get a reduced variance compared to the uniform prior equivalent. This is more pronounced in the case $k = 0.5$. In general, as $k$ gets smaller, this can be interpreted as having less prior information and so the variance of $\rho$ is increased. Note that increasing $s$ has also yielded a more skewed prior as larger $s$ puts greater density on beta distributions with large $\alpha$ and $\beta$ (whether symmetric or not). To understand the effect of increasing $k$, we plot the sampled variances and expectations of $\rho$ for six different values of $k$ but with other parameters fixed, $r = -0.3$, $s = 3$.

```{r, message=FALSE, warning=FALSE,eval=FALSE, results='hide', silent=TRUE}
kvals <- c(0.1, 0.2, 0.4, 0.8, 1.6, 3.2)
#parvals <- cbind(rep(0.3, 6), rep(-1, 6), kvals)
parvals <- cbind(rep(-0.3, 6), rep(3, 6), kvals)
rhovarplots <- list()
rhomeanplots <- list()
for (i in 1:6){
  fit_cor <- rstan::sampling(cor_pri_st, data = list(cor_p=parvals[i,]), iter = 2000, chains=4)
  ex.fit <- rstan::extract(fit_cor)
  rhovar_density <- density(ex.fit$rhovars[,1,2])
  rhomean_density <- density(ex.fit$rhovars[,2,1])
  rhovar_data <- data.frame(cbind(rhovar_density$x, rhovar_density$y, rep(kvals[i], length(rhovar_density$x))))
  rhomean_data <- data.frame(cbind(rhomean_density$x, rhomean_density$y, rep(kvals[i], length(rhomean_density$x))))
  names(rhovar_data) <- c("Variance", "Density", "k")
  names(rhomean_data) <- c("Expectation", "Density", "k")
  rhovarplots[[i]] <- rhovar_data
  rhomeanplots[[i]] <- rhomean_data
}
rhovarplot_data <- do.call(rbind, rhovarplots)
rhomeanplot_data <- do.call(rbind, rhomeanplots)
rhovarplot <- ggplot(rhovarplot_data) + geom_area(aes(x = Variance, y = Density), color = "blue", fill = "blue", alpha = 0.3, position = "identity") + geom_vline(xintercept = 0.2, color = "green", linetype = "dashed", linewidth = 0.8) + facet_wrap(vars(k), nrow = 2, ncol = 3) + scale_x_continuous(sec.axis = sec_axis(~., name = "k", breaks = NULL, labels = NULL))
rhomeanplot <- ggplot(rhomeanplot_data) + geom_area(aes(x = Expectation, y = Density), color = "blue", fill = "blue", alpha = 0.3, position = "identity") + geom_vline(xintercept = 0, color = "green", linetype = "dashed", linewidth = 0.8) + facet_wrap(vars(k), nrow = 2, ncol = 3) + scale_x_continuous(sec.axis = sec_axis(~., name = "k", breaks = NULL, labels = NULL))
```

```{r, echo = FALSE, out.width="600px", out.height = "500px"}
knitr::include_graphics("data/rhovarplot.png")
```

```{r, eval = FALSE, echo = FALSE}
rhovarplot
```
Note that in this case increasing $k$ leads to increased variance of $\rho$. The dashed green line at 0.2 indicates the variance of the marginal distribution of LKJ(1) correlation parameters, the uniform equivalent.

```{r, echo = FALSE, out.width = "600px", out.height = "500px"}
knitr::include_graphics("data/rhomeanplot.png")
```

```{r, eval=FALSE, fig.dim = c(8,5), echo = FALSE}
rhomeanplot
```

Note that as $k$ increases the variance of the expectation of $\rho$ decreases. 

## Prior predictive distribution
As an example of using the Beta conjugate prior for the short-term individual discrepancy correlations, we sample from the prior predictive distribution of the ensemble model in the case of four species with $r = 0.25, s = 3$ and $k = 4$ as our hyperparameters. We first draw from the the marginal prior distribution of the first off-diagonal correlation matrix entry $\rho_{m,1,2}$. We plot these draws below: 
```{r, eval = FALSE, message = FALSE, warning = FALSE, silent = TRUE, results = 'hide'}
fit_cor <- rstan::sampling(cor_pri_st, data = list(cor_p=c(0.25, 3, 4), iter = 2000, chains=4))
ex.fit <- rstan::extract(fit_cor)
rhoplot_data  <- data.frame(ex.fit$rho[,1,1,2])
names(rhoplot_data) <- "rho"
rhoplot <- ggplot(rhoplot_data) + geom_histogram(aes(x = rho), color = "blue", fill = "blue", alpha = 0.3, binwidth = 0.05) + scale_x_continuous(bquote(rho))
```

```{r, echo = FALSE, out.width = "400px", out.height = "400px"}
knitr::include_graphics("data/rhoplot.png")
```

```{r, eval = FALSE, echo = FALSE, warning = FALSE, fig.dim = c(4,3)}
rhoplot
```

The prior predictive distribution is then
```{r,eval=FALSE,message=FALSE, warning = FALSE, results = 'hide',silent=TRUE}
priors <- EnsemblePrior(4,
                        ind_st_params =IndSTPrior("hierarchical_beta_conjugate",list(-3,1,8,4),
                                                  list(0.25,3,4),AR_params=c(2,2)),
                        ind_lt_params = IndLTPrior("lkj",list(1,1),1),
                        sha_st_params = ShaSTPrior("lkj",list(1,10),1,AR_params=c(2,2)),
                        sha_lt_params = 5
)
prior_density <- prior_ensemble_model(priors, M = 4)
samples <- sample_prior(observations = list(SSB_obs, Sigma_obs),
             simulators = list(list(SSB_ewe, Sigma_ewe,"EwE"),
                   list(SSB_fs ,  Sigma_fs,"FishSums"),
                   list(SSB_lm ,  Sigma_lm,"LeMans"),
                   list(SSB_miz, Sigma_miz,"mizer")),
             priors=priors,
             sam_priors = prior_density)
```

```{r, eval = FALSE, fig.dim = c(7, 6)}
plot(samples)
```

```{r, echo = FALSE, out.height="500px", out.width="800px"}
knitr::include_graphics("data/p_priors_beta_conj.png")
```

# References

---
title: "Effective sample size"
author: "Thomas I. J. Bartos, Luke E. K. Broadbent, Michael A. Spence and Michael J. Thomson"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Effective sample size}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}#
references:
- id: stan_diagnostics
  author:
  - family: "Stan Development Team"
  title: "How to Diagnose and Resolve Convergence Problems"
  url: "https://mc-stan.org/learn-stan/diagnostics-warnings.html#bulk-and-tail-ess"
  type: webpage
  accessed:
  - year: 2026
    month: 4
    day: 15

nocite: |
  @stan_diagnostics
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

# Summary

The effective sample size (ESS) is important as it indicates the information gained from your Markov Chain Monte Carlo (MCMC) by accounting for the correlation between your samples. Receiving an ESS warning requires further work to understand if your model is trustworthy for your purposes. If you are only interested in your variable of interest, you should use the `get_ESS_diag` function to find the ESS and check that each satisfies the stan's recommended ESS of 400 (number of chains * 100). This will tell you whether you can ignore this error. If this is not satisfied, you need to use more iterations (by setting the argument `iter`).

## What is the warning?

The warning `Bulk / Tail Effective Samples Size (ESS) is too low` is printed by the Markov Chain Monte Carlo (MCMC) software stan when it has identified that the model may not have necessary information about the sample for confidence in the results. 

The user should check whether this warning is relevant to their model and goal before concluding if the results are reliable. 

## What to do?

If ESS is too low, you must use more iterations than the default of 1000 per chain. This is changed in the `fit_ensemble` function by the `iter` argument. However, depending on your goal, ESS may only be of concern for certain parameters.

## How to check?

`EcoEnsemble` provides a function to calculate the ESS for each of your parameters, but you may only be interested in the variable of interest.

We compare the ESS across all the parameters so that it is clear why the ESS error is given, and when it can be disregarded.

We run the example given in the documentation.

```{r, eval=FALSE}
fit <- fit_ensemble_model(observations = list(SSB_obs, Sigma_obs),
                                          simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
                                                            list(SSB_lm,  Sigma_lm,  "LeMans"),
                                                            list(SSB_miz, Sigma_miz, "mizer"),
                                                            list(SSB_fs,  Sigma_fs,  "FishSUMS")),
                                          priors = priors,
                                          sampler = "explicit")
```

And, we find the ESS using an `EcoEnsemble` function.

```{r, eval=FALSE}
ESS_fit <- get_ESS_diag(fit, only_voi = F)
```

The object contains a list of two matrices, one for the bulk and one for the tail ESS (See stan's guidance for [further information](https://mc-stan.org/learn-stan/diagnostics-warnings.html#bulk-and-tail-ess)). We plot the bulk/tail ESS for each parameter, to see why the error is printed.

```{r, echo=F}
knitr::include_graphics("data/ESS_plot.png")
```

For some parameters, the ESS is too low. Stan recommends an ESS higher than the number of chains * 100, and as we used the default 4 chains, there are parameters that will trigger the warning. However, depending on our goal, it may not influence the utility of our model. If we only wanted to understand the variable of interest we will only want to look at parameters concerning our variable of interest.

To check if the these parameters ESS is too low, we use the `only_voi` argument. The `only_voi` will return in the same structure as before.   

```{r, eval = FALSE}

ESS_fit_voi <- get_ESS_diag(fit, only_voi = T)

```
```{r,echo=F}
knitr::include_graphics("data/ESS_plot1.png")
```

When looking at the parameters used in the variable of interest, their ESS can be considered acceptable (>400). In this case, the user shouldn't concern themselves with the warning message and can continue analysis. However, users may consider values different than 400 to be acceptable and should change the number of iterations to reflect their decision.

## What is effective sample size?
 
Suppose we are interested in the expectation of some function of a random variable $\theta$ with distribution $p(\theta)$, that is
$$\mathbb{E}(f(\theta))\,.$$
Given an independent, identically distributed sample $\theta_{1},\theta_{2},\ldots,\theta_{N}$ drawn from $p(\theta)$, the Monte Carlo estimator of the above expectation is
$$I_{N}(f) = \frac{1}{N}\sum_{i = 1}^{N}f(\theta_{i})\,.$$
For large $N$, this estimator is approximately normally distributed with mean equal to the true expectation and variance dependent on the sample size $N$:
$$I_{N}(f) \approx \mathcal{N}\bigg(\mathbb{E}(f(\theta))\,,\,\frac{\text{Var}(f(\theta))}{N}\bigg)\,.$$
In `EcoEnsemble` we use MCMC to sample the posterior which gives us a correlated sample $\widehat{\theta}_{1},\widehat{\theta}_{2},\ldots,\widehat{\theta}_{N}$ from $p(\theta)$. The correlation in the sample means that the amount of information contained in this sample is lower than that of an independent sample of the same size. Informally, the effective sample size is the size of an independent sample containing the same information about $\mathbb{E}(f(\theta))$ as the correlated sample. Specifically, the effective sample size is the value $N_{\text{eff}}(f)$ such that
$$\widehat{I}_{N}(f) =\frac{1}{N}\sum_{i = 1}^{N}f(\widehat{\theta}_{i})\, \approx \mathcal{N}\bigg(\mathbb{E}(f(\theta))\,,\,\frac{\text{Var}(f(\theta))}{N_{\text{eff}}(f)}\bigg)\,.$$


# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EnsembleSample-class.R
\docType{class}
\name{EnsembleSample-class}
\alias{EnsembleSample-class}
\title{A class to hold  samples of the ensemble model}
\description{
\code{EnsembleSample} objects are generated using the \code{generate_sample} function.
}
\section{Slots}{

\describe{
\item{\code{ensemble_fit}}{An \code{EnsembleFit} object containing the fitted ensemble model.}

\item{\code{mle}}{An \code{array} of dimension \eqn{T \times (M + 2)\times N_{sample}} containing MLE point estimates from the \code{ensemble_fit} object, where \eqn{T} is the total time, \eqn{M} is the number of simulators and \eqn{N_{sample}} is the number of samples. For each time step, the \code{t}th element of the array is a \code{matrix} where each column is a sample and the rows are the variables:
\deqn{\left( y^{(t)}, \eta^{(t)}, z_1^{(t)}, z_2^{(t)}, \ldots, z_M^{(t)}\right)'}
where \eqn{y^{(t)}} is the ensemble model's prediction of the latent truth value at time \eqn{t},
\eqn{\eta^{(t)}} is the shared short-term discrepancy at time \eqn{t},
\eqn{z_i^{(t)}} is the individual short-term discrepancy of simulator \eqn{i} at time \eqn{t}.}

\item{\code{samples}}{An \code{array} of dimension \eqn{T \times (M + 2)\times N_{sample}} containing samples from the \code{ensemble_fit} object, where \eqn{T} is the total time, \eqn{M} is the number of simulators and \eqn{N_{sample}} is the number of samples. For each time step, the \code{t}th element of the array is a \code{matrix} where each column is a sample and the rows are the variables:
\deqn{\left( y^{(t)}, \eta^{(t)}, z_1^{(t)}, z_2^{(t)}, \ldots, z_M^{(t)}\right)'}
where \eqn{y^{(t)}} is the ensemble model's prediction of the latent truth value at time \eqn{t},
\eqn{\eta^{(t)}} is the shared short-term discrepancy at time \eqn{t},
\eqn{z_i^{(t)}} is the individual short-term discrepancy of simulator \eqn{i} at time \eqn{t}.}
}}

\seealso{
\code{\link{EnsembleSample}}, \code{\link{generate_sample}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{KalmanFilter_back}
\alias{KalmanFilter_back}
\title{Backwards Kalman filter}
\usage{
KalmanFilter_back(rhos, dee, R, Q, C, P, xhat, Time, y, obs)
}
\arguments{
\item{rhos}{A \code{numeric} containing the diagonal elements of the transition matrix of the evolution equation.}

\item{dee}{A \code{numeric} of the same length as \code{rho} containing the discrepancies or biases in the observation process.}

\item{R}{A \code{numeric} representing the variances of the observation process.}

\item{Q}{A \code{matrix} of dimensions \code{length(rho)} and \code{length(rho)} representing the covariance of the evolution process.}

\item{C}{A a \code{matrix} of dimensions \code{length(rho)} and \code{length(R)} representing the observation operator of the observation process.}

\item{P}{A \code{matrix} of dimensions \code{length(rho)} and \code{length(rho)} representing the covariance matrix of the first state of the evolution process.}

\item{xhat}{A \code{numeric} of the same length as \code{rho} representing the expectation of the first state of the evolution process.}

\item{Time}{A numeric The length of time of the dynamical linear model.}

\item{y}{A \code{matrix} of dimensions \code{Time} and \code{length(R)} of observations of the observation process.}

\item{obs}{A \code{matrix} of dimensions \code{Time} and \code{length(R)}. \code{1} means in the \code{i},\code{j}th element means that the \code{j}th output is observed at tim \code{i}.}
}
\value{
A \code{matrix} with dimensions \code{nrow(time)} and \code{length(xhat)} representing the most likely values of the latent variables.
}
\description{
Finds the most likely path through a dynamical linear model.
}
\details{
For the model with the evolution process \deqn{x_{t+1}\sim{}N(\rho{}\cdot{}x_{t},Q)} and observation process \deqn{y_{t}\sim{}N(\rho{}(x_{t} + \delta),diag(R))}.

Using the sequential Kalman filter, the function gives the mostly path of \eqn{x_{t}} for all \eqn{t}.
}
\examples{
\donttest{
fit <- fit_ensemble_model(observations = list(SSB_obs, Sigma_obs),
               simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
                                 list(SSB_fs,  Sigma_fs, "FishSUMS"),
                                 list(SSB_lm,  Sigma_lm, "LeMans"),
                                 list(SSB_miz, Sigma_miz, "Mizer")),
               priors = EnsemblePrior(4,
               ind_st_params = IndSTPrior(parametrisation_form = "lkj",
               var_params= list(1,1), cor_params = 10, AR_params = c(2, 2))),
               full_sample = FALSE) #Only optimise in this case
transformed_data <- get_transformed_data(fit)
ex.fit <- fit@point_estimate$par
params <- get_parameters(ex.fit)
ret <- KalmanFilter_back(params$AR_params, params$lt_discrepancies,
                          transformed_data$all_eigenvalues_cov,params$SIGMA,
                          transformed_data$bigM, params$SIGMA_init, params$x_hat,
                          fit@ensemble_data@stan_input$time,transformed_data$new_data,
                          transformed_data$observation_available)
}
}
\references{
Chui, C.K. & Chen, G. (2009) Kalman Filtering with Real-Time Applications. Springer, Berlin, Heidelberg, Fourth Edtion.

Kalman, R. E. (1960) A new approach to linear filtering and prediction problems. Trans. ASME, J. Basic Eng., 82, pp. 35-45.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Sigma_obs}
\alias{Sigma_obs}
\title{Stock assessment Sigma}
\format{
A 4x4 \code{matrix}.
}
\usage{
Sigma_obs
}
\description{
A 4x4 covariance matrix quantifying the covariances of the stock assessment estimates of biomass.
}
\references{
Herring Assessment Working Group for the Area South of 62 N (HAWG).Technical report,
ICES Scientific Reports. ACOM:07. 960 pp, ICES, Copenhagen.

Report of the Working Group on the Assessment of Demersal Stocks inthe North Sea and
Skagerrak. Technical report, ICES Scientific Reports. ACOM:22.pp, ICES, Copenhagen.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DiscrepancyPrior-ShaSTPrior-class.R
\docType{class}
\name{ShaSTPrior-class}
\alias{ShaSTPrior-class}
\title{A class to hold the priors for the ensemble model.}
\description{
An \code{ShaSTPrior} object encapsulates the prior information for the short-term discrepancies of the shared discrepancy of the ensemble model.
}
\details{
Shared short-term discrepancies \eqn{\mathbf{\eta}^{(t)}} are modelled as an AR(1) process so that \deqn{\mathbf{\eta}^{(t+1)} \sim N(R_\eta \mathbf{\eta}, \Lambda_\eta).} Accepted parametrisation forms for this discrepancy are \code{lkj}, \code{beta}, or \code{inv_wishart}. See details of the \code{EnsemblePrior()} constructor for more details.
}
\section{Slots}{

\describe{
\item{\code{AR_param}}{The parameters giving the beta parameters for the main parameter of the AR(1) process.}

\item{\code{parametrisation_form}}{The parametrisation by which the covariance matrix of the noise of the AR process is decomposed.}

\item{\code{var_params}}{The parameters characterising the variance of the AR process on the shared short-term discrepancy.}

\item{\code{cor_params}}{The parameters characterising the correlations of the AR process on the shared short-term discrepancy.}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Sigma_fs}
\alias{Sigma_fs}
\title{FishSUMS Sigma}
\format{
A 3x3 \code{matrix}.
}
\usage{
Sigma_fs
}
\description{
A 3x3 covariance matrix quantifying the parameter uncertainty of FishSUMS
}
\references{
Spence, M. A., Blanchard, J. L., Rossberg, A. G., Heath, M. R., Heymans, J. J., Mackinson, S., Serpetti, N., Speirs, D. C., Thorpe, R. B., and Blackwell, P. G. (2018). A general framework for combining ecosystem models. Fish and Fisheries, 19(6), 1031–1042.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_stan_outputs.R,
%   R/get_stan_outputs_withdrivers.R
\name{generate_sample}
\alias{generate_sample}
\alias{get_transformed_data}
\alias{get_parameters}
\alias{get_mle}
\alias{gen_sample}
\alias{get_transformed_data_dri}
\title{Generate samples from a fitted ensemble model.}
\usage{
generate_sample(fit, num_samples = 1)

get_transformed_data(fit)

get_parameters(ex.fit, x = 1)

get_mle(x = 1, ex.fit, transformed_data, time)

gen_sample(x = 1, ex.fit, transformed_data, time)

get_transformed_data_dri(fit)
}
\arguments{
\item{fit}{An \code{EnsembleFit} object.}

\item{num_samples}{A \code{numeric} specifying the number of samples to be generated. The default is 1.}

\item{ex.fit}{A \code{list} containing the samples / point estimate from the \code{EnsembleFit} object.}

\item{x}{A \code{numeric} specifying which sample from the posterior to use. The default is 1.}

\item{transformed_data}{A \code{list} of transformed input data.}

\item{time}{A \code{numeric} specifying the time for which the ensemble model was run.}
}
\value{
\code{generate_sample} gives a \code{list} of length 2, the first element being the MLE of latent variables and the second element being a set of samples of the latent variables.
\itemize{
\item If \code{fit} is a sampling of the ensemble model parameters, then:
\itemize{
\item \code{mle} is a \code{time}\eqn{\times (3M + 2) \times} \code{num_samples} \code{array}, where \eqn{M} is the number of simulators and \code{num_samples} is the number of samples from the ensemble model, giving the MLE of the latent variables for each available sample from the ensemble model.
\item \code{sample} is a \code{time}\eqn{\times (3M + 2) \times} \code{num_samples} \code{array}, giving a sample of the latent variables for each available sample of the ensemble model.
}
\item If \code{fit} is a point estimate of the ensemble model parameters, then:
\itemize{
\item \code{mle} is a \code{time}\eqn{\times (3M + 2) \times} 1 \code{array} giving the MLE of the latent variables for the point estimate of the ensemble model.
\item \code{sample} is a \code{time}\eqn{\times (3M + 2) \times} \code{num_samples} \code{array}, giving \code{num_samples} samples of the latent variables for the single point estimate of the ensemble model.
}
}

\code{get_transformed_data} gives a \code{list} of transformed input data.

\code{get_parameters} gives a \code{list} of ensemble parameters from the requested sample.

\code{get_mle} If \code{fit} is a sampling of the ensemble model parameters, then this is a \code{time}\eqn{\times (3M + 2) \times} \code{num_samples} \code{array}. If \code{fit} is a point estimate of the ensemble model parameters, then this is a \code{time}\eqn{\times (3M + 2) \times} 1 \code{array} giving the MLE of the latent variables for the point estimate of the ensemble model.

\code{gen_sample} If \code{fit} is a sampling of the ensemble model parameters, then this is a \code{time}\eqn{\times (3M + 2) \times} \code{num_samples} \code{array}, giving a sample of the latent variables for each available sample of the ensemble model. If \code{fit} is a point estimate of the ensemble model parameters, then this is a \code{time}\eqn{\times (3M + 2) \times} \code{num_samples} \code{array}.
}
\description{
Methods to generates samples of the latent variables from a fitted ensemble model.
}
\details{
The samples are created using the methods described in Strickland et. al. (2009) and Durbin and Koopman (2002).
}
\examples{
\donttest{
fit <- fit_ensemble_model(observations = list(SSB_obs, Sigma_obs),
               simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
                                 list(SSB_fs,  Sigma_fs, "FishSUMS"),
                                 list(SSB_lm,  Sigma_lm, "LeMans"),
                                 list(SSB_miz, Sigma_miz, "Mizer")),
               priors = EnsemblePrior(4,
               ind_st_params = IndSTPrior(parametrisation_form = "lkj",
               var_params= list(1,1), cor_params = 10, AR_params = c(2, 2))),
               full_sample = FALSE) #Only optimise in this case
samples <- generate_sample(fit, num_samples = 2000)
}
}
\references{
Durbin, J. and Koopman, S. J. (2002). "A simple and efficient simulation smoother for state space time series analysis." Biometrika, 89(3), 603--616.

Chris M.Strickland, Ian. W.Turner, Robert Denhamb, Kerrie L.Mengersena. Efficient Bayesian estimation of multivariate state space models Computational Statistics & Data Analysis Volume 53, Issue 12, 1 October 2009, Pages 4116-4125
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EnsembleFit-class.R
\docType{class}
\name{EnsembleFit-class}
\alias{EnsembleFit-class}
\title{A class to hold the samples or point estimates from the ensemble model.}
\description{
An \code{EnsembleFit} object is returned by the \code{fit_ensemble_model} function.
The object contains a slot for the \code{EnsembleData} object originally used to
fit the ensemble model. The \code{samples} slot contains the samples from the MCMC
if a full sampling was completed, otherwise the \code{point_estimate} slot contains
information about a point estimate.
}
\section{Slots}{

\describe{
\item{\code{ensemble_data}}{An \code{EnsembleData} object encapsulating the data used to fit the ensemble model.}

\item{\code{samples}}{A \code{stanfit} object containing the samples drawn from the fitted model. The default value is \code{NULL}.}

\item{\code{point_estimate}}{A \code{list} output of the optimised model. The default value is \code{NULL}.}
}}

\references{
Stan Development Team (2020). RStan: the R interface to Stan. R package version 2.21.2. https://mc-stan.org
}
\seealso{
\code{\link{EnsembleFit}}, \code{\link{fit_ensemble_model}}, \code{\link{generate_sample}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EnsembleData-class.R
\name{EnsembleData}
\alias{EnsembleData}
\title{Constructor for the \code{EnsembleData} class}
\usage{
EnsembleData(observations, simulators, priors, drivers = FALSE, MMod)
}
\arguments{
\item{observations}{A \code{list} of length 2 containing observations and a covariance matrix. The first element is a \code{data.frame} or \code{matrix} with each column giving observations of each output of interest and each row a time. Rows should be named with the times and columns should be named the variables. The second element is is a \eqn{d \times d} \code{matrix} where \eqn{d} is the number of columns of the observations data frame / matrix. This matrix is the covariance matrix of the observations.}

\item{simulators}{A \code{list} with length equal to the number of simulators. For each simulator, there is a \code{list} of 2 objects containing the simulator output and covariance matrix. The first element is a \code{data.frame} or \code{matrix} with each column giving a simulator outputs of interest and each row a time. Rows should be named with the times and columns should be named the variables. The second element is a \eqn{n_k \times n_k} \code{matrix} where \eqn{n_k} is the number of columns of the simulators output data frame / matrix. This matrix is the covariance matrix of the simulator outputs.}

\item{priors}{An \code{EnsemblePrior} object specifying the prior distributions for the ensemble.}

\item{drivers}{A \code{logical} indicating whether drivers have been used in combination with simulators. Default value is FALSE.}

\item{MMod}{Not currently implemented.}
}
\value{
An object of class \code{EnsembleData}
}
\description{
A constructor for the \code{EnsembleData} class. This is used to convert input data into the required form for \code{fit_ensemble_model}.
}
\examples{
ensemble_data <- EnsembleData(observations = list(SSB_obs, Sigma_obs),
                             simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
                                           list(SSB_fs,  Sigma_fs, "FishSUMS"),
                                           list(SSB_lm,  Sigma_lm, "LeMans"),
                                           list(SSB_miz, Sigma_miz, "mizer")),
                              priors = EnsemblePrior(4))
}
\seealso{
\code{\linkS4class{EnsembleData}}, \code{\link{EnsemblePrior}}, \code{\link{fit_ensemble_model}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Prior_sampling.R
\name{prior_ensemble_model}
\alias{prior_ensemble_model}
\title{Generate samples of parameters from prior distribution}
\usage{
prior_ensemble_model(
  priors,
  M = 1,
  MM = NULL,
  full_sample = TRUE,
  control = list(adapt_delta = 0.95),
  ...
)
}
\arguments{
\item{priors}{An \code{EnsemblePrior} object specifying the prior distributions for the ensemble.}

\item{M}{A \code{numeric} that represents the number of simulators. The default is 1.}

\item{MM}{A \code{numeric} that represents the number of drivers. The default is \code{NULL}.}

\item{full_sample}{A \code{logical} that runs a full sampling of the prior density of the ensemble model if \code{TRUE}. If \code{FALSE}, returns the point estimate which maximises the prior density of the ensemble model.}

\item{control}{If creating a full sample, this is a named \code{list} of parameters to control Stan's sampling behaviour. See the documentation of the \code{stan()} function in the \code{rstan} package for details. The default value is \code{list(adapt_delta = 0.95)}. If optimizing, this value is ignored.}

\item{...}{Additional arguments passed to the function \code{rstan::sampling} or  \code{rstan::optimizing}.}
}
\value{
A \code{list} containing two items named \code{samples} and \code{point_estimate}. If \code{full_sample==TRUE}, \code{samples} is a \code{stanfit} and \code{point_estimate} is a \code{NULL} object, else \code{samples} is a \code{NULL} and \code{point_estimate} is a \code{list} object. It is possible to generate a point estimate for the prior if the individual short-term discrepancy prior follows a hierarchical parameterisation.
}
\description{
Methods to generates samples of the parameters from the prior distribution of the ensemble model.
}
\examples{
\donttest{
priors <- EnsemblePrior(4)
prior_density <- prior_ensemble_model(priors, M = 4)
}
}
\references{
Stan Development Team (2020). RStan: the R interface to Stan. R package version 2.21.2. https://mc-stan.org
}
\seealso{
\code{\linkS4class{EnsembleFit}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{SSB_ewe}
\alias{SSB_ewe}
\title{Ecopath with EcoSim SSB}
\format{
A \code{data.frame} with 60 rows and 4 variables:
\describe{
\item{\code{N.pout}}{Spawning stock biomass of Norway Pout, in log tonnes.}
\item{\code{Herring}}{Spawning stock biomass of Herring, in log tonnes.}
\item{\code{Cod}}{Spawning stock biomass of Cod, in log tonnes.}
\item{\code{Sole}}{Spawning stock biomass of Sole, in log tonnes.}
}
}
\usage{
SSB_ewe
}
\description{
A dataset containing the predictions for spawning stock biomass of Norway Pout, Herring, Cod, and Sole in the North Sea between 1991-2050 under an MSY fishing scenario from EcoPath with EcoSim.
}
\references{
ICES (2016). Working Group on Multispecies Assessment Methods (WGSAM). Technical report, International Council for Exploration of the Seas.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{SSB_miz}
\alias{SSB_miz}
\title{mizer SSB}
\format{
A data frame with 67 rows and 4 variables:
\describe{
\item{\code{N.pout}}{Spawning stock biomass of Norway Pout, in log tonnes.}
\item{\code{Herring}}{Spawning stock biomass of Herring, in log tonnes.}
\item{\code{Cod}}{Spawning stock biomass of Cod, in log tonnes.}
\item{\code{Sole}}{Spawning stock biomass of Sole, in log tonnes.}
}
}
\usage{
SSB_miz
}
\description{
A \code{data.frame} containing the predictions for spawning stock biomass of Norway Pout, Herring, Cod, and Sole
in the North Sea between 1984-2050 under an MSY fishing scenario from mizer.
}
\references{
Blanchard, J. L., Andersen, K. H., Scott, F., Hintzen, N. T., Piet, G., and Jennings, S. (2014). Evaluating targets and trade-offs among fisheries and conservation objectives using a multispecies size spectrum model. Journal of Applied Ecology, 51(3), 612–622
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_ensemble.R
\name{fit_ensemble_model}
\alias{fit_ensemble_model}
\title{Fits the ensemble model}
\usage{
fit_ensemble_model(
  observations,
  simulators,
  priors,
  full_sample = TRUE,
  control = list(adapt_delta = 0.95),
  drivers = FALSE,
  sampler = c("explicit", "kalman"),
  MMod,
  ...
)
}
\arguments{
\item{observations}{A \code{list} of length 2 containing observations and a covariance matrix. The first element is a \code{data.frame} or \code{matrix} with each column giving observations of each output of interest and each row a time. Rows should be named with the times and columns should be named the variables. The second element is is a \eqn{d \times d} \code{matrix} where \eqn{d} is the number of columns of the observations data frame / matrix. This matrix is the covariance matrix of the observations.}

\item{simulators}{A \code{list} with length equal to the number of simulators. For each simulator, there is a \code{list} of 2 objects containing the simulator output and covariance matrix. The first element is a \code{data.frame} or \code{matrix} with each column giving a simulator outputs of interest and each row a time. Rows should be named with the times and columns should be named the variables. The second element is a \eqn{n_k \times n_k} \code{matrix} where \eqn{n_k} is the number of columns of the simulators output data frame / matrix. This matrix is the covariance matrix of the simulator outputs.}

\item{priors}{An \code{EnsemblePrior} object specifying the prior distributions for the ensemble.}

\item{full_sample}{A \code{logical} that runs a full sampling of the posterior density of the ensemble model if \code{TRUE}. If \code{FALSE}, returns the point estimate which maximises the posterior density of the ensemble model.}

\item{control}{If creating a full sample, this is a named \code{list} of paramaters to control Stan's sampling behaviour. See the documentation of the \code{stan()} function in the \code{rstan} package for details. The default value is \code{list(adapt_delta = 0.95)}. If optimizing, this value is ignored.}

\item{drivers}{A \code{logical} indicating whether drivers have been used in combination with simulators. Default value is FALSE.}

\item{sampler}{A character string choosing between the "explicit" latent
sampler (default) and the legacy "kalman" implementation.}

\item{MMod}{Not currently implemented.}

\item{...}{Additional arguments passed to the function \code{rstan::sampling} or  \code{rstan::optimizing}.}
}
\value{
An \code{EnsembleFit} object.
}
\description{
\code{fit_ensemble_model} runs an MCMC of the ensemble model. This process can take a long time depending on the size of the datasets.
}
\examples{
\donttest{

fit <- fit_ensemble_model(observations = list(SSB_obs, Sigma_obs),
               simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
                                 list(SSB_fs,  Sigma_fs, "FishSUMS"),
                                 list(SSB_lm,  Sigma_lm, "LeMans"),
                                 list(SSB_miz, Sigma_miz, "Mizer")),
               priors = EnsemblePrior(4,
               ind_st_params = IndSTPrior(parametrisation_form = "lkj",
               var_params= list(1,1), cor_params = 10, AR_params = c(2, 2))),
               full_sample = FALSE) #Only optimise in this case
}
}
\references{
Stan Development Team (2020). RStan: the R interface to Stan. R package version 2.21.2. https://mc-stan.org
}
\seealso{
\code{\linkS4class{EnsembleFit}}, \code{\link{EnsembleSample}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_functions.R
\name{plot.EnsembleSample}
\alias{plot.EnsembleSample}
\title{Plot the ensemble output}
\usage{
\method{plot}{EnsembleSample}(x, variable = NULL, quantiles = c(0.05, 0.95), ...)
}
\arguments{
\item{x}{An \code{EnsembleSample} object.}

\item{variable}{The name of the variable to plot. This can either be a \code{character} string in the same form as the observation variable, or an index for the column in the observations data frame.}

\item{quantiles}{A \code{numeric} vector of length 2 giving the quantiles for which to plot ribbons if doing a full sampling of the ensemble model. The default is \code{c(0.05,0.95)}.}

\item{...}{Other arguments passed on to methods. Not currently used.}
}
\value{
The \code{ggplot} object.
}
\description{
Plots the latent variables predicted by the ensemble model, along with simulator outputs and observations.
}
\examples{
\donttest{
priors <- EnsemblePrior(4)
prior_density <- prior_ensemble_model(priors, M = 4)
samples <- sample_prior(observations = list(SSB_obs, Sigma_obs),
             simulators = list(list(SSB_miz, Sigma_miz),
                               list(SSB_ewe, Sigma_ewe),
                               list(SSB_fs, Sigma_fs),
                               list(SSB_lm, Sigma_lm)),
             priors = priors,
             sam_priors = prior_density)
plot(samples) #Plot the prior predictive density.
plot(samples, variable="Herring")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{SSB_fs}
\alias{SSB_fs}
\title{FishSUMS SSB}
\format{
A \code{data.frame} with 67 rows and 3 variables:
\describe{
\item{\code{N.pout}}{Spawning stock biomass of Norway Pout, in log tonnes.}
\item{\code{Herring}}{Spawning stock biomass of Herring, in log tonnes.}
\item{\code{Cod}}{Spawning stock biomass of Cod, in log tonnes.}
}
}
\usage{
SSB_fs
}
\description{
A \verb{data frame} containing the predictions for spawning stock biomass of Norway Pout, Herring, and Cod
in the North Sea between 1984-2050 under an MSY fishing scenario from FishSUMS. Note that FishSUMS does not
produce outputs for Sole.
}
\references{
Speirs, D., Greenstreet, S., and Heath, M. (2016). Modelling the effects of fishing on the North Sea fish community size composition. Ecological Modelling, 321, 35–45
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Prior_sampling.R
\name{sample_prior}
\alias{sample_prior}
\title{Generate samples of latent variables from prior predictive distribution}
\usage{
sample_prior(
  observations,
  simulators,
  priors,
  sam_priors,
  num_samples = 1,
  full_sample = TRUE,
  drivers = FALSE,
  ...
)
}
\arguments{
\item{observations}{A \code{list} of length 2 containing observations and a covariance matrix. The first element is a \code{data.frame} or \code{matrix} with each column giving observations of each output of interest and each row a time. Rows should be named with the times and columns should be named the variables. The second element is is a \eqn{d \times d} \code{matrix} where \eqn{d} is the number of columns of the observations data frame / matrix. This matrix is the covariance matrix of the observations.}

\item{simulators}{A \code{list} with length equal to the number of simulators. For each simulator, there is a \code{list} of 2 objects containing the simulator output and covariance matrix. The first element is a \code{data.frame} or \code{matrix} with each column giving a simulator outputs of interest and each row a time. Rows should be named with the times and columns should be named the variables. The second element is a \eqn{n_k \times n_k} \code{matrix} where \eqn{n_k} is the number of columns of the simulators output data frame / matrix. This matrix is the covariance matrix of the simulator outputs.}

\item{priors}{An \code{EnsemblePrior} object specifying the prior distributions for the ensemble.}

\item{sam_priors}{A \code{list} containing two items named \code{samples} and \code{point_estimate}. \code{samples} is either a \code{NULL} or a \code{stanfit} object containing the samples drawn from the prior distribution of the ensemble model and \code{point_estimate} is either a \code{NULL} or a \code{list} object containing the optimised prior distribution of the ensemble model. If this object is \code{missing} then \code{sample_prior} generates it.}

\item{num_samples}{A \code{numeric} specifying the number of samples to be generated. The default is 1.}

\item{full_sample}{A \code{logical} that runs a full sampling of the prior density of the ensemble model if \code{TRUE}. If \code{FALSE}, returns the point estimate which maximises the prior density of the ensemble model.}

\item{drivers}{A \code{logical} indicating whether drivers have been used in combination with simulators. Default value is FALSE.}

\item{...}{Additional arguments passed to the function \code{rstan::sampling} or  \code{rstan::optimizing}.}
}
\value{
An \code{EnsembleSample} object.
}
\description{
Methods to generates samples of the latent variables from the prior predictive distribution of the ensemble model.
}
\details{
The samples are created using the methods described in Strickland et. al. (2009) and Durbin and Koopman (2002).
}
\examples{
\donttest{
priors <- EnsemblePrior(4)
prior_density <- prior_ensemble_model(priors, M = 4)
samples <- sample_prior(observations = list(SSB_obs, Sigma_obs),
             simulators = list(list(SSB_miz, Sigma_miz),
                               list(SSB_ewe, Sigma_ewe),
                               list(SSB_fs, Sigma_fs),
                               list(SSB_lm, Sigma_lm)),
             priors = priors,
             sam_priors = prior_density)
plot(samples) #Plot the prior predictive density.
}
}
\references{
Durbin, J. and Koopman, S. J. (2002). "A simple and efficient simulation smoother for state space time series analysis." Biometrika, 89(3), 603--616.

Chris M.Strickland, Ian. W.Turner, RobertDenhamb, Kerrie L.Mengersena. Efficient Bayesian estimation of multivariate state space models Computational Statistics & Data Analysis Volume 53, Issue 12, 1 October 2009, Pages 4116-4125
}
\seealso{
\code{\linkS4class{EnsembleFit}}, \code{\link{EnsembleSample}}, \code{\link{generate_sample}}, \code{\link{prior_ensemble_model}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DiscrepancyPrior-IndSTPrior-class.R
\docType{class}
\name{IndSTPrior-class}
\alias{IndSTPrior-class}
\title{A class to hold the priors for the ensemble model.}
\description{
An \code{IndSTPrior} object encapsulates the prior information for the short-term discrepancies of the individual simulators within the ensemble model.
}
\details{
Individual short-term discrepancies \eqn{z_k^{(t)}} are modelled as an AR(1) process so that \deqn{z_k^{(t+1)} \sim N(R_k z_k^{(t)}, \Lambda_k).} Accepted parametrisation forms for this discrepancy are \code{lkj}, \code{beta}, \code{inv_wishart}, \code{hierarchical} or \code{hierarchical_beta_conjugate}. See details of the \code{EnsemblePrior()} constructor for more details.
}
\section{Slots}{

\describe{
\item{\code{AR_param}}{The parameters giving the beta parameters for the autoregressive parameter of the AR(1) process.}

\item{\code{parametrisation_form}}{The parametrisation by which the covariance matrix of the noise of the AR process is decomposed.}

\item{\code{var_params}}{The parameters characterising the variance of the AR process on the individual short-term discrepancy.}

\item{\code{cor_params}}{The parameters characterising the correlations of the AR process on the individual short-term discrepancy. .}
}}

\seealso{
\code{\linkS4class{IndLTPrior}}, \code{\linkS4class{ShaSTPrior}}, \code{\linkS4class{TruthPrior}},
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DiscrepancyPrior-IndLTPrior-class.R
\docType{class}
\name{IndLTPrior-class}
\alias{IndLTPrior-class}
\title{A class to hold the priors for the ensemble model.}
\description{
An \code{IndLTPrior} object encapsulates the prior information for the long-term discrepancies of the individual simulators within the ensemble model.
}
\details{
Individual long-term discrepancies \eqn{\gamma_k} are drawn from a distribtion \deqn{\gamma_k \sim N(0, C_\gamma).} Accepted parametrisation forms for this discrepancy are \code{lkj}, \code{beta}, or \code{inv_wishart}. See details of the \code{EnsemblePrior()} constructor for more details.
}
\section{Slots}{

\describe{
\item{\code{parametrisation_form}}{The parametrisation by which the covariance matrix of the noise of the AR process is decomposed.}

\item{\code{var_params}}{The parameters characterising the variance of the distribution of the individual long-term discrepancy.}

\item{\code{cor_params}}{The parameters characterising the correlations of the distribution of the individual long-term discrepancy.}
}}

\seealso{
\code{\linkS4class{IndSTPrior}}, \code{\linkS4class{ShaSTPrior}}, \code{\linkS4class{TruthPrior}},
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{SSB_obs}
\alias{SSB_obs}
\title{Stock assessment SSB}
\format{
A data frame with 34 rows and 4 variables:
\describe{
\item{\code{N.pout}}{Spawning stock biomass of Norway Pout, in log tonnes.}
\item{\code{Herring}}{Spawning stock biomass of Herring, in log tonnes.}
\item{\code{Cod}}{Spawning stock biomass of Cod, in log tonnes.}
\item{\code{Sole}}{Spawning stock biomass of Sole, in log tonnes.}
}
}
\usage{
SSB_obs
}
\description{
A \code{data.frame} containing the single species stock assessment estimates of spawning stock
biomass of Norway Pout, Herring, Cod, and Sole in the North Sea between 1984-2017.
}
\references{
Herring Assessment Working Group for the Area South of 62 N (HAWG)(2020).Technical report,
ICES Scientific Reports. ACOM:07. 960 pp, ICES, Copenhagen.

Report of the Working Group on the Assessment of Demersal Stocks in the North Sea and
Skagerrak (2020). Technical report, ICES Scientific Reports. ACOM:22.pp, ICES, Copenhagen.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_samples_array.R
\name{generate_sample_array}
\alias{generate_sample_array}
\alias{get_mle_array}
\alias{get_parameters_array}
\alias{get_param_idx}
\alias{gen_sample_array}
\alias{get_ESS_diag}
\alias{calc_ess}
\title{Generate samples from a fitted ensemble model to get MCMC effective sample size diagnostics}
\usage{
generate_sample_array(fit)

get_mle_array(ex.fit, transformed_data, time)

get_parameters_array(ex.fit)

get_param_idx(arr, stan_name)

gen_sample_array(ex.fit, transformed_data, time)

get_ESS_diag(fit, only_voi = TRUE)

calc_ess(sammy)
}
\arguments{
\item{fit}{An \code{EnsembleFit} object.}

\item{ex.fit}{A named \code{numeric}.}

\item{transformed_data}{A \code{list} of transformed input data.}

\item{time}{A \code{numeric} specifying the time for which the ensemble model was run.}

\item{arr}{A \verb{named numeric}}

\item{stan_name}{A \code{character} object}

\item{only_voi}{A \code{logical} indicating whether only the effective sample size of the variables of interest should be returned. Default value is \code{TRUE}.}

\item{sammy}{An \code{array} of size the number of parameters \eqn{\times} \code{num_samples} \eqn{\times} \code{nchains}.}
}
\value{
\code{generate_sample_array} gives an array of size  \code{time * (MM + M + 2)*N} \eqn{\times} \code{num_samples} \eqn{\times} \code{nchains} where \code{M} is the number of simulators, \code{MM} is the number of drivers (usually 1), \code{N} is the number of variables of interest \code{num_samples} is the number of samples from the ensemble model for each chain and \code{nchains} is the number of chains in the \code{EnsembleFit} object.

\code{get_parameters_array} gives a \code{list} of ensemble parameters from the requested sample.

\code{get_mle_array} gives an array of size \code{time * (MM + M + 2)*N} \eqn{\times} \code{num_samples} \eqn{\times} \code{nchains}.

\code{gen_sample_array} gives a \code{list} of two arrays of dimensions \code{time * (MM + M + 2)*N} \eqn{\times} \code{num_samples} \eqn{\times} \code{nchains}

\code{get_ESS_diag} and \code{calc_ess} gives a list of length two containing \code{ESS_bulk} and \code{ESS_tail} which has the bulk effective sample size and the tail effective sample size. Each list is of dimensions \code{time} \eqn{\times} \code{(MM + M + 2)*N}.
}
\description{
Methods to generates samples of the latent variables from a fitted ensemble model in the same order as the MCMC and to calculate diagnostics effective sample size for the MCMC.
}
\details{
Effective sample size is calculated from these samples, only for the quantity of interest if desired.
}
\examples{
\donttest{
fit <- fit_ensemble_model(observations = list(SSB_obs, Sigma_obs),
               simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
                                 list(SSB_fs,  Sigma_fs, "FishSUMS"),
                                 list(SSB_lm,  Sigma_lm, "LeMans"),
                                 list(SSB_miz, Sigma_miz, "Mizer")),
               priors = EnsemblePrior(4,
               ind_st_params = IndSTPrior(parametrisation_form = "lkj",
               var_params= list(1,1), cor_params = 10, AR_params = c(2, 2))))
get_ESS_diag(fit)
}
}
\seealso{
See \code{\link[=generate_sample]{generate_sample()}} for information about the sampling and \code{\link[posterior]{ess_bulk}} and \code{\link[posterior]{ess_tail}} for information about the effective sample size calculations.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Sigma_miz}
\alias{Sigma_miz}
\title{mizer Sigma}
\format{
A 4x4 \code{matrix}.
A 4x4 covariance matrix quantifying the parameter uncertainty of mizer
}
\usage{
Sigma_miz
}
\description{
mizer Sigma
}
\references{
Spence, M. A., Blackwell, P. G., and Blanchard, J. L. (2016). Parameter uncertainty of a dynamic multispecies size spectrum model. Canadian Journal of Fisheries and Aquatic Sciences, 73(4), 589–59
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EnsembleFit-class.R
\name{EnsembleFit}
\alias{EnsembleFit}
\title{The constructor for the \code{EnsembleFit} object}
\usage{
EnsembleFit(ensemble_data, samples = NULL, point_estimate = NULL)
}
\arguments{
\item{ensemble_data}{An \code{EnsembleData} object encapsulating the data used to fit the ensemble model.}

\item{samples}{A \code{stanfit} object containing the samples drawn from the fitted model. The default value is \code{NULL}.}

\item{point_estimate}{A \code{list} output of the optimised model. The default value is \code{NULL}.}
}
\value{
An object of class \code{EnsembleFit}
}
\description{
The constructor for an \code{EnsembleFit} object. This function need not be called as an \code{EnsembleFit} object is constructed automatically by the \code{fit_ensemble_model} function.
The \code{samples} slot contains the samples from the MCMC if a full sampling was completed, otherwise the \code{point_estimate} slot contains information about a point estimate.
}
\references{
Stan Development Team (2020). RStan: the R interface to Stan. R package version 2.21.2. https://mc-stan.org
}
\seealso{
\code{\linkS4class{EnsembleFit}}, \code{\link{fit_ensemble_model}},
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EnsembleData-class.R
\docType{class}
\name{EnsembleData-class}
\alias{EnsembleData-class}
\title{A class to hold the Ensemble data}
\description{
A class that holds the observation data, simulator outputs, and prior information to convert into the required form for \code{fit_ensemble_model}.
}
\section{Slots}{

\describe{
\item{\code{stan_input}}{A \code{list} of parameters in the correct form to fit the ensemble model in Stan.}

\item{\code{observations}}{A \code{list} of length 2 containing observations and a covariance matrix. The first element is a \code{data.frame} or \code{matrix} with each column giving observations of each output of interest and each row a time. Rows should be named with the times and columns should be named the variables. The second element is the covariance matrix of the observations.}

\item{\code{simulators}}{A \code{list} with length equal to the number of simulators. For each simulator, there is a \code{list} of 2 objects containing the simulator output and covariance matrix. The first element is a \code{data.frame} or \code{matrix} with each column giving a simulator outputs of interest and each row a time. Rows should be named with the times and columns should be named the variables. The second element is the covariance matrix of the simulator outputs.}

\item{\code{priors}}{An \code{EnsemblePrior} object specifying the prior distributions for the ensemble.}
}}

\seealso{
\code{\link{EnsembleData}}, \code{\link{EnsemblePrior}}, \code{\link{fit_ensemble_model}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Sigma_lm}
\alias{Sigma_lm}
\title{LeMans Sigma}
\format{
A 4x4 \code{matrix}.
A 4x4 covariance matrix quantifying the parameter uncertainty of LeMans
}
\usage{
Sigma_lm
}
\description{
LeMans Sigma
}
\references{
Thorpe, R. B., Le Quesne, W. J. F., Luxford, F., Collie, J. S., and Jennings, S. (2015). Evaluation and management implications of uncertainty in a multispecies size-structured model of population and community responses to fishing. Methods in Ecology and Evolution, 6(1), 49–58.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_ensemble.R
\name{get_mcmc_ensemble_model}
\alias{get_mcmc_ensemble_model}
\title{Return the compiled ensemble model Stan object.}
\usage{
get_mcmc_ensemble_model(
  priors,
  likelihood = TRUE,
  drivers = FALSE,
  sampler = c("explicit", "kalman")
)
}
\arguments{
\item{priors}{An \code{EnsemblePrior} object specifying the prior distributions for the ensemble for which the compiled \code{stanmodel} object will be obtained.}

\item{likelihood}{A \code{logical} that returns the compiled \code{stanmodel} object including the likelihood (the Kalman filter) for given priors if \code{TRUE}. If \code{FALSE} returns the compiled \code{stanmodel} object without the likelihood for sampling from the prior.}

\item{drivers}{A \code{logical} indicating whether drivers have been used in combination with simulators. Default value is FALSE.}

\item{sampler}{A character string selecting which state-space sampler to
use. Either \code{explicit} (the default), which samples latent states
directly in the Stan program, or \code{kalman} which retains the Kalman
filter implementation.}
}
\value{
The \code{stanmodel} object encoding the ensemble model.
}
\description{
Gets the unfit, compiled \code{stanmodel} object encoding the ensemble model. This allows for
manual fitting of the ensemble model directly using \code{rstan::sampling}.
}
\examples{
priors <- EnsemblePrior(4)
mod <- get_mcmc_ensemble_model(priors)

ensemble_data <- EnsembleData(observations = list(SSB_obs, Sigma_obs),
                             simulators = list(list(SSB_ewe, Sigma_ewe, "EwE"),
                                           list(SSB_fs,  Sigma_fs, "FishSUMS"),
                                           list(SSB_lm,  Sigma_lm, "LeMans"),
                                           list(SSB_miz, Sigma_miz, "mizer")),
                              priors = priors)
\donttest{
out <- rstan::sampling(mod, ensemble_data@stan_input, chains = 1)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EnsemblePrior-class.R
\docType{class}
\name{EnsemblePrior-class}
\alias{EnsemblePrior-class}
\title{A class to hold the priors for the ensemble model.}
\description{
An \code{EnsemblePrior} object encapsulates the prior information for the ensemble model.
}
\section{Slots}{

\describe{
\item{\code{d}}{A \code{numeric} specifying the number of variables of interest in the ensemble.}

\item{\code{ind_st_params}}{A \code{list} containing a prior specification for the individual short-term discrepancies \eqn{z_k^{(t)}}. See details of the \code{EnsemblePrior()} constructor.}

\item{\code{ind_lt_params}}{A \code{list} containing a prior specification for the individual long-term discrepancies \eqn{\gamma_k}. See details of the \code{EnsemblePrior()} constructor.}

\item{\code{sha_st_params}}{A \code{list} containing a prior specification for the shared short-term discrepancies \eqn{\eta^{(t)}}. See details of the \code{EnsemblePrior()} constructor.}

\item{\code{sha_lt_params}}{A \code{numeric} containing the standard deviations for the normal prior used on the shared short-term discrepancy \eqn{\mu}. If a single value is supplied,  this is repeated for each variable}

\item{\code{truth_params}}{A \code{list} containing a prior specification for the processes on the truth \eqn{y^{(t)}}. See details of the \code{EnsemblePrior()} constructor. The default value is \code{TruthPrior(d)}.}

\item{\code{priors_stan_input}}{A \code{list} containing the prior data in the correct form to fit the model in Stan. This information is automatically generated by the constructor.}
}}

\references{
Stan Development Team (2020). RStan: the R interface to Stan. R package version 2.21.2. https://mc-stan.org

Lewandowski, Daniel, Dorota Kurowicka, and Harry Joe. 2009. "Generating Random Correlation Matrices Based on Vines and Extended Onion Method." Journal of Multivariate Analysis 100: 1989--2001.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{SSB_lm}
\alias{SSB_lm}
\title{LeMans SSB}
\format{
A \code{data.frame} with 65 rows and 4 variables:
\describe{
\item{\code{N.pout}}{Spawning stock biomass of Norway Pout, in log tonnes.}
\item{\code{Herring}}{Spawning stock biomass of Herring, in log tonnes.}
\item{\code{Cod}}{Spawning stock biomass of Cod, in log tonnes.}
\item{\code{Sole}}{Spawning stock biomass of Sole, in log tonnes.}
}
}
\usage{
SSB_lm
}
\description{
A \code{data.frame} containing the predictions for spawning stock biomass of Norway Pout, Herring, Cod, and Sole
in the North Sea between 1986-2050 under an MSY fishing scenario from the LeMaRns package.
}
\references{
Thorpe, R. B., Le Quesne, W. J. F., Luxford, F., Collie, J. S., and Jennings, S. (2015). Evaluation and management implications of uncertainty in a multispecies size-structured model of population and community responses to fishing. Methods in Ecology and Evolution, 6(1), 49–58.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DiscrepancyPrior-TruthPrior-class.R
\docType{class}
\name{TruthPrior-class}
\alias{TruthPrior-class}
\title{A class to hold the priors for the truth model in the ensemble framework}
\description{
An \code{TruthPrior} object encapsulates the prior information for the short-term discrepancies of the shared discrepancy of the ensemble model.
}
\details{
The truth \eqn{\mathbf{y}^{(t)}} is modelled as a random walk such that \deqn{\mathbf{y}^{(t+1)} \sim N(\mathbf{y}^{(t)}, \Lambda_y).} The covariance matrix \eqn{\Lambda_y} is parameterised by an inverse Wishart distribution (contained in the \code{rw_covariance} slot) and the initial value is modelled as drawn from a normal distribution.
}
\section{Slots}{

\describe{
\item{\code{d}}{A \code{numeric} giving the number of variables of interest in the ensemble model.}

\item{\code{initial_mean}}{A \code{numeric} giving the standard deviation of the normal prior on the initial mean value of the random walk. This is the same standard deviation for each variable of interest.}

\item{\code{initial_var}}{A \code{list} of length \code{2} containing the shape and scale parameters (respectively) for the gamma priors on the variance of the initial value of the truth.}

\item{\code{rw_covariance}}{A \code{list} of length \code{2} containing the inverse-Wishart parameters for the covariance of the random walk of the truth.}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EcoEnsemble-package.R
\docType{package}
\name{EcoEnsemble-package}
\alias{EcoEnsemble}
\alias{EcoEnsemble-package}
\title{A general framework for combining ecosystem models}
\description{
The \code{EcoEnsemble} package implements the framework for combining ecosystem models laid out in Spence et al (2018).
}
\details{
The ensemble model can be implemented in three main stages:
\enumerate{
\item Eliciting priors on discrepancy terms: This is done by using the \code{EnsemblePrior} constructor.
\item Fitting the ensemble model: Using \code{fit_ensemble_model} with simulator outputs, observations and prior information. The ensemble model can be fit, obtaining either the point estimate, which maximises the posterior density, or running Markov chain Monte Carlo to generate a sample from the posterior denisty of the ensemble model.
\item Sampling the latent variables from the fitted model: Using \code{generate_sample} with the fitted ensemble object, the discrepancy terms and the ensemble's best guess of the truth can be generated. Similarly to \code{fit_ensemble_model}, this can either be a point estimate or a full sample.
}
}
\references{
Stan Development Team (2020). RStan: the R interface to Stan. R package version 2.21.2. \url{https://mc-stan.org}

Spence, M. A., J. L. Blanchard, A. G. Rossberg, M. R. Heath, J. J. Heymans, S. Mackinson, N. Serpetti, D. C. Speirs, R. B. Thorpe, and P. G. Blackwell. 2018. "A General Framework for Combining Ecosystem Models." Fish and Fisheries 19: 1013-42. \url{https://onlinelibrary.wiley.com/doi/abs/10.1111/faf.12310}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/CefasRepRes/EcoEnsemble}
  \item Report bugs at \url{https://github.com/CefasRepRes/EcoEnsemble/issues}
}

}
\author{
\strong{Maintainer}: Michael A. Spence \email{michael.spence@cefas.gov.uk} (\href{https://orcid.org/0000-0002-3445-7979}{ORCID})

Authors:
\itemize{
  \item James A. Martindale (\href{https://orcid.org/0000-0002-1913-5592}{ORCID})
  \item Michael J. Thomson (\href{https://orcid.org/0000-0003-0284-0129}{ORCID})
  \item Thomas I. J. Bartos
  \item Luke E. K. Broadbent
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DiscrepancyPrior-IndLTPrior-class.R,
%   R/DiscrepancyPrior-IndSTPrior-class.R,
%   R/DiscrepancyPrior-ShaSTPrior-class.R,
%   R/DiscrepancyPrior-TruthPrior-class.R, R/EnsemblePrior-class.R
\name{IndLTPrior}
\alias{IndLTPrior}
\alias{IndSTPrior}
\alias{ShaSTPrior}
\alias{TruthPrior}
\alias{EnsemblePrior}
\title{Constructor for the \code{EnsemblePrior} class}
\usage{
IndLTPrior(
  parametrisation_form = "lkj",
  var_params = list(1, 1),
  cor_params = 1
)

IndSTPrior(
  parametrisation_form = "hierarchical",
  var_params = list(-3, 1, 8, 4),
  cor_params = list(0.1, 0.1, 0.1, 0.1),
  AR_params = c(2, 2)
)

ShaSTPrior(
  parametrisation_form = "lkj",
  var_params = list(1, 10),
  cor_params = 1,
  AR_params = c(2, 2)
)

TruthPrior(
  d,
  initial_mean = 0,
  initial_var = 100,
  rw_covariance = list(2 * d, diag(d))
)

EnsemblePrior(
  d,
  ind_st_params = IndSTPrior(),
  ind_lt_params = IndLTPrior(),
  sha_st_params = ShaSTPrior(),
  sha_lt_params = 5,
  truth_params = TruthPrior(d)
)
}
\arguments{
\item{parametrisation_form}{The parametrisation by which the covariance matrix of the noise of the AR process (in the case of \code{IndSTPrior} and \code{ShaSTPrior} objects or the covariance of the distribution of long-term discrepancies for \code{IndLTPrior} objects) is decomposed. The default is \code{hierarchical} for \code{IndSTPrior} objects, and \code{lkj} otherwise. See details.}

\item{var_params}{The parameters characterising the variance of the AR process (in the case of \code{IndSTPrior} and \code{ShaSTPrior} objects or the variance of the distribution of long-term discrepancies for \code{IndLTPrior} objects) on the discrepancy. The default value is \code{list(-3, 1, 8, 4)} for \code{IndSTPrior} objects, \code{list(1, 1)} for \code{IndLTPrior} objects, and \code{list(1, 10)} for \code{ShaSTPrior} objects. See details.}

\item{cor_params}{The parameters characterising the correlations of the AR process (or the distribution of long-term discrepancies) on the short-term discrepancies. The default value in this case is to use \code{list(0.1,0.1,0.1,0.1)} for \code{IndSTPrior} objects, and \code{1} for \code{IndLTPrior} and \code{ShaSTPrior} objects. See details.}

\item{AR_params}{The parameters giving the beta parameters for the prior distribution on the autoregressive parameter of the AR(1) process. The default is \code{c(2,2)}. See details.}

\item{d}{A \code{numeric} specifying the number of variables of interest in the ensemble.}

\item{initial_mean}{A \code{numeric} giving the mean of the normal distribution giving the prior on the initial value of the random walk. This is the same value for each variable Default value is \code{0}.}

\item{initial_var}{A \code{numeric} giving the variance of the normal distribution giving the prior on the initial value of the random walk. This is the same value for each variable Default value is \code{100}.}

\item{rw_covariance}{A \code{list} of length \code{2} containing the inverse-Wishart parameters for the covariance of the random walk of the truth. The default value is \code{list(2*d, diag(d))}.}

\item{ind_st_params}{An \code{IndSTPrior} object specifying priors for the individual short-term discrepancies \eqn{z_k^{(t)}}. The default value is \code{IndSTPrior("hierarchical", list(-3, 1, 8, 4), list(0.1, 0.1, 0.1, 0.1), c(2, 2))}.}

\item{ind_lt_params}{An \code{IndLTPrior} object specifying priors for the individual long-term discrepancies \eqn{\gamma_k}. The default value is \code{IndLTPrior("lkj",list(1, 1), 1)}.}

\item{sha_st_params}{A \code{ShaSTPrior} object specifying priors for the shared short-term discrepancies \eqn{\eta^{(t)}}. The default value is \code{ShaSTPrior("lkj",list(1, 10), 1, c(2,2))}.}

\item{sha_lt_params}{A \code{numeric} of length \code{d} or \code{1} containing the standard deviations for the normal prior used on the shared short-term discrepancy \eqn{\mu}. If a single value is supplied,  this is repeated for each variable of interest. The default value is \code{5}.}

\item{truth_params}{A \code{TruthPrior} object specifying priors for the processes on the truth \eqn{y^{(t)}}. The default value is \code{TruthPrior(d)}.}
}
\value{
\code{EnsemblePrior} returns an object of class \code{EnsemblePrior}.
\code{IndSTPrior} returns an object of class \code{IndSTPrior}.
\code{IndLTPrior} returns an object of class \code{IndLTPrior}.
\code{ShaSTPrior} returns an object of class \code{ShaSTPrior}.
\code{TruthPrior} returns an object of class \code{TruthPrior}.
}
\description{
Constructors for the \code{EnsemblePrior} class and related classes. These functions are used to encode prior information for the ensemble model. The \code{IndSTPrior}, \code{IndLTPrior}, \code{ShaSTPrior}, and \code{TruthPrior} constructors encapsulate prior information.
}
\details{
\code{IndSTPrior} and \code{ShaSTPrior} discrepancy prior parameter objects contain 4 slots corresponding to:
\enumerate{
\item \code{parametrisation_form} - A \code{character} specifying how the priors are parametrised. Currently supported priors are \code{'lkj'}, \code{'inv_wishart'}, \code{'beta'}, \code{'hierarchical'}, or \code{'hierarchical_beta_conjugate'} (\code{'hierarchical'} and \code{'hierarchical_beta_conjugate'} are only supported for \code{IndSTPrior} objects).
\item \code{var_params} - The prior parameters for the discrepancy variances, either a \code{list} of length \code{2} or a \code{numeric} of length \code{4}. See below.
\item \code{cor_params} - The correlation matrix parameters, either a \code{list} of length \code{2}, a \code{numeric} of length \code{3} or a \code{numeric} of length \code{4}. See below.
\item \code{AR_params} - Parameters for the autoregressive parameter as a \code{numeric} of length \code{2}.
}

\code{IndLTPrior} discrepancy prior parameter objects contain the slots \code{parametrisation_form}, \code{var_params}, and \code{cor_params}.

There are currently five supported prior distributions on covariance matrices. As in Spence et. al. (2018), the individual and shared short-term discrepancy covariances, \eqn{\Lambda_k} and \eqn{\Lambda_\eta}, as well as the individual long-term discrepancy covariance, \eqn{\Lambda_\gamma},  are decomposed into a vector of variances and a correlation matrix \deqn{\Lambda = \sqrt{\mathrm{diag}(\pi)}  P \sqrt{\mathrm{diag}(\pi)},} where \eqn{\pi} is the vector of variances for each variable of interest (VoI), and \eqn{P} is the correlation matrix.

Selecting \code{'lkj'}, \code{'inv_wishart'}, \code{'beta'}, \code{'hierarchical'} or \code{'hierarchical_beta_conjugate'} refers to setting LKJ, inverse Wishart, beta or hierarchical (with gamma-distributed hyperparameters or beta-conjugate-distributed hyperparameters) prior distributions on the covariance matrix respectively. The variance parameters should be passed through as the \code{var_params} slot of the object and the correlation parameters should be passed through as the \code{cor_params}. For \code{'lkj'}, \code{'inv_wishart'}, and \code{'beta'} selections, variances are parameterised by gamma distributions, so the \code{var_params} slot should be a \code{list} of length two, where each element gives the shape and rate parameters for each VoI (either as a single value which is the same for each VoI or a \code{numeric} with the same length as the number of VoI). For example, setting \code{var_params  = list(c(5,6,7,8), c(4,3,2,1))} would correspond to a \code{Gamma(5, 4)} prior on the variance of the first VoI, a \code{Gamma(6, 3)} prior on the variance of the second VoI, etc...
The correlations should be in the following form:
\itemize{
\item If \code{'lkj'} is selected, then \code{cor_params} should be a \code{numeric} \eqn{\eta} giving the LKJ shape parameter, such  that the probability density is given by  (Lewandowski et. al. 2009) \deqn{f(\Sigma | \eta)\propto \mathrm{det} (\Sigma)^{\eta - 1}.} Variances are parameterised by gamma distributions.
\item If \code{'inv_wishart'} is selected, then  \code{cor_params} should be a \code{list} containing a scalar value \eqn{\eta} (giving the degrees of freedom) and a symmetric, positive definite matrix \eqn{S} (giving the scale matrix). The dimensions of \eqn{S} should be the same as the correlation matrix it produces (i.e \eqn{d \times d} where \eqn{d} is the number of VoI). The density of an inverse Wishart is given by  \deqn{f(W|\eta, S) = \frac{1}{2^{\eta d/2} \Gamma_N \left( \frac{\eta}{2} \right)} |S|^{\eta/2} |W|^{-(\eta + d + 1)/2}  \exp \left(- \frac{1}{2} \mathrm{tr}\left(SW^{-1} \right) \right),} where \eqn{\Gamma_N} is the multivariate gamma function and \eqn{\mathrm{tr \left(X \right)}} is the trace of \eqn{X}.  Note that inverse Wishart distributions act over the space of all covariance matrices. When used for a correlation  matrix, only the subset of valid covariance matrices that are also valid correlation matrices are considered. Variances are parameterised by gamma distributions.
\item If \code{'beta'} is selected, then  \code{cor_params} should be a  \code{list} containing two symmetric \code{d}\eqn{\times}\code{d} matrices \eqn{A} and \eqn{B} giving the prior success parameters and prior failure parameters respectively. The correlation between the \code{i}th and \code{j}th VoI is \eqn{\rho_{i, j}} with \deqn{\frac{1}{\pi} \tan^{-1} \frac{\rho_{i, j}}{\sqrt{1-\rho_{i, j}^2}} + \frac{1}{2} \sim \mathrm{beta}(A_{i, j}, B_{i, j}).} Variances are parameterised by gamma distributions.
\item If \code{'hierarchical'} or \code{'hierarchical_beta_conjugate'} is selected, then variances are parameterised by log-normal distributions:
\deqn{\log \pi_{k, i} \sim \mathrm{N}(\mu_i, \sigma^2_i)} with priors
\deqn{\mu_i \sim \mathrm{N}(\alpha_\pi, \beta_\pi),}
\deqn{\sigma^2_i \sim \mathrm{InvGamma}(\gamma_\pi, \delta_\pi).}
The \code{var_params} slot should then be a \code{numeric} of length 4, giving the \eqn{\alpha_\pi, \beta_\pi, \gamma_\pi, \delta_\pi} hyperparameters respectively. Correlations (\eqn{\rho_{k, i, j}} where \eqn{\rho_{k, i, j}} is the correlation between VoI \eqn{i} and \eqn{j} for the \eqn{k}th simulator) are parameterised by hierarchical beta distributions.
\deqn{\frac{\rho_{k, i, j} + 1}{2} \sim \mathrm{beta}(c_{k, i, j}, d_{k, i, j})} with priors
\deqn{c_{k, i, j} \sim \mathrm{gamma}(\alpha_\rho, \beta_\rho),}
\deqn{d_{k, i, j} \sim \mathrm{gamma}(\gamma_\rho, \delta_\rho).}
NOTE: These options is only supported for the individual short-term discrepancy terms.
\item If \code{'hierarchical'} is selected, then the \code{cor_params} slot should be a \code{numeric} of length 4 giving the \eqn{\alpha_\rho, \beta_\rho, \gamma_\rho, \delta_\rho} hyperparameters. respectively. NOTE: This option is only supported for the individual short-term discrepancy terms.
\item If \code{'hierarchical_beta_conjugate'} is selected, then the \code{cor_params} slot should be a \code{numeric} of length 3. Denoting the values by \eqn{r,s,k}, they map to the hyperparameters \eqn{p, q, k} of the beta conjugate distribution via \eqn{k = k}, \eqn{p^{-1/k} = (1+e^{-s})(1+e^{-r})} and \eqn{q^{1/k} = e^{-r}(1+e^{-s})^{-1}(1+e^{-r})^{-1}}. The density of the beta conjugate distribution is defined up to a constant of proportionality by
\deqn{p(\alpha_\rho, \beta_\rho\,|\,p, q, k) \propto \frac{\Gamma(\alpha_\rho + \beta_\rho)^{k}p^{\alpha_\rho}q^{\beta_\rho}}{\Gamma(\alpha_\rho)^{k}\Gamma(\beta_\rho)^{k}}\,.}
NOTE: This option is only supported for the individual short-term discrepancy terms.
Priors may also be specified for the autoregressive parameters for discrepancies modelled using autoregressive processes (i.e. for \code{IndSTPrior} and \code{ShaSTPrior} objects). These are parametrised via beta distributions such that the autoregressive parameter \eqn{R \in (-1,1)} satisfies \deqn{\frac{R+1}{2} \sim \mathrm{Beta}(\alpha, \beta)}.
}

In addition to priors on the discrepancy terms, it is also possible to add prior information on the truth. We require priors on the truth at \eqn{t=0}. By default, a \eqn{N(0, 10)} prior is used on the initial values., however this can be configured by the \code{truth_params} argument. The covariance matrix of the random walk of the truth \eqn{\Lambda_y} can be configured using an inverse-Wishart prior. The \code{truth_params} argument should be a \code{TruthPrior} object.
}
\examples{
##### Different forms of the individual long term discrepancy priors
#LKJ(10) priors on correlation matrices and gamma(5, 3) priors on the variances
ist_lkj <- IndSTPrior("lkj", list(5, 3), 10)#

#Same as above but with an additional beta(2, 4) prior on
#the autoregressive parameter of the AR process.
ist_lkj <- IndSTPrior("lkj", list(5, 3), 10, AR_params = c(2, 4))

#Same as above but with different variance priors for 5 different variables of interest.
#This encodes that there is a gamma(1, 1) prior on the variance of the first variable,
#a gamma(23, 1) on the second variable etc...
ist_lkj <- IndSTPrior("lkj", list(c(1,23,24,6,87), c(1,1,1,1,5)), 10, AR_params = c(2, 4))

#Hierarchical priors with gamma(1,2) and gamma(10, 1) on the variance hyperparameters and
#gamma(3,4), gamma(5,6) on the correlation hyperparameters
ist_hie <- IndSTPrior("hierarchical", list(1,2,10,1), list(3,4,5,6))

#Hierarchical priors with gamma(1,2) and gamma(10, 1) on the variance hyperparameters and
#the beta conjugate prior with parameters (p = 0.75, q = 0.75, k = 0.2) on the
#correlation hyperparameters
ist_hie_beta_conj <- IndSTPrior("hierarchical_beta_conjugate",
list(1,2,10,1), list(0.75,0.75,0.2))

#Inverse Wishart correlation priors. Gamma(2, 1/3) priors are on the variances and
#inv-Wishart(5, diag(5)) on the correlation matrices.
ist_inW <- IndSTPrior("inv_wishart", list(2, 1/3),list(5, diag(5)))

##### TruthPrior
#Simple default truth prior with 7 variables of interest
truth_def <- TruthPrior(7)
# A more fine-tuned truth prior for an ensemble with 7 species.
truth_cus <- TruthPrior(7, initial_mean = 2, initial_var = 10, rw_covariance = list(10, diag(7)))

#The default priors for an ensemble with 8 variables of interest
priors <- EnsemblePrior(8)

#With 4 variables of interest.
priors <- EnsemblePrior(4)

#Defining custom priors for a model with 4 species.
num_species <- 5
priors <- EnsemblePrior(
  d = num_species,
  ind_st_params = IndSTPrior("lkj",  list(3, 2), 3, AR_params = c(2,4)),
  ind_lt_params = IndLTPrior(
    "beta",
    list(c(10,4,8, 7,6),c(2,3,1, 4,4)),
    list(matrix(5, num_species, num_species),
         matrix(0.5, num_species, num_species))
  ),
  sha_st_params = ShaSTPrior("inv_wishart",list(2, 1/3),list(5, diag(num_species))),
  sha_lt_params = 5,
  truth_params = TruthPrior(d = num_species, initial_mean = 5, initial_var = 10,
                            rw_covariance = list(10, diag(10)))
)

}
\references{
Spence et. al. (2018). A general framework for combining ecosystem models. \emph{Fish and Fisheries}, 19(6):1031-1042.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Sigma_ewe}
\alias{Sigma_ewe}
\title{Ecopath with EcoSim Sigma}
\format{
A 4x4 \code{matrix}.
}
\usage{
Sigma_ewe
}
\description{
A 4x4 covariance matrix quantifying the parameter uncertainty of Ecopath with EcoSim
}
\references{
Mackinson, S., Platts, M., Garcia, C., and Lynam, C. (2018). Evaluating the fishery and ecological consequences of the proposed North Sea multi-annual plan. PLOS ONE, 13(1), 1–23.
}
\keyword{datasets}
